@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/attribute_manager.cc,v 1.13 2007/02/16 22:45:10 pavlin Exp $"
+#ident "$XORP: xorp/bgp/attribute_manager.cc,v 1.16 2008/07/23 05:09:31 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 #include "bgp_module.h"
@@ -45,8 +45,6 @@ AttributeManager<A>::add_attribute_list(
     if (i == _attribute_lists.end()) {
 	new_att->clone_data();
 	_attribute_lists.insert(new_att);
-	debug_msg("ATMgr: Inserting new attribute %p\n",
-		  new_att->attribute());
 	return new_att->attribute();
     }
 
