@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/bgp.cc,v 1.81 2007/02/16 22:45:10 pavlin Exp $"
+#ident "$XORP: xorp/bgp/bgp.cc,v 1.93 2008/07/23 05:09:31 pavlin Exp $"
 
 // #define DEBUG_MAXIMUM_DELAY
 // #define DEBUG_LOGGING
@@ -39,40 +39,37 @@
 #include "profile_vars.hh"
 
 
-// ----------------------------------------------------------------------------
-// Static class members
-
-EventLoop BGPMain::_eventloop;
-
 // ----------------------------------------------------------------------------
 // BGPMain implementation
 
-BGPMain::BGPMain()
-    : _exit_loop(false),
-      _local_data(_eventloop),
+BGPMain::BGPMain(EventLoop& eventloop)
+    : _eventloop(eventloop),
+      _exit_loop(false),
       _component_count(0),
       _ifmgr(NULL),
-      _is_ifmgr_ready(false)
+      _is_ifmgr_ready(false),
+      _first_policy_push(false)
 {
     debug_msg("BGPMain object created\n");
     /*
     ** Ideally these data structures should be created inline. However
     ** we need to finely control the order of destruction.
     */
+    _local_data = new LocalData(_eventloop),
     _peerlist = new BGPPeerList();
     _deleted_peerlist = new BGPPeerList();
-    _xrl_router = new XrlStdRouter(eventloop(), "bgp");
+    _xrl_router = new XrlStdRouter(_eventloop, "bgp");
     _xrl_target = new XrlBgpTarget(_xrl_router, *this);
 
-    wait_until_xrl_router_is_ready(eventloop(), *_xrl_router);
+    wait_until_xrl_router_is_ready(_eventloop, *_xrl_router);
 
     _rib_ipc_handler = new RibIpcHandler(*_xrl_router, *this);
     _aggregation_handler = new AggregationHandler();
     _next_hop_resolver_ipv4 = new NextHopResolver<IPv4>(_xrl_router,
-							eventloop(),
+							_eventloop,
 							*this);
     _next_hop_resolver_ipv6 = new NextHopResolver<IPv6>(_xrl_router,
-							eventloop(),
+							_eventloop,
 							*this);
     _plumbing_unicast = new BGPPlumbing(SAFI_UNICAST,
 					_rib_ipc_handler,
@@ -90,7 +87,7 @@ BGPMain::BGPMain()
 					  *this);
     _rib_ipc_handler->set_plumbing(_plumbing_unicast, _plumbing_multicast);
 
-    _process_watch = new ProcessWatch(_xrl_router, eventloop(),
+    _process_watch = new ProcessWatch(_xrl_router, _eventloop,
 				      bgp_mib_name().c_str(),
 				      ::callback(this,
 						 &BGPMain::terminate));
@@ -194,6 +191,10 @@ BGPMain::~BGPMain()
     debug_msg("Deleting peerlist\n");
     delete _peerlist;
 
+    debug_msg("-------------------------------------------\n");
+    debug_msg("Deleting localdata\n");
+    delete _local_data;
+
     debug_msg("-------------------------------------------\n");
     debug_msg("Deleting plumbing unicast\n");
     delete _plumbing_unicast;
@@ -236,21 +237,24 @@ BGPMain::status(string& reason)
     } else if (! _is_ifmgr_ready) {
 	s = PROC_NOT_READY;
 	reason = "Waiting for interface manager";
+    } else if (!_first_policy_push) {
+	s = PROC_NOT_READY;
+	reason = "Waiting for policy manager";
     }
 
     return s;
 }
 
-bool
+int
 BGPMain::startup()
 {
     //
     // XXX: when the startup is completed,
     // IfMgrHintObserver::tree_complete() will be called.
     //
-    if (_ifmgr->startup() != true) {
+    if (_ifmgr->startup() != XORP_OK) {
 	ServiceBase::set_status(SERVICE_FAILED);
-	return (false);
+	return (XORP_ERROR);
     }
 
     component_up("startup");
@@ -258,10 +262,10 @@ BGPMain::startup()
     register_address_status(callback(this, &BGPMain::address_status_change4));
     register_address_status(callback(this, &BGPMain::address_status_change6));
 
-    return (true);
+    return (XORP_OK);
 }
 
-bool
+int
 BGPMain::shutdown()
 {
     //
@@ -329,7 +333,7 @@ BGPMain::is_interface_enabled(const string& interface) const
 {
     debug_msg("Interface %s\n", interface.c_str());
 
-    const IfMgrIfAtom* fi = ifmgr_iftree().find_if(interface);
+    const IfMgrIfAtom* fi = ifmgr_iftree().find_interface(interface);
     if (fi == NULL)
 	return false;
 
@@ -426,11 +430,11 @@ BGPMain::get_mtu(const string& interface)
 {
     debug_msg("Interface %s\n", interface.c_str());
 
-    const IfMgrIfAtom* fi = ifmgr_iftree().find_if(interface);
+    const IfMgrIfAtom* fi = ifmgr_iftree().find_interface(interface);
     if (fi == NULL)
 	return 0;
 
-    return (fi->mtu_bytes());
+    return (fi->mtu());
 }
 
 void
@@ -449,8 +453,8 @@ BGPMain::updates_made()
 {
     IfMgrIfTree::IfMap::const_iterator ii;
     IfMgrIfAtom::VifMap::const_iterator vi;
-    IfMgrVifAtom::V4Map::const_iterator ai4;
-    IfMgrVifAtom::V6Map::const_iterator ai6;
+    IfMgrVifAtom::IPv4Map::const_iterator ai4;
+    IfMgrVifAtom::IPv6Map::const_iterator ai6;
     const IfMgrIfAtom* if_atom;
     const IfMgrIfAtom* other_if_atom;
     const IfMgrVifAtom* vif_atom;
@@ -463,7 +467,7 @@ BGPMain::updates_made()
     //
     // Check whether the old interfaces, vifs and addresses are still there
     //
-    for (ii = _iftree.ifs().begin(); ii != _iftree.ifs().end(); ++ii) {
+    for (ii = _iftree.interfaces().begin(); ii != _iftree.interfaces().end(); ++ii) {
 	bool is_old_interface_enabled = false;
 	bool is_new_interface_enabled = false;
 	bool is_old_vif_enabled = false;
@@ -476,7 +480,7 @@ BGPMain::updates_made()
 	is_old_interface_enabled &= (! if_atom->no_carrier());
 
 	// Check the interface
-	other_if_atom = ifmgr_iftree().find_if(if_atom->name());
+	other_if_atom = ifmgr_iftree().find_interface(if_atom->name());
 	if (other_if_atom == NULL) {
 	    // The interface has disappeared
 	    is_new_interface_enabled = false;
@@ -581,13 +585,13 @@ BGPMain::updates_made()
     //
     // Check for new interfaces, vifs and addresses
     //
-    for (ii = ifmgr_iftree().ifs().begin();
-	 ii != ifmgr_iftree().ifs().end();
+    for (ii = ifmgr_iftree().interfaces().begin();
+	 ii != ifmgr_iftree().interfaces().end();
 	 ++ii) {
 	if_atom = &ii->second;
 
 	// Check the interface
-	other_if_atom = _iftree.find_if(if_atom->name());
+	other_if_atom = _iftree.find_interface(if_atom->name());
 	if (other_if_atom == NULL) {
 	    // A new interface
 	    if (if_atom->enabled()
@@ -679,7 +683,7 @@ BGPMain::address_status_change4(const string& interface, const string& vif,
 {
     debug_msg("interface %s vif %s address %s prefix_len %u state %s\n",
 	      interface.c_str(), vif.c_str(), cstring(source), prefix_len,
-	      pb(state));
+	      bool_c_str(state));
 
     if (state) {
 	_interfaces_ipv4.insert(make_pair(source, prefix_len));
@@ -697,7 +701,7 @@ BGPMain::address_status_change6(const string& interface, const string& vif,
 {
     debug_msg("interface %s vif %s address %s prefix_len %u state %s\n",
 	      interface.c_str(), vif.c_str(), cstring(source), prefix_len,
-	      pb(state));
+	      bool_c_str(state));
 
     if (state) {
 	_interfaces_ipv6.insert(make_pair(source, prefix_len));
@@ -801,12 +805,14 @@ BGPMain::main_loop()
 }
 
 void
-BGPMain::local_config(const uint32_t& as, const IPv4& id)
+BGPMain::local_config(const uint32_t& as, const IPv4& id,
+		      bool use_4byte_asnums)
 {
     LocalData *local = get_local_data();
 
     local->set_as(AsNum(as));
     local->set_id(id);
+    local->set_use_4byte_asnums(use_4byte_asnums);
 
 //     _plumbing_unicast->set_my_as_number(local->as());
 //     _plumbing_multicast->set_my_as_number(local->as());
@@ -1037,7 +1043,7 @@ BGPMain::create_peer(BGPPeerData *pd)
 
     SocketClient *sock = new SocketClient(pd->iptuple(), eventloop(), md5sig);
 
-    p = new BGPPeer(&_local_data, pd, sock, this);
+    p = new BGPPeer(_local_data, pd, sock, this);
     //    sock->set_eventloop(eventloop());
     sock->set_callback(callback(p, &BGPPeer::get_message));
 
@@ -1152,8 +1158,11 @@ BGPMain::local_ip_changed(string local_address)
 	const Iptuple& iptuple = (*i)->peerdata()->iptuple();
 	debug_msg("Trying: %s ", iptuple.str().c_str());
 
-	if (iptuple.get_local_addr() == local_address)
+	if (iptuple.get_local_addr() == local_address) {
+	    XLOG_INFO("Interface address %s changed bouncing peer",
+		      local_address.c_str());
 	    bounce_peer(iptuple);
+	}
     }
 }
 
@@ -1172,7 +1181,7 @@ BGPMain::change_tuple(const Iptuple& iptuple, const Iptuple& nptuple)
 	return true;
 
     BGPPeerData *pd = new 
-	BGPPeerData(_local_data, nptuple,
+	BGPPeerData(*_local_data, nptuple,
 		    peer->peerdata()->as(),
 		    peer->peerdata()->get_v4_local_addr(),
 		    peer->peerdata()->get_configured_hold_time());
@@ -1461,6 +1470,11 @@ BGPMain::activate(const Iptuple& iptuple)
 
     peer->set_activate_state(true);
 
+    // Delay the activation of the peers until we have seen the first
+    // policy push.
+    if (!_first_policy_push)
+	return true;
+
     if (peer->get_current_peer_state() == peer->get_next_peer_state())
 	return true;
 
@@ -1473,6 +1487,27 @@ BGPMain::activate(const Iptuple& iptuple)
     return true;
 }
 
+bool
+BGPMain::activate_all_peers()
+{
+    list<BGPPeer *>& peers = _peerlist->get_list();
+    list<BGPPeer *>::iterator i;
+    for (i = peers.begin(); i != peers.end(); i++) {
+	const Iptuple& iptuple = (*i)->peerdata()->iptuple();
+	debug_msg("Trying: %s ", iptuple.str().c_str());
+	if ((*i)->get_current_peer_state() == (*i)->get_next_peer_state())
+	    continue;
+
+	if ((*i)->get_next_peer_state()) {
+	    enable_peer(iptuple);
+	} else {
+	    disable_peer(iptuple);
+	}
+    }
+
+    return true;
+}
+
 bool
 BGPMain::set_peer_md5_password(const Iptuple& iptuple, const string& password)
 {
@@ -1696,7 +1731,7 @@ BGPMain::create_listener(const Iptuple& iptuple)
 LocalData*
 BGPMain::get_local_data()
 {
-    return &_local_data;
+    return _local_data;
 }
 
 void
@@ -1792,7 +1827,7 @@ BGPMain::originate_route(const IPv4Net& nlri, const IPv4& next_hop,
     debug_msg("nlri %s next hop %s unicast %d multicast %d\n",
 	      nlri.str().c_str(), next_hop.str().c_str(), unicast, multicast);
 
-    AsPath aspath;
+    ASPath aspath;
 
     return _rib_ipc_handler->originate_route(IGP, aspath, nlri,
 					     next_hop, unicast,
@@ -1807,7 +1842,7 @@ BGPMain::originate_route(const IPv6Net& nlri, const IPv6& next_hop,
     debug_msg("nlri %s next hop %s unicast %d multicast %d\n",
 	      nlri.str().c_str(), next_hop.str().c_str(), unicast, multicast);
 
-    AsPath aspath;
+    ASPath aspath;
 
     return _rib_ipc_handler->originate_route(IGP, aspath, nlri,
 					      next_hop, unicast,
@@ -1947,7 +1982,7 @@ BGPMain::set_parameter(const Iptuple& iptuple , const string& parameter,
 	return false;
     }
     
-    debug_msg("Peer %s. %s\n", iptuple.str().c_str(),toggle ? "true" :"false");
+    debug_msg("Peer %s. %s\n", iptuple.str().c_str(), bool_c_str(toggle));
 
     BGPPeerData *peerdata = const_cast<BGPPeerData *>(peer->peerdata());
     if (toggle) {
@@ -1959,17 +1994,25 @@ BGPMain::set_parameter(const Iptuple& iptuple , const string& parameter,
 }
 
 void
-BGPMain::configure_filter(const uint32_t& filter, const string& conf) {
+BGPMain::configure_filter(const uint32_t& filter, const string& conf)
+{
     _policy_filters.configure(filter,conf);
 }
 
 void
-BGPMain::reset_filter(const uint32_t& filter) {
+BGPMain::reset_filter(const uint32_t& filter)
+{
     _policy_filters.reset(filter);
 }
 
 void
-BGPMain::push_routes() {
+BGPMain::push_routes()
+{
     _plumbing_unicast->push_routes();
     _plumbing_multicast->push_routes();
+
+    if (!_first_policy_push) {
+	_first_policy_push = true;
+	activate_all_peers();
+    }
 }
