@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/bgp_trie.cc,v 1.19 2007/02/16 22:45:10 pavlin Exp $"
+#ident "$XORP: xorp/bgp/bgp_trie.cc,v 1.24 2008/07/23 05:09:31 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 
@@ -110,7 +110,16 @@ BgpTrie<A>::erase(const IPNet& net)
     debug_msg("deleting route for %s with attributes %p", net.str().c_str(),
 	   found->attributes());
     typename PathmapType::iterator pmi = _pathmap.find(found->attributes());
-    XLOG_ASSERT(pmi != _pathmap.end());
+    if (pmi == _pathmap.end()) {
+	XLOG_ERROR("Error deleting route for %s with attributes %p %s", 
+		   net.str().c_str(), found->attributes(), 
+		   found->attributes()->str().c_str());
+	XLOG_INFO("Pathmap dump follows: \n");
+	for (pmi == _pathmap.begin(); pmi != _pathmap.end(); pmi++) {
+	    XLOG_INFO("%p %s\n\n", pmi->first, pmi->second->str().c_str());
+	}
+	XLOG_FATAL("Exiting\n");
+    }
     if (pmi->second == found) {		// this was the head node
 	if (found->next() == found) {	 // it's the only node in the chain
 	    _pathmap.erase(pmi);
