@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/dummy_main.cc,v 1.19 2007/02/16 22:45:11 pavlin Exp $"
+#ident "$XORP: xorp/bgp/dummy_main.cc,v 1.24 2008/07/23 05:09:32 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -26,10 +26,10 @@
 #include "iptuple.hh"
 
 
-EventLoop BGPMain::_eventloop;
-
-BGPMain::BGPMain() : _local_data(_eventloop)
+BGPMain::BGPMain(EventLoop& eventloop)
+    : _eventloop(eventloop)
 {
+    _local_data = new LocalData(_eventloop);
     _xrl_router = NULL;
 }
 
@@ -41,19 +41,19 @@ BGPMain::~BGPMain() {
     if (_xrl_router != NULL)
 	while(_xrl_router->pending())
 	    eventloop().run();
-    
+    delete _local_data;
 }
 
-bool
+int
 BGPMain::startup()
 {
-    return (true);
+    return (XORP_OK);
 }
 
-bool
+int
 BGPMain::shutdown()
 {
-    return (true);
+    return (XORP_OK);
 }
 
 void
@@ -108,7 +108,7 @@ BGPMain::main_loop()
 }	
 
 void 
-BGPMain::local_config(const uint32_t&, const IPv4&)
+BGPMain::local_config(const uint32_t&, const IPv4&, bool)
 {
 }
 
