@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/dump_iterators.cc,v 1.32 2007/02/16 22:45:11 pavlin Exp $"
+#ident "$XORP: xorp/bgp/dump_iterators.cc,v 1.35 2008/07/23 05:09:32 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 //#define DEBUG_PRINT_FUNCTION_NAME
@@ -57,7 +57,7 @@ string
 PeerDumpState<A>::str() const
 {
     return c_format("peer: %p routes_dumped: %s last_net: %s, genid: %d\n",
-		    _peer, _routes_dumped ? "true" : "false",
+		    _peer, bool_c_str(_routes_dumped),
 		    _last_net_before_down.str().c_str(),
 		    XORP_INT_CAST(_genid));
 
