@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/coord.cc,v 1.27 2007/02/16 22:45:25 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/coord.cc,v 1.31 2008/07/23 05:09:43 pavlin Exp $"
 
 #include "bgp/bgp_module.h"
 
@@ -33,7 +33,6 @@
 #include "coord.hh"
 #include "command.hh"
 
-
 static const char SERVER[] = "coord";/* This servers name */
 static const char SERVER_VERSION[] = "0.1";
 
@@ -94,8 +93,7 @@ XrlCoordTarget::coord_0_1_command(const string&	command)
     } catch(const XorpException& e) {
 	_incommand--;
 	return XrlCmdError::COMMAND_FAILED(e.why() + "\nPending operation: " +
-					   (_coord.pending() ?
-					   "true" : "false"));
+					   bool_c_str(_coord.pending()));
     }
     _incommand--;
 
@@ -113,8 +111,7 @@ XrlCoordTarget::coord_0_1_status(const string& peer, string& status)
     } catch(const XorpException& e) {
 	_incommand--;
 	return XrlCmdError::COMMAND_FAILED(e.why() + "\nPending operation: " +
-					   (_coord.pending() ?
-					   "true" : "false"));
+					   bool_c_str(_coord.pending()));
     }
     _incommand--;
 
