@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.45 2007/02/16 22:45:26 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.52 2008/07/23 05:09:43 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -131,6 +131,22 @@ XrlTestPeerTarget::test_peer_0_1_packetisation(const string& protocol)
     return XrlCmdError::OKAY();
 }
 
+/* test peer needs to know whether to assume ASPaths are 2-byte or
+   4-byte encoded */
+
+XrlCmdError
+XrlTestPeerTarget::test_peer_0_1_use_4byte_asnums(const bool& use)
+{
+    debug_msg("\n");
+
+    if(_trace)
+	printf("use_4byte_asnums(%d)\n", use);
+
+    _test_peer.use_4byte_asnums(use);
+
+    return XrlCmdError::OKAY();
+}
+
 XrlCmdError
 XrlTestPeerTarget::test_peer_0_1_connect(const string&	host,
 					 const uint32_t&  port)
@@ -249,7 +265,8 @@ XrlTestPeerTarget::test_peer_0_1_terminate()
 */
 TestPeer::TestPeer(EventLoop& eventloop, XrlRouter& xrlrouter,
 		   const char *server, bool verbose)
-    : _eventloop(eventloop), _xrlrouter(xrlrouter), _server(server),
+    : _eventloop(eventloop), _xrlrouter(xrlrouter), 
+      _server(server),
       _verbose(verbose),
      _done(false),
       _async_writer(0),
@@ -257,10 +274,14 @@ TestPeer::TestPeer(EventLoop& eventloop, XrlRouter& xrlrouter,
       _flying(0),
      _bgp_bytes(0)
 {
+    _localdata = new LocalData(eventloop);
+    _peerdata = new BGPPeerData(*_localdata, Iptuple(), AsNum(0), IPv4(), 0);
 }
 
 TestPeer::~TestPeer()
 {
+    delete _peerdata;
+    delete _localdata;
     delete _async_writer;
 }
 
@@ -303,6 +324,16 @@ TestPeer::packetisation(const string& protocol)
     return true;
 }
 
+void
+TestPeer::use_4byte_asnums(bool use)
+{
+    _localdata->set_use_4byte_asnums(use);
+
+    // normally this is negotiated, but this is a test peer so we want
+    // to be able to force the parsing of ASnums either way.
+    _peerdata->set_use_4byte_asnums(use);
+}
+
 bool
 TestPeer::connect(const string& host, const uint32_t& port,
 		  string& error_string)
@@ -409,6 +440,12 @@ TestPeer::listen(const string& host, const uint32_t& port,
 	return false;
     }
 
+    if (comm_listen(s, COMM_LISTEN_DEFAULT_BACKLOG) != XORP_OK) {
+	error_string = c_format("comm_listen() failed: %s\n",
+				comm_get_last_error_str());
+	return false;
+    }
+
     if(!_eventloop.add_ioevent_cb(s, IOT_ACCEPT,
 				callback(this,
 					 &TestPeer::connect_attempt))) {
@@ -499,7 +536,7 @@ TestPeer::send(const vector<uint8_t>& data, string& error_string)
     _async_writer->start();
 
     if(_verbose && _bgp)
-	printf("Sending: %s", bgppp(buf, len).c_str());
+	printf("Sending: %s", bgppp(buf, len, _peerdata).c_str());
 
     return true;
 }
@@ -748,7 +785,7 @@ TestPeer::datain(status st, uint8_t *ptr, size_t len, string error)
 	switch(st) {
 	case GOOD:
 	    if(_bgp)
-		printf("Received: %s", bgppp(ptr, len).c_str());
+		printf("Received: %s", bgppp(ptr, len, _peerdata).c_str());
 	    break;
 	case CLOSED:
 	    printf("Connection closed by peer\n");
