@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_peering2.sh,v 1.55 2006/08/16 22:10:14 atanu Exp $
+# $XORP: xorp/bgp/harness/test_peering2.sh,v 1.62 2008/04/25 19:10:01 atanu Exp $
 #
 
 #
@@ -48,22 +48,38 @@ HOST=127.0.0.1
 LOCALHOST=$HOST
 ID=192.150.187.78
 AS=65008
+USE4BYTEAS=false
 
-# IBGP
+# IBGP - IPV4
 PORT1=10001
 PEER1_PORT=20001
 PEER1_AS=$AS
 
-# EBGP
+# EBGP - IPV4
 PORT2=10002
 PEER2_PORT=20002
 PEER2_AS=65000
 
-# EBGP
+# EBGP - IPV4
 PORT3=10003
 PEER3_PORT=20003
 PEER3_AS=65003
 
+# IBGP - IPV6
+PORT4=10004
+PEER4_PORT=20004
+PEER4_AS=$AS
+
+# EBGP - IPV6
+PORT5=10005
+PEER5_PORT=20005
+PEER5_AS=65000
+
+# EBGP - IPV6
+PORT6=10006
+PEER6_PORT=20006
+PEER6_AS=65003
+
 HOLDTIME=5
 
 TRAFFIC_DIR="${srcdir}/../../../data/bgp"
@@ -76,28 +92,52 @@ EXT=${LOGNAME:-unknown}
 
 configure_bgp()
 {
-    local_config $AS $ID
+    local_config $AS $ID $USE4BYTEAS
 
     # Don't try and talk to the rib.
     register_rib ""
 
+    # IBGP - IPV4
     PEER=$HOST
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT1 $PEER $PEER1_PORT $PEER1_AS $NEXT_HOP $HOLDTIME
     set_parameter $LOCALHOST $PORT1 $PEER $PEER1_PORT MultiProtocol.IPv4.Unicast true
     enable_peer $LOCALHOST $PORT1 $PEER $PEER1_PORT
 
+    # EBGP - IPV4
     PEER=$HOST
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT2 $PEER $PEER2_PORT $PEER2_AS $NEXT_HOP $HOLDTIME
     set_parameter $LOCALHOST $PORT2 $PEER $PEER2_PORT MultiProtocol.IPv4.Unicast true
     enable_peer $LOCALHOST $PORT2 $PEER $PEER2_PORT
 
+    # EBGP - IPV4
     PEER=$HOST
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT3 $PEER $PEER3_PORT $PEER3_AS $NEXT_HOP $HOLDTIME
     set_parameter $LOCALHOST $PORT3 $PEER $PEER3_PORT MultiProtocol.IPv4.Unicast true
     enable_peer $LOCALHOST $PORT3 $PEER $PEER3_PORT
+
+    # IBGP - IPV6
+    PEER=$HOST
+    NEXT_HOP=192.150.187.78
+    add_peer $LOCALHOST $PORT4 $PEER $PEER4_PORT $PEER4_AS $NEXT_HOP $HOLDTIME
+    set_parameter $LOCALHOST $PORT4 $PEER $PEER4_PORT MultiProtocol.IPv6.Unicast true
+    enable_peer $LOCALHOST $PORT4 $PEER $PEER4_PORT
+
+    # EBGP - IPV6
+    PEER=$HOST
+    NEXT_HOP=192.150.187.78
+    add_peer $LOCALHOST $PORT5 $PEER $PEER5_PORT $PEER5_AS $NEXT_HOP $HOLDTIME
+    set_parameter $LOCALHOST $PORT5 $PEER $PEER5_PORT MultiProtocol.IPv6.Unicast true
+    enable_peer $LOCALHOST $PORT5 $PEER $PEER5_PORT
+
+    # EBGP - IPV6
+    PEER=$HOST
+    NEXT_HOP=192.150.187.78
+    add_peer $LOCALHOST $PORT6 $PEER $PEER6_PORT $PEER6_AS $NEXT_HOP $HOLDTIME
+    set_parameter $LOCALHOST $PORT6 $PEER $PEER6_PORT MultiProtocol.IPv6.Unicast true
+    enable_peer $LOCALHOST $PORT6 $PEER $PEER6_PORT
 }
 
 wait_for_peerdown()
@@ -166,7 +206,7 @@ bgp_peer_unchanged()
 	status $1
 
 	a=$(status $1)
-	sleep 2
+	sleep 10
 	b=$(status $1)
 	if [ "$a" = "$b" ]
 	then
@@ -223,6 +263,29 @@ test1()
     coord peer2 assert established
 }
 
+# This function exists to test saved IPv6 feeds and is not normally run.
+test1_ipv6()
+{
+    TFILE=$TFILE
+
+    echo "TEST1 IPV6 - Inject a saved feed then drop peering - $TFILE"
+
+    # Reset the peers
+    coord reset
+    coord target $HOST $PORT5
+    coord initialise attach peer2
+
+    coord peer2 establish AS $PEER5_AS holdtime 0 id 192.150.187.102 ipv6 true
+    coord peer2 assert established
+
+    NOBLOCK=true coord peer2 send dump mrtd update $TFILE
+
+    # Wait for the file to be transmitted by the test peer.
+    bgp_peer_unchanged peer2
+
+    coord peer2 assert established
+}
+
 test2()
 {
     TFILE=$1
@@ -249,10 +312,7 @@ test2()
     # Bring up another peering to test the dump code.
     for i in 1 2
     do
-	coord reset
-
-	coord target $HOST $PORT1
-	coord initialise attach peer1
+        coord peer1 disconnect
 
 	# Wait for the peering to be dropped
 	while status peer1 | grep established
@@ -263,18 +323,21 @@ test2()
 	sleep 2
 
 	coord peer1 establish AS $PEER1_AS holdtime 0 id 192.150.187.101
+	sleep 2
 	coord peer1 assert established
 
 	# Wait for the BGP process to send all the updates to peer1
 	bgp_peer_unchanged peer1
     done
 
-    # The reset above will have removed state about peer2 from the coordinator
-    # but the tcp connection between test_peer2 and the bgp process will not
-    # have been dropped. So by re-initialising like this we will toggle the
-    # original connection.
-    coord target $HOST $PORT2
-    coord initialise attach peer2
+    coord peer2 disconnect
+
+    while status peer2 | grep established
+    do
+	sleep 2
+    done
+
+    sleep 2
 
     coord peer2 establish AS $PEER2_AS holdtime 0 id 192.150.187.102
     coord peer2 assert established
@@ -349,20 +412,7 @@ test4()
     # Bring up a second peering and wait for all the updates to arrive
     coord peer1 establish AS $PEER1_AS holdtime 0 id 192.150.187.101
 
-    while :
-    do
-	# debug
-	status peer1
-	status peer2
-
-	a=$(status peer1)
-	sleep 2
-	b=$(status peer1)
-	if [ "$a" = "$b" ]
-	then
-	    break
-	fi
-    done
+    bgp_peer_unchanged peer1
 
     # Make sure that the peer1 connection is still established
     coord peer1 assert established
@@ -397,20 +447,7 @@ test5()
     # Bring up a second peering and wait for all the updates to arrive
     coord peer1 establish AS $PEER1_AS holdtime 0 id 192.150.187.101
 
-    while :
-    do
-	# debug
-	status peer1
-	status peer2
-
-	a=$(status peer1)
-	sleep 2
-	b=$(status peer1)
-	if [ "$a" = "$b" ]
-	then
-	    break
-	fi
-    done
+    bgp_peer_unchanged peer1
 
     coord reset
 
@@ -469,20 +506,7 @@ test6()
     # Bring up a second peering and wait for all the updates to arrive
     coord peer1 establish AS $PEER1_AS holdtime 0 id 192.150.187.101
 
-    while :
-    do
-	# debug
-	status peer1
-	status peer2
-
-	a=$(status peer1)
-	sleep 2
-	b=$(status peer1)
-	if [ "$a" = "$b" ]
-	then
-	    break
-	fi
-    done
+    bgp_peer_unchanged peer1
 
     status peer1
     status peer2
@@ -777,6 +801,7 @@ test15()
 
     # Establish the EBGP peering.
     coord peer1 establish AS $PEER1_AS holdtime 0 id 192.150.187.101
+    sleep 2
     coord peer1 assert established
 
     # Save the traffic that we send on peer1
