@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_rib_fea1.sh,v 1.17 2006/08/16 22:10:14 atanu Exp $
+# $XORP: xorp/bgp/harness/test_rib_fea1.sh,v 1.19 2008/05/14 07:00:47 pavlin Exp $
 #
 
 #
@@ -46,6 +46,7 @@ if [ "X${srcdir}" = "X" ] ; then srcdir=`dirname $0` ; fi
 
 HOST=127.0.0.1
 AS=65008
+USE4BYTEAS=false
 
 # EBGP - IPV4
 PORT1=10001
@@ -95,7 +96,7 @@ configure_bgp()
 {
     LOCALHOST=$HOST
     ID=192.150.187.78
-    local_config $AS $ID
+    local_config $AS $ID $USE4BYTEAS
 
     # EBGP - IPV4
     PEER=$HOST
@@ -204,6 +205,12 @@ configure_fea()
     $FEA_FUNCS set_prefix6 $tid $VIF0_IPV6 $VIF0_IPV6 $NH1_IPV6 24
     $FEA_FUNCS set_prefix6 $tid $VIF1_IPV6 $VIF1_IPV6 $NH2_IPV6 24
 
+    $FEA_FUNCS enable_address4 $tid $VIF0 $VIF0 $NH1
+    $FEA_FUNCS enable_address4 $tid $VIF1 $VIF1 $NH2
+
+    $FEA_FUNCS enable_address6 $tid $VIF0_IPV6 $VIF0_IPV6 $NH1_IPV6
+    $FEA_FUNCS enable_address6 $tid $VIF1_IPV6 $VIF1_IPV6 $NH2_IPV6
+
     $FEA_FUNCS commit_fea_transaction $tid
 }
 
