@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_routing1.sh,v 1.24 2006/08/16 22:10:14 atanu Exp $
+# $XORP: xorp/bgp/harness/test_routing1.sh,v 1.26 2007/12/10 23:26:33 mjh Exp $
 #
 
 #
@@ -59,6 +59,7 @@ PEER_PORT1_IPV6=20004
 PEER_PORT2_IPV6=20005
 PEER_PORT3_IPV6=20006
 AS=65008
+USE4BYTEAS=false 
 PEER1_AS=64001
 PEER2_AS=64002
 PEER3_AS=$AS
@@ -66,7 +67,7 @@ PEER1_AS_IPV6=64004
 PEER2_AS_IPV6=64005
 PEER3_AS_IPV6=$AS
 
-HOLDTIME=20
+HOLDTIME=40
 
 # Next Hops
 #NH1=10.10.10.10
@@ -91,7 +92,8 @@ configure_bgp()
     LOCALHOST=$HOST
     AS=65008
     ID=192.150.187.78
-    local_config $AS $ID
+    USE4BYTEAS=false 
+    local_config $AS $ID $USE4BYTEAS
 
     # EBGP - IPV4
     PEER=$HOST
