@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/notification_packet.cc,v 1.31 2007/02/16 22:45:13 pavlin Exp $"
+#ident "$XORP: xorp/bgp/notification_packet.cc,v 1.35 2008/07/23 05:09:33 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -54,7 +54,7 @@ NotificationPacket::NotificationPacket(uint8_t		ec,
     } else
 	_error_data = 0;
 
-    debug_msg(str().c_str());
+    debug_msg("%s", str().c_str());
 }
 
 NotificationPacket::NotificationPacket(const uint8_t *d, uint16_t l)
@@ -82,13 +82,16 @@ NotificationPacket::NotificationPacket(const uint8_t *d, uint16_t l)
     return;
 }
 
-const uint8_t *
-NotificationPacket::encode(size_t& len, uint8_t *buf) const
+bool
+NotificationPacket::encode(uint8_t *buf, size_t& len, const BGPPeerData *peerdata) const
 {
+    UNUSED(peerdata);
+    XLOG_ASSERT(buf != 0);
     debug_msg("Encode in NotificationPacket called (%u)\n",
 	      XORP_UINT_CAST(_Length));
-    if (buf != 0)		// have a buffer, check length
-	assert(len >= _Length);	// XXX should become an exception
+
+    if (len < _Length)
+	return false;
 
     len = _Length;
 
@@ -99,7 +102,7 @@ NotificationPacket::encode(size_t& len, uint8_t *buf) const
     if (_error_data != 0)
 	memcpy(buf + BGPPacket::MINNOTIFICATIONPACKET, _error_data,
 		len - BGPPacket::MINNOTIFICATIONPACKET);
-    return buf;
+    return true;
 }
 
 bool
