@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer_data.cc,v 1.33 2007/02/16 22:45:14 pavlin Exp $"
+#ident "$XORP: xorp/bgp/peer_data.cc,v 1.36 2008/07/23 05:09:34 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -31,6 +31,7 @@ BGPPeerData::BGPPeerData(const LocalData& local_data, const Iptuple& iptuple,
 			 AsNum as,
 			 const IPv4& next_hop, const uint16_t holdtime)
     : _local_data(local_data), _iptuple(iptuple), _as(as),
+      _use_4byte_asnums(false),
       _route_reflector(false), _confederation(false),
       _prefix_limit(0, false),
       _delay_open_time(0),
@@ -42,6 +43,12 @@ BGPPeerData::BGPPeerData(const LocalData& local_data, const Iptuple& iptuple,
 
     set_retry_duration(2 * 60);	// Connect retry time.
 
+    // if we're configured to use 4byte AS numbers, make sure we send
+    // the relevant capability.
+    if (_local_data.use_4byte_asnums()) {
+	add_sent_parameter( new BGP4ByteASCapability(as) );
+    }
+
     // we support route refresh
     // add_sent_parameter( new BGPRefreshCapability() );
     // add_sent_parameter( new BGPMultiRouteCapability() );
@@ -392,6 +399,20 @@ BGPPeerData::open_negotiation()
 	    }
 	}
     }
+
+    // if we are configured for 4 byte AS numbers, and the other side
+    // sent this capability too, then extract their AS number from it.
+    if (_local_data.use_4byte_asnums()) {
+	for(iter_recv = _recv_parameters.begin(); 
+	    iter_recv != _recv_parameters.end(); iter_recv++) {
+	    ParameterNode& pn = *iter_recv;
+	    if(const BGP4ByteASCapability *cap4byte = 
+	       dynamic_cast<const BGP4ByteASCapability *>(pn.get())) {
+		_use_4byte_asnums = true;
+		_as = AsNum(cap4byte->as());
+	    }
+	}
+    }
 }
 
 void
