@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer_list.cc,v 1.24 2007/02/16 22:45:14 pavlin Exp $"
+#ident "$XORP: xorp/bgp/peer_list.cc,v 1.28 2008/07/23 05:09:34 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -104,21 +104,25 @@ BGPPeerList::detach_peer(BGPPeer *p)
     //really frequest operation, we don't mind iterating through the
     //whole map of readers.
     map <uint32_t, list<BGPPeer *>::iterator>::iterator mi;
-    for (mi = _readers.begin(); mi != _readers.end(); mi++) {
+    for (mi = _readers.begin(); mi != _readers.end(); ) {
+	uint32_t token = mi->first;
 	list<BGPPeer *>::iterator pli = mi->second;
+	++mi;
 	if (*pli == p) {
 	    pli++;
-	    _readers[mi->first] = pli;
+	    _readers.erase(token);
+	    _readers.insert(make_pair(token, pli));
 	}
     }
 
     //Now it's safe to do the deletion.
     list<BGPPeer *>::iterator i;
-    for(i = _peers.begin(); i != _peers.end(); i++)
-	if(p == *i) {
+    for (i = _peers.begin(); i != _peers.end(); i++) {
+	if (p == *i) {
 	    _peers.erase(i);
 	    return;
 	}
+    }
     XLOG_FATAL("Peer %s not found in peerlist", p->str().c_str());
 }
 
@@ -173,6 +177,7 @@ BGPPeerList::get_peer_list_next(const uint32_t& token,
 	_readers.erase(mi);
 	return false;
     }
-    _readers[token] = i;
+    _readers.erase(token);
+    _readers.insert(make_pair(token, i));
     return true;
 }
