@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.75 2007/03/06 23:44:20 atanu Exp $"
+#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.78 2008/07/23 05:09:35 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -290,7 +290,7 @@ RibIpcHandler::push_packet()
 }
 
 bool 
-RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
+RibIpcHandler::originate_route(const OriginType origin, const ASPath& aspath,
 			       const IPv4Net& nlri, const IPv4& next_hop,
 			       const bool& unicast, const bool& multicast, 
 			       const PolicyTags& policytags)
@@ -343,7 +343,7 @@ RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
 }
 
 bool 
-RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
+RibIpcHandler::originate_route(const OriginType origin, const ASPath& aspath,
 			       const IPv6Net& nlri, const IPv6& next_hop,
 			       const bool& unicast, const bool& multicast,
 			       const PolicyTags& policytags)
