@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_decision.cc,v 1.44 2007/02/16 22:45:16 pavlin Exp $"
+#ident "$XORP: xorp/bgp/route_table_decision.cc,v 1.49 2008/07/23 05:09:36 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -435,6 +435,8 @@ DecisionTable<A>::lookup_route(const IPNet<A> &net,
 	genid = winner->genid();
 	return winner->route();
     }
+    XLOG_UNREACHABLE();
+    return NULL;
 }
 
 
@@ -605,7 +607,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	    i = alternatives.erase(i);
 	} else if (lp > test_pref) {
 	    test_pref = lp;
-	    alternatives.pop_front();
+	    alternatives.erase(alternatives.begin(), i);
 	    i++;
 	} else {
 	    i++;
@@ -642,7 +644,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	    i = alternatives.erase(i);
 	} else if (len < test_aspath_length) {
 	    test_aspath_length = len;
-	    alternatives.pop_front();
+	    alternatives.erase(alternatives.begin(), i);
 	} else {
 	    i++;
 	}
@@ -666,7 +668,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	    i = alternatives.erase(i);
 	} else if (origin < test_origin) {
 	    test_origin = origin;
-	    alternatives.pop_front();
+	    alternatives.erase(alternatives.begin(), i);
 	    i++;
 	} else {
 	    i++;
@@ -683,7 +685,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
     */
     typename list <RouteData<A> >::iterator j;
     for (i=alternatives.begin(); i!=alternatives.end();) {
-	AsPath aspath1 = i->route()->attributes()->aspath();
+	ASPath aspath1 = i->route()->attributes()->aspath();
  	AsNum asnum1 = (0 == aspath1.path_length()) ? 
  	    AsNum(AsNum::AS_INVALID) : aspath1.first_asnum();
 	int med1 = med(i->route());
@@ -691,7 +693,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	for (j=alternatives.begin(); j!=alternatives.end();) {
 	    bool del_j = false;
 	    if (i != j) {
-		AsPath aspath2 = j->route()->attributes()->aspath();
+		ASPath aspath2 = j->route()->attributes()->aspath();
 		AsNum asnum2 = (0 == aspath2.path_length()) ? 
 		    AsNum(AsNum::AS_INVALID) : aspath2.first_asnum();
 		int med2 = med(j->route());
@@ -735,7 +737,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	    i = alternatives.erase(i);
 	} else if (test_ibgp && !ibgp) {
 	    //test route is internal, alternative is external
-	    alternatives.pop_front();
+	    alternatives.erase(alternatives.begin(), i);
 	    test_ibgp = ibgp;
 	    i++;
 	} else {
@@ -760,7 +762,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	if (test_igp_distance < igp_dist) {
 	    i = alternatives.erase(i);
 	} else if (test_igp_distance > igp_dist) {
-	    alternatives.pop_front();
+	    alternatives.erase(alternatives.begin(), i);
 	    test_igp_distance = igp_dist;
 	    i++;
 	} else {
@@ -785,7 +787,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	if (test_id < id) {
 	    i = alternatives.erase(i);
 	} else if (test_id > id) {
-	    alternatives.pop_front();
+	    alternatives.erase(alternatives.begin(), i);
 	    test_id = id;
 	    i++;
 	} else {
@@ -812,7 +814,7 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
 	if (test_address < address) {
 	    i = alternatives.erase(i);
 	} else if (test_address > address) {
-	    alternatives.pop_front();
+	    alternatives.erase(alternatives.begin(), i);
 	    test_address = address;
 	    i++;
 	} else {
