@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/socket.cc,v 1.46 2007/02/16 22:45:19 pavlin Exp $"
+#ident "$XORP: xorp/bgp/socket.cc,v 1.56 2008/07/23 05:09:38 pavlin Exp $"
 
 // #define DEBUG_LOGGING 
 // #define DEBUG_PRINT_FUNCTION_NAME 
@@ -64,6 +64,9 @@ Socket::create_listener()
     if (!_s.is_valid()) {
 	XLOG_ERROR("comm_bind_tcp failed");
     }
+    if (comm_listen(_s, COMM_LISTEN_DEFAULT_BACKLOG) != XORP_OK) {
+	XLOG_ERROR("comm_listen failed");
+    }
 }
 
 /* **************** BGPSocket - PROTECTED METHODS *********************** */
@@ -488,6 +491,7 @@ SocketClient::connect_socket_complete(XorpFd sock, IoEventType type,
 				      ConnectCallback cb)
 {
     int soerror;
+    int is_connected = 0;
     socklen_t len = sizeof(soerror);
 
     debug_msg("connect socket complete %s %d\n", sock.str().c_str(), type);
@@ -502,11 +506,16 @@ SocketClient::connect_socket_complete(XorpFd sock, IoEventType type,
     eventloop().remove_ioevent_cb(sock);
 
     // Did the connection succeed?
-    if (comm_sock_is_connected(sock) != XORP_OK) {
+    if (comm_sock_is_connected(sock, &is_connected) != XORP_OK) {
 	debug_msg("connect failed (comm_sock_is_connected: %s) %s\n",
 		  comm_get_last_error_str(), sock.str().c_str());
 	goto failed;
     }
+    if (is_connected == 0) {
+	debug_msg("connect failed (comm_sock_is_connected: false) %s\n",
+		  sock.str().c_str());
+	goto failed;
+    }
 
     // Check for a pending socket error if one exists.
     if (getsockopt(sock, SOL_SOCKET, SO_ERROR, XORP_SOCKOPT_CAST(&soerror),
@@ -546,13 +555,17 @@ SocketClient::async_add(XorpFd sock)
 	XLOG_FATAL("Failed to go non-blocking");
 
     XLOG_ASSERT(0 == _async_writer);
-    // we use high priority for the file writer so that we don't
-    // generally receive data in faster than we can send it back out
-    _async_writer = new AsyncFileWriter(eventloop(), sock,
-					XorpTask::PRIORITY_HIGH);
+    _async_writer = new AsyncFileWriter(eventloop(), sock);
 
     XLOG_ASSERT(0 == _async_reader);
-    _async_reader = new AsyncFileReader(eventloop(), sock);
+    //
+    // XXX: We use lowest priority for the file reader so that we don't
+    // generally receive data in faster than we can send it back out.
+    // Also, the priority is lower than the tasks' background priority
+    // to avoid being overloaded by high volume data from the peers.
+    //
+    _async_reader = new AsyncFileReader(eventloop(), sock,
+					XorpTask::PRIORITY_LOWEST);
 
     async_read_start();
 }
