@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_nhlookup.cc,v 1.34 2007/02/16 22:45:20 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_nhlookup.cc,v 1.38 2008/07/23 05:09:39 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -76,7 +76,8 @@ test_nhlookup(TestInfo& /*info*/)
     free(tmppath);
 #endif
 
-    BGPMain bgpmain;
+    EventLoop eventloop;
+    BGPMain bgpmain(eventloop);
     LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
     BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
@@ -117,19 +118,19 @@ test_nhlookup(TestInfo& /*info*/)
 
     OriginAttribute igp_origin_att(IGP);
 
-    AsPath aspath1;
+    ASPath aspath1;
     aspath1.prepend_as(AsNum(1));
     aspath1.prepend_as(AsNum(2));
     aspath1.prepend_as(AsNum(3));
     ASPathAttribute aspathatt1(aspath1);
 
-    AsPath aspath2;
+    ASPath aspath2;
     aspath2.prepend_as(AsNum(4));
     aspath2.prepend_as(AsNum(5));
     aspath2.prepend_as(AsNum(6));
     ASPathAttribute aspathatt2(aspath2);
 
-    AsPath aspath3;
+    ASPath aspath3;
     aspath3.prepend_as(AsNum(7));
     aspath3.prepend_as(AsNum(8));
     aspath3.prepend_as(AsNum(9));
