@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_plumbing.cc,v 1.22 2007/02/16 22:45:21 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_plumbing.cc,v 1.26 2008/07/23 05:09:39 pavlin Exp $"
 #include "bgp_module.h"
 
 #include "libxorp/debug.h"
@@ -83,8 +83,8 @@ PlumbingTest::test1()
     printf("ADD A ROUTE\n");
     IPv4 nhaddr("20.20.20.1");
     NextHopAttribute<IPv4> nexthop(nhaddr);
-    AsPath as_path;
-    AsSegment as_seq;
+    ASPath as_path;
+    ASSegment as_seq;
     as_seq.set_type(AS_SEQUENCE);
     as_seq.add_as(AsNum(666));
     as_path.add_segment(as_seq);
@@ -285,8 +285,8 @@ PlumbingTest::test2()
     printf("ADD A ROUTE\n");
     IPv4 nhaddr("20.20.20.1");
     NextHopAttribute<IPv4> nexthop(nhaddr);
-    AsPath as_path;
-    AsSegment as_seq;
+    ASPath as_path;
+    ASSegment as_seq;
     as_seq.set_type(AS_SEQUENCE);
     as_seq.add_as(AsNum(666));
     as_path.add_segment(as_seq);
@@ -394,7 +394,8 @@ int main(int /* argc */, char *argv[])
     }
 #endif	// !HOST_OS_WINDOWS
 
-	BGPMain bgpm;
+	EventLoop eventloop;
+	BGPMain bgpm(eventloop);
 
 	PlumbingTest *tester;
  	tester = (PlumbingTest*)(bgpm.plumbing_unicast());
