@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2006-2007 International Computer Science Institute
+// Copyright (c) 2006-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_policy.cc,v 1.7 2007/02/16 22:45:22 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_policy.cc,v 1.11 2008/07/23 05:09:39 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -51,7 +51,8 @@ test_policy_export(TestInfo& /*info*/)
     string filename = string(tmppath) + "test_policy_export";
     free(tmppath);
 #endif
-    BGPMain bgpmain;
+    EventLoop eventloop;
+    BGPMain bgpmain(eventloop);
     LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
     BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
@@ -94,19 +95,19 @@ test_policy_export(TestInfo& /*info*/)
 
     OriginAttribute igp_origin_att(IGP);
 
-    AsPath aspath1;
+    ASPath aspath1;
     aspath1.prepend_as(AsNum(1));
     aspath1.prepend_as(AsNum(2));
     aspath1.prepend_as(AsNum(3));
     ASPathAttribute aspathatt1(aspath1);
 
-    AsPath aspath2;
+    ASPath aspath2;
     aspath2.prepend_as(AsNum(4));
     aspath2.prepend_as(AsNum(5));
     aspath2.prepend_as(AsNum(6));
     ASPathAttribute aspathatt2(aspath2);
 
-    AsPath aspath3;
+    ASPath aspath3;
     aspath3.prepend_as(AsNum(7));
     aspath3.prepend_as(AsNum(8));
     aspath3.prepend_as(AsNum(9));
@@ -281,7 +282,8 @@ test_policy(TestInfo& /*info*/)
     string filename = string(tmppath) + "test_policy";
     free(tmppath);
 #endif
-    BGPMain bgpmain;
+    EventLoop eventloop;
+    BGPMain bgpmain(eventloop);
     LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
     BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
@@ -336,19 +338,19 @@ test_policy(TestInfo& /*info*/)
 
     OriginAttribute igp_origin_att(IGP);
 
-    AsPath aspath1;
+    ASPath aspath1;
     aspath1.prepend_as(AsNum(1));
     aspath1.prepend_as(AsNum(2));
     aspath1.prepend_as(AsNum(3));
     ASPathAttribute aspathatt1(aspath1);
 
-    AsPath aspath2;
+    ASPath aspath2;
     aspath2.prepend_as(AsNum(4));
     aspath2.prepend_as(AsNum(5));
     aspath2.prepend_as(AsNum(6));
     ASPathAttribute aspathatt2(aspath2);
 
-    AsPath aspath3;
+    ASPath aspath3;
     aspath3.prepend_as(AsNum(7));
     aspath3.prepend_as(AsNum(8));
     aspath3.prepend_as(AsNum(9));
@@ -554,7 +556,8 @@ test_policy_dump(TestInfo& /*info*/)
     string filename = string(tmppath) + "test_policy_dump";
     free(tmppath);
 #endif
-    BGPMain bgpmain;
+    EventLoop eventloop;
+    BGPMain bgpmain(eventloop);
     LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
     BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
@@ -646,19 +649,19 @@ test_policy_dump(TestInfo& /*info*/)
 
     OriginAttribute igp_origin_att(IGP);
 
-    AsPath aspath1;
+    ASPath aspath1;
     aspath1.prepend_as(AsNum(1));
     aspath1.prepend_as(AsNum(2));
     aspath1.prepend_as(AsNum(3));
     ASPathAttribute aspathatt1(aspath1);
 
-    AsPath aspath2;
+    ASPath aspath2;
     aspath2.prepend_as(AsNum(4));
     aspath2.prepend_as(AsNum(5));
     aspath2.prepend_as(AsNum(6));
     ASPathAttribute aspathatt2(aspath2);
 
-    AsPath aspath3;
+    ASPath aspath3;
     aspath3.prepend_as(AsNum(7));
     aspath3.prepend_as(AsNum(8));
     aspath3.prepend_as(AsNum(9));
