@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_ribin.cc,v 1.37 2007/02/16 22:45:22 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_ribin.cc,v 1.41 2008/07/23 05:09:39 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -53,7 +53,8 @@ test_ribin_dump(TestInfo& /*info*/)
     free(tmppath);
 #endif
 
-    BGPMain bgpmain;
+    EventLoop eventloop;
+    BGPMain bgpmain(eventloop);
     LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
     BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
@@ -77,7 +78,7 @@ test_ribin_dump(TestInfo& /*info*/)
     IPv4 nexthop1("2.0.0.1");
     NextHopAttribute<IPv4> nhatt1(nexthop1);
     OriginAttribute igp_origin_att(IGP);
-    AsPath aspath1("3,2,1");
+    ASPath aspath1("3,2,1");
     ASPathAttribute aspathatt1(aspath1);
     PathAttributeList<IPv4>* palist1 =
 	new PathAttributeList<IPv4>(nhatt1, aspathatt1, igp_origin_att);
@@ -198,7 +199,8 @@ test_ribin(TestInfo& /*info*/)
     free(tmppath);
 #endif
 
-    BGPMain bgpmain;
+    EventLoop eventloop;
+    BGPMain bgpmain(eventloop);
     LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
     BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
@@ -229,15 +231,15 @@ test_ribin(TestInfo& /*info*/)
 
     OriginAttribute igp_origin_att(IGP);
 
-    AsPath aspath1;
+    ASPath aspath1;
     aspath1.prepend_as(AsNum(1));
     aspath1.prepend_as(AsNum(2));
     aspath1.prepend_as(AsNum(3));
-    AsPath foo("3,2,1");
+    ASPath foo("3,2,1");
     assert (foo == aspath1);
     ASPathAttribute aspathatt1(aspath1);
 
-    AsPath aspath2;
+    ASPath aspath2;
     aspath2.prepend_as(AsNum(3));
     aspath2.prepend_as(AsNum(4));
     aspath2.prepend_as(AsNum(5));
