@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_command.cc,v 1.30 2007/02/16 22:45:28 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_command.cc,v 1.34 2008/07/23 05:09:46 pavlin Exp $"
 
 
 //
@@ -138,7 +138,7 @@ CliCommand::add_command(CliCommand *child_command, string& error_msg)
 	_child_command_list.insert(insert_pos, child_command);
     child_command->set_root_command(this->root_command());
     
-    return(XORP_OK);
+    return (XORP_OK);
 }
 
 //
@@ -203,7 +203,7 @@ CliCommand::add_command(const string& init_command_name,
 				 command_name_string,
 				 init_command_help);
     
-    if (parent_cli_command->add_command(cli_command, error_msg) < 0) {
+    if (parent_cli_command->add_command(cli_command, error_msg) != XORP_OK) {
 	delete cli_command;
 	goto error_label_failed;
     }
@@ -364,7 +364,7 @@ CliCommand::delete_command(const string& delete_command_name)
     if (delete_cli_command == NULL)
 	goto error_label;
     
-    if (parent_cli_command->delete_command(delete_cli_command) < 0)
+    if (parent_cli_command->delete_command(delete_cli_command) != XORP_OK)
 	goto error_label;
     return (XORP_OK);
     
