@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_node.cc,v 1.36 2007/02/16 22:45:28 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_node.cc,v 1.42 2008/07/23 05:09:46 pavlin Exp $"
 
 
 //
@@ -115,7 +115,7 @@ CliNode::start()
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
-    if (ProtoNode<Vif>::start() < 0)
+    if (ProtoNode<Vif>::start() != XORP_OK)
 	return (XORP_ERROR);
     
     // Perform misc. CLI-specific start operations
@@ -157,7 +157,7 @@ CliNode::stop()
     if (! is_up())
 	return (XORP_ERROR);
 
-    if (ProtoNode<Vif>::pending_stop() < 0)
+    if (ProtoNode<Vif>::pending_stop() != XORP_OK)
 	return (XORP_ERROR);
 
     // Perform misc. CLI-specific stop operations
@@ -165,10 +165,11 @@ CliNode::stop()
 
     delete_pointers_list(_client_list);
 
-    eventloop().remove_ioevent_cb(_cli_socket, IOT_ACCEPT);
+    if (_cli_socket.is_valid())
+	eventloop().remove_ioevent_cb(_cli_socket, IOT_ACCEPT);
     sock_serv_close();
 
-    if (ProtoNode<Vif>::stop() < 0)
+    if (ProtoNode<Vif>::stop() != XORP_OK)
 	return (XORP_ERROR);
 
     XLOG_TRACE(is_log_trace(), "CLI stopped");
@@ -426,7 +427,7 @@ CliNode::find_cli_by_session_id(uint32_t session_id) const
  * otherwise %XORP_ERROR.
  **/
 int
-CliNode::xlog_output(void *obj, const char *msg)
+CliNode::xlog_output(void *obj, xlog_level_t level, const char *msg)
 {
     CliClient *cli_client = static_cast<CliClient *>(obj);
 
@@ -436,6 +437,7 @@ CliNode::xlog_output(void *obj, const char *msg)
 	    return ret_value;
 	}
     return  XORP_ERROR;
+    UNUSED(level);
 }
 
 //
@@ -620,18 +622,16 @@ CliNode::add_client(XorpFd input_fd, XorpFd output_fd, bool is_network,
 }
 
 int
-CliNode::delete_client(CliClient *cli_client, string& error_msg)
+CliNode::remove_client(CliClient *cli_client, string& error_msg)
 {
     if (delete_connection(cli_client, error_msg) != XORP_OK)
 	return (XORP_ERROR);
 
-    // XXX: delete the client itself if it is still around
+    // XXX: Remove the client itself if it is still around
     list<CliClient *>::iterator iter;
-
     iter = find(_client_list.begin(), _client_list.end(), cli_client);
     if (iter != _client_list.end()) {
 	_client_list.erase(iter);
-	delete cli_client;
     }
 
     return (XORP_OK);
