@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.56 2007/02/16 22:45:29 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.68 2008/07/23 05:09:46 pavlin Exp $"
 
 
 //
@@ -109,7 +109,8 @@ sigwinch_handler(int signo)
  * 
  * Open a socket for the CLI to listen on for connections.
  * 
- * Return value: The new socket to listen on success, othewise %XORP_ERROR.
+ * Return value: The new socket to listen on success, othewise a XockFd
+ * that contains an invalid socket.
  **/
 XorpFd
 CliNode::sock_serv_open()
@@ -121,7 +122,8 @@ CliNode::sock_serv_open()
 	break;
 #ifdef HAVE_IPV6
     case AF_INET6:
-	_cli_socket = comm_bind_tcp6(NULL, _cli_port, COMM_SOCK_NONBLOCKING);
+	_cli_socket = comm_bind_tcp6(NULL, 0, _cli_port,
+				     COMM_SOCK_NONBLOCKING);
 	break;
 #endif // HAVE_IPV6
     default:
@@ -129,7 +131,9 @@ CliNode::sock_serv_open()
 	_cli_socket.clear();
 	break;
     }
-    
+    if (comm_listen(_cli_socket, COMM_LISTEN_DEFAULT_BACKLOG) != XORP_OK) {
+	_cli_socket.clear();
+    }
     return (_cli_socket);
 }
 
@@ -149,7 +153,7 @@ CliNode::sock_serv_close()
     if (!_cli_socket.is_valid())
 	return (XORP_OK);	// Nothing to do
 
-    if (comm_close(_cli_socket) < 0)
+    if (comm_close(_cli_socket) != XORP_OK)
 	ret_value = XORP_ERROR;
 
     _cli_socket.clear();
@@ -221,8 +225,8 @@ ENABLE_PROCESSED_OUTPUT | ENABLE_WRAP_AT_EOL_OUTPUT);
 	struct sockaddr_storage ss;
 	socklen_t len = sizeof(ss);
 	// XXX
-	if (getpeername(cli_client->input_fd(), (struct sockaddr *)&ss,
-			&len) < 0) {
+	if (getpeername(cli_client->input_fd(), (struct sockaddr *)&ss, &len)
+	    < 0) {
 	    error_msg = c_format("Cannot get peer name");
 	    // Error getting peer address
 	    delete_connection(cli_client, dummy_error_msg);
@@ -268,7 +272,7 @@ ENABLE_PROCESSED_OUTPUT | ENABLE_WRAP_AT_EOL_OUTPUT);
 	return (NULL);
     }
     
-    if (cli_client->start_connection(error_msg) < 0) {
+    if (cli_client->start_connection(error_msg) != XORP_OK) {
 	// Error connecting to the client
 	delete_connection(cli_client, dummy_error_msg);
 	return (NULL);
@@ -399,8 +403,10 @@ CliClient::start_connection(string& error_msg)
 	send(input_fd(), will_sga_cmd, sizeof(will_sga_cmd), 0);
 	send(input_fd(), dont_linemode_cmd, sizeof(dont_linemode_cmd), 0);
 	send(input_fd(), do_window_size_cmd, sizeof(do_window_size_cmd), 0);
-	send(input_fd(), do_transmit_binary_cmd, sizeof(do_transmit_binary_cmd), 0);
-	send(input_fd(), will_transmit_binary_cmd, sizeof(will_transmit_binary_cmd), 0);
+	send(input_fd(), do_transmit_binary_cmd,
+	     sizeof(do_transmit_binary_cmd), 0);
+	send(input_fd(), will_transmit_binary_cmd,
+	     sizeof(will_transmit_binary_cmd), 0);
     }
 #endif
 
@@ -783,8 +789,9 @@ CliClient::process_input_data()
 	
 	if (is_telnet()) {
 	    // Filter-out the Telnet commands
-	    int ret = process_telnet_option(val);
-	    if (ret < 0) {
+	    bool is_telnet_option = false;
+	    int ret = process_telnet_option(val, is_telnet_option);
+	    if (ret != XORP_OK) {
 		// Kick-out the client
 		// TODO: print more informative message about the client:
 		// E.g. where it came from, etc.
@@ -795,7 +802,7 @@ CliClient::process_input_data()
 		cli_node().delete_connection(this, dummy_error_msg);
 		return;
 	    }
-	    if (ret == 0) {
+	    if (is_telnet_option) {
 		// Telnet option processed
 		continue;
 	    }
@@ -815,7 +822,7 @@ CliClient::process_input_data()
 
 	preprocess_char(val, stop_processing);
 
-	if (is_waiting_for_data()) {
+	if (is_waiting_for_data() && (! is_page_mode())) {
 	    stop_processing = true;
 	    ignore_current_character = true;
 	}
@@ -884,9 +891,16 @@ void
 CliClient::schedule_process_input_data()
 {
     EventLoop& eventloop = cli_node().eventloop();
-    OneoffTaskCallback cb = callback(this, &CliClient::process_input_data);
+    OneoffTimerCallback cb = callback(this, &CliClient::process_input_data);
 
-    _process_pending_input_data_task = eventloop.new_oneoff_task(cb);
+    //
+    // XXX: Schedule the processing after 10ms to avoid increasing
+    // the CPU usage.
+    //
+    _process_pending_input_data_timer = eventloop.new_oneoff_after(
+	TimeVal(0, 10),
+	cb,
+	XorpTask::PRIORITY_LOWEST);
 }
 
 //
@@ -897,6 +911,9 @@ CliClient::preprocess_char(uint8_t val, bool& stop_processing)
 {
     stop_processing = false;
 
+    if (is_page_mode())
+	return (XORP_OK);
+
     if ((val == '\n') || (val == '\r')) {
 	// New command
 	if (is_waiting_for_data())
@@ -934,27 +951,20 @@ CliClient::preprocess_char(uint8_t val, bool& stop_processing)
     return (XORP_OK);
 }
 
-//
-// Process octet that may be part of a telnet option.
-// Parameter 'val' is the value of the next octet.
-// Return -1 if the caller should remove this client.
-// Return 0 if @val is part of a telnet option, and the telnet option was
-// processed.
-// Return 1 if @val is not part of a telnet option and should be processed
-// as input data.
-//
 int
-CliClient::process_telnet_option(int val)
+CliClient::process_telnet_option(int val, bool& is_telnet_option)
 {
 #ifdef HOST_OS_WINDOWS
-    return (1);
     UNUSED(val);
+    is_telnet_option = false;
+    return (XORP_OK);
 #else
+    is_telnet_option = true;
     if (val == IAC) {
 	// Probably a telnet command
 	if (! telnet_iac()) {
 	    set_telnet_iac(true);
-	    return (0);
+	    return (XORP_OK);
 	}
 	set_telnet_iac(false);
     }
@@ -1077,40 +1087,43 @@ CliClient::process_telnet_option(int val)
 	    break;
 	}
 	set_telnet_iac(false);
-	return (0);
+	return (XORP_OK);
     }
     //
     // Cleanup the telnet options state
     //
     if (telnet_sb()) {
 	// A negotiated option value
-	if (telnet_sb_buffer().add_data(val) < 0) {
+	if (telnet_sb_buffer().add_data(val) != XORP_OK) {
 	    // This client is sending too much options. Kick it out!
 	    return (XORP_ERROR);
 	}
-	return (0);
+	return (XORP_OK);
     }
     if (telnet_dont()) {
 	// Telnet DONT option code
 	set_telnet_dont(false);
-	return (0);
+	return (XORP_OK);
     }
     if (telnet_do()) {
 	// Telnet DO option code
 	set_telnet_do(false);
-	return (0);
+	return (XORP_OK);
     }
     if (telnet_wont()) {
 	// Telnet WONT option code
 	set_telnet_wont(false);
-	return (0);
+	return (XORP_OK);
     }
     if (telnet_will()) {
 	// Telnet WILL option code
 	set_telnet_will(false);
-	return (0);
+	return (XORP_OK);
     }
-    
-    return (1);
+
+    // XXX: Not a telnet option
+    is_telnet_option = false;
+
+    return (XORP_OK);
 #endif // HOST_OS_WINDOWS
 }
