@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/xrl_cli_node.cc,v 1.30 2007/02/16 22:45:29 pavlin Exp $"
+#ident "$XORP: xorp/cli/xrl_cli_node.cc,v 1.34 2008/07/23 05:09:47 pavlin Exp $"
 
 #include "cli_module.h"
 #include "libxorp/xorp.h"
@@ -74,7 +74,7 @@ XrlCliNode::start_cli()
 {
     int ret_code = XORP_OK;
     
-    if (cli_node().start() < 0)
+    if (cli_node().start() != XORP_OK)
 	ret_code = XORP_ERROR;
     
     return (ret_code);
@@ -85,7 +85,7 @@ XrlCliNode::stop_cli()
 {
     int ret_code = XORP_OK;
     
-    if (cli_node().stop() < 0)
+    if (cli_node().stop() != XORP_OK)
 	ret_code = XORP_ERROR;
     
     return (ret_code);
@@ -125,7 +125,7 @@ XrlCliNode::common_0_1_get_target_name(
     // Output values, 
     string&	name)
 {
-    name = my_xrl_target_name();
+    name = xrl_router().class_name();
     return XrlCmdError::OKAY();
 }
 
