@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $XORP: xorp/docs/kdoc/gen-kdoc.sh,v 1.25 2007/03/14 01:33:58 pavlin Exp $
+# $XORP: xorp/docs/kdoc/gen-kdoc.sh,v 1.28 2008/07/23 03:32:05 pavlin Exp $
 #
 
 #
@@ -38,7 +38,7 @@ HTML_CSS="--html-css http://www.xorp.org/xorp-kdoc.css"
 #
 # Misc. pre-defined variables
 #
-HTML_LOGO="http://www.xorp.org/images/xorp-logo-medium.jpg"
+HTML_LOGO="http://www.xorp.org/images/mad/logo-xorp_org.gif"
 HTML_LOGO_LINK="http://www.xorp.org/"
 
 #
@@ -455,7 +455,7 @@ kdoc_mld6igmp()
     html_start_page="index.html"
     files="mld6igmp/*.h mld6igmp/*.hh"
     excludes=""
-    xref="libxorp libxorp-callback libcomm libxipc libproto mrt cli fea xrl-interfaces xrl-targets"
+    xref="libxorp libxorp-callback libcomm libxipc libproto mrt cli fea xrl-interfaces xrl-targets libfeaclient"
     kdocify
 }
 
@@ -469,7 +469,7 @@ kdoc_pim()
     html_start_page="index.html"
     files="pim/*.h pim/*.hh"
     excludes=""
-    xref="libxorp libxorp-callback libcomm libxipc xrl-interfaces xrl-targets libproto mrt cli fea mld6igmp"
+    xref="libxorp libxorp-callback libcomm libxipc xrl-interfaces xrl-targets libfeaclient libproto mrt cli fea mld6igmp"
     kdocify
 }
 
@@ -627,10 +627,29 @@ kdoc_static_routes()
     kdocify
 }
 
+#
+# contrib directories
+#
+
+#
+# olsr
+#
+kdoc_contrib_olsr()
+{
+    lib="olsr"
+    desc="OLSR daemon"
+    html_start_page="index.html"
+    files="contrib/olsr/*.h contrib/olsr/*.hh"
+    excludes=""
+    xref="libxorp libxorp-callback libcomm libxipc xrl-interfaces xrl-targets libfeaclient libpolicybackend"
+    kdocify
+}
 
 KDOC_ALL_TGTS="libxorp callback libcomm libxipc libproto xrl_interfaces \
 	       xrl_targets mrt cli libfeaclient fea mld6igmp pim policycommon \
-	       libpolicybackend policy bgp fib2mrib mibs ospf rib rip rtrmgr static_routes"
+	       libpolicybackend policy bgp fib2mrib mibs ospf rib rip rtrmgr \
+	       static_routes \
+	       contrib_olsr"
 : ${KDOC_TGTS:=${KDOC_ALL_TGTS}}
 for i in ${KDOC_TGTS} ; do
     kdoc_$i
