@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.50 2007/02/16 22:45:45 pavlin Exp $"
+#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.63 2008/07/23 05:10:11 pavlin Exp $"
 
 //
 // Multicast routing kernel-access specific implementation.
@@ -32,11 +32,28 @@
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif
+//
+// XXX: We need to include <linux/types.h> before <sys/sysctl.h> because
+// the latter is broken on many systems (e.g., Linux Debian-3.1, Linux
+// Ubuntu-7.04, Gentoo-2006.1): file <linux/types.h> is needed if
+// <linux/mroute.h> is included, but <sys/sysctl.h> contains some hacks
+// to prevent the inclusion of <linux/types.h>.
+//
+#ifdef HAVE_LINUX_TYPES_H
+#include <linux/types.h>
+#endif
+#ifdef HAVE_SYS_SYSCTL_H
+#include <sys/sysctl.h>
+#endif
 #ifdef HAVE_NET_IF_H
 #include <net/if.h>
 #endif
 #ifdef HAVE_NET_IF_VAR_H
 #include <net/if_var.h>
+// if_name() is a macro in FreeBSD 7.0 conflicting with our method names.
+#ifdef if_name
+#undef if_name
+#endif
 #endif
 #ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
@@ -62,12 +79,12 @@
 #include "mrt/max_vifs.h"
 #include "mrt/multicast_defs.h"
 
+#include "fea_node.hh"
 #include "mfea_node.hh"
 #include "mfea_vif.hh"
 #include "mfea_kernel_messages.hh"
 #include "mfea_osdep.hh"
 #include "mfea_mrouter.hh"
-#include "mfea_proto_comm.hh"
 
 
 //
@@ -108,8 +125,15 @@ typedef char *caddr_t;
  **/
 MfeaMrouter::MfeaMrouter(MfeaNode& mfea_node)
     : ProtoUnit(mfea_node.family(), mfea_node.module_id()),
-      _mfea_node(mfea_node)
+      _mfea_node(mfea_node),
+      _mrt_api_mrt_mfc_flags_disable_wrongvif(false),
+      _mrt_api_mrt_mfc_flags_border_vif(false),
+      _mrt_api_mrt_mfc_rp(false),
+      _mrt_api_mrt_mfc_bw_upcall(false),
+      _multicast_forwarding_enabled(false)
 {
+    string error_msg;
+
     // Allocate the buffers
     _rcvbuf0 = new uint8_t[IO_BUF_SIZE];
     _sndbuf0 = new uint8_t[IO_BUF_SIZE];
@@ -159,10 +183,27 @@ MfeaMrouter::MfeaMrouter(MfeaNode& mfea_node)
     _sndmh.msg_controllen	= 0;
 #endif // ! HOST_OS_WINDOWS
 
-    _mrt_api_mrt_mfc_flags_disable_wrongvif = false;
-    _mrt_api_mrt_mfc_flags_border_vif = false;
-    _mrt_api_mrt_mfc_rp = false;
-    _mrt_api_mrt_mfc_bw_upcall = false;
+    //
+    // Get the old state from the underlying system
+    //
+    int ret_value = XORP_OK;
+    switch (family()) {
+    case AF_INET:
+	ret_value = multicast_forwarding_enabled4(_multicast_forwarding_enabled,
+						  error_msg);
+	break;
+#ifdef HAVE_IPV6
+    case AF_INET6:
+	ret_value = multicast_forwarding_enabled6(_multicast_forwarding_enabled,
+						  error_msg);
+	break;
+#endif // HAVE_IPV6
+    default:
+	XLOG_UNREACHABLE();
+    }
+    if (ret_value != XORP_OK) {
+	XLOG_FATAL("%s", error_msg.c_str());
+    }
 }
 
 MfeaMrouter::~MfeaMrouter()
@@ -189,6 +230,8 @@ MfeaMrouter::~MfeaMrouter()
 int
 MfeaMrouter::start()
 {
+    string error_msg;
+
 #ifdef HOST_OS_WINDOWS
     XLOG_ERROR("Multicast routing is not supported on Windows");
     return (XORP_ERROR);
@@ -200,7 +243,7 @@ MfeaMrouter::start()
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
-    if (ProtoUnit::start() < 0)
+    if (ProtoUnit::start() != XORP_OK)
 	return (XORP_ERROR);
     
 #ifndef HOST_OS_WINDOWS
@@ -211,13 +254,28 @@ MfeaMrouter::start()
 	// return (XORP_ERROR);
     }
 #endif // ! HOST_OS_WINDOWS
-    
-    // Open kernel multicast routing access socket
-    if (!open_mrouter_socket().is_valid())
+
+    // Register as multicast upcall receiver
+    IoIpManager& io_ip_manager = mfea_node().fea_node().io_ip_manager();
+    uint8_t ip_protocol = kernel_mrouter_ip_protocol();
+    if (io_ip_manager.register_system_multicast_upcall_receiver(
+	    family(),
+	    ip_protocol,
+	    callback(this, &MfeaMrouter::kernel_call_process),
+	    _mrouter_socket,
+	    error_msg)
+	!= XORP_OK) {
+	XLOG_ERROR("Cannot register multicast upcall receiver: %s",
+		   error_msg.c_str());
 	return (XORP_ERROR);
-    
+    }
+    if (! _mrouter_socket.is_valid()) {
+	XLOG_ERROR("Failed to assign the multicast routing socket");
+	return (XORP_ERROR);
+    }
+
     // Start the multicast routing in the kernel
-    if (start_mrt() < 0)
+    if (start_mrt() != XORP_OK)
 	return (XORP_ERROR);
     
     return (XORP_OK);
@@ -234,17 +292,56 @@ MfeaMrouter::start()
 int
 MfeaMrouter::stop()
 {
+    string error_msg;
+
     if (is_down())
 	return (XORP_OK);
 
-    if (ProtoUnit::stop() < 0)
+    if (ProtoUnit::stop() != XORP_OK)
 	return (XORP_ERROR);
-    
+
     // Stop the multicast routing in the kernel
     stop_mrt();
     
-    // Close kernel multicast routing access socket
-    close_mrouter_socket();
+    // Clear kernel multicast routing access socket
+    _mrouter_socket.clear();
+
+    // Unregister as multicast upcall receiver
+    IoIpManager& io_ip_manager = mfea_node().fea_node().io_ip_manager();
+    uint8_t ip_protocol = kernel_mrouter_ip_protocol();
+    if (io_ip_manager.unregister_system_multicast_upcall_receiver(
+	    family(),
+	    ip_protocol,
+	    error_msg)
+	!= XORP_OK) {
+	XLOG_ERROR("Cannot unregister multicast upcall receiver: %s",
+		   error_msg.c_str());
+	return (XORP_ERROR);
+    }
+
+    //
+    // Restore the old forwarding state in the underlying system.
+    //
+    int ret_value = XORP_OK;
+    switch (family()) {
+    case AF_INET:
+	ret_value = set_multicast_forwarding_enabled4(_multicast_forwarding_enabled,
+						      error_msg);
+	break;
+#ifdef HAVE_IPV6
+    case AF_INET6:
+	ret_value = set_multicast_forwarding_enabled6(_multicast_forwarding_enabled,
+						      error_msg);
+	break;
+#endif // HAVE_IPV6
+    default:
+	XLOG_UNREACHABLE();
+    }
+    if (ret_value != XORP_OK) {
+	XLOG_ERROR("Cannot restore the multicast forwarding state: %s",
+		   error_msg.c_str());
+	return (XORP_ERROR);
+    }
     
     return (XORP_OK);
 }
@@ -339,13 +436,280 @@ MfeaMrouter::have_multicast_routing6() const
 #endif // HAVE_IPV6_MULTICAST_ROUTING
 }
 
+/**
+ * Test whether the IPv4 multicast forwarding engine is enabled or disabled
+ * to forward packets.
+ * 
+ * @param ret_value if true on return, then the IPv4 multicast forwarding
+ * is enabled, otherwise is disabled.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+int
+MfeaMrouter::multicast_forwarding_enabled4(bool& ret_value,
+					   string& error_msg) const
+{
+    int enabled = 0;
+
+    UNUSED(error_msg);
+
+    // XXX: always return true if running in dummy mode
+    if (mfea_node().is_dummy()) {
+	ret_value = true;
+	return (XORP_OK);
+    }
+
+    //
+    // XXX: Don't check whether the system supports IPv4 multicast routing,
+    // because this might require to run as a root.
+    //
+
+#if defined(CTL_NET) && defined(IPPROTO_IP) && defined(IPCTL_MFORWARDING)
+    {
+	size_t sz = sizeof(enabled);
+	int mib[4];
+	
+	mib[0] = CTL_NET;
+	mib[1] = AF_INET;
+	mib[2] = IPPROTO_IP;
+	mib[3] = IPCTL_MFORWARDING;
+	
+	if (sysctl(mib, sizeof(mib) / sizeof(mib[0]), &enabled, &sz, NULL, 0)
+	    != 0) {
+	    error_msg = c_format("Get sysctl(IPCTL_MFORWARDING) failed: %s",
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+    }
+
+#else // ! IPCTL_MFORWARDING
+
+    // XXX: Not all systems have such additional control mechanism to
+    // explicitly enable multicast forwarding, hence assume it is enabled.
+    enabled = 1;
+#endif // ! IPCTL_MFORWARDING
+
+    if (enabled > 0)
+	ret_value = true;
+    else
+	ret_value = false;
+    
+    return (XORP_OK);
+}
+
+/**
+ * Test whether the IPv6 multicast forwarding engine is enabled or disabled
+ * to forward packets.
+ * 
+ * @param ret_value if true on return, then the IPv6 multicast forwarding
+ * is enabled, otherwise is disabled.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+int
+MfeaMrouter::multicast_forwarding_enabled6(bool& ret_value,
+					   string& error_msg) const
+{
+    int enabled = 0;
+
+    UNUSED(error_msg);
+
+    // XXX: always return true if running in dummy mode
+    if (mfea_node().is_dummy()) {
+	ret_value = true;
+	return (XORP_OK);
+    }
+
+    //
+    // XXX: Don't check whether the system supports IPv6 multicast routing,
+    // because this might require to run as a root.
+    //
+
+#if defined(CTL_NET) && defined(IPPROTO_IPV6) && defined(IPV6CTL_MFORWARDING)
+    {
+	size_t sz = sizeof(enabled);
+	int mib[4];
+	
+	mib[0] = CTL_NET;
+	mib[1] = AF_INET6;
+	mib[2] = IPPROTO_IPV6;
+	mib[3] = IPV6CTL_MFORWARDING;
+	
+	if (sysctl(mib, sizeof(mib) / sizeof(mib[0]), &enabled, &sz, NULL, 0)
+	    != 0) {
+	    error_msg = c_format("Get sysctl(IPV6CTL_MFORWARDING) failed: %s",
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+    }
+
+#else // ! IPV6CTL_MFORWARDING
+    //
+    // XXX: Not all systems have such additional control mechanism to
+    // explicitly enable multicast forwarding, hence assume it is enabled.
+    //
+    enabled = 1;
+#endif // ! IPV6CTL_MFORWARDING
+
+    if (enabled > 0)
+	ret_value = true;
+    else
+	ret_value = false;
+    
+    return (XORP_OK);
+}
+
+/**
+ * Set the IPv4 multicast forwarding engine to enable or disable forwarding
+ * of packets.
+ * 
+ * @param v if true, then enable IPv4 multicast forwarding, otherwise
+ * disable it.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+int
+MfeaMrouter::set_multicast_forwarding_enabled4(bool v, string& error_msg)
+{
+    // XXX: don't do anything if running in dummy mode
+    if (mfea_node().is_dummy())
+	return (XORP_OK);
+
+    if (! have_multicast_routing4()) {
+	if (! v) {
+	    //
+	    // XXX: we assume that "not supported" == "disable", hence
+	    // return OK.
+	    //
+	    return (XORP_OK);
+	}
+	error_msg = c_format("Cannot set IPv4 multicast forwarding to %s: "
+			     "IPv4 multicast routing is not supported",
+			     bool_c_str(v));
+	XLOG_ERROR("%s", error_msg.c_str());
+	return (XORP_ERROR);
+    }
+
+    int enable = (v) ? 1 : 0;
+    bool old_value;
+
+    UNUSED(enable);
+
+    if (multicast_forwarding_enabled4(old_value, error_msg) != XORP_OK)
+	return (XORP_ERROR);
+
+    if (old_value == v)
+	return (XORP_OK);	// Nothing changed
+
+#if defined(CTL_NET) && defined(IPPROTO_IP) && defined(IPCTL_MFORWARDING)
+    {
+	size_t sz = sizeof(enable);
+	int mib[4];
+
+	mib[0] = CTL_NET;
+	mib[1] = AF_INET;
+	mib[2] = IPPROTO_IP;
+	mib[3] = IPCTL_MFORWARDING;
+	
+	if (sysctl(mib, sizeof(mib) / sizeof(mib[0]), NULL, NULL, &enable, sz)
+	    != 0) {
+	    error_msg = c_format("Set sysctl(IPCTL_MFORWARDING) to %s failed: %s",
+				 bool_c_str(v), strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+    }
+
+#else // ! IPCTL_MFORWARDING
+    //
+    // XXX: Not all systems have such additional control mechanism to
+    // explicitly enable multicast forwarding, hence don't do anything
+    //
+#endif // ! IPCTL_MFORWARDING
+
+    return (XORP_OK);
+}
+
+/**
+ * Set the IPv6 multicast forwarding engine to enable or disable forwarding
+ * of packets.
+ * 
+ * @param v if true, then enable IPv6 multicast forwarding, otherwise
+ * disable it.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+int
+MfeaMrouter::set_multicast_forwarding_enabled6(bool v, string& error_msg)
+{
+    // XXX: don't do anything if running in dummy mode
+    if (mfea_node().is_dummy())
+	return (XORP_OK);
+
+    if (! have_multicast_routing6()) {
+	if (! v) {
+	    //
+	    // XXX: we assume that "not supported" == "disable", hence
+	    // return OK.
+	    //
+	    return (XORP_OK);
+	}
+	error_msg = c_format("Cannot set IPv6 multicast forwarding to %s: "
+			     "IPv6 multicast routing is not supported",
+			     bool_c_str(v));
+	XLOG_ERROR("%s", error_msg.c_str());
+	return (XORP_ERROR);
+    }
+
+    int enable = (v) ? 1 : 0;
+    bool old_value;
+
+    UNUSED(enable);
+
+    if (multicast_forwarding_enabled6(old_value, error_msg) != XORP_OK)
+	return (XORP_ERROR);
+
+    if (old_value == v)
+	return (XORP_OK);	// Nothing changed
+
+#if defined(CTL_NET) && defined(IPPROTO_IPV6) && defined(IPV6CTL_MFORWARDING)
+    {
+	size_t sz = sizeof(enable);
+	int mib[4];
+
+	mib[0] = CTL_NET;
+	mib[1] = AF_INET6;
+	mib[2] = IPPROTO_IPV6;
+	mib[3] = IPV6CTL_MFORWARDING;
+	
+	if (sysctl(mib, sizeof(mib) / sizeof(mib[0]), NULL, NULL, &enable, sz)
+	    != 0) {
+	    error_msg = c_format("Set sysctl(IPV6CTL_MFORWARDING) to %s failed: %s",
+				 bool_c_str(v), strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+    }
+
+#else // ! IPV6CTL_MFORWARDING
+    //
+    // XXX: Not all systems have such additional control mechanism to
+    // explicitly enable multicast forwarding, hence don't do anything
+    //
+#endif // ! IPV6CTL_MFORWARDING
+
+    return (XORP_OK);
+}
+
 /**
  * MfeaMrouter::kernel_mrouter_ip_protocol:
  * @: 
  * 
  * Get the protocol that would be used in case of mrouter socket.
  * 
- * Return value: the protocol number on success, otherwise %XORP_ERROR.
+ * Return value: the protocol number on success, otherwise -1.
  **/
 int
 MfeaMrouter::kernel_mrouter_ip_protocol() const
@@ -359,188 +723,10 @@ MfeaMrouter::kernel_mrouter_ip_protocol() const
 #endif // HAVE_IPV6
     default:
 	XLOG_UNREACHABLE();
-	return (XORP_ERROR);
+	return (-1);
     }
     
-    return (XORP_ERROR);
-}
-
-/**
- * MfeaMrouter::open_mrouter_socket:
- * @: 
- * 
- * Open the mrouter socket (used for various multicast routing kernel calls).
- * 
- * Return value: The socket value.
- **/
-XorpFd
-MfeaMrouter::open_mrouter_socket()
-{
-    XorpFd badfd;
-
-    if (_mrouter_socket.is_valid())
-	return (_mrouter_socket);
-
-    if (kernel_mrouter_ip_protocol() < 0)
-	return (badfd);
-    
-    //
-    // XXX: if we have already IGMP or ICMPV6 socket, then reuse it
-    //
-    do {
-	ProtoComm *proto_comm;
-	proto_comm = mfea_node().proto_comm_find_by_ip_protocol(kernel_mrouter_ip_protocol());
-	if (proto_comm == NULL)
-	    break;
-	_mrouter_socket = proto_comm->proto_socket_in();
-	if (_mrouter_socket.is_valid())
-	    return (_mrouter_socket);
-	break;
-    } while (false);
-    
-    _mrouter_socket = socket(family(), SOCK_RAW, kernel_mrouter_ip_protocol());
-    if (!_mrouter_socket.is_valid()) {
-	XLOG_ERROR("Cannot open mrouter socket");
-	return (badfd);
-    }
-    
-    if (!adopt_mrouter_socket().is_valid())
-	return (badfd);
-    
-    return (_mrouter_socket);
-}
-
-/**
- * MfeaMrouter::adopt_mrouter_socket:
- * @: 
- * 
- * Adopt control over the mrouter socket.
- * When the #MfeaMrouter adopts control over the mrouter socket,
- * it is the one that will be reading from that socket.
- * 
- * Return value: The adopted socket value.
- **/
-XorpFd
-MfeaMrouter::adopt_mrouter_socket()
-{
-    XorpFd badfd;
-
-    if (!_mrouter_socket.is_valid())
-	return (badfd);
-    
-    XLOG_ASSERT(is_up());
-    
-    // Set receiving buffer size
-    if (comm_sock_set_rcvbuf(_mrouter_socket, SO_RCV_BUF_SIZE_MAX,
-			     SO_RCV_BUF_SIZE_MIN)
-	< SO_RCV_BUF_SIZE_MIN) {
-	comm_close(_mrouter_socket);
-	_mrouter_socket.clear();
-	return (badfd);
-    }
-    
-    // Protocol-specific setup
-    switch (family()) {
-    case AF_INET:
-#ifndef HAVE_IPV4_MULTICAST_ROUTING
-	XLOG_ERROR("adopt_mrouter_socket() failed: "
-		   "IPv4 multicast routing not supported");
-	return (badfd);
-#endif
-	break;
-#ifdef HAVE_IPV6
-    case AF_INET6:
-    {
-#ifndef HAVE_IPV6_MULTICAST_ROUTING
-	XLOG_ERROR("adopt_mrouter_socket() failed: "
-		   "IPv6 multicast routing not supported");
-	return (badfd);
-#else
-	struct icmp6_filter filter;
-	// Filter all ICMPv6 messages
-	ICMP6_FILTER_SETBLOCKALL(&filter);
-	if (setsockopt(_mrouter_socket, IPPROTO_ICMPV6, ICMP6_FILTER,
-		       (void *)&filter, sizeof(filter)) < 0) {
-	    XLOG_ERROR("setsockopt(ICMP6_FILTER) failed: %s", strerror(errno));
-	    comm_close(_mrouter_socket);
-	    _mrouter_socket.clear();
-	    return (badfd);
-	}
-#endif // HAVE_IPV6_MULTICAST_ROUTING
-    }
-    break;
-#endif // HAVE_IPV6
-    default:
-	XLOG_UNREACHABLE();
-	return (badfd);
-    }
-    
-    // Assign a method to read from this socket
-    if (mfea_node().eventloop().add_ioevent_cb(_mrouter_socket, IOT_READ,
-				callback(this,
-					 &MfeaMrouter::mrouter_socket_read))
-	== false) {
-	XLOG_ERROR("Cannot add mrouter socket to the set of sockets "
-		   "to read from in the event loop");
-	comm_close(_mrouter_socket);
-	_mrouter_socket.clear();
-	return (badfd);
-    }
-    
-    return (_mrouter_socket);
-}
-
-/**
- * MfeaMrouter::close_mrouter_socket:
- * @: 
- * 
- * Close the mrouter socket.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
-int
-MfeaMrouter::close_mrouter_socket()
-{
-    if (!_mrouter_socket.is_valid())
-	return (XORP_ERROR);
-    
-    if (kernel_mrouter_ip_protocol() < 0)
-	return (XORP_ERROR);
-    
-    //
-    // XXX: if we have already IGMP or ICMPV6 socket, then we must be
-    // using the same socket, hence don't close it.
-    //
-    do {
-	ProtoComm *proto_comm;
-	proto_comm = mfea_node().proto_comm_find_by_ip_protocol(kernel_mrouter_ip_protocol());
-	if (proto_comm == NULL)
-	    break;
-	// If there is already IGMP or ICMPV6 socket, then don't close it.
-	if (_mrouter_socket == proto_comm->proto_socket_in()) {
-	    // Transfer the I/O event callback to the protocol socket
-	    mfea_node().eventloop().remove_ioevent_cb(_mrouter_socket);
-	    proto_comm->add_proto_socket_in_callback();
-	    _mrouter_socket.clear();
-	    return (XORP_OK);
-	}
-	break;
-    } while (false);
-    
-    // Remove the function for reading from this socket    
-    mfea_node().eventloop().remove_ioevent_cb(_mrouter_socket);
-    
-    // Close the socket and set its handle to an invalid value.
-    if (comm_close(_mrouter_socket) == XORP_ERROR) {
-	XLOG_ERROR("Cannot close mrouter socket: %s",
-		    comm_get_last_error_str());
-	_mrouter_socket.clear();
-	return (XORP_ERROR);
-    }
-    
-    _mrouter_socket.clear();
-    
-    return (XORP_OK);
+    return (-1);
 }
 
 /**
@@ -555,9 +741,10 @@ int
 MfeaMrouter::start_mrt()
 {
     int mrouter_version = 1;	// XXX: hardcoded version
-#if !defined(HAVE_IPV4_MULTICAST_ROUTING) && !defined(HAVE_IPV6_MULTICAST_ROUTING)
-    UNUSED(mrouter_version)
-#endif
+    string error_msg;
+
+    UNUSED(mrouter_version);
+    UNUSED(error_msg);
     
     switch (family()) {
     case AF_INET:
@@ -566,6 +753,11 @@ MfeaMrouter::start_mrt()
 		   "IPv4 multicast routing not supported");
 	return (XORP_ERROR);
 #else
+	if (set_multicast_forwarding_enabled4(true, error_msg) != XORP_OK) {
+	    XLOG_ERROR("Cannot enable IPv4 multicast forwarding: %s",
+		       error_msg.c_str());
+	    return (XORP_ERROR);
+	}
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_INIT,
 		       (void *)&mrouter_version, sizeof(mrouter_version))
 	    < 0) {
@@ -584,6 +776,11 @@ MfeaMrouter::start_mrt()
 		   "IPv6 multicast routing not supported");
 	return (XORP_ERROR);
 #else
+	if (set_multicast_forwarding_enabled6(true, error_msg) != XORP_OK) {
+	    XLOG_ERROR("Cannot enable IPv6 multicast forwarding: %s",
+		       error_msg.c_str());
+	    return (XORP_ERROR);
+	}
 	if (setsockopt(_mrouter_socket, IPPROTO_IPV6, MRT6_INIT,
 		       (void *)&mrouter_version, sizeof(mrouter_version))
 	    < 0) {
@@ -757,6 +954,10 @@ MfeaMrouter::start_mrt()
 int
 MfeaMrouter::stop_mrt()
 {
+    string error_msg;
+
+    UNUSED(error_msg);
+
     _mrt_api_mrt_mfc_flags_disable_wrongvif = false;
     _mrt_api_mrt_mfc_flags_border_vif = false;
     _mrt_api_mrt_mfc_rp = false;
@@ -772,6 +973,11 @@ MfeaMrouter::stop_mrt()
 		   "IPv4 multicast routing not supported");
 	return (XORP_ERROR);
 #else
+	if (set_multicast_forwarding_enabled4(false, error_msg) != XORP_OK) {
+	    XLOG_ERROR("Cannot disable IPv4 multicast forwarding: %s",
+		       error_msg.c_str());
+	    return (XORP_ERROR);
+	}
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_DONE, NULL, 0)
 	    < 0) {
 	    XLOG_ERROR("setsockopt(MRT_DONE) failed: %s", strerror(errno));
@@ -787,6 +993,11 @@ MfeaMrouter::stop_mrt()
 		   "IPv6 multicast routing not supported");
 	return (XORP_ERROR);
 #else
+	if (set_multicast_forwarding_enabled6(false, error_msg) != XORP_OK) {
+	    XLOG_ERROR("Cannot disable IPv6 multicast forwarding: %s",
+		       error_msg.c_str());
+	    return (XORP_ERROR);
+	}
 	if (setsockopt(_mrouter_socket, IPPROTO_IPV6, MRT6_DONE, NULL, 0)
 	    < 0) {
 	    XLOG_ERROR("setsockopt(MRT6_DONE) failed: %s", strerror(errno));
@@ -804,31 +1015,22 @@ MfeaMrouter::stop_mrt()
     return (XORP_OK);
 }
 
-
-/**
- * MfeaMrouter::start_pim:
- * @: 
- * 
- * Start/enable PIM routing in the kernel.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
-MfeaMrouter::start_pim()
+MfeaMrouter::start_pim(string& error_msg)
 {
     int v = 1;
 
     switch (family()) {
     case AF_INET:
 #ifndef HAVE_IPV4_MULTICAST_ROUTING
-	XLOG_ERROR("start_pim() failed: "
-		   "IPv4 multicast routing not supported");
+	error_msg = c_format("start_pim() failed: "
+			     "IPv4 multicast routing not supported");
 	return (XORP_ERROR);
 #else
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_PIM,
 		       (void *)&v, sizeof(v)) < 0) {
-	    XLOG_ERROR("setsockopt(MRT_PIM, %u) failed: %s",
-		       v, strerror(errno));
+	    error_msg = c_format("setsockopt(MRT_PIM, %u) failed: %s",
+				 v, strerror(errno));
 	    return (XORP_ERROR);
 	}
 #endif // HAVE_IPV4_MULTICAST_ROUTING
@@ -838,14 +1040,14 @@ MfeaMrouter::start_pim()
     case AF_INET6:
     {
 #ifndef HAVE_IPV6_MULTICAST_ROUTING
-	XLOG_ERROR("start_pim() failed: "
-		   "IPv6 multicast routing not supported");
+	error_msg = c_format("start_pim() failed: "
+			     "IPv6 multicast routing not supported");
 	return (XORP_ERROR);
 #else
 	if (setsockopt(_mrouter_socket, IPPROTO_IPV6, MRT6_PIM,
 		       (void *)&v, sizeof(v)) < 0) {
-	    XLOG_ERROR("setsockopt(MRT6_PIM, %u) failed: %s",
-		       v, strerror(errno));
+	    error_msg = c_format("setsockopt(MRT6_PIM, %u) failed: %s",
+				 v, strerror(errno));
 	    return (XORP_ERROR);
 	}
 #endif // HAVE_IPV6_MULTICAST_ROUTING
@@ -862,17 +1064,8 @@ MfeaMrouter::start_pim()
     return (XORP_OK);
 }
 
-
-/**
- * MfeaMrouter::stop_pim:
- * @: 
- * 
- * Stop/disable PIM routing in the kernel.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
-MfeaMrouter::stop_pim()
+MfeaMrouter::stop_pim(string& error_msg)
 {
     int v = 0;
 
@@ -882,15 +1075,15 @@ MfeaMrouter::stop_pim()
     switch (family()) {
     case AF_INET:
 #ifndef HAVE_IPV4_MULTICAST_ROUTING
-	XLOG_ERROR("stop_pim() failed: "
-		   "IPv4 multicast routing not supported");
+	error_msg = c_format("stop_pim() failed: "
+			     "IPv4 multicast routing not supported");
 	return (XORP_ERROR);
 #else
 	v = 0;
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_PIM,
 		       (void *)&v, sizeof(v)) < 0) {
-	    XLOG_ERROR("setsockopt(MRT_PIM, %u) failed: %s",
-		       v, strerror(errno));
+	    error_msg = c_format("setsockopt(MRT_PIM, %u) failed: %s",
+				 v, strerror(errno));
 	    return (XORP_ERROR);
 	}
 #endif
@@ -900,15 +1093,15 @@ MfeaMrouter::stop_pim()
     case AF_INET6:
     {
 #ifndef HAVE_IPV6_MULTICAST_ROUTING
-	XLOG_ERROR("stop_pim() failed: "
-		   "IPv6 multicast routing not supported");
+	error_msg = c_format("stop_pim() failed: "
+			     "IPv6 multicast routing not supported");
 	return (XORP_ERROR);
 #else
 	v = 0;
 	if (setsockopt(_mrouter_socket, IPPROTO_IPV6, MRT6_PIM,
 		       (void *)&v, sizeof(v)) < 0) {
-	    XLOG_ERROR("setsockopt(MRT6_PIM, %u) failed: %s",
-		       v, strerror(errno));
+	    error_msg = c_format("setsockopt(MRT6_PIM, %u) failed: %s",
+				 v, strerror(errno));
 	    return (XORP_ERROR);
 	}
 #endif // HAVE_IPV6_MULTICAST_ROUTING
@@ -1133,7 +1326,7 @@ MfeaMrouter::delete_multicast_vif(uint32_t vif_index)
  * @rp_addr: The RP address.
  * 
  * Install/modify a Multicast Forwarding Cache (MFC) entry in the kernel.
- * If the MFC entry specified by (@source, @group) pair was not
+ * If the MFC entry specified by (source, group) pair was not
  * installed before, a new MFC entry will be created in the kernel;
  * otherwise, the existing entry's fields will be modified.
  * 
@@ -1183,7 +1376,7 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	return (XORP_ERROR);
 #else
 
-#if defined(HAVE_MFCCTL2) && defined(ENABLE_ADVANCED_MULTICAST_API)
+#if defined(HAVE_STRUCT_MFCCTL2) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct mfcctl2 mc;
 #else
 	struct mfcctl mc;
@@ -1195,11 +1388,11 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	mc.mfcc_parent = iif_vif_index;
 	for (uint32_t i = 0; i < mfea_node().maxvifs(); i++) {
 	    mc.mfcc_ttls[i] = oifs_ttl[i];
-#if defined(HAVE_MFCC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
+#if defined(HAVE_STRUCT_MFCCTL2_MFCC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	    mc.mfcc_flags[i] = oifs_flags[i];
 #endif
 	}
-#if defined(HAVE_MFCC_RP) && defined(ENABLE_ADVANCED_MULTICAST_API)
+#if defined(HAVE_STRUCT_MFCCTL2_MFCC_RP) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	if (_mrt_api_mrt_mfc_rp)
 	    rp_addr.copy_out(mc.mfcc_rp);
 #endif
@@ -1223,7 +1416,7 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	return (XORP_ERROR);
 #else
 	
-#if defined(HAVE_MF6CCTL2) && defined(ENABLE_ADVANCED_MULTICAST_API)
+#if defined(HAVE_STRUCT_MF6CCTL2) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct mf6cctl2 mc;
 #else
 	struct mf6cctl mc;
@@ -1237,11 +1430,11 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	for (uint32_t i = 0; i < mfea_node().maxvifs(); i++) {
 	    if (oifs_ttl[i] > 0)
 		IF_SET(i, &mc.mf6cc_ifset);
-#if defined(HAVE_MF6CC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
+#if defined(HAVE_STRUCT_MF6CCTL2_MF6CC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	    mc.mf6cc_flags[i] = oifs_flags[i];
 #endif
 	}
-#if defined(HAVE_MF6CC_RP) && defined(ENABLE_ADVANCED_MULTICAST_API)
+#if defined(HAVE_STRUCT_MF6CCTL2_MF6CC_RP) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	if (_mrt_api_mrt_mfc_rp)
 	    rp_addr.copy_out(mc.mf6cc_rp);
 #endif
@@ -1416,8 +1609,8 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
 			     "the GEQ and LEQ flags are mutually exclusive "
 			     "(GEQ = %s; LEQ = %s)",
 			     cstring(source), cstring(group),
-			     (is_geq_upcall)? "true" : "false",
-			     (is_leq_upcall)? "true" : "false");
+			     bool_c_str(is_geq_upcall),
+			     bool_c_str(is_leq_upcall));
 	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);		// Invalid arguments
     }
@@ -1428,8 +1621,8 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
 			     "(is_threshold_in_packets = %s; "
 			     "is_threshold_in_bytes = %s)",
 			     cstring(source), cstring(group),
-			     (is_threshold_in_packets)? "true" : "false",
-			     (is_threshold_in_bytes)? "true" : "false");
+			     bool_c_str(is_threshold_in_packets),
+			     bool_c_str(is_threshold_in_bytes));
 	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);		// Invalid arguments
     }
@@ -1614,8 +1807,8 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
 			     "the GEQ and LEQ flags are mutually exclusive "
 			     "(GEQ = %s; LEQ = %s)",
 			     cstring(source), cstring(group),
-			     (is_geq_upcall)? "true" : "false",
-			     (is_leq_upcall)? "true" : "false");
+			     bool_c_str(is_geq_upcall),
+			     bool_c_str(is_leq_upcall));
 	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);		// Invalid arguments
     }
@@ -1626,8 +1819,8 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
 			     "(is_threshold_in_packets = %s; "
 			     "is_threshold_in_bytes = %s)",
 			     cstring(source), cstring(group),
-			     (is_threshold_in_packets)? "true" : "false",
-			     (is_threshold_in_bytes)? "true" : "false");
+			     bool_c_str(is_threshold_in_packets),
+			     bool_c_str(is_threshold_in_bytes));
 	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);		// Invalid arguments
     }
@@ -2042,152 +2235,6 @@ MfeaMrouter::get_vif_count(uint32_t vif_index, VifCount& vif_count)
     return (XORP_OK);
 }
 
-/**
- * MfeaMrouter::mrouter_socket_read:
- * @fd: file descriptor of arriving data.
- * @type: The event type that describes the status of @fd.
- * 
- * Read data from the mrouter socket, and then call the appropriate method
- * to process it.
- **/
-void
-MfeaMrouter::mrouter_socket_read(XorpFd fd, IoEventType type)
-{
-    ssize_t nbytes;
-    
-    UNUSED(fd);
-    UNUSED(type);
- 
-#ifndef HOST_OS_WINDOWS
-    // Zero and reset various fields
-    _rcvmh.msg_controllen = CMSG_BUF_SIZE;
-
-    // TODO: when resetting _from4 and _from6 do we need to set the address
-    // family and the sockaddr len?
-    switch (family()) {
-    case AF_INET:
-	memset(&_from4, 0, sizeof(_from4));
-	_rcvmh.msg_namelen = sizeof(_from4);
-	break;
-
-#ifdef HAVE_IPV6
-    case AF_INET6:
-    {
-#ifndef HAVE_IPV6_MULTICAST_ROUTING
-	XLOG_FATAL("mrouter_socket_read() failed: "
-		   "IPv6 multicast routing not supported");
-	return;
-#else
-	memset(&_from6, 0, sizeof(_from6));
-	_rcvmh.msg_namelen = sizeof(_from6);
-#endif // HAVE_IPV6_MULTICAST_ROUTING
-    }
-    break;
-#endif // HAVE_IPV6
-
-    default:
-	XLOG_UNREACHABLE();
-	return;			// Error
-    }
-
-    // Read from the socket
-    nbytes = recvmsg(_mrouter_socket, &_rcvmh, 0);
-    if (nbytes < 0) {
-	if (errno == EINTR)
-	    return;		// OK: restart receiving
-	XLOG_ERROR("recvmsg() on socket %p failed: %s",
-		   _mrouter_socket.str().c_str(), strerror(errno));
-	return;			// Error
-    }
-
-#else // HOST_OS_WINDOWS
-    UNUSED(nbytes);
-    XLOG_FATAL("Multicast routing is not supported on Windows");
-#endif
-    
-    // Check if it is a signal from the kernel to the user-level
-    switch (family()) {
-    case AF_INET:
-    {
-#ifndef HAVE_IPV4_MULTICAST_ROUTING
-	XLOG_FATAL("mrouter_socket_read() failed: "
-		   "IPv4 multicast routing not supported");
-	return;
-#else
-	if (nbytes < (ssize_t)sizeof(struct igmpmsg)) {
-	    XLOG_WARNING("mrouter_socket_read() failed: "
-			 "kernel signal packet size %d is smaller than minimum size %u",
-			 XORP_INT_CAST(nbytes),
-			 XORP_UINT_CAST(sizeof(struct igmpmsg)));
-	    return;		// Error
-	}
-	struct igmpmsg igmpmsg;
-	memcpy(&igmpmsg, _rcvbuf0, sizeof(igmpmsg));
-	if (igmpmsg.im_mbz == 0) {
-	    //
-	    // XXX: Packets sent up from kernel to daemon have
-	    //      igmpmsg.im_mbz = ip->ip_p = 0
-	    //
-	    kernel_call_process(_rcvbuf0, nbytes);
-	    return;		// OK
-	}
-#endif // HAVE_IPV4_MULTICAST_ROUTING
-    }
-    break;
-    
-#ifdef HAVE_IPV6
-    case AF_INET6:
-    {
-#ifndef HAVE_IPV6_MULTICAST_ROUTING
-	XLOG_FATAL("mrouter_socket_read() failed: "
-		   "IPv6 multicast routing not supported");
-	return;
-#else
-	if ((nbytes < (ssize_t)sizeof(struct mrt6msg))
-	    && (nbytes < (ssize_t)MLD_MINLEN)) {
-	    XLOG_WARNING("mrouter_socket_read() failed: "
-			 "kernel signal packet size %d is smaller than minimum size %u",
-			 XORP_INT_CAST(nbytes),
-			 XORP_UINT_CAST(min(sizeof(struct mrt6msg),
-					    (size_t)MLD_MINLEN)));
-	    return;		// Error
-	}
-	struct mrt6msg mrt6msg;
-	memcpy(&mrt6msg, _rcvbuf0, sizeof(mrt6msg));
-	if ((mrt6msg.im6_mbz == 0) || (_rcvmh.msg_controllen == 0)) {
-	    //
-	    // XXX: Packets sent up from kernel to daemon have
-	    //      mrt6msg.im6_mbz = icmp6_hdr->icmp6_type = 0
-	    // Because we set ICMP6 filters on the socket,
-	    // we should never see a real ICMPv6 packet
-	    // with icmp6_type = 0 .
-	    //
-	    //
-	    // TODO: XXX: (msg_controllen == 0) is presumably
-	    // true for older IPv6 systems (e.g. KAME circa
-	    // April 2000, FreeBSD-4.0) which don't have the
-	    //     'icmp6_type = 0' mechanism.
-	    //
-	    kernel_call_process(_rcvbuf0, nbytes);
-	    return;		// OK
-	}
-#endif // HAVE_IPV6_MULTICAST_ROUTING
-    }
-    break;
-#endif // HAVE_IPV6
-    
-    default:
-	XLOG_UNREACHABLE();
-	break;
-    }
-    
-    //
-    // Not a kernel signal. Ignore it.
-    //
-    
-    return;			// OK
-}
-
 /**
  * kernel_call_process:
  * @databuf: The data buffer.
@@ -2200,7 +2247,7 @@ MfeaMrouter::mrouter_socket_read(XorpFd fd, IoEventType type)
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
+MfeaMrouter::kernel_call_process(const uint8_t *databuf, size_t datalen)
 {
     uint32_t	iif_vif_index;
     int		message_type;
@@ -2294,7 +2341,6 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 	// Deliver the signal
 	//
 	mfea_node().signal_message_recv("",
-					module_id(),
 					message_type,
 					iif_vif_index, src, dst,
 					databuf + sizeof(struct igmpmsg),
@@ -2384,7 +2430,6 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 	// Deliver the signal
 	//
 	mfea_node().signal_message_recv("",
-					module_id(),
 					message_type,
 					iif_vif_index, src, dst,
 					databuf + sizeof(struct mrt6msg),
