@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/profile_vars.cc,v 1.5 2007/02/16 22:45:49 pavlin Exp $"
+#ident "$XORP: xorp/fea/profile_vars.cc,v 1.8 2008/07/23 05:10:12 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -28,8 +28,8 @@ struct profile_vars {
     string var;
     string comment;
 } profile_vars[] = {
-    {profile_route_in, 	"Routes entering FEA"},
-    {profile_route_out,	"Routes being sent to the KERNEL"},
+    { profile_route_in, 	"Routes entering FEA" },
+    { profile_route_out,	"Routes being sent to the KERNEL" }
 };
 
 void
