@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.16 2007/02/16 22:45:51 pavlin Exp $"
+#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.25 2008/07/23 05:10:12 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -34,9 +34,6 @@
 #include "xrl/interfaces/socket4_xif.hh"
 #include "xrl/targets/test_socket4_base.hh"
 
-#include "xrl_socket_server.hh"
-#include "addr_table.hh"
-
 
 static const uint8_t FILLER_VALUE = 0xe7;
 
@@ -45,8 +42,8 @@ static const uint8_t FILLER_VALUE = 0xe7;
 
 static bool s_verbose = false;
 
-inline bool verbose()           { return s_verbose; }
-inline void set_verbose(bool v) { s_verbose = v; }
+bool verbose()           { return s_verbose; }
+void set_verbose(bool v) { s_verbose = v; }
 
 #define verbose_log(x...)                                                     \
 do {                                                                          \
@@ -65,80 +62,15 @@ do {                                                                          \
 } while(0)
 
 
-// ----------------------------------------------------------------------------
-// TestAddressTable definition and implementation
-
-class TestAddressTable : public AddressTableBase {
-public:
-    void add_address(const IPv4& addr);
-    void remove_address(const IPv4& addr);
-    bool address_valid(const IPv6& addr) const;
-
-    void add_address(const IPv6& addr);
-    void remove_address(const IPv6& addr);
-    bool address_valid(const IPv4& addr) const;
-
-    uint32_t address_pif_index(const IPv4&) const { return 0; }
-    uint32_t address_pif_index(const IPv6&) const { return 0; }
-
-protected:
-    set<IPv4> _v4s;
-    set<IPv6> _v6s;
-};
-
-void
-TestAddressTable::add_address(const IPv4& a)
-{
-    _v4s.insert(a);
-}
-
-void
-TestAddressTable::remove_address(const IPv4& a)
-{
-    set<IPv4>::iterator i = _v4s.find(a);
-    if (i != _v4s.end()) {
-	_v4s.erase(i);
-	invalidate_address(a, "invalidated");
-    }
-}
-
-bool
-TestAddressTable::address_valid(const IPv4& a) const
-{
-    return _v4s.find(a) != _v4s.end();
-}
-
-void
-TestAddressTable::add_address(const IPv6& a)
-{
-    _v6s.insert(a);
-}
-
-void
-TestAddressTable::remove_address(const IPv6& a)
-{
-    set<IPv6>::iterator i = _v6s.find(a);
-    if (i != _v6s.end()) {
-	_v6s.erase(i);
-	invalidate_address(a, "invalidated");
-    }
-}
-
-bool
-TestAddressTable::address_valid(const IPv6& a) const
-{
-    return _v6s.find(a) != _v6s.end();
-}
-
 
 // ----------------------------------------------------------------------------
 // UDP client
 
 class TestSocket4UDP : public XrlTestSocket4TargetBase {
 public:
-    TestSocket4UDP(EventLoop& e, const string& ssname,
+    TestSocket4UDP(EventLoop& e, const string& fea_target_name,
 		   IPv4 finder_host, uint16_t finder_port)
-	: _e(e), _ssname(ssname),
+	: _e(e), _fea_target_name(fea_target_name),
 	  _p_snd(0), _p_rcv(0), _b_snd(0), _b_rcv(0), _x_err(0), _closed(false)
     {
 	_r = new XrlStdRouter(_e, "test_xrl_socket", finder_host, finder_port);
@@ -153,12 +85,12 @@ public:
 	_r = 0;
     }
 
-    inline uint32_t bytes_received() const	{ return _b_rcv; }
-    inline uint32_t bytes_sent() const		{ return _b_snd; }
-    inline uint32_t packets_received() const	{ return _p_rcv; }
-    inline uint32_t packets_sent() const	{ return _p_snd; }
-    inline uint32_t xrl_errors() const		{ return _x_err; }
-    inline bool	    closed() const		{ return _closed; }
+    uint32_t bytes_received() const	{ return _b_rcv; }
+    uint32_t bytes_sent() const		{ return _b_snd; }
+    uint32_t packets_received() const	{ return _p_rcv; }
+    uint32_t packets_sent() const	{ return _p_snd; }
+    uint32_t xrl_errors() const		{ return _x_err; }
+    bool closed() const			{ return _closed; }
 
     /**
      * Bind to interface and port.
@@ -175,9 +107,9 @@ public:
     {
 	XrlSocket4V0p1Client c(_r);
 	verbose_log("Sending bind request (\"%s\", %s/%u)\n",
-		    _ssname.c_str(), addr.str().c_str(), port);
+		    _fea_target_name.c_str(), addr.str().c_str(), port);
 	bool s = c.send_udp_open_and_bind(
-	    _ssname.c_str(), _r->instance_name(), addr, port, false,
+	    _fea_target_name.c_str(), _r->instance_name(), addr, port,
 	    callback(this, &TestSocket4UDP::bind_cb));
 	return s;
     }
@@ -195,7 +127,7 @@ public:
     close()
     {
 	XrlSocket4V0p1Client c(_r);
-	bool s = c.send_close(_ssname.c_str(), _sockid,
+	bool s = c.send_close(_fea_target_name.c_str(), _sockid,
 			      callback(this, &TestSocket4UDP::close_cb));
 	return s;
     }
@@ -233,8 +165,8 @@ protected:
 	vector<uint8_t> d(bytes, FILLER_VALUE);
 
 	XrlSocket4V0p1Client c(_r);
-	bool s = c.send_send_to(_ssname.c_str(), _sockid, host, port, d,
-				callback(this, &TestSocket4UDP::send_data_cb));
+	bool s = c.send_send_to(_fea_target_name.c_str(), _sockid, host, port,
+				d, callback(this, &TestSocket4UDP::send_data_cb));
 	if (s) {
 	    verbose_log("Sent %u bytes to %s/%u\n",
 			bytes, host.str().c_str(), port);
@@ -309,10 +241,15 @@ protected:
 
     XrlCmdError
     socket4_user_0_1_recv_event(const string&	sockid,
+				const string&	if_name,
+				const string&	vif_name,
 				const IPv4&	src_host,
 				const uint32_t&	src_port,
 				const vector<uint8_t>&	data)
     {
+	UNUSED(if_name);
+	UNUSED(vif_name);
+
 	XLOG_ASSERT(sockid == _sockid);
 
 	verbose_log("Received data from %s %u\n",
@@ -328,11 +265,17 @@ protected:
     }
 
     XrlCmdError
-    socket4_user_0_1_connect_event(const string&	/* sockid */,
-				   const IPv4&		/* src_host */,
-				   const uint32_t&	/* src_port */,
-				   const string&	/* new_sockid */,
-				   bool&		/* accept */)
+    socket4_user_0_1_inbound_connect_event(const string&	/* sockid */,
+					   const IPv4&		/* src_host */,
+					   const uint32_t&	/* src_port */,
+					   const string&	/* new_sockid */,
+					   bool&		/* accept */)
+    {
+	return XrlCmdError::COMMAND_FAILED("Not implemented");
+    }
+
+    XrlCmdError
+    socket4_user_0_1_outgoing_connect_event(const string&	/* sockid */)
     {
 	return XrlCmdError::COMMAND_FAILED("Not implemented");
     }
@@ -348,11 +291,10 @@ protected:
     }
 
     XrlCmdError
-    socket4_user_0_1_close_event(const string&	sockid,
-				 const string&	reason)
+    socket4_user_0_1_disconnect_event(const string&	sockid)
     {
 	XLOG_ASSERT(sockid == _sockid);
-	verbose_log("close event: %s\n", reason.c_str());
+	verbose_log("disconnect event\n");
 	_closed = true;
 	return XrlCmdError::OKAY();
     }
@@ -360,7 +302,7 @@ protected:
 private:
     EventLoop&	_e;
     XrlRouter*	_r;
-    string	_ssname;
+    string	_fea_target_name;
     string	_sockid;
 
     uint32_t	_p_snd;		// packets sent
@@ -390,76 +332,15 @@ ticker()
     return true;
 }
 
-static void
-create_socket_server(EventLoop*		pe,
-		     TestAddressTable*	ptat,
-		     IPv4		addr,
-		     uint16_t		port,
-		     XrlSocketServer**	ppxss)
-{
-    XrlSocketServer* pxss = new XrlSocketServer(*pe, *ptat, addr, port);
-    *ppxss = pxss;
-    verbose_log("Created socket server.\n");
-}
-
-static void
-destroy_socket_server(XrlSocketServer** ppxss)
-{
-    delete *ppxss;
-    *ppxss = 0;
-    verbose_log("Destroyed socket server.\n");
-}
-
-static void
-start_socket_server(XrlSocketServer** ppxss)
-{
-    XrlSocketServer* pxss = *ppxss;
-    pxss->startup();
-    verbose_log("Starting socket server.\n");
-}
-
-static void
-shutdown_socket_server(XrlSocketServer** ppxss)
-{
-    XrlSocketServer* pxss = *ppxss;
-    pxss->shutdown();
-    verbose_log("Shutting down socket server.\n");
-}
-
-static void
-verify_socket_owner_count(XrlSocketServer** ppxss, int n, bool* eflag)
-{
-    XrlSocketServer* pxss = *ppxss;
-    int m = pxss->socket_owner_count();
-    if (m != n) {
-	verbose_err("Socket owner count (%d) does not match expected (%d)\n",
-		    m, n);
-	*eflag = true;
-    }
-}
-
-static void
-verify_socket4_count(XrlSocketServer** ppxss, int n, bool* eflag)
-{
-    XrlSocketServer* pxss = *ppxss;
-    int m =  pxss->ipv4_socket_count();
-    if (m != n) {
-	verbose_err("Socket4 count (%d) does not match expected (%d)\n", m, n);
-	*eflag = true;
-    }
-}
-
 static void
 create_test_socket(EventLoop*		pe,
-		   XrlSocketServer**	ppxss,
+		   string		fea_target_name,
 		   IPv4			finder_host,
 		   uint16_t		finder_port,
 		   TestSocket4UDP**	ppu)
 {
-    XrlSocketServer* pxss = *ppxss;
     verbose_log("Creating TestSocket4UDP instance.\n");
-    *ppu = new TestSocket4UDP(*pe, pxss->instance_name(),
-			      finder_host, finder_port);
+    *ppu = new TestSocket4UDP(*pe, fea_target_name, finder_host, finder_port);
 }
 
 static void
@@ -528,12 +409,6 @@ verify_closed(TestSocket4UDP** ppu, bool closed, bool* eflag)
     }
 }
 
-static void
-remove_address(TestAddressTable* ta, IPv4 addr)
-{
-    ta->remove_address(addr);
-}
-
 // ----------------------------------------------------------------------------
 // Test Main
 
@@ -541,36 +416,22 @@ int
 test_main(IPv4 finder_host, uint16_t finder_port)
 {
     EventLoop e;
-    TestAddressTable tat;
-
-    tat.add_address(IPv4::LOOPBACK());
-
+    const string fea_target_name = "fea";
     vector<XorpTimer> ev;	// Vector for timed events
     bool eflag(false);		// Error flag set by timed events
 
     if (verbose())
 	ev.push_back(e.new_periodic_ms(100, callback(ticker)));
 
-    //
-    // Create Socket server.
-    //
-    XrlSocketServer* xss;
-    ev.push_back(e.new_oneoff_after_ms(0,
-				       callback(create_socket_server,
-						&e, &tat, finder_host,
-						finder_port, &xss)));
-    ev.push_back(e.new_oneoff_after_ms(500,
-				       callback(start_socket_server,
-						&xss)));
-
     //
     // Create udp1 socket and bind to port 5000.
     //
     TestSocket4UDP* udp1;
     ev.push_back(e.new_oneoff_after_ms(1000,
 				       callback(create_test_socket,
-						&e, &xss, finder_host,
-						finder_port, &udp1)));
+						&e, fea_target_name,
+						finder_host, finder_port,
+						&udp1)));
 
     ev.push_back(e.new_oneoff_after_ms(1500,
 				       callback(bind_test_socket, &udp1,
@@ -583,8 +444,9 @@ test_main(IPv4 finder_host, uint16_t finder_port)
     TestSocket4UDP* udp2;
     ev.push_back(e.new_oneoff_after_ms(2000,
 				       callback(create_test_socket,
-						&e, &xss, finder_host,
-						finder_port, &udp2)));
+						&e, fea_target_name,
+						finder_host, finder_port,
+						&udp2)));
 
     ev.push_back(e.new_oneoff_after_ms(2500,
 				       callback(bind_test_socket, &udp2,
@@ -647,32 +509,24 @@ test_main(IPv4 finder_host, uint16_t finder_port)
     //
     // Check udp2 is not closed
     //
-    ev.push_back(e.new_oneoff_after_ms(14800,
+    ev.push_back(e.new_oneoff_after_ms(15000,
 				       callback(verify_closed, &udp2,
 						false, &eflag)));
 
     //
-    // Invalidate IPv4::LOOPBACK(), which should cause udp2 to be closed by
-    // server.
+    // Close udp2
     //
-    ev.push_back(e.new_oneoff_after_ms(15000,
-				       callback(remove_address,
-						&tat, IPv4::LOOPBACK())));
+    ev.push_back(e.new_oneoff_after_ms(15500, callback(close_test_socket,
+						       &udp2)));
 
     //
     // Check udp2 is closed.
     //
-    ev.push_back(e.new_oneoff_after_ms(15500,
+    ev.push_back(e.new_oneoff_after_ms(16000,
 				       callback(verify_closed, &udp2,
 						true, &eflag)));
 
     //
-    // Check no sockets open
-    //
-    ev.push_back(e.new_oneoff_after_ms(16000,
-				       callback(verify_socket4_count,
-						&xss, 0, &eflag)));
-    //
     // Destroy udp1 and udp2.
     //
     ev.push_back(e.new_oneoff_after_ms(19998,
@@ -681,29 +535,6 @@ test_main(IPv4 finder_host, uint16_t finder_port)
     ev.push_back(e.new_oneoff_after_ms(19998,
 				       callback(destroy_test_socket, &udp2)));
 
-    //
-    // Check socket server is holding no resources.
-    //
-    ev.push_back(e.new_oneoff_after_ms(21000,
-				       callback(verify_socket_owner_count,
-						&xss, 0, &eflag)));
-
-    ev.push_back(e.new_oneoff_after_ms(21000,
-				       callback(verify_socket4_count,
-						&xss, 0, &eflag)));
-    //
-    // Shutdown socket server.
-    //
-    ev.push_back(e.new_oneoff_after_ms(24000,
-				       callback(shutdown_socket_server,
-						&xss)));
-    //
-    // Destroy socket server.
-    //
-    ev.push_back(e.new_oneoff_after_ms(25000,
-				       callback(destroy_socket_server,
-						&xss)));
-
     //
     // Force exit.
     //
@@ -715,7 +546,7 @@ test_main(IPv4 finder_host, uint16_t finder_port)
     }
 
     if (eflag)
-	return -1;
+	return 1;
 
     return 0;
 }
@@ -827,5 +658,7 @@ main(int argc, char* const argv[])
     xlog_stop();
     xlog_exit();
 
-    return r;
+    if (r != 0)
+	return (1);
+    return (0);
 }
