@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/tools/show_interfaces.cc,v 1.19 2007/02/16 22:45:57 pavlin Exp $"
+#ident "$XORP: xorp/fea/tools/show_interfaces.cc,v 1.29 2008/07/23 05:10:35 pavlin Exp $"
 
 #include "fea/fea_module.h"
 
@@ -62,7 +62,7 @@ InterfaceMonitor::~InterfaceMonitor()
     _ifmgr.unset_observer(this);
 }
 
-bool
+int
 InterfaceMonitor::startup()
 {
     //
@@ -70,10 +70,10 @@ InterfaceMonitor::startup()
     //
     if ((ServiceBase::status() == SERVICE_STARTING)
 	|| (ServiceBase::status() == SERVICE_RUNNING))
-	return true;
+	return (XORP_OK);
 
     if (ServiceBase::status() != SERVICE_READY)
-	return false;
+	return (XORP_ERROR);
 
     //
     // Transition to SERVICE_RUNNING occurs when all transient startup
@@ -85,16 +85,16 @@ InterfaceMonitor::startup()
     //
     // Startup the interface manager
     //
-    if (ifmgr_startup() != true) {
+    if (ifmgr_startup() != XORP_OK) {
 	XLOG_ERROR("Cannot startup the interface mirroring manager");
 	ServiceBase::set_status(SERVICE_FAILED);
-	return false;
+	return (XORP_ERROR);
     }
 
-    return true;
+    return (XORP_OK);
 }
 
-bool
+int
 InterfaceMonitor::shutdown()
 {
     //
@@ -102,11 +102,11 @@ InterfaceMonitor::shutdown()
     //
     if ((ServiceBase::status() == SERVICE_SHUTDOWN)
 	|| (ServiceBase::status() == SERVICE_FAILED)) {
-	return true;
+	return (XORP_OK);
     }
 
     if (ServiceBase::status() != SERVICE_RUNNING)
-	return false;
+	return (XORP_ERROR);
 
     //
     // Transition to SERVICE_SHUTDOWN occurs when all transient shutdown
@@ -118,13 +118,13 @@ InterfaceMonitor::shutdown()
     //
     // Shutdown the interface manager
     //
-    if (ifmgr_shutdown() != true) {
+    if (ifmgr_shutdown() != XORP_OK) {
 	XLOG_ERROR("Cannot shutdown the interface mirroring manager");
 	ServiceBase::set_status(SERVICE_FAILED);
-	return false;
+	return (XORP_ERROR);
     }
 
-    return true;
+    return (XORP_OK);
 }
 
 void
@@ -244,10 +244,10 @@ InterfaceMonitor::updates_made()
 
 }
 
-bool
+int
 InterfaceMonitor::ifmgr_startup()
 {
-    bool ret_value;
+    int ret_value;
 
     // TODO: XXX: we should startup the ifmgr only if it hasn't started yet
     incr_startup_requests_n();
@@ -259,13 +259,13 @@ InterfaceMonitor::ifmgr_startup()
     // will be called.
     //
 
-    return ret_value;
+    return (ret_value);
 }
 
-bool
+int
 InterfaceMonitor::ifmgr_shutdown()
 {
-    bool ret_value;
+    int ret_value;
 
     incr_shutdown_requests_n();
 
@@ -276,7 +276,7 @@ InterfaceMonitor::ifmgr_shutdown()
     // will be called.
     //
 
-    return ret_value;
+    return (ret_value);
 }
 
 void
@@ -285,16 +285,16 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
     bool iface_found = false;
     IfMgrIfTree::IfMap::const_iterator ifmgr_iface_iter;
     IfMgrIfAtom::VifMap::const_iterator ifmgr_vif_iter;
-    IfMgrVifAtom::V4Map::const_iterator a4_iter;
-    IfMgrVifAtom::V6Map::const_iterator a6_iter;
+    IfMgrVifAtom::IPv4Map::const_iterator a4_iter;
+    IfMgrVifAtom::IPv6Map::const_iterator a6_iter;
 
     debug_msg("Begin iftree loop\n");
 
     //
     // Iterate for all interface, vifs, and addresses, and print them
     //
-    for (ifmgr_iface_iter = ifmgr_iftree().ifs().begin();
-	 ifmgr_iface_iter != ifmgr_iftree().ifs().end();
+    for (ifmgr_iface_iter = ifmgr_iftree().interfaces().begin();
+	 ifmgr_iface_iter != ifmgr_iftree().interfaces().end();
 	 ++ifmgr_iface_iter) {
 	const IfMgrIfAtom& ifmgr_iface = ifmgr_iface_iter->second;
 	const string& ifmgr_iface_name = ifmgr_iface.name();
@@ -327,14 +327,79 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
 		fprintf(stdout, "ENABLED");
 		prev = true;
 	    }
-	    fprintf(stdout, "> mtu %d\n", ifmgr_iface.mtu_bytes());
+	    if (ifmgr_iface.discard()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "DISCARD");
+		prev = true;
+	    }
+	    if (ifmgr_iface.unreachable()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "UNREACHABLE");
+		prev = true;
+	    }
+	    if (ifmgr_iface.management()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "MANAGEMENT");
+		prev = true;
+	    }
+	    fprintf(stdout, "> mtu %d", ifmgr_iface.mtu());
+
+	    //
+	    // Print the baudrate in bps/Kbps/Mbps/Gbps/Tbps
+	    //
+	    fprintf(stdout, " speed ");
+	    if (ifmgr_iface.baudrate() == 0) {
+		fprintf(stdout, "unknown");
+	    } else {
+		do {
+		    uint64_t b = ifmgr_iface.baudrate();
+
+		    // Test if bps
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u bps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Kbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Kbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Mbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Mbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Gbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Gbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Tbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Tbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Default
+		    fprintf(stdout, "%u Tbps", XORP_UINT_CAST(b));
+
+		    break;
+		} while (false);
+	    }
+	    fprintf(stdout, "\n");
 
 	    //
 	    // Print the physical interface index and MAC address
 	    //
 	    fprintf(stdout, "        physical index %d\n",
 		    ifmgr_iface.pif_index());
-	    if (!ifmgr_iface.mac().empty())
+	    if (!ifmgr_iface.mac().is_zero())
 		fprintf(stdout, "        ether %s\n",
 			ifmgr_iface.mac().str().c_str());
 	}
@@ -365,6 +430,24 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
 		fprintf(stdout, "ENABLED");
 		prev = true;
 	    }
+	    if (ifmgr_iface.discard()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "DISCARD");
+		prev = true;
+	    }
+	    if (ifmgr_iface.unreachable()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "UNREACHABLE");
+		prev = true;
+	    }
+	    if (ifmgr_iface.management()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "MANAGEMENT");
+		prev = true;
+	    }
 	    if (ifmgr_vif.broadcast_capable()) {
 		if (prev)
 		    fprintf(stdout, ",");
@@ -389,7 +472,60 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
 		fprintf(stdout, "POINTTOPOINT");
 		prev = true;
 	    }
-	    fprintf(stdout, "> mtu %d\n", ifmgr_iface.mtu_bytes());
+	    if (ifmgr_vif.pim_register()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "PIM-REGISTER");
+		prev = true;
+	    }
+	    fprintf(stdout, "> mtu %d", ifmgr_iface.mtu());
+
+	    //
+	    // Print the baudrate in bps/Kbps/Mbps/Gbps/Tbps
+	    //
+	    fprintf(stdout, " speed ");
+	    if (ifmgr_iface.baudrate() == 0) {
+		fprintf(stdout, "unknown");
+	    } else {
+		do {
+		    uint64_t b = ifmgr_iface.baudrate();
+
+		    // Test if bps
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u bps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Kbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Kbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Mbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Mbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Gbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Gbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Test if Tbps
+		    b /= 1000;
+		    if (b / 1000 == 0) {
+			fprintf(stdout, "%u Tbps", XORP_UINT_CAST(b));
+			break;
+		    }
+		    // Default
+		    fprintf(stdout, "%u Tbps", XORP_UINT_CAST(b));
+
+		    break;
+		} while (false);
+	    }
+	    fprintf(stdout, "\n");
 
 	    //
 	    // Print the IPv6 addresses
@@ -439,9 +575,18 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
 	    //
 	    fprintf(stdout, "        physical index %d\n",
 		    ifmgr_vif.pif_index());
-	    if (!ifmgr_iface.mac().empty())
+	    if (!ifmgr_iface.mac().is_zero()) {
 		fprintf(stdout, "        ether %s\n",
 			ifmgr_iface.mac().str().c_str());
+	    }
+
+	    //
+	    // Print the VLAN-related information
+	    //
+	    if (ifmgr_vif.is_vlan()) {
+		fprintf(stdout, "        vlan: %u parent interface: %s\n",
+			ifmgr_vif.vlan_id(), ifmgr_iface_name.c_str());
+	    }
 	}
     }
 
