@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fib2mrib/fib2mrib_node.cc,v 1.33 2007/02/16 22:45:58 pavlin Exp $"
+#ident "$XORP: xorp/fib2mrib/fib2mrib_node.cc,v 1.39 2008/07/23 05:10:35 pavlin Exp $"
 
 //
 // Fib2mrib node implementation.
@@ -46,7 +46,7 @@ Fib2mribNode::~Fib2mribNode()
     shutdown();
 }
 
-bool
+int
 Fib2mribNode::startup()
 {
     //
@@ -54,10 +54,10 @@ Fib2mribNode::startup()
     //
     if ((ServiceBase::status() == SERVICE_STARTING)
 	|| (ServiceBase::status() == SERVICE_RUNNING)) {
-	return true;
+	return (XORP_OK);
     }
     if (ServiceBase::status() != SERVICE_READY) {
-	return false;
+	return (XORP_ERROR);
     }
 
     //
@@ -82,10 +82,10 @@ Fib2mribNode::startup()
     //
     rib_register_startup();
 
-    return true;
+    return (XORP_OK);
 }
 
-bool
+int
 Fib2mribNode::shutdown()
 {
     //
@@ -94,14 +94,14 @@ Fib2mribNode::shutdown()
     if ((ServiceBase::status() == SERVICE_SHUTDOWN)
 	|| (ServiceBase::status() == SERVICE_SHUTTING_DOWN)
 	|| (ServiceBase::status() == SERVICE_FAILED)) {
-	return true;
+	return (XORP_OK);
     }
     if ((ServiceBase::status() != SERVICE_RUNNING)
 	&& (ServiceBase::status() != SERVICE_STARTING)
 	&& (ServiceBase::status() != SERVICE_PAUSING)
 	&& (ServiceBase::status() != SERVICE_PAUSED)
 	&& (ServiceBase::status() != SERVICE_RESUMING)) {
-	return false;
+	return (XORP_ERROR);
     }
 
     //
@@ -131,7 +131,7 @@ Fib2mribNode::shutdown()
     //
     update_status();
 
-    return true;
+    return (XORP_OK);
 }
 
 void
@@ -319,7 +319,7 @@ Fib2mribNode::updates_made()
 	    const IfMgrIfAtom* if_atom;
 	    const IfMgrVifAtom* vif_atom;
 
-	    if_atom = _iftree.find_if(fib2mrib_route.ifname());
+	    if_atom = _iftree.find_interface(fib2mrib_route.ifname());
 	    vif_atom = _iftree.find_vif(fib2mrib_route.ifname(),
 					fib2mrib_route.vifname());
 	    if ((if_atom != NULL) && (if_atom->enabled())
@@ -328,7 +328,7 @@ Fib2mribNode::updates_made()
 		is_old_up = true;
 	    }
 
-	    if_atom = ifmgr_iftree().find_if(fib2mrib_route.ifname());
+	    if_atom = ifmgr_iftree().find_interface(fib2mrib_route.ifname());
 	    vif_atom = ifmgr_iftree().find_vif(fib2mrib_route.ifname(),
 					       fib2mrib_route.vifname());
 	    if ((if_atom != NULL) && (if_atom->enabled())
@@ -1049,7 +1049,7 @@ Fib2mribNode::is_accepted_by_nexthop(const Fib2mribRoute& route) const
 	const IfMgrVifAtom* vif_atom;
 	bool is_up = false;
 
-	if_atom = _iftree.find_if(route.ifname());
+	if_atom = _iftree.find_interface(route.ifname());
 	vif_atom = _iftree.find_vif(route.ifname(), route.vifname());
 	if ((if_atom != NULL) && (if_atom->enabled())
 	    && (! if_atom->no_carrier())
@@ -1158,8 +1158,8 @@ Fib2mribNode::update_route(const IfMgrIfTree& iftree, Fib2mribRoute& route)
     // or the address of the next-hop router.
     //
     IfMgrIfTree::IfMap::const_iterator if_iter;
-    for (if_iter = iftree.ifs().begin();
-	 if_iter != iftree.ifs().end();
+    for (if_iter = iftree.interfaces().begin();
+	 if_iter != iftree.interfaces().end();
 	 ++if_iter) {
 	const IfMgrIfAtom& iface = if_iter->second;
 
@@ -1173,7 +1173,7 @@ Fib2mribNode::update_route(const IfMgrIfTree& iftree, Fib2mribRoute& route)
 	    // Check the IPv4 subnets
 	    //
 	    if (route.is_ipv4()) {
-		IfMgrVifAtom::V4Map::const_iterator a4_iter;
+		IfMgrVifAtom::IPv4Map::const_iterator a4_iter;
 		for (a4_iter = vif.ipv4addrs().begin();
 		     a4_iter != vif.ipv4addrs().end();
 		     ++a4_iter) {
@@ -1203,7 +1203,7 @@ Fib2mribNode::update_route(const IfMgrIfTree& iftree, Fib2mribRoute& route)
 	    // Check the IPv6 subnets
 	    //
 	    if (route.is_ipv6()) {
-		IfMgrVifAtom::V6Map::const_iterator a6_iter;
+		IfMgrVifAtom::IPv6Map::const_iterator a6_iter;
 		for (a6_iter = vif.ipv6addrs().begin();
 		     a6_iter != vif.ipv6addrs().end();
 		     ++a6_iter) {
