@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fib2mrib/xrl_fib2mrib_node.cc,v 1.36 2007/02/16 22:45:58 pavlin Exp $"
+#ident "$XORP: xorp/fib2mrib/xrl_fib2mrib_node.cc,v 1.42 2008/07/23 05:10:36 pavlin Exp $"
 
 #include "fib2mrib_module.h"
 
@@ -39,8 +39,6 @@ XrlFib2mribNode::XrlFib2mribNode(EventLoop&	eventloop,
 		   finder_port),
       XrlFib2mribTargetBase(&xrl_router()),
       _eventloop(eventloop),
-      _class_name(xrl_router().class_name()),
-      _instance_name(xrl_router().instance_name()),
       _xrl_fea_fti_client(&xrl_router()),
       _xrl_fea_fib_client(&xrl_router()),
       _xrl_rib_client(&xrl_router()),
@@ -80,16 +78,16 @@ XrlFib2mribNode::~XrlFib2mribNode()
     _ifmgr.unset_observer(dynamic_cast<Fib2mribNode*>(this));
 }
 
-bool
+int
 XrlFib2mribNode::startup()
 {
-    return Fib2mribNode::startup();
+    return (Fib2mribNode::startup());
 }
 
-bool
+int
 XrlFib2mribNode::shutdown()
 {
-    return Fib2mribNode::shutdown();
+    return (Fib2mribNode::shutdown());
 }
 
 //
@@ -153,7 +151,7 @@ XrlFib2mribNode::fea_register_startup()
     // Register interest in the FEA with the Finder
     //
     success = _xrl_finder_client.send_register_class_event_interest(
-	_finder_target.c_str(), _instance_name, _fea_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _fea_target,
 	callback(this, &XrlFib2mribNode::finder_register_interest_fea_cb));
 
     if (! success) {
@@ -184,7 +182,7 @@ XrlFib2mribNode::finder_register_interest_fea_cb(const XrlError& xrl_error)
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot register interest in finder events: %s",
+	XLOG_FATAL("Cannot register interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -193,7 +191,7 @@ XrlFib2mribNode::finder_register_interest_fea_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -218,7 +216,7 @@ XrlFib2mribNode::finder_register_interest_fea_cb(const XrlError& xrl_error)
 	// (unless the timer is already running).
 	//
 	if (! _fea_register_startup_timer.scheduled()) {
-	    XLOG_ERROR("Failed to register interest in finder events: %s. "
+	    XLOG_ERROR("Failed to register interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _fea_register_startup_timer = _eventloop.new_oneoff_after(
@@ -264,7 +262,7 @@ XrlFib2mribNode::fea_register_shutdown()
     // De-register interest in the FEA with the Finder
     //
     success = _xrl_finder_client.send_deregister_class_event_interest(
-	_finder_target.c_str(), _instance_name, _fea_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _fea_target,
 	callback(this, &XrlFib2mribNode::finder_deregister_interest_fea_cb));
 
     if (! success) {
@@ -303,7 +301,7 @@ XrlFib2mribNode::finder_deregister_interest_fea_cb(const XrlError& xrl_error)
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot deregister interest in finder events: %s",
+	XLOG_FATAL("Cannot deregister interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -312,7 +310,7 @@ XrlFib2mribNode::finder_deregister_interest_fea_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -338,7 +336,7 @@ XrlFib2mribNode::finder_deregister_interest_fea_cb(const XrlError& xrl_error)
 	// (unless the timer is already running).
 	//
 	if (! _fea_register_shutdown_timer.scheduled()) {
-	    XLOG_ERROR("Failed to deregister interest in finder events: %s. "
+	    XLOG_ERROR("Failed to deregister interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _fea_register_shutdown_timer = _eventloop.new_oneoff_after(
@@ -379,7 +377,7 @@ XrlFib2mribNode::rib_register_startup()
     // Register interest in the RIB with the Finder
     //
     success = _xrl_finder_client.send_register_class_event_interest(
-	_finder_target.c_str(), _instance_name, _rib_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _rib_target,
 	callback(this, &XrlFib2mribNode::finder_register_interest_rib_cb));
 
     if (! success) {
@@ -411,7 +409,7 @@ XrlFib2mribNode::finder_register_interest_rib_cb(const XrlError& xrl_error)
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot register interest in finder events: %s",
+	XLOG_FATAL("Cannot register interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -420,7 +418,7 @@ XrlFib2mribNode::finder_register_interest_rib_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -445,7 +443,7 @@ XrlFib2mribNode::finder_register_interest_rib_cb(const XrlError& xrl_error)
 	// (unless the timer is already running).
 	//
 	if (! _rib_register_startup_timer.scheduled()) {
-	    XLOG_ERROR("Failed to register interest in finder events: %s. "
+	    XLOG_ERROR("Failed to register interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _rib_register_startup_timer = _eventloop.new_oneoff_after(
@@ -489,7 +487,7 @@ XrlFib2mribNode::rib_register_shutdown()
     // De-register interest in the RIB with the Finder
     //
     success = _xrl_finder_client.send_deregister_class_event_interest(
-	_finder_target.c_str(), _instance_name, _rib_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _rib_target,
 	callback(this, &XrlFib2mribNode::finder_deregister_interest_rib_cb));
 
     if (! success) {
@@ -522,7 +520,7 @@ XrlFib2mribNode::finder_deregister_interest_rib_cb(const XrlError& xrl_error)
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot deregister interest in finder events: %s",
+	XLOG_FATAL("Cannot deregister interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -531,7 +529,7 @@ XrlFib2mribNode::finder_deregister_interest_rib_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -557,7 +555,7 @@ XrlFib2mribNode::finder_deregister_interest_rib_cb(const XrlError& xrl_error)
 	// (unless the timer is already running).
 	//
 	if (! _rib_register_shutdown_timer.scheduled()) {
-	    XLOG_ERROR("Failed to deregister interest in finder events: %s. "
+	    XLOG_ERROR("Failed to deregister interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _rib_register_shutdown_timer = _eventloop.new_oneoff_after(
@@ -612,7 +610,7 @@ XrlFib2mribNode::send_fea_add_fib_client()
     if (_fea_have_ipv4 && ! _is_fea_fib_client4_registered) {
 	success = _xrl_fea_fib_client.send_add_fib_client4(
 	    _fea_target.c_str(),
-	    my_xrl_target_name(),
+	    xrl_router().class_name(),
 	    true,		/* send_updates */
 	    false,		/* send_resolves */
 	    callback(this, &XrlFib2mribNode::fea_fib_client_send_add_fib_client4_cb));
@@ -626,7 +624,7 @@ XrlFib2mribNode::send_fea_add_fib_client()
     if (_fea_have_ipv6 && ! _is_fea_fib_client6_registered) {
 	success = _xrl_fea_fib_client.send_add_fib_client6(
 	    _fea_target.c_str(),
-	    my_xrl_target_name(),
+	    xrl_router().class_name(),
 	    true,		/* send_updates */
 	    false,		/* send_resolves */
 	    callback(this, &XrlFib2mribNode::fea_fib_client_send_add_fib_client6_cb));
@@ -680,7 +678,7 @@ XrlFib2mribNode::fea_fti_client_send_have_ipv4_cb(const XrlError& xrl_error,
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -749,7 +747,7 @@ XrlFib2mribNode::fea_fti_client_send_have_ipv6_cb(const XrlError& xrl_error,
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -814,7 +812,7 @@ XrlFib2mribNode::fea_fib_client_send_add_fib_client4_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -878,7 +876,7 @@ XrlFib2mribNode::fea_fib_client_send_add_fib_client6_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -929,7 +927,7 @@ XrlFib2mribNode::send_fea_delete_fib_client()
 	bool success4;
 	success4 = _xrl_fea_fib_client.send_delete_fib_client4(
 	    _fea_target.c_str(),
-	    my_xrl_target_name(),
+	    xrl_router().class_name(),
 	    callback(this, &XrlFib2mribNode::fea_fib_client_send_delete_fib_client4_cb));
 	if (success4 != true) {
 	    XLOG_ERROR("Failed to deregister IPv4 FIB client with the FEA. "
@@ -942,7 +940,7 @@ XrlFib2mribNode::send_fea_delete_fib_client()
 	bool success6;
 	success6 = _xrl_fea_fib_client.send_delete_fib_client6(
 	    _fea_target.c_str(),
-	    my_xrl_target_name(),
+	    xrl_router().class_name(),
 	    callback(this, &XrlFib2mribNode::fea_fib_client_send_delete_fib_client6_cb));
 	if (success6 != true) {
 	    XLOG_ERROR("Failed to deregister IPv6 FIB client with the FEA. "
@@ -986,7 +984,7 @@ XrlFib2mribNode::fea_fib_client_send_delete_fib_client4_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -1054,7 +1052,7 @@ XrlFib2mribNode::fea_fib_client_send_delete_fib_client6_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -1108,8 +1106,8 @@ XrlFib2mribNode::send_rib_add_tables()
 	success = _xrl_rib_client.send_add_igp_table4(
 	    _rib_target.c_str(),
 	    Fib2mribNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    false,	/* unicast */
 	    true,	/* multicast */
 	    callback(this, &XrlFib2mribNode::rib_client_send_add_igp_table4_cb));
@@ -1124,8 +1122,8 @@ XrlFib2mribNode::send_rib_add_tables()
 	success = _xrl_rib_client.send_add_igp_table6(
 	    _rib_target.c_str(),
 	    Fib2mribNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    false,	/* unicast */
 	    true,	/* multicast */
 	    callback(this, &XrlFib2mribNode::rib_client_send_add_igp_table6_cb));
@@ -1174,7 +1172,7 @@ XrlFib2mribNode::rib_client_send_add_igp_table4_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -1237,7 +1235,7 @@ XrlFib2mribNode::rib_client_send_add_igp_table6_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -1289,8 +1287,8 @@ XrlFib2mribNode::send_rib_delete_tables()
 	success4 = _xrl_rib_client.send_delete_igp_table4(
 	    _rib_target.c_str(),
 	    Fib2mribNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    false,	/* unicast */
 	    true,	/* multicast */
 	    callback(this, &XrlFib2mribNode::rib_client_send_delete_igp_table4_cb));
@@ -1306,8 +1304,8 @@ XrlFib2mribNode::send_rib_delete_tables()
 	success6 = _xrl_rib_client.send_delete_igp_table6(
 	    _rib_target.c_str(),
 	    Fib2mribNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    false,	/* unicast */
 	    true,	/* multicast */
 	    callback(this, &XrlFib2mribNode::rib_client_send_delete_igp_table6_cb));
@@ -1351,7 +1349,7 @@ XrlFib2mribNode::rib_client_send_delete_igp_table4_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -1415,7 +1413,7 @@ XrlFib2mribNode::rib_client_send_delete_igp_table6_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -1465,7 +1463,7 @@ XrlFib2mribNode::common_0_1_get_target_name(
     // Output values, 
     string&	name)
 {
-    name = my_xrl_target_name();
+    name = xrl_router().class_name();
 
     return XrlCmdError::OKAY();
 }
@@ -1505,7 +1503,7 @@ XrlFib2mribNode::common_0_1_shutdown()
 {
     string error_msg;
 
-    if (shutdown() != true) {
+    if (shutdown() != XORP_OK) {
 	error_msg = c_format("Failed to shutdown Fib2mrib");
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
@@ -1532,7 +1530,7 @@ XrlFib2mribNode::finder_event_observer_0_1_xrl_target_birth(
 	// IfMgrHintObserver::tree_complete() will be called.
 	//
 	_is_fea_alive = true;
-	if (_ifmgr.startup() != true) {
+	if (_ifmgr.startup() != XORP_OK) {
 	    Fib2mribNode::ServiceBase::set_status(SERVICE_FAILED);
 	    Fib2mribNode::update_status();
 	} else {
@@ -1633,7 +1631,7 @@ XrlFib2mribNode::fea_fib_client_0_1_add_route4(
 	      XORP_UINT_CAST(metric),
 	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
-	      (xorp_route)? "true" : "false");
+	      bool_c_str(xorp_route));
 
     if (Fib2mribNode::add_route4(network, nexthop, ifname, vifname,
 				 metric, admin_distance,
@@ -1670,7 +1668,7 @@ XrlFib2mribNode::fea_fib_client_0_1_add_route6(
 	      XORP_UINT_CAST(metric),
 	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
-	      (xorp_route)? "true" : "false");
+	      bool_c_str(xorp_route));
 
     if (Fib2mribNode::add_route6(network, nexthop, ifname, vifname,
 				 metric, admin_distance,
@@ -1731,7 +1729,7 @@ XrlFib2mribNode::fea_fib_client_0_1_replace_route4(
 	      XORP_UINT_CAST(metric),
 	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
-	      (xorp_route)? "true" : "false");
+	      bool_c_str(xorp_route));
 
     if (Fib2mribNode::replace_route4(network, nexthop, ifname, vifname,
 				     metric, admin_distance,
@@ -1768,7 +1766,7 @@ XrlFib2mribNode::fea_fib_client_0_1_replace_route6(
 	      XORP_UINT_CAST(metric),
 	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
-	      (xorp_route)? "true" : "false");
+	      bool_c_str(xorp_route));
 
     if (Fib2mribNode::replace_route6(network, nexthop, ifname, vifname,
 				     metric, admin_distance,
@@ -2185,7 +2183,7 @@ XrlFib2mribNode::send_rib_route_change_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
