@@ -32,7 +32,7 @@
  */
 
 /*
- * $XORP: xorp/libcomm/comm_api.h,v 1.22 2006/03/01 12:55:33 bms Exp $
+ * $XORP: xorp/libcomm/comm_api.h,v 1.33 2008/07/07 11:27:07 bms Exp $
  */
 
 #ifndef __LIBCOMM_COMM_API_H__
@@ -62,6 +62,8 @@
 #define COMM_SOCK_BLOCKING		1
 #define COMM_SOCK_NONBLOCKING		0
 
+#define COMM_LISTEN_DEFAULT_BACKLOG	5
+
 #ifndef AF_LOCAL
 #define AF_LOCAL		AF_UNIX	   /* XXX: AF_UNIX is the older name */
 #endif
@@ -96,30 +98,36 @@ __BEGIN_DECLS
  */
 
 /**
- * Init stuff. Need to be called only once (during startup).
+ * Library initialization. Need be called only once, during startup.
+ *
+ * Note: Currently it is not thread-safe.
  *
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 extern int	comm_init(void);
 
 /**
- * Finalization. Must be called when client is finished with the library.
+ * Library termination/cleanup. Must be called at process exit.
+ *
+ * Note: Currently it is not thread-safe.
  */
 extern void	comm_exit(void);
 
 /**
  * Retrieve the most recently occured error for the current thread.
  *
- * @return Operating system specific error code for this thread's
- *         last socket operation.
+ * @return operating system specific error code for this thread's
+ * last socket operation.
  */
 extern int	comm_get_last_error(void);
 
 /**
  * Retrieve a human readable string (in English) for the given error code.
  *
+ * Note: Currently it is not thread-safe.
+ *
  * @param serrno the socket error number returned by comm_get_last_error().
- * @return Pointer to a string giving more information about the error.
+ * @return a pointer to a string giving more information about the error.
  */
 extern char const *	comm_get_error_str(int serrno);
 
@@ -131,19 +139,47 @@ extern char const *	comm_get_error_str(int serrno);
 extern char const *comm_get_last_error_str(void);
 
 /**
- * Indicate presence of IPv4 support.
+ * Test whether the underlying system has IPv4 support.
  *
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 extern int	comm_ipv4_present(void);
 
 /**
- * Indicate presence of IPv6 support.
+ * Test whether the underlying system has IPv6 support.
  *
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 extern int	comm_ipv6_present(void);
 
+/**
+ * Test whether the underlying system has SO_BINDTODEVICE support.
+ *
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_bindtodevice_present(void);
+
+/**
+ * Test whether the underlying system has IP_ONESBCAST support.
+ *
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_onesbcast_present(void);
+
+/**
+ * Test whether the underlying system has IP_TOS support.
+ *
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_tos_present(void);
+
+/**
+ * Test whether the underlying system has IP_TTL support.
+ *
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_unicast_ttl_present(void);
+
 /**
  * Open a TCP socket.
  *
@@ -172,6 +208,15 @@ extern xsock_t	comm_open_udp(int family, int is_blocking);
  */
 extern int	comm_close(xsock_t sock);
 
+/**
+ * Listen on a socket.
+ *
+ * @param sock the socket to listen on.
+ * @param backlog the maximum queue size for pending connections.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_listen(xsock_t sock, int backlog);
+
 /**
  * Open an IPv4 TCP socket and bind it to a local address and a port.
  *
@@ -190,13 +235,17 @@ extern xsock_t	comm_bind_tcp4(const struct in_addr *my_addr,
  *
  * @param my_addr the local IPv6 address to bind to (in network order).
  * If it is NULL, will bind to `any' local address.
+ * @param my_ifindex the local network interface index to bind to.
+ * It is required if @ref my_addr is a link-local address, otherwise it
+ * should be set to 0.
  * @param my_port the local port to bind to (in network order).
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
  * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
 extern xsock_t	comm_bind_tcp6(const struct in6_addr *my_addr,
-			       unsigned short my_port, int is_blocking);
+			       unsigned int my_ifindex, unsigned short my_port,
+			       int is_blocking);
 
 /**
  * Open a TCP (IPv4 or IPv6) socket and bind it to a local address and a port.
@@ -228,13 +277,17 @@ extern xsock_t	comm_bind_udp4(const struct in_addr *my_addr,
  *
  * @param my_addr the local IPv6 address to bind to (in network order).
  * If it is NULL, will bind to `any' local address.
+ * @param my_ifindex the network interface index to bind to.
+ * It is required if @ref my_addr is a link-local address, otherwise it
+ * should be set to 0.
  * @param my_port the local port to bind to (in network order).
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
  * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
 extern xsock_t	comm_bind_udp6(const struct in6_addr *my_addr,
-			       unsigned short my_port, int is_blocking);
+			       unsigned int my_ifindex, unsigned short my_port,
+			       int is_blocking);
 
 /**
  * Open an IPv4 UDP socket on an interface, bind it to a port,
@@ -249,19 +302,14 @@ extern xsock_t	comm_bind_udp6(const struct in6_addr *my_addr,
  * one of the local interface addresses and the same port number.
  *
  * @param mcast_addr the multicast address to join.
- *
  * @param join_if_addr the local unicast interface address (in network order)
  * to join the multicast group on. If it is NULL, the system will choose the
  * interface each time a datagram is sent.
- *
  * @param my_port the port to bind to (in network order).
- *
  * @param reuse_flag if true, allow other sockets to bind to the same multicast
  * address and port, otherwise disallow it.
- *
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- *
  * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
 extern xsock_t	comm_bind_join_udp4(const struct in_addr *mcast_addr,
@@ -282,23 +330,18 @@ extern xsock_t	comm_bind_join_udp4(const struct in_addr *mcast_addr,
  * one of the local interface addresses and the same port number.
  *
  * @param mcast_addr the multicast address to join.
- *
- * @param join_if_index the local unicast interface index to join the multicast
+ * @param my_ifindex the local network interface index to join the multicast
  * group on. If it is 0, the system will choose the interface each time a
  * datagram is sent.
- *
  * @param my_port the port to bind to (in network order).
- *
  * @param reuse_flag if true, allow other sockets to bind to the same multicast
  * address and port, otherwise disallow it.
- *
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- *
  * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
 extern xsock_t	comm_bind_join_udp6(const struct in6_addr *mcast_addr,
-				    unsigned int join_if_index,
+				    unsigned int my_ifindex,
 				    unsigned short my_port,
 				    int reuse_flag, int is_blocking);
 
@@ -378,6 +421,60 @@ extern xsock_t	comm_connect_udp6(const struct in6_addr *remote_addr,
 				  int is_blocking,
 				  int *in_progress);
 
+/**
+ * Open an IPv4 TCP socket, bind it to a local address and a port,
+ * and connect it to a remote address and port.
+ *
+ * @param local_addr the local address to bind to.
+ * If it is NULL, will bind to `any' local address.
+ * @param local_port the local port to bind to.
+ * @param remote_addr the remote address to connect to.
+ * @param remote_port the remote port to connect to.
+ * @param is_blocking if true then the socket will be blocking, otherwise
+ * non-blocking.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
+ */
+extern xsock_t	comm_bind_connect_tcp4(const struct in_addr *local_addr,
+				       unsigned short local_port,
+				       const struct in_addr *remote_addr,
+				       unsigned short remote_port,
+				       int is_blocking,
+				       int *in_progress);
+
+/**
+ * Open an IPv6 TCP socket, bind it to a local address and a port,
+ * and connect it to a remote address and port.
+ *
+ * @param local_addr the local address to bind to.
+ * If it is NULL, will bind to `any' local address.
+ * @param my_ifindex the network interface index to bind to.
+ * It is required if @ref local_addr is a link-local address, otherwise it
+ * should be set to 0.
+ * @param local_port the local port to bind to.
+ * @param remote_addr the remote address to connect to.
+ * @param remote_port the remote port to connect to.
+ * @param is_blocking if true then the socket will be blocking, otherwise
+ * non-blocking.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
+ */
+extern xsock_t	comm_bind_connect_tcp6(const struct in6_addr *local_addr,
+				       unsigned int my_ifindex,
+				       unsigned short local_port,
+				       const struct in6_addr *remote_addr,
+				       unsigned short remote_port,
+				       int is_blocking,
+				       int *in_progress);
+
 /**
  * Open an IPv4 UDP socket, bind it to a local address and a port,
  * and connect it to a remote address and port.
@@ -409,6 +506,9 @@ extern xsock_t	comm_bind_connect_udp4(const struct in_addr *local_addr,
  *
  * @param local_addr the local address to bind to.
  * If it is NULL, will bind to `any' local address.
+ * @param my_ifindex the network interface index to bind to.
+ * It is required if @ref local_addr is a link-local address, otherwise it
+ * should be set to 0.
  * @param local_port the local port to bind to.
  * @param remote_addr the remote address to connect to.
  * @param remote_port the remote port to connect to.
@@ -422,6 +522,7 @@ extern xsock_t	comm_bind_connect_udp4(const struct in_addr *local_addr,
  * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
 extern xsock_t	comm_bind_connect_udp6(const struct in6_addr *local_addr,
+				       unsigned int my_ifindex,
 				       unsigned short local_port,
 				       const struct in6_addr *remote_addr,
 				       unsigned short remote_port,
@@ -434,8 +535,7 @@ extern xsock_t	comm_bind_connect_udp6(const struct in6_addr *local_addr,
  */
 
 /**
- * Open a socket of domain = @ref domain, type = @ref type,
- * protocol = @protocol, and blocking = @blocking.
+ * Open a socket for given domain, type and protocol.
  *
  * The sending and receiving buffer size are set, and the socket
  * itself is set with TCP_NODELAY (if a TCP socket).
@@ -456,19 +556,19 @@ extern xsock_t	comm_sock_open(int domain, int type, int protocol,
  * The sockets will be created in the blocking state by default, and
  * with no additional socket options set.
  *
- * Currently a domain of AF_UNIX and a type of SOCK_STREAM must be
- * specified. On Windows platforms, the sockets created will actually
- * be in the AF_INET domain.
+ * On Windows platforms, a domain of AF_UNIX, AF_INET, or AF_INET6 must
+ * be specified. For the AF_UNIX case, the sockets created will actually
+ * be in the AF_INET domain. The protocol field is ignored.
+ *
+ * On UNIX, this function simply wraps the socketpair(2) system call.
  *
  * @param domain the domain of the socket (e.g., AF_INET, AF_INET6).
  * @param type the type of the socket (e.g., SOCK_STREAM, SOCK_DGRAM).
  * @param protocol the particular protocol to be used with the socket.
  * @param sv pointer to an array of two xsock_t handles to receive the
- *        allocated socket pair.
- *
- * @return XORP_OK if the socket pair was created, otherwise if any error
- * is encountered, XORP_ERROR.
- **/
+ * allocated socket pair.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
 extern int	comm_sock_pair(int domain, int type, int protocol,
 			       xsock_t sv[2]);
 
@@ -490,10 +590,14 @@ extern int	comm_sock_bind4(xsock_t sock, const struct in_addr *my_addr,
  * @param sock the socket to bind.
  * @param my_addr the address to bind to (in network order).
  * If it is NULL, will bind to `any' local address.
+ * @param my_ifindex the network interface index to bind to.
+ * It is required if @ref my_addr is a link-local address, otherwise it
+ * should be set to 0.
  * @param my_port the port to bind to (in network order).
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 extern int	comm_sock_bind6(xsock_t sock, const struct in6_addr *my_addr,
+				unsigned int my_ifindex,
 				unsigned short my_port);
 
 /**
@@ -525,7 +629,7 @@ extern int	comm_sock_join4(xsock_t sock,
  *
  * @param sock he socket to join the group.
  * @param mcast_addr the multicast address to join.
- * @param my_ifindex the local unicast interface index to join.
+ * @param my_ifindex the local network interface index to join.
  * If it is 0, the interface is chosen by the kernel.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
@@ -551,7 +655,7 @@ extern int	comm_sock_leave4(xsock_t sock,
  *
  * @param sock he socket to leave the group.
  * @param mcast_addr the multicast address to leave.
- * @param my_ifindex the local unicast interface index to leave.
+ * @param my_ifindex the local network interface index to leave.
  * If it is 0, the interface is chosen by the kernel.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
@@ -634,6 +738,15 @@ extern int	comm_sock_connect(xsock_t sock, const struct sockaddr *sin,
  */
 extern xsock_t	comm_sock_accept(xsock_t sock);
 
+/**
+ * Listen for connections on a socket.
+ *
+ * @param sock the socket to listen on.
+ * @param backlog the maximum queue size for pending connections
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_sock_listen(xsock_t sock, int backlog);
+
 /**
  * Close a socket.
  *
@@ -642,6 +755,25 @@ extern xsock_t	comm_sock_accept(xsock_t sock);
  */
 extern int	comm_sock_close(xsock_t sock);
 
+/**
+ * Set/reset the SO_BROADCAST option on a socket.
+ *
+ * @param sock the socket whose option we want to set/reset.
+ * @param val if non-zero, the option will be set, otherwise will be reset.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_set_send_broadcast(xsock_t sock, int val);
+
+/**
+ * Set/reset the IP_RECEIVE_BROADCAST option on a socket.
+ * This option is specific to Windows Server 2003 and later.
+ *
+ * @param sock the socket whose option we want to set/reset.
+ * @param val if non-zero, the option will be set, otherwise will be reset.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_set_receive_broadcast(xsock_t sock, int val);
+
 /**
  * Set/reset the TCP_NODELAY option on a TCP socket.
  *
@@ -654,7 +786,7 @@ extern int	comm_set_nodelay(xsock_t sock, int val);
 /**
  * Set/reset the SO_REUSEADDR option on a socket.
  *
- * XXX: if the OS doesn't support this option, XORP_ERROR is returned.
+ * Note: If the OS doesn't support this option, then XORP_ERROR is returned.
  *
  * @param sock the socket whose option we want to set/reset.
  * @param val if non-zero, the option will be set, otherwise will be reset.
@@ -665,7 +797,7 @@ extern int	comm_set_reuseaddr(xsock_t sock, int val);
 /**
  * Set/reset the SO_REUSEPORT option on a socket.
  *
- * XXX: if the OS doesn't support this option, XORP_ERROR is returned.
+ * Note: If the OS doesn't support this option, then XORP_ERROR is returned.
  *
  * @param sock the socket whose option we want to set/reset.
  * @param val if non-zero, the option will be set, otherwise will be reset.
@@ -685,7 +817,7 @@ extern int	comm_set_loopback(xsock_t sock, int val);
 /**
  * Set/reset the TCP_MD5SIG option on a socket.
  *
- * XXX: if the OS doesn't support this option, XORP_ERROR is returned.
+ * Note: If the OS doesn't support this option, XORP_ERROR is returned.
  *
  * @param sock the socket whose option we want to set/reset.
  * @param val if non-zero, the option will be set, otherwise will be reset.
@@ -693,6 +825,24 @@ extern int	comm_set_loopback(xsock_t sock, int val);
  */
 extern int	comm_set_tcpmd5(xsock_t sock, int val);
 
+/**
+ * Set the ip_tos bits of the outgoing packets on a socket.
+ *
+ * @param sock the socket whose ip_tos we want to set.
+ * @param val the ip_tos of the outgoing packets.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_set_tos(xsock_t sock, int val);
+
+/**
+ * Set the TTL of the outgoing unicast/broadcast packets on a socket.
+ *
+ * @param sock the socket whose TTL we want to set.
+ * @param val the TTL of the outgoing unicast/broadcast packets.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_set_unicast_ttl(xsock_t sock, int val);
+
 /**
  * Set the TTL of the outgoing multicast packets on a socket.
  *
@@ -700,7 +850,7 @@ extern int	comm_set_tcpmd5(xsock_t sock, int val);
  * @param val the TTL of the outgoing multicast packets.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_ttl(xsock_t sock, int val);
+extern int	comm_set_multicast_ttl(xsock_t sock, int val);
 
 /**
  * Set default interface for IPv4 outgoing multicast on a socket.
@@ -717,12 +867,40 @@ extern int	comm_set_iface4(xsock_t sock, const struct in_addr *in_addr);
  * Set default interface for IPv6 outgoing multicast on a socket.
  *
  * @param sock the socket whose default multicast interface to set.
- * @param ifindex the IPv6 interface index of the default interface to set.
- * If @ref ifindex is 0, the system will choose the interface each time
+ * @param my_ifindex the local network interface index of the default
+ * interface to set. If it is 0, the system will choose the interface each time
  * a datagram is sent.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_iface6(xsock_t sock, u_int ifindex);
+extern int	comm_set_iface6(xsock_t sock, unsigned int my_ifindex);
+
+/**
+ * Set the interface to which a socket is bound.
+ *
+ * XXX: This exists to support XORP's use of the 255.255.255.255
+ * address in Linux for MANET protocols, as well as certain limited
+ * uses with raw IPv4 sockets, and SHOULD NOT be used in new code.
+ *
+ * @param sock the socket to be bound to @param my_ifname
+ * @param my_ifname the name of the local network interface to which the
+ *                  socket should be bound.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_set_bindtodevice(xsock_t sock, const char * my_ifname);
+
+/**
+ * Set the option which causes sends to directed IPv4 broadcast addresses 
+ * to go to 255.255.255.255 instead.
+ *
+ * XXX: This exists to support XORP's use of the 255.255.255.255
+ * address in BSD-derived systems for MANET protocols. It SHOULD NOT
+ * be used in new code.
+ *
+ * @param sock the socket to enable undirected broadcasts for.
+ * @param enabled zero to disable the option, non-zero to enable it.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_set_onesbcast(xsock_t sock, int enabled);
 
 /**
  * Set the sending buffer size of a socket.
@@ -750,30 +928,43 @@ extern int	comm_sock_set_rcvbuf(xsock_t sock, int desired_bufsize,
 
 /**
  * Get the address family of a socket.
- * Idea taken from W. Stevens' UNPv1, 2e (pp 109).
+ *
+ * Note: Idea taken from W. Stevens' UNPv1, 2e (pp 109).
  *
  * @param sock the socket whose address family we need to get.
  * @return the address family on success, otherwise XORP_ERROR.
  */
 extern int	comm_sock_get_family(xsock_t sock);
 
+/**
+ * Get the type of a socket.
+ *
+ * Examples of socket type are SOCK_STREAM for TCP, SOCK_DGRAM for UDP,
+ * and SOCK_RAW for raw protocol sockets.
+ *
+ * @param sock the socket whose type we need to get.
+ * @return the socket type on success, otherwise XORP_ERROR.
+ */
+extern int	comm_sock_get_type(xsock_t sock);
+
 /**
  * Set the blocking or non-blocking mode of an existing socket.
+ *
  * @param sock the socket whose blocking mode is to be set.
  * @param is_blocking if non-zero, then set socket to blocking mode.
- *
- * @return XORP_OK if the operation was successful, otherwise XORP_ERROR.
- **/
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
 extern int	comm_sock_set_blocking(xsock_t sock, int is_blocking);
 
 /**
  * Determine if an existing socket is in the connected state.
- * @param sock the socket whose connected state is to be queried.
  *
- * @return XORP_OK if the socket is in the connected state, otherwise
- * if it is not, or any other error is encountered, XORP_ERROR.
- **/
-extern int	comm_sock_is_connected(xsock_t sock);
+ * @param sock the socket whose connected state is to be queried.
+ * @param is_connected if the socket is in the connected state, then the
+ * referenced value is set to 1, otherwise it is set to 0.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_sock_is_connected(xsock_t sock, int *is_connected);
 
 __END_DECLS
 
