@@ -1,5 +1,5 @@
 /* -*-  Mode:C; c-basic-offset:4; tab-width:8; indent-tabs-mode:t -*- */
-/* vim:set sts=4 ts=8: */
+/* vim:set sts=4 ts=8 sw=4: */
 /*
  * Copyright (c) 2001
  * YOID Project.
@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-#ident "$XORP: xorp/libcomm/comm_sock.c,v 1.34 2006/10/12 01:24:45 pavlin Exp $"
+#ident "$XORP: xorp/libcomm/comm_sock.c,v 1.46 2008/04/22 16:03:43 pavlin Exp $"
 
 /*
  * COMM socket library lower `sock' level implementation.
@@ -42,6 +42,7 @@
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/random.h"
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -85,20 +86,6 @@ const struct in6_addr in6addr_any = { { IN6ADDR_ANY_INIT } };
 const struct in6_addr in6addr_loopback = { { IN6ADDR_LOOPBACK_INIT } };
 #endif
 
-/**
- * comm_sock_open:
- * @domain: The domain of the socket (e.g., %AF_INET, %AF_INET6).
- * @type: The type of the socket (e.g., %SOCK_STREAM, %SOCK_DGRAM).
- * @protocol: The particular protocol to be used with the socket.
- * @is_blocking: If true, then the socket will be blocking, otherwise
- * non-blocking.
- *
- * Open a socket of domain = @domain, type = @type, and protocol = @protocol.
- * The sending and receiving buffer size are set, and the socket
- * itself is set with %TCP_NODELAY (if a TCP socket).
- *
- * Return value: The open socket on success, otherwise %XORP_BAD_SOCKET.
- **/
 xsock_t
 comm_sock_open(int domain, int type, int protocol, int is_blocking)
 {
@@ -146,30 +133,6 @@ comm_sock_open(int domain, int type, int protocol, int is_blocking)
     return (sock);
 }
 
-/**
- * comm_sock_pair:
- *
- * Create a pair of connected sockets. The sockets will be created in
- * the blocking state by default, and with no additional socket options set.
- *
- * On Windows platforms, a domain of AF_UNIX, AF_INET, or AF_INET6 must
- * be specified. For the AF_UNIX case, the sockets created will actually
- * be in the AF_INET domain. The protocol field is ignored.
- *
- * On UNIX, this function simply wraps the socketpair() system call.
- *
- * XXX: There may be UNIX platforms lacking socketpair() where we
- * have to emulate it.
- *
- * @param domain the domain of the socket (e.g., AF_INET, AF_INET6).
- * @param type the type of the socket (e.g., SOCK_STREAM, SOCK_DGRAM).
- * @param protocol the particular protocol to be used with the socket.
- * @param sv pointer to an array of two xsock_t handles to receive the
- *        allocated socket pair.
- *
- * @return XORP_OK if the socket pair was created, otherwise if any error
- * is encountered, XORP_ERROR.
- **/
 int
 comm_sock_pair(int domain, int type, int protocol, xsock_t sv[2])
 {
@@ -233,7 +196,7 @@ comm_sock_pair(int domain, int type, int protocol, xsock_t sv[2])
 
     numtries = 3;
     do {
-	port = htons((rand() % (CSP_LOWPORT - CSP_HIGHPORT)) + CSP_LOWPORT);
+	port = htons((xorp_random() % (CSP_LOWPORT - CSP_HIGHPORT)) + CSP_LOWPORT);
 #ifdef HAVE_IPV6
 	if (intdomain == AF_INET6)
 	    psin6->sin6_port = port;
@@ -314,17 +277,6 @@ error:
 #endif /* HOST_OS_WINDOWS */
 }
 
-/**
- * comm_sock_bind4:
- * @sock: The socket to bind.
- * @my_addr: The address to bind to (in network order).
- * If it is NULL, will bind to `any' local address.
- * @my_port: The port to bind to (in network order).
- *
- * Bind an IPv4 socket to an address and a port.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_sock_bind4(xsock_t sock, const struct in_addr *my_addr,
 		unsigned short my_port)
@@ -340,7 +292,7 @@ comm_sock_bind4(xsock_t sock, const struct in_addr *my_addr,
     }
 
     memset(&sin_addr, 0, sizeof(sin_addr));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin_addr.sin_len = sizeof(sin_addr);
 #endif
     sin_addr.sin_family = (u_char)family;
@@ -364,20 +316,9 @@ comm_sock_bind4(xsock_t sock, const struct in_addr *my_addr,
     return (XORP_OK);
 }
 
-/**
- * comm_sock_bind6:
- * @sock: The socket to bind.
- * @my_addr: The address to bind to (in network order).
- * If it is NULL, will bind to `any' local address.
- * @my_port: The port to bind to (in network order).
- *
- * Bind an IPv6 socket to an address and a port.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_sock_bind6(xsock_t sock, const struct in6_addr *my_addr,
-		unsigned short my_port)
+		unsigned int my_ifindex, unsigned short my_port)
 {
 #ifdef HAVE_IPV6
     int family;
@@ -391,7 +332,7 @@ comm_sock_bind6(xsock_t sock, const struct in6_addr *my_addr,
     }
 
     memset(&sin6_addr, 0, sizeof(sin6_addr));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6_addr.sin6_len = sizeof(sin6_addr);
 #endif
     sin6_addr.sin6_family = (u_char)family;
@@ -401,8 +342,10 @@ comm_sock_bind6(xsock_t sock, const struct in6_addr *my_addr,
 	memcpy(&sin6_addr.sin6_addr, my_addr, sizeof(sin6_addr.sin6_addr));
     else
 	memcpy(&sin6_addr.sin6_addr, &in6addr_any, sizeof(sin6_addr.sin6_addr));
-
-    sin6_addr.sin6_scope_id = 0;		/* XXX: unused (?)	     */
+    if (IN6_IS_ADDR_LINKLOCAL(&sin6_addr.sin6_addr))
+	sin6_addr.sin6_scope_id = my_ifindex;
+    else
+	sin6_addr.sin6_scope_id = 0;
 
     if (bind(sock, (struct sockaddr *)&sin6_addr, sizeof(sin6_addr)) < 0) {
 	char addr_str[sizeof "ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255"];
@@ -419,20 +362,11 @@ comm_sock_bind6(xsock_t sock, const struct in6_addr *my_addr,
 
     return (XORP_OK);
 #else /* ! HAVE_IPV6 */
-    comm_sock_no_ipv6("comm_sock_bind6", sock, my_addr, my_port);
+    comm_sock_no_ipv6("comm_sock_bind6", sock, my_addr, my_ifindex, my_port);
     return (XORP_ERROR);
 #endif /* ! HAVE_IPV6 */
 }
 
-/**
- * Bind a socket (IPv4 or IPv6) to an address and a port.
- *
- * @param sock the socket to bind.
- * @param sin agnostic sockaddr containing the local address (If it is
- * NULL, will bind to `any' local address.)  and the local port to
- * bind to all in network order.
- * @return XORP_OK on success, otherwise XORP_ERROR.
- */
 int
 comm_sock_bind(xsock_t sock, const struct sockaddr *sin)
 {
@@ -447,7 +381,8 @@ comm_sock_bind(xsock_t sock, const struct sockaddr *sin)
     case AF_INET6:
 	{
 	    const struct sockaddr_in6 *sin6 = (const struct sockaddr_in6 *)((const void *)sin);
-	    return comm_sock_bind6(sock, &sin6->sin6_addr, sin6->sin6_port);
+	    return comm_sock_bind6(sock, &sin6->sin6_addr, sin6->sin6_scope_id,
+				   sin6->sin6_port);
 	}
 	break;
 #endif /* HAVE_IPV6 */
@@ -461,17 +396,6 @@ comm_sock_bind(xsock_t sock, const struct sockaddr *sin)
     return XORP_ERROR;
 }
 
-/**
- * comm_sock_join4:
- * @sock: The socket to join the group.
- * @mcast_addr: The multicast address to join.
- * @my_addr: The local unicast address of an interface to join.
- * If it is NULL, the interface is chosen by the kernel.
- *
- * Join an IPv4 multicast group on a socket (and an interface).
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_sock_join4(xsock_t sock, const struct in_addr *mcast_addr,
 		const struct in_addr *my_addr)
@@ -515,17 +439,6 @@ comm_sock_join4(xsock_t sock, const struct in_addr *mcast_addr,
     return (XORP_OK);
 }
 
-/**
- * comm_sock_join6:
- * @sock: The socket to join the group.
- * @mcast_addr: The multicast address to join.
- * @my_ifindex: The local unicast interface index to join.
- * If it is 0, the interface is chosen by the kernel.
- *
- * Join an IPv6 multicast group on a socket (and an interface).
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_sock_join6(xsock_t sock, const struct in6_addr *mcast_addr,
 		unsigned int my_ifindex)
@@ -564,17 +477,6 @@ comm_sock_join6(xsock_t sock, const struct in6_addr *mcast_addr,
 #endif /* ! HAVE_IPV6 */
 }
 
-/**
- * comm_sock_leave4:
- * @sock: The socket to leave the group.
- * @mcast_addr: The multicast address to leave.
- * @my_addr: The local unicast address of an interface to leave.
- * If it is NULL, the interface is chosen by the kernel.
- *
- * Leave an IPv4 multicast group on a socket (and an interface).
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_sock_leave4(xsock_t sock, const struct in_addr *mcast_addr,
 		const struct in_addr *my_addr)
@@ -618,17 +520,6 @@ comm_sock_leave4(xsock_t sock, const struct in_addr *mcast_addr,
     return (XORP_OK);
 }
 
-/**
- * comm_sock_leave6:
- * @sock: The socket to leave the group.
- * @mcast_addr: The multicast address to leave.
- * @my_ifindex: The local unicast interface index to leave.
- * If it is 0, the interface is chosen by the kernel.
- *
- * Leave an IPv6 multicast group on a socket (and an interface).
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_sock_leave6(xsock_t sock, const struct in6_addr *mcast_addr,
 		unsigned int my_ifindex)
@@ -667,23 +558,6 @@ comm_sock_leave6(xsock_t sock, const struct in6_addr *mcast_addr,
 #endif /* ! HAVE_IPV6 */
 }
 
-/**
- * Connect to a remote IPv4 address.
- *
- * Note that we can use this function not only for TCP, but for UDP sockets
- * as well.
- *
- * @param sock the socket to use to connect.
- * @param remote_addr the remote address to connect to.
- * @param remote_port the remote port to connect to.
- * @param is_blocking if true, the socket is blocking, otherwise non-blocking.
- * @param in_progress if the socket is non-blocking and the connect cannot be
- * completed immediately, then the referenced value is set to 1, and the
- * return value is XORP_ERROR. For all other errors or if the non-blocking
- * socket was connected, the referenced value is set to 0. If the return value
- * is XORP_OK or if the socket is blocking, then the return value is undefined.
- * @return XORP_OK on success, otherwise XORP_ERROR.
- */
 int
 comm_sock_connect4(xsock_t sock, const struct in_addr *remote_addr,
 		   unsigned short remote_port, int is_blocking,
@@ -703,7 +577,7 @@ comm_sock_connect4(xsock_t sock, const struct in_addr *remote_addr,
     }
 
     memset(&sin_addr, 0, sizeof(sin_addr));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin_addr.sin_len = sizeof(sin_addr);
 #endif
     sin_addr.sin_family = (u_char)family;
@@ -739,23 +613,6 @@ comm_sock_connect4(xsock_t sock, const struct in_addr *remote_addr,
     return (XORP_OK);
 }
 
-/**
- * Connect to a remote IPv6 address.
- *
- * Note that we can use this function not only for TCP, but for UDP sockets
- * as well.
- *
- * @param sock the socket to use to connect.
- * @param remote_addr the remote address to connect to.
- * @param remote_port the remote port to connect to.
- * @param is_blocking if true, the socket is blocking, otherwise non-blocking.
- * @param in_progress if the socket is non-blocking and the connect cannot be
- * completed immediately, then the referenced value is set to 1, and the
- * return value is XORP_ERROR. For all other errors or if the non-blocking
- * socket was connected, the referenced value is set to 0. If the return value
- * is XORP_OK or if the socket is blocking, then the return value is undefined.
- * @return XORP_OK on success, otherwise XORP_ERROR.
- */
 int
 comm_sock_connect6(xsock_t sock, const struct in6_addr *remote_addr,
 		   unsigned short remote_port, int is_blocking,
@@ -776,7 +633,7 @@ comm_sock_connect6(xsock_t sock, const struct in6_addr *remote_addr,
     }
 
     memset(&sin6_addr, 0, sizeof(sin6_addr));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6_addr.sin6_len = sizeof(sin6_addr);
 #endif
     sin6_addr.sin6_family = (u_char)family;
@@ -828,24 +685,6 @@ comm_sock_connect6(xsock_t sock, const struct in6_addr *remote_addr,
 #endif /* ! HAVE_IPV6 */
 }
 
-/**
- * Connect to a remote address (IPv4 or IPv6).
- *
- * Note that we can use this function not only for TCP, but for UDP sockets
- * as well.
- *
- * @param sock the socket to use to connect.
- * @param sin agnostic sockaddr containing the local address (If it is
- * NULL, will bind to `any' local address.)  and the local port to
- * bind to all in network order.
- * @param is_blocking if true, the socket is blocking, otherwise non-blocking.
- * @param in_progress if the socket is non-blocking and the connect cannot be
- * completed immediately, then the referenced value is set to 1, and the
- * return value is XORP_ERROR. For all other errors or if the non-blocking
- * socket was connected, the referenced value is set to 0. If the return value
- * is XORP_OK or if the socket is blocking, then the return value is undefined.
- * @return XORP_OK on success, otherwise XORP_ERROR.
- */
 int
 comm_sock_connect(xsock_t sock, const struct sockaddr *sin, int is_blocking,
 		  int *in_progress)
@@ -878,14 +717,6 @@ comm_sock_connect(xsock_t sock, const struct sockaddr *sin, int is_blocking,
     return XORP_ERROR;
 }
 
-/**
- * comm_sock_accept:
- * @sock: The listening socket to accept on.
- *
- * Accept a connection on a listening socket.
- *
- * Return value: The accepted socket on success, otherwise %XORP_BAD_SOCKET.
- **/
 xsock_t
 comm_sock_accept(xsock_t sock)
 {
@@ -918,14 +749,22 @@ comm_sock_accept(xsock_t sock)
     return (sock_accept);
 }
 
-/**
- * comm_sock_close:
- * @sock: The socket to close.
- *
- * Close a socket.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
+int
+comm_sock_listen(xsock_t sock, int backlog)
+{
+    int ret;
+    ret = listen(sock, backlog);
+
+    if (ret < 0) {
+	_comm_set_serrno();
+	XLOG_ERROR("Error listening on socket (socket = %d) : %s",
+		   sock, comm_get_error_str(comm_get_last_error()));
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+}
+
 int
 comm_sock_close(xsock_t sock)
 {
@@ -948,15 +787,49 @@ comm_sock_close(xsock_t sock)
     return (XORP_OK);
 }
 
-/**
- * comm_set_nodelay:
- * @sock: The socket whose option we want to set/reset.
- * @val: If non-zero, the option will be set, otherwise will be reset.
- *
- * Set/reset the %TCP_NODELAY option on a TCP socket.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
+int
+comm_set_send_broadcast(xsock_t sock, int val)
+{
+    if (setsockopt(sock, SOL_SOCKET, SO_BROADCAST,
+		   XORP_SOCKOPT_CAST(&val), sizeof(val)) < 0) {
+	_comm_set_serrno();
+	XLOG_ERROR("Error %s SO_BROADCAST on socket %d: %s",
+		   (val)? "set": "reset",  sock,
+		   comm_get_error_str(comm_get_last_error()));
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+}
+
+int
+comm_set_receive_broadcast(xsock_t sock, int val)
+{
+#if defined(HOST_OS_WINDOWS) && defined(IP_RECEIVE_BROADCAST)
+    /*
+     * With Windows Server 2003 and later, you have to explicitly
+     * ask to receive broadcast packets.
+     */
+    DWORD ip_rx_bcast = (DWORD)val;
+
+    if (setsockopt(sock, IPPROTO_IP, IP_RECEIVE_BROADCAST,
+		   XORP_SOCKOPT_CAST(&ip_rx_bcast),
+		   sizeof(ip_rx_bcast)) < 0) {
+	_comm_set_serrno();
+	XLOG_ERROR("Error %s IP_RECEIVE_BROADCAST on socket %d: %s",
+		   (val)? "set": "reset",  sock,
+		   comm_get_error_str(comm_get_last_error()));
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+#else
+    UNUSED(sock);
+    UNUSED(val);
+    return (XORP_OK);
+#endif
+}
+
 int
 comm_set_nodelay(xsock_t sock, int val)
 {
@@ -972,16 +845,6 @@ comm_set_nodelay(xsock_t sock, int val)
     return (XORP_OK);
 }
 
-/**
- * comm_set_reuseaddr:
- * @sock: The socket whose option we want to set/reset.
- * @val: If non-zero, the option will be set, otherwise will be reset.
- *
- * Set/reset the %SO_REUSEADDR option on a socket.
- * XXX: if the OS doesn't support this option, %XORP_ERROR is returned.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_set_reuseaddr(xsock_t sock, int val)
 {
@@ -1006,16 +869,6 @@ comm_set_reuseaddr(xsock_t sock, int val)
 #endif /* ! SO_REUSEADDR */
 }
 
-/**
- * comm_set_reuseport:
- * @sock: The socket whose option we want to set/reset.
- * @val: If non-zero, the option will be set, otherwise will be reset.
- *
- * Set/reset the %SO_REUSEPORT option on a socket.
- * XXX: if the OS doesn't support this option, %XORP_OK is returned.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_set_reuseport(xsock_t sock, int val)
 {
@@ -1040,15 +893,6 @@ comm_set_reuseport(xsock_t sock, int val)
 #endif /* ! SO_REUSEPORT */
 }
 
-/**
- * comm_set_loopback:
- * @sock: The socket whose option we want to set/reset.
- * @val: If non-zero, the option will be set, otherwise will be reset.
- *
- * Set/reset the multicast loopback option on a socket.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_set_loopback(xsock_t sock, int val)
 {
@@ -1072,7 +916,7 @@ comm_set_loopback(xsock_t sock, int val)
 #ifdef HAVE_IPV6
     case AF_INET6:
     {
-	u_int loop6 = val;
+	unsigned int loop6 = val;
 
 	if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_LOOP,
 		       XORP_SOCKOPT_CAST(&loop6), sizeof(loop6)) < 0) {
@@ -1094,16 +938,6 @@ comm_set_loopback(xsock_t sock, int val)
     return (XORP_OK);
 }
 
-/**
- * comm_set_tcpmd5:
- * @sock: The socket whose option we want to set/reset.
- * @val: If non-zero, the option will be set, otherwise will be reset.
- *
- * Set/reset the %TCP_MD5SIG option on a TCP socket.
- * XXX: if the OS doesn't support this option, %XORP_ERROR is returned.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_set_tcpmd5(xsock_t sock, int val)
 {
@@ -1128,29 +962,66 @@ comm_set_tcpmd5(xsock_t sock, int val)
 #endif /* ! TCP_MD5SIG */
 }
 
-/**
- * comm_set_ttl:
- * @sock: The socket whose TTL we want to set.
- * @val: The TTL of the outgoing multicast packets.
- *
- * Set the TTL of the outgoing multicast packets on a socket.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
-comm_set_ttl(xsock_t sock, int val)
+comm_set_tos(xsock_t sock, int val)
+{
+#ifdef IP_TOS
+    /*
+     * Most implementations use 'int' to represent the value of
+     * the IP_TOS option.
+     */
+    int family, ip_tos;
+
+    family = comm_sock_get_family(sock);
+    if (family != AF_INET) {
+	XLOG_FATAL("Error %s setsockopt IP_TOS on socket %d: "
+		   "invalid family = %d",
+		   (val)? "set": "reset", sock, family);
+	return (XORP_ERROR);
+    }
+
+    /*
+     * Note that it is not guaranteed that the TOS will be successfully
+     * set or indeed propagated where the host platform is running
+     * its own traffic classifier; the use of comm_set_tos() is
+     * intended for link-scoped traffic.
+     */
+    ip_tos = val;
+    if (setsockopt(sock, IPPROTO_IP, IP_TOS,
+		   XORP_SOCKOPT_CAST(&ip_tos), sizeof(ip_tos)) < 0) {
+	_comm_set_serrno();
+	XLOG_ERROR("setsockopt IP_TOS 0x%x: %s",
+	       ip_tos, comm_get_error_str(comm_get_last_error()));
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+#else
+    UNUSED(sock);
+    UNUSED(val);
+    XLOG_WARNING("IP_TOS Undefined!");
+
+    return (XORP_ERROR);
+#endif /* ! IP_TOS */
+}
+
+int
+comm_set_unicast_ttl(xsock_t sock, int val)
 {
     int family = comm_sock_get_family(sock);
 
     switch (family) {
     case AF_INET:
     {
-	u_char ip_ttl = val;	/* XXX: In IPv4 the value arg is 'u_char' */
+	/* XXX: Most platforms now use int for this option;
+	 * legacy BSD specified u_char.
+	 */
+	int ip_ttl = val;
 
-	if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL,
+	if (setsockopt(sock, IPPROTO_IP, IP_TTL,
 		       XORP_SOCKOPT_CAST(&ip_ttl), sizeof(ip_ttl)) < 0) {
 	    _comm_set_serrno();
-	    XLOG_ERROR("setsockopt IP_MULTICAST_TTL %u: %s",
+	    XLOG_ERROR("setsockopt IP_TTL %u: %s",
 		       ip_ttl, comm_get_error_str(comm_get_last_error()));
 	    return (XORP_ERROR);
 	}
@@ -1161,15 +1032,66 @@ comm_set_ttl(xsock_t sock, int val)
     {
 	int ip_ttl = val;
 
-	if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
+	if (setsockopt(sock, IPPROTO_IPV6, IPV6_UNICAST_HOPS,
 		       XORP_SOCKOPT_CAST(&ip_ttl), sizeof(ip_ttl)) < 0) {
 	    _comm_set_serrno();
-	    XLOG_ERROR("setsockopt IPV6_MULTICAST_HOPS %u: %s",
+	    XLOG_ERROR("setsockopt IPV6_UNICAST_HOPS %u: %s",
 		       ip_ttl, comm_get_error_str(comm_get_last_error()));
 	    return (XORP_ERROR);
 	}
 	break;
     }
+#endif /* HAVE_IPV6 */
+    default:
+	XLOG_FATAL("Error %s setsockopt IP_TTL/IPV6_UNICAST_HOPS "
+		   "on socket %d: invalid family = %d",
+		   (val)? "set": "reset", sock, family);
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+}
+
+int
+comm_set_multicast_ttl(xsock_t sock, int val)
+{
+    int family = comm_sock_get_family(sock);
+
+    switch (family) {
+    case AF_INET:
+    {
+	/* XXX: Most platforms now use int for this option;
+	 * legacy BSD specified u_char.
+	 */
+	u_char ip_multicast_ttl = val;
+
+	if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL,
+		       XORP_SOCKOPT_CAST(&ip_multicast_ttl),
+		       sizeof(ip_multicast_ttl)) < 0) {
+	    _comm_set_serrno();
+	    XLOG_ERROR("setsockopt IP_MULTICAST_TTL %u: %s",
+		       ip_multicast_ttl,
+		       comm_get_error_str(comm_get_last_error()));
+	    return (XORP_ERROR);
+	}
+	break;
+    }
+#ifdef HAVE_IPV6
+    case AF_INET6:
+    {
+	int ip_multicast_ttl = val;
+
+	if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
+		       XORP_SOCKOPT_CAST(&ip_multicast_ttl),
+		       sizeof(ip_multicast_ttl)) < 0) {
+	    _comm_set_serrno();
+	    XLOG_ERROR("setsockopt IPV6_MULTICAST_HOPS %u: %s",
+		       ip_multicast_ttl,
+		       comm_get_error_str(comm_get_last_error()));
+	    return (XORP_ERROR);
+	}
+	break;
+    }
 #endif /* HAVE_IPV6 */
     default:
 	XLOG_FATAL("Error %s setsockopt IP_MULTICAST_TTL/IPV6_MULTICAST_HOPS "
@@ -1181,17 +1103,6 @@ comm_set_ttl(xsock_t sock, int val)
     return (XORP_OK);
 }
 
-/**
- * comm_set_iface4:
- * @sock: The socket whose default multicast interface to set.
- * @in_addr: The IPv4 address of the default interface to set.
- * If @in_addr is NULL, the system will choose the interface each time
- * a datagram is sent.
- *
- * Set default interface for IPv4 outgoing multicast on a socket.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
 comm_set_iface4(xsock_t sock, const struct in_addr *in_addr)
 {
@@ -1220,19 +1131,8 @@ comm_set_iface4(xsock_t sock, const struct in_addr *in_addr)
     return (XORP_OK);
 }
 
-/**
- * comm_set_iface6:
- * @sock: The socket whose default multicast interface to set.
- * @ifindex: The IPv6 interface index of the default interface to set.
- * If @ifindex is 0, the system will choose the interface each time
- * a datagram is sent.
- *
- * Set default interface for IPv6 outgoing multicast on a socket.
- *
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
-comm_set_iface6(xsock_t sock, u_int ifindex)
+comm_set_iface6(xsock_t sock, unsigned int my_ifindex)
 {
 #ifdef HAVE_IPV6
     int family = comm_sock_get_family(sock);
@@ -1244,32 +1144,91 @@ comm_set_iface6(xsock_t sock, u_int ifindex)
     }
 
     if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_IF,
-		   XORP_SOCKOPT_CAST(&ifindex), sizeof(ifindex)) < 0) {
+		   XORP_SOCKOPT_CAST(&my_ifindex), sizeof(my_ifindex)) < 0) {
 	_comm_set_serrno();
-	XLOG_ERROR("setsockopt IPV6_MULTICAST_IF for ifindex %d: %s",
-		   ifindex, comm_get_error_str(comm_get_last_error()));
+	XLOG_ERROR("setsockopt IPV6_MULTICAST_IF for interface index %d: %s",
+		   my_ifindex, comm_get_error_str(comm_get_last_error()));
 	return (XORP_ERROR);
     }
 
     return (XORP_OK);
 
 #else /* ! HAVE_IPV6 */
-    comm_sock_no_ipv6("comm_set_iface6", sock, ifindex);
+    comm_sock_no_ipv6("comm_set_iface6", sock, my_ifindex);
     return (XORP_ERROR);
 #endif /* ! HAVE_IPV6 */
 }
 
-/**
- * comm_sock_set_sndbuf:
- * @sock: The socket whose sending buffer size to set.
- * @desired_bufsize: The preferred buffer size.
- * @min_bufsize: The smallest acceptable buffer size.
- *
- * Set the sending buffer size of a socket.
- *
- * Return value: The successfully set buffer size on success,
- * otherwise %XORP_ERROR.
- **/
+int
+comm_set_onesbcast(xsock_t sock, int enabled)
+{
+#ifdef IP_ONESBCAST
+    int family = comm_sock_get_family(sock);
+
+    if (family != AF_INET) {
+	XLOG_ERROR("Invalid family of socket %d: family = %d (expected %d)",
+		   sock, family, AF_INET);
+	return (XORP_ERROR);
+    }
+
+    if (setsockopt(sock, IPPROTO_IP, IP_ONESBCAST,
+		   XORP_SOCKOPT_CAST(&enabled), sizeof(enabled)) < 0) {
+	_comm_set_serrno();
+	XLOG_ERROR("setsockopt IP_ONESBCAST %d: %s", enabled,
+		   comm_get_error_str(comm_get_last_error()));
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+
+#else /* ! IP_ONESBCAST */
+    XLOG_ERROR("setsockopt IP_ONESBCAST %u: %s", enabled,
+	       "IP_ONESBCAST support not present.");
+    UNUSED(sock);
+    return (XORP_ERROR);
+#endif /* ! IP_ONESBCAST */
+}
+
+int
+comm_set_bindtodevice(xsock_t sock, const char * my_ifname)
+{
+#ifdef SO_BINDTODEVICE
+    char tmp_ifname[IFNAMSIZ];
+
+    /*
+     * Bind a socket to an interface by name.
+     *
+     * Under Linux, use of the undirected broadcast address
+     * 255.255.255.255 requires that the socket is bound to the
+     * underlying interface, as it is an implicitly scoped address
+     * with no meaning on its own. This is not architecturally OK,
+     * and requires additional support for SO_BINDTODEVICE.
+     * See socket(7) man page in Linux.
+     *
+     * Note: strlcpy() is not present in glibc; strncpy() is used
+     * instead to avoid introducing a circular dependency on the
+     * C++ library libxorp.
+     */
+    strncpy(tmp_ifname, my_ifname, IFNAMSIZ-1);
+    tmp_ifname[IFNAMSIZ-1] = '\0';
+    if (setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE, tmp_ifname,
+		   sizeof(tmp_ifname)) < 0) {
+	_comm_set_serrno();
+	XLOG_ERROR("setsockopt SO_BINDTODEVICE %s: %s",
+		   tmp_ifname, comm_get_error_str(comm_get_last_error()));
+
+	return (XORP_ERROR);
+    }
+
+    return (XORP_OK);
+#else
+    XLOG_ERROR("setsockopt SO_BINDTODEVICE %s: %s",
+	       my_ifname, "SO_BINDTODEVICE support not present.");
+    UNUSED(sock);
+    return (XORP_ERROR);
+#endif
+}
+
 int
 comm_sock_set_sndbuf(xsock_t sock, int desired_bufsize, int min_bufsize)
 {
@@ -1313,17 +1272,6 @@ comm_sock_set_sndbuf(xsock_t sock, int desired_bufsize, int min_bufsize)
     return (desired_bufsize);
 }
 
-/**
- * comm_sock_set_rcvbuf:
- * @sock: The socket whose receiving buffer size to set.
- * @desired_bufsize: The preferred buffer size.
- * @min_bufsize: The smallest acceptable buffer size.
- *
- * Set the receiving buffer size of a socket.
- *
- * Return value: The successfully set buffer size on success,
- * otherwise %XORP_ERROR.
- **/
 int
 comm_sock_set_rcvbuf(xsock_t sock, int desired_bufsize, int min_bufsize)
 {
@@ -1367,15 +1315,6 @@ comm_sock_set_rcvbuf(xsock_t sock, int desired_bufsize, int min_bufsize)
     return (desired_bufsize);
 }
 
-/**
- * comm_sock_get_family:
- * @sock: The socket whose address family we need to get.
- *
- * Get the address family of a socket.
- * XXX: idea taken from W. Stevens' UNPv1, 2e (pp 109)
- *
- * Return value: The address family on success, otherwise %XORP_ERROR.
- **/
 int
 comm_sock_get_family(xsock_t sock)
 {
@@ -1418,17 +1357,24 @@ comm_sock_get_family(xsock_t sock)
 #endif /* ! HOST_OS_WINDOWS */
 }
 
-/**
- * comm_sock_set_blocking:
- *
- * Set the blocking or non-blocking mode of an existing socket.
- * @sock: The socket whose blocking mode is to be set.
- * @is_blocking: If non-zero, then the socket will be blocking, otherwise
- * non-blocking.
- *
- * Return value: XORP_OK if the operation was successful, otherwise
- *               if any error is encountered, XORP_ERROR.
- **/
+int
+comm_sock_get_type(xsock_t sock)
+{
+    int err, type;
+    socklen_t len = sizeof(type);
+
+    err = getsockopt(sock, SOL_SOCKET, SO_TYPE,
+		     XORP_SOCKOPT_CAST(&type), &len);
+    if (err != 0)  {
+	_comm_set_serrno();
+	XLOG_ERROR("Error getsockopt(SO_TYPE) for socket %d: %s",
+		   sock, comm_get_error_str(comm_get_last_error()));
+	return (XORP_ERROR);
+    }
+
+    return type;
+}
+
 int
 comm_sock_set_blocking(xsock_t sock, int is_blocking)
 {
@@ -1474,44 +1420,50 @@ comm_sock_set_blocking(xsock_t sock, int is_blocking)
     return (XORP_OK);
 }
 
-/**
- * comm_sock_is_connected:
- *
- * Determine if an existing socket is in the connected state.
- *
- * Return value: XORP_OK if the socket is in the connected state, otherwise
- * if it is not, or any other error is encountered, XORP_ERROR.
- **/
 int
-comm_sock_is_connected(xsock_t sock)
+comm_sock_is_connected(xsock_t sock, int *is_connected)
 {
     struct sockaddr_storage ss;
     int err;
     socklen_t sslen;
 
+    if (is_connected == NULL) {
+	XLOG_ERROR("comm_sock_is_connected() error: "
+		   "return value pointer is NULL");
+	return (XORP_ERROR);
+    }
+    *is_connected = 0;
+
     sslen = sizeof(ss);
     memset(&ss, 0, sslen);
     err = getpeername(sock, (struct sockaddr *)&ss, &sslen);
-    if (err != 0) {
+
+#ifdef HOST_OS_WINDOWS
+    if (err == SOCKET_ERROR) {
+	if ((WSAGetLastError() == WSAENOTCONN)
+	    || (WSAGetLastError() == WSAEINPROGRESS)) {
+	    return (XORP_OK);	/* Socket is not connected */
+	}
 	_comm_set_serrno();
 	return (XORP_ERROR);
     }
 
+#else /* ! HOST_OS_WINDOWS */
+    if (err != 0) {
+	if ((err == ENOTCONN) || (err == ECONNRESET)) {
+	    return (XORP_OK);	/* Socket is not connected */
+	}
+	_comm_set_serrno();
+	return (XORP_ERROR);
+    }
+#endif /* ! HOST_OS_WINDOWS */
+
+    /*  Socket is connected */
+    *is_connected = 1;
+
     return (XORP_OK);
 }
 
-/**
- * comm_sock_no_ipv6:
- *
- * Log an error when an IPv6 specific method is called when IPv6 is
- * not preset. Set an appropriate error code relevant to the platform
- * we're running under.
- *
- * XXX This is currently done with knowledge of how the error code is
- * internally stored, which is a design bug (we're not thread friendly).
- *
- * @param method C-style string denoting the ipv6 function called.
- **/
 void
 comm_sock_no_ipv6(const char* method, ...)
 {
@@ -1523,14 +1475,6 @@ comm_sock_no_ipv6(const char* method, ...)
     XLOG_ERROR("%s: IPv6 support not present.", method);
 }
 
-/**
- * _comm_set_serrno:
- *
- * Fetch the socket layer error code from the underlying system, clear
- * the general error condition, and record it.
- *
- * XXX: Currently not thread-safe. Internal use only.
- **/
 void
 _comm_set_serrno(void)
 {
