@@ -2,7 +2,7 @@
 /* vim:set sts=4 ts=8: */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libcomm/test_comm.c,v 1.13 2007/02/16 22:45:59 pavlin Exp $"
+#ident "$XORP: xorp/libcomm/test_comm.c,v 1.19 2008/07/23 05:10:36 pavlin Exp $"
 
 
 /*
@@ -151,6 +151,23 @@ main(int argc, char *argv[])
     } else {
 	printf("OK: open, bind and join UDP socket to group %s and port %d\n",
 	       inet_ntoa(mcast_addr), ntohs(port));
+	comm_close(sock);
+    }
+
+    /*
+     * Test 'listen on socket'
+     */
+    sock = comm_bind_tcp4(NULL, port, COMM_SOCK_BLOCKING);
+    if (sock == XORP_BAD_SOCKET) {
+	printf("ERROR: cannot open and bind TCP socket to port %d,"
+	       " for listening\n", ntohs(port));
+    } else if (comm_listen(sock, COMM_LISTEN_DEFAULT_BACKLOG) != XORP_OK) {
+	printf("ERROR: listening TCP socket on port %d\n", ntohs(port));
+	comm_close(sock);
+    } else {
+        printf("OK: open, bind and listen TCP socket on port %d\n",
+	       ntohs(port));
+	comm_close(sock);
     }
 
 
