@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,126 +12,55 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_atoms.cc,v 1.16 2007/02/16 22:45:59 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_atoms.cc,v 1.28 2008/07/23 05:10:37 pavlin Exp $"
 
 #include "ifmgr_atoms.hh"
 
-// ----------------------------------------------------------------------------
-// Find method helpers
 
-static inline const IfMgrIfAtom*
-find_interface(const IfMgrIfTree* tree, const string& ifname)
-{
-    const IfMgrIfTree::IfMap& ifs = tree->ifs();
-    IfMgrIfTree::IfMap::const_iterator ii = ifs.find(ifname);
-    if (ifs.end() == ii)
-	return 0;
-    return &ii->second;
-}
-
-static inline IfMgrIfAtom*
-find_interface(IfMgrIfTree* tree, const string& ifname)
-{
-    IfMgrIfTree::IfMap& ifs = tree->ifs();
-    IfMgrIfTree::IfMap::iterator ii = ifs.find(ifname);
-    if (ifs.end() == ii)
-	return 0;
-    return &ii->second;
-}
-
-
-static inline const IfMgrVifAtom*
-find_virtual_interface(const IfMgrIfAtom* ifa, const string& vifname)
-{
-    const IfMgrIfAtom::VifMap& vifs = ifa->vifs();
-    IfMgrIfAtom::VifMap::const_iterator vi = vifs.find(vifname);
-    if (vifs.end() == vi)
-	return 0;
-    return &vi->second;
-}
-
-static inline IfMgrVifAtom*
-find_virtual_interface(IfMgrIfAtom* ifa, const string& vifname)
-{
-    IfMgrIfAtom::VifMap& vifs = ifa->vifs();
-    IfMgrIfAtom::VifMap::iterator vi = vifs.find(vifname);
-    if (vifs.end() == vi)
-	return 0;
-    return &vi->second;
-}
-
-
-static inline const IfMgrIPv4Atom*
-find_virtual_interface_addr(const IfMgrVifAtom* vifa, const IPv4& addr)
-{
-    const IfMgrVifAtom::V4Map& addrs = vifa->ipv4addrs();
-    IfMgrVifAtom::V4Map::const_iterator ai = addrs.find(addr);
-    if (addrs.end() == ai)
-	return 0;
-    return &ai->second;
-}
-
-static inline IfMgrIPv4Atom*
-find_virtual_interface_addr(IfMgrVifAtom* vifa, const IPv4& addr)
-{
-    IfMgrVifAtom::V4Map& addrs = vifa->ipv4addrs();
-    IfMgrVifAtom::V4Map::iterator ai = addrs.find(addr);
-    if (addrs.end() == ai)
-	return 0;
-    return &ai->second;
-}
-
-static inline const IfMgrIPv6Atom*
-find_virtual_interface_addr(const IfMgrVifAtom* vifa, const IPv6& addr)
-{
-    const IfMgrVifAtom::V6Map& addrs = vifa->ipv6addrs();
-    IfMgrVifAtom::V6Map::const_iterator ai = addrs.find(addr);
-    if (addrs.end() == ai)
-	return 0;
-    return &ai->second;
-}
-
-static inline IfMgrIPv6Atom*
-find_virtual_interface_addr(IfMgrVifAtom* vifa, const IPv6& addr)
-{
-    IfMgrVifAtom::V6Map& addrs = vifa->ipv6addrs();
-    IfMgrVifAtom::V6Map::iterator ai = addrs.find(addr);
-    if (addrs.end() == ai)
-	return 0;
-    return &ai->second;
-}
+const IPv4 IfMgrIPv4Atom::_ZERO_ADDR = IPv4::ZERO();
+const IPv6 IfMgrIPv6Atom::_ZERO_ADDR = IPv6::ZERO();
 
 // ----------------------------------------------------------------------------
 // IfMgrIfTree Methods
 
 const IfMgrIfAtom*
-IfMgrIfTree::find_if(const string& ifname) const
+IfMgrIfTree::find_interface(const string& ifname) const
 {
-    return find_interface(this, ifname);
+    IfMgrIfTree::IfMap::const_iterator ii = interfaces().find(ifname);
+    if (ii == interfaces().end())
+	return (NULL);
+
+    return (&ii->second);
 }
 
 IfMgrIfAtom*
-IfMgrIfTree::find_if(const string& ifname)
+IfMgrIfTree::find_interface(const string& ifname)
 {
-    return find_interface(this, ifname);
+    IfMgrIfTree::IfMap::iterator ii = interfaces().find(ifname);
+    if (ii == interfaces().end())
+	return (NULL);
+
+    return (&ii->second);
 }
 
 const IfMgrVifAtom*
 IfMgrIfTree::find_vif(const string& ifname, const string& vifname) const
 {
-    const IfMgrIfAtom* ifa = find_interface(this, ifname);
-    if (ifa == 0)
-	return 0;
-    return find_virtual_interface(ifa, vifname);
+    const IfMgrIfAtom* ifa = find_interface(ifname);
+    if (ifa == NULL)
+	return (NULL);
+
+    return (ifa->find_vif(vifname));
 }
 
 IfMgrVifAtom*
 IfMgrIfTree::find_vif(const string& ifname, const string& vifname)
 {
-    IfMgrIfAtom* ifa = find_interface(this, ifname);
-    if (ifa == 0)
-	return 0;
-    return find_virtual_interface(ifa, vifname);
+    IfMgrIfAtom* ifa = find_interface(ifname);
+    if (ifa == NULL)
+	return (NULL);
+
+    return (ifa->find_vif(vifname));
 }
 
 const IfMgrIPv4Atom*
@@ -139,13 +68,11 @@ IfMgrIfTree::find_addr(const string& ifname,
 		       const string& vifname,
 		       const IPv4&   addr) const
 {
-    const IfMgrIfAtom* ifa = find_interface(this, ifname);
-    if (ifa == 0)
-	return 0;
-    const IfMgrVifAtom* vifa = find_virtual_interface(ifa, vifname);
-    if (vifa == 0)
-	return 0;
-    return find_virtual_interface_addr(vifa, addr);
+    const IfMgrVifAtom* vifa = find_vif(ifname, vifname);
+    if (vifa == NULL)
+	return (NULL);
+
+    return (vifa->find_addr(addr));
 }
 
 IfMgrIPv4Atom*
@@ -153,13 +80,11 @@ IfMgrIfTree::find_addr(const string& ifname,
 		       const string& vifname,
 		       const IPv4&   addr)
 {
-    IfMgrIfAtom* ifa = find_interface(this, ifname);
-    if (ifa == 0)
-	return 0;
-    IfMgrVifAtom* vifa = find_virtual_interface(ifa, vifname);
-    if (vifa == 0)
-	return 0;
-    return find_virtual_interface_addr(vifa, addr);
+    IfMgrVifAtom* vifa = find_vif(ifname, vifname);
+    if (vifa == NULL)
+	return (NULL);
+
+    return (vifa->find_addr(addr));
 }
 
 const IfMgrIPv6Atom*
@@ -167,13 +92,11 @@ IfMgrIfTree::find_addr(const string& ifname,
 		       const string& vifname,
 		       const IPv6&   addr) const
 {
-    const IfMgrIfAtom* ifa = find_interface(this, ifname);
-    if (ifa == 0)
-	return 0;
-    const IfMgrVifAtom* vifa = find_virtual_interface(ifa, vifname);
-    if (vifa == 0)
-	return 0;
-    return find_virtual_interface_addr(vifa, addr);
+    const IfMgrVifAtom* vifa = find_vif(ifname, vifname);
+    if (vifa == NULL)
+	return (NULL);
+
+    return (vifa->find_addr(addr));
 }
 
 IfMgrIPv6Atom*
@@ -181,19 +104,17 @@ IfMgrIfTree::find_addr(const string& ifname,
 		       const string& vifname,
 		       const IPv6&   addr)
 {
-    IfMgrIfAtom* ifa = find_interface(this, ifname);
-    if (ifa == 0)
-	return 0;
-    IfMgrVifAtom* vifa = find_virtual_interface(ifa, vifname);
-    if (vifa == 0)
-	return 0;
-    return find_virtual_interface_addr(vifa, addr);
+    IfMgrVifAtom* vifa = find_vif(ifname, vifname);
+    if (vifa == NULL)
+	return (NULL);
+
+    return (vifa->find_addr(addr));
 }
 
 bool
 IfMgrIfTree::operator==(const IfMgrIfTree& o) const
 {
-    return o.ifs() == ifs();
+    return (o.interfaces() == interfaces());
 }
 
 bool
@@ -202,7 +123,7 @@ IfMgrIfTree::is_my_addr(const IPv4& addr, string& ifname,
 {
     IfMgrIfTree::IfMap::const_iterator if_iter;
 
-    for (if_iter = ifs().begin(); if_iter != ifs().end(); ++if_iter) {
+    for (if_iter = interfaces().begin(); if_iter != interfaces().end(); ++if_iter) {
 	const IfMgrIfAtom& iface = if_iter->second;
 
 	// Test if interface is enabled and the link state is up
@@ -220,7 +141,7 @@ IfMgrIfTree::is_my_addr(const IPv4& addr, string& ifname,
 		continue;
 
 	    // Test if there is matching IPv4 address
-	    IfMgrVifAtom::V4Map::const_iterator a4_iter;
+	    IfMgrVifAtom::IPv4Map::const_iterator a4_iter;
 
 	    for (a4_iter = vif.ipv4addrs().begin();
 		 a4_iter != vif.ipv4addrs().end();
@@ -242,6 +163,7 @@ IfMgrIfTree::is_my_addr(const IPv4& addr, string& ifname,
 
     ifname = "";
     vifname = "";
+
     return (false);
 }
 
@@ -251,7 +173,7 @@ IfMgrIfTree::is_my_addr(const IPv6& addr, string& ifname,
 {
     IfMgrIfTree::IfMap::const_iterator if_iter;
 
-    for (if_iter = ifs().begin(); if_iter != ifs().end(); ++if_iter) {
+    for (if_iter = interfaces().begin(); if_iter != interfaces().end(); ++if_iter) {
 	const IfMgrIfAtom& iface = if_iter->second;
 
 	// Test if interface is enabled and the link state is up
@@ -269,7 +191,7 @@ IfMgrIfTree::is_my_addr(const IPv6& addr, string& ifname,
 		continue;
 
 	    // Test if there is matching IPv6 address
-	    IfMgrVifAtom::V6Map::const_iterator a6_iter;
+	    IfMgrVifAtom::IPv6Map::const_iterator a6_iter;
 
 	    for (a6_iter = vif.ipv6addrs().begin();
 		 a6_iter != vif.ipv6addrs().end();
@@ -291,6 +213,7 @@ IfMgrIfTree::is_my_addr(const IPv6& addr, string& ifname,
 
     ifname = "";
     vifname = "";
+
     return (false);
 }
 
@@ -317,7 +240,7 @@ IfMgrIfTree::is_directly_connected(const IPv4& addr, string& ifname,
 {
     IfMgrIfTree::IfMap::const_iterator if_iter;
 
-    for (if_iter = ifs().begin(); if_iter != ifs().end(); ++if_iter) {
+    for (if_iter = interfaces().begin(); if_iter != interfaces().end(); ++if_iter) {
 	const IfMgrIfAtom& iface = if_iter->second;
 
 	// Test if interface is enabled and the link state is up
@@ -335,7 +258,7 @@ IfMgrIfTree::is_directly_connected(const IPv4& addr, string& ifname,
 		continue;
 
 	    // Test if there is matching IPv4 address
-	    IfMgrVifAtom::V4Map::const_iterator a4_iter;
+	    IfMgrVifAtom::IPv4Map::const_iterator a4_iter;
 
 	    for (a4_iter = vif.ipv4addrs().begin();
 		 a4_iter != vif.ipv4addrs().end();
@@ -374,6 +297,7 @@ IfMgrIfTree::is_directly_connected(const IPv4& addr, string& ifname,
 
     ifname = "";
     vifname = "";
+
     return (false);
 }
 
@@ -383,7 +307,7 @@ IfMgrIfTree::is_directly_connected(const IPv6& addr, string& ifname,
 {
     IfMgrIfTree::IfMap::const_iterator if_iter;
 
-    for (if_iter = ifs().begin(); if_iter != ifs().end(); ++if_iter) {
+    for (if_iter = interfaces().begin(); if_iter != interfaces().end(); ++if_iter) {
 	const IfMgrIfAtom& iface = if_iter->second;
 
 	// Test if interface is enabled and the link state is up
@@ -401,7 +325,7 @@ IfMgrIfTree::is_directly_connected(const IPv6& addr, string& ifname,
 		continue;
 
 	    // Test if there is matching IPv6 address
-	    IfMgrVifAtom::V6Map::const_iterator a6_iter;
+	    IfMgrVifAtom::IPv6Map::const_iterator a6_iter;
 
 	    for (a6_iter = vif.ipv6addrs().begin();
 		 a6_iter != vif.ipv6addrs().end();
@@ -440,6 +364,7 @@ IfMgrIfTree::is_directly_connected(const IPv6& addr, string& ifname,
 
     ifname = "";
     vifname = "";
+
     return (false);
 }
 
@@ -470,10 +395,14 @@ IfMgrIfAtom::operator==(const IfMgrIfAtom& o) const
     return (
 	    name()			== o.name()			&&
 	    enabled()			== o.enabled()			&&
-	    mtu_bytes()			== o.mtu_bytes()		&&
+	    discard()			== o.discard()			&&
+	    unreachable()		== o.unreachable()		&&
+	    management()		== o.management()		&&
+	    mtu()			== o.mtu()			&&
 	    mac()			== o.mac()			&&
 	    pif_index()			== o.pif_index()		&&
 	    no_carrier()		== o.no_carrier()		&&
+	    baudrate()			== o.baudrate()			&&
 	    vifs()			== o.vifs()
 	    );
 }
@@ -481,13 +410,21 @@ IfMgrIfAtom::operator==(const IfMgrIfAtom& o) const
 const IfMgrVifAtom*
 IfMgrIfAtom::find_vif(const string& vifname) const
 {
-    return find_virtual_interface(this, vifname);
+    IfMgrIfAtom::VifMap::const_iterator vi = vifs().find(vifname);
+    if (vi == vifs().end())
+	return (NULL);
+
+    return (&vi->second);
 }
 
 IfMgrVifAtom*
 IfMgrIfAtom::find_vif(const string& vifname)
 {
-    return find_virtual_interface(this, vifname);
+    IfMgrIfAtom::VifMap::iterator vi = vifs().find(vifname);
+    if (vi == vifs().end())
+	return (NULL);
+
+    return (&vi->second);
 }
 
 
@@ -504,7 +441,11 @@ IfMgrVifAtom::operator==(const IfMgrVifAtom& o) const
 	    broadcast_capable()	  	== o.broadcast_capable()	&&
 	    p2p_capable()		== o.p2p_capable()		&&
 	    loopback()			== o.loopback()			&&
+	    pim_register()		== o.pim_register()		&&
 	    pif_index()			== o.pif_index()		&&
+	    vif_index()			== o.vif_index()		&&
+	    is_vlan()			== o.is_vlan()			&&
+	    vlan_id()			== o.vlan_id()			&&
 	    ipv4addrs()			== o.ipv4addrs()		&&
 	    ipv6addrs()			== o.ipv6addrs()
 	    );
@@ -513,25 +454,41 @@ IfMgrVifAtom::operator==(const IfMgrVifAtom& o) const
 const IfMgrIPv4Atom*
 IfMgrVifAtom::find_addr(const IPv4& addr) const
 {
-    return find_virtual_interface_addr(this, addr);
+    IfMgrVifAtom::IPv4Map::const_iterator ai = ipv4addrs().find(addr);
+    if (ai == ipv4addrs().end())
+	return (NULL);
+
+    return (&ai->second);
 }
 
 IfMgrIPv4Atom*
 IfMgrVifAtom::find_addr(const IPv4& addr)
 {
-    return find_virtual_interface_addr(this, addr);
+    IfMgrVifAtom::IPv4Map::iterator ai = ipv4addrs().find(addr);
+    if (ai == ipv4addrs().end())
+	return (NULL);
+
+    return (&ai->second);
 }
 
 const IfMgrIPv6Atom*
 IfMgrVifAtom::find_addr(const IPv6& addr) const
 {
-    return find_virtual_interface_addr(this, addr);
+    IfMgrVifAtom::IPv6Map::const_iterator ai = ipv6addrs().find(addr);
+    if (ai == ipv6addrs().end())
+	return (NULL);
+
+    return (&ai->second);
 }
 
 IfMgrIPv6Atom*
 IfMgrVifAtom::find_addr(const IPv6& addr)
 {
-    return find_virtual_interface_addr(this, addr);
+    IfMgrVifAtom::IPv6Map::iterator ai = ipv6addrs().find(addr);
+    if (ai == ipv6addrs().end())
+	return (NULL);
+
+    return (&ai->second);
 }
 
 
