@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_cmd_queue.cc,v 1.13 2007/02/16 22:45:59 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_cmd_queue.cc,v 1.24 2008/07/23 05:10:37 pavlin Exp $"
 
 #include <algorithm>
 #include <iterator>
@@ -58,7 +58,7 @@ IfMgrCommandDispatcher::IfMgrCommandDispatcher(IfMgrIfTree& tree)
 void
 IfMgrCommandDispatcher::push(const Cmd& cmd)
 {
-    if (_cmd.get() != 0) {
+    if (_cmd.get() != NULL) {
 	XLOG_WARNING("Dropping buffered command.");
     }
     _cmd = cmd;
@@ -68,7 +68,7 @@ bool
 IfMgrCommandDispatcher::execute()
 {
     bool success = false;
-    if (_cmd.get()) {
+    if (_cmd.get() != NULL) {
 	success = _cmd->execute(_iftree);
 	_cmd = 0;
     }
@@ -121,7 +121,7 @@ void
 IfMgrCommandIfClusteringQueue::push(const Cmd& cmd)
 {
     IfMgrIfCommandBase* ifcmd = dynamic_cast<IfMgrIfCommandBase*>(cmd.get());
-    XLOG_ASSERT(ifcmd != 0);
+    XLOG_ASSERT(ifcmd != NULL);
     if (ifcmd->ifname() == _current_ifname) {
 	_current_cmds.push_back(cmd);
     } else {
@@ -158,7 +158,7 @@ IfMgrCommandIfClusteringQueue::pop_front()
 	Cmd& c = _current_cmds.front();
 	IfMgrIfCommandBase* ifcmd =
 	    dynamic_cast<IfMgrIfCommandBase*>(c.get());
-	XLOG_ASSERT(ifcmd != 0);
+	XLOG_ASSERT(ifcmd != NULL);
 	_current_ifname = ifcmd->ifname();
 	_current_cmds.pop_front();
     }
@@ -174,14 +174,13 @@ IfMgrCommandIfClusteringQueue::pop_front()
 //
 class InterfaceNameOfQueuedCmdMatches {
 public:
-    inline InterfaceNameOfQueuedCmdMatches(const string& ifname)
+    InterfaceNameOfQueuedCmdMatches(const string& ifname)
 	: _ifname(ifname)
     {}
 
-    inline bool operator() (IfMgrCommandIfClusteringQueue::Cmd c)
-    {
+    bool operator() (IfMgrCommandIfClusteringQueue::Cmd c) {
 	IfMgrIfCommandBase* ifcmd = dynamic_cast<IfMgrIfCommandBase*>(c.get());
-	XLOG_ASSERT(ifcmd != 0);
+	XLOG_ASSERT(ifcmd != NULL);
 	return ifcmd->ifname() == _ifname;
     }
 protected:
@@ -201,7 +200,7 @@ IfMgrCommandIfClusteringQueue::change_active_interface()
     Cmd& c = _future_cmds.front();
     IfMgrIfCommandBase* ifcmd =
 	dynamic_cast<IfMgrIfCommandBase*>(c.get());
-    XLOG_ASSERT(ifcmd != 0);
+    XLOG_ASSERT(ifcmd != NULL);
     _current_ifname = ifcmd->ifname();
     back_insert_iterator<CmdList> bi(_current_cmds);
     remove_copy_if(_future_cmds.begin(), _future_cmds.end(), bi,
@@ -215,10 +214,10 @@ IfMgrCommandIfClusteringQueue::change_active_interface()
 void
 IfMgrIfTreeToCommands::convert(IfMgrCommandSinkBase& s) const
 {
-    const IfMgrIfTree::IfMap& ifs = _tree.ifs();
+    const IfMgrIfTree::IfMap& interfaces = _tree.interfaces();
     IfMgrIfTree::IfMap::const_iterator cii;
 
-    for (cii = ifs.begin(); cii != ifs.end(); ++cii) {
+    for (cii = interfaces.begin(); cii != interfaces.end(); ++cii) {
 	IfMgrIfAtomToCommands(cii->second).convert(s);
     }
     s.push(new IfMgrHintTreeComplete());
@@ -229,10 +228,14 @@ IfMgrIfAtomToCommands::convert(IfMgrCommandSinkBase& s) const
 {
     s.push(new IfMgrIfAdd(_i.name()));
     s.push(new IfMgrIfSetEnabled(_i.name(), _i.enabled()));
-    s.push(new IfMgrIfSetMtu(_i.name(), _i.mtu_bytes()));
+    s.push(new IfMgrIfSetDiscard(_i.name(), _i.discard()));
+    s.push(new IfMgrIfSetUnreachable(_i.name(), _i.unreachable()));
+    s.push(new IfMgrIfSetManagement(_i.name(), _i.management()));
+    s.push(new IfMgrIfSetMtu(_i.name(), _i.mtu()));
     s.push(new IfMgrIfSetMac(_i.name(), _i.mac()));
     s.push(new IfMgrIfSetPifIndex(_i.name(), _i.pif_index()));
     s.push(new IfMgrIfSetNoCarrier(_i.name(), _i.no_carrier()));
+    s.push(new IfMgrIfSetBaudrate(_i.name(), _i.baudrate()));
 
     const IfMgrIfAtom::VifMap& vifs = _i.vifs();
     IfMgrIfAtom::VifMap::const_iterator cvi;
@@ -253,16 +256,20 @@ IfMgrVifAtomToCommands::convert(IfMgrCommandSinkBase& s) const
     s.push(new IfMgrVifSetBroadcastCapable(ifn, vifn, _v.broadcast_capable()));
     s.push(new IfMgrVifSetP2PCapable(ifn, vifn, _v.p2p_capable()));
     s.push(new IfMgrVifSetLoopbackCapable(ifn, vifn, _v.loopback()));
+    s.push(new IfMgrVifSetPimRegister(ifn, vifn, _v.pim_register()));
     s.push(new IfMgrVifSetPifIndex(ifn, vifn, _v.pif_index()));
+    s.push(new IfMgrVifSetVifIndex(ifn, vifn, _v.vif_index()));
+    s.push(new IfMgrVifSetIsVlan(ifn, vifn, _v.is_vlan()));
+    s.push(new IfMgrVifSetVlanId(ifn, vifn, _v.vlan_id()));
 
-    const IfMgrVifAtom::V4Map& v4s = _v.ipv4addrs();
-    for (IfMgrVifAtom::V4Map::const_iterator cai = v4s.begin();
+    const IfMgrVifAtom::IPv4Map& v4s = _v.ipv4addrs();
+    for (IfMgrVifAtom::IPv4Map::const_iterator cai = v4s.begin();
 	 cai != v4s.end(); ++cai) {
 	IfMgrIPv4AtomToCommands(ifn, vifn, cai->second).convert(s);
     }
 
-    const IfMgrVifAtom::V6Map& v6s = _v.ipv6addrs();
-    for (IfMgrVifAtom::V6Map::const_iterator cai = v6s.begin();
+    const IfMgrVifAtom::IPv6Map& v6s = _v.ipv6addrs();
+    for (IfMgrVifAtom::IPv6Map::const_iterator cai = v6s.begin();
 	 cai != v6s.end(); ++cai) {
 	IfMgrIPv6AtomToCommands(ifn, vifn, cai->second).convert(s);
     }
