@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_cmds.cc,v 1.17 2007/02/16 22:45:59 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_cmds.cc,v 1.30 2008/07/23 05:10:37 pavlin Exp $"
 
 #include "libxorp/c_format.hh"
 
@@ -25,12 +25,6 @@
 // ----------------------------------------------------------------------------
 // Helper functions
 
-static const char*
-bool2str(bool t)
-{
-    return t ? "true" : "false";
-}
-
 /*
  * Makes string starting: IfMgrIfXXX("if0"
  */
@@ -127,21 +121,21 @@ IfMgrCommandBase::~IfMgrCommandBase()
 bool
 IfMgrIfAdd::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    if (ifs.find(n) != ifs.end())
+    if (interfaces.find(n) != interfaces.end())
 	return true;	// Not a failure to add something that already exists
 
     pair<IfMgrIfTree::IfMap::iterator, bool> r =
-	ifs.insert( make_pair(n, IfMgrIfAtom(n)) );
+	interfaces.insert( make_pair(n, IfMgrIfAtom(n)) );
     return r.second;
 }
 
 bool
-IfMgrIfAdd::forward(XrlSender&		  sender,
-		    const string&	  xrl_target,
-		    const IfMgrXrlSendCB& xcb) const
+IfMgrIfAdd::forward(XrlSender&			sender,
+		    const string&		xrl_target,
+		    const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
@@ -160,14 +154,14 @@ IfMgrIfAdd::str() const
 bool
 IfMgrIfRemove::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
-    if (i == ifs.end())
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
 	return true;	// Not a failure if the interface doesn't exist anymore
 
-    ifs.erase(i);
+    interfaces.erase(i);
     return true;
 }
 
@@ -193,33 +187,33 @@ IfMgrIfRemove::str() const
 bool
 IfMgrIfSetEnabled::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
-    if (i == ifs.end())
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
 	return false;
 
     IfMgrIfAtom& interface = i->second;
-    interface.set_enabled(en());
+    interface.set_enabled(enabled());
     return true;
 }
 
 bool
-IfMgrIfSetEnabled::forward(XrlSender&		 sender,
-			   const string&	 xrl_target,
-			   const IfMgrXrlSendCB& xcb) const
+IfMgrIfSetEnabled::forward(XrlSender&			sender,
+			   const string&		xrl_target,
+			   const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    return c.send_interface_set_enabled(xt, ifname(), en(), xcb);
+    return c.send_interface_set_enabled(xt, ifname(), enabled(), xcb);
 }
 
 string
 IfMgrIfSetEnabled::str() const
 {
     return if_str_begin(this, "SetEnabled")
-	+ "\", " + bool2str(en()) + if_str_end();
+	+ "\", " + bool_c_str(enabled()) + if_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -228,11 +222,11 @@ IfMgrIfSetEnabled::str() const
 bool
 IfMgrIfSetDiscard::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
-    if (i == ifs.end())
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
 	return false;
 
     IfMgrIfAtom& interface = i->second;
@@ -241,9 +235,9 @@ IfMgrIfSetDiscard::execute(IfMgrIfTree& t) const
 }
 
 bool
-IfMgrIfSetDiscard::forward(XrlSender&		 sender,
-			   const string&	 xrl_target,
-			   const IfMgrXrlSendCB& xcb) const
+IfMgrIfSetDiscard::forward(XrlSender&			sender,
+			   const string&		xrl_target,
+			   const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
@@ -254,7 +248,77 @@ string
 IfMgrIfSetDiscard::str() const
 {
     return if_str_begin(this, "SetDiscard")
-	+ "\", " + bool2str(discard()) + if_str_end();
+	+ "\", " + bool_c_str(discard()) + if_str_end();
+}
+
+// ----------------------------------------------------------------------------
+// IfMgrIfSetUnreachable
+
+bool
+IfMgrIfSetUnreachable::execute(IfMgrIfTree& t) const
+{
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
+    const string& n = ifname();
+
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
+	return false;
+
+    IfMgrIfAtom& interface = i->second;
+    interface.set_unreachable(unreachable());
+    return true;
+}
+
+bool
+IfMgrIfSetUnreachable::forward(XrlSender&		sender,
+			       const string&		xrl_target,
+			       const IfMgrXrlSendCB&	xcb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_interface_set_unreachable(xt, ifname(), unreachable(), xcb);
+}
+
+string
+IfMgrIfSetUnreachable::str() const
+{
+    return if_str_begin(this, "SetUnreachable")
+	+ "\", " + bool_c_str(unreachable()) + if_str_end();
+}
+
+// ----------------------------------------------------------------------------
+// IfMgrIfSetManagement
+
+bool
+IfMgrIfSetManagement::execute(IfMgrIfTree& t) const
+{
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
+    const string& n = ifname();
+
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
+	return false;
+
+    IfMgrIfAtom& interface = i->second;
+    interface.set_management(management());
+    return true;
+}
+
+bool
+IfMgrIfSetManagement::forward(XrlSender&		sender,
+			      const string&		xrl_target,
+			      const IfMgrXrlSendCB&	xcb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_interface_set_management(xt, ifname(), management(), xcb);
+}
+
+string
+IfMgrIfSetManagement::str() const
+{
+    return if_str_begin(this, "SetManagement")
+	+ "\", " + bool_c_str(management()) + if_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -263,33 +327,33 @@ IfMgrIfSetDiscard::str() const
 bool
 IfMgrIfSetMtu::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
-    if (i == ifs.end())
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
 	return false;
 
     IfMgrIfAtom& interface = i->second;
-    interface.set_mtu_bytes(mtu_bytes());
+    interface.set_mtu(mtu());
     return true;
 }
 
 bool
-IfMgrIfSetMtu::forward(XrlSender&	     sender,
-		       const string&	     xrl_target,
-		       const IfMgrXrlSendCB& xcb) const
+IfMgrIfSetMtu::forward(XrlSender&		sender,
+		       const string&		xrl_target,
+		       const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    return c.send_interface_set_mtu(xt, ifname(), mtu_bytes(), xcb);
+    return c.send_interface_set_mtu(xt, ifname(), mtu(), xcb);
 }
 
 string
 IfMgrIfSetMtu::str() const
 {
     return if_str_begin(this, "Mtu") + ", " +
-	c_format("%u", XORP_UINT_CAST(mtu_bytes())) + if_str_end();
+	c_format("%u", XORP_UINT_CAST(mtu())) + if_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -298,11 +362,11 @@ IfMgrIfSetMtu::str() const
 bool
 IfMgrIfSetMac::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
-    if (i == ifs.end())
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
 	return false;
 
     IfMgrIfAtom& interface = i->second;
@@ -332,11 +396,11 @@ IfMgrIfSetMac::str() const
 bool
 IfMgrIfSetPifIndex::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
-    if (i == ifs.end())
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
 	return false;
 
     IfMgrIfAtom& interface = i->second;
@@ -367,11 +431,11 @@ IfMgrIfSetPifIndex::str() const
 bool
 IfMgrIfSetNoCarrier::execute(IfMgrIfTree& t) const
 {
-    IfMgrIfTree::IfMap& ifs = t.ifs();
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
     const string& n = ifname();
 
-    IfMgrIfTree::IfMap::iterator i = ifs.find(n);
-    if (i == ifs.end())
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
 	return false;
 
     IfMgrIfAtom& interface = i->second;
@@ -380,9 +444,9 @@ IfMgrIfSetNoCarrier::execute(IfMgrIfTree& t) const
 }
 
 bool
-IfMgrIfSetNoCarrier::forward(XrlSender&		sender,
-			     const string&	xrl_target,
-			     const IfMgrXrlSendCB& xcb) const
+IfMgrIfSetNoCarrier::forward(XrlSender&			sender,
+			     const string&		xrl_target,
+			     const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
@@ -393,7 +457,42 @@ string
 IfMgrIfSetNoCarrier::str() const
 {
     return if_str_begin(this, "NoCarrier") + ", " +
-	c_format("%s", no_carrier()? "true" : "false") + if_str_end();
+	c_format("%s", bool_c_str(no_carrier())) + if_str_end();
+}
+
+// ----------------------------------------------------------------------------
+// IfMgrIfSetBaudrate
+
+bool
+IfMgrIfSetBaudrate::execute(IfMgrIfTree& t) const
+{
+    IfMgrIfTree::IfMap& interfaces = t.interfaces();
+    const string& n = ifname();
+
+    IfMgrIfTree::IfMap::iterator i = interfaces.find(n);
+    if (i == interfaces.end())
+	return false;
+
+    IfMgrIfAtom& interface = i->second;
+    interface.set_baudrate(baudrate());
+    return true;
+}
+
+bool
+IfMgrIfSetBaudrate::forward(XrlSender&			sender,
+			    const string&		xrl_target,
+			    const IfMgrXrlSendCB&	xcb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_interface_set_baudrate(xt, ifname(), baudrate(), xcb);
+}
+
+string
+IfMgrIfSetBaudrate::str() const
+{
+    return if_str_begin(this, "Baudrate") + ", " +
+	c_format("%u", XORP_UINT_CAST(baudrate())) + if_str_end();
 }
 
 
@@ -410,7 +509,7 @@ IfMgrIfSetNoCarrier::str() const
 bool
 IfMgrVifAdd::execute(IfMgrIfTree& tree) const
 {
-    IfMgrIfAtom* ifa = tree.find_if(ifname());
+    IfMgrIfAtom* ifa = tree.find_interface(ifname());
     if (ifa == NULL)
 	return false;
 
@@ -426,9 +525,9 @@ IfMgrVifAdd::execute(IfMgrIfTree& tree) const
 }
 
 bool
-IfMgrVifAdd::forward(XrlSender&		   sender,
-		     const string&	   xrl_target,
-		     const IfMgrXrlSendCB& xscb) const
+IfMgrVifAdd::forward(XrlSender&			sender,
+		     const string&		xrl_target,
+		     const IfMgrXrlSendCB&	xscb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
@@ -447,7 +546,7 @@ IfMgrVifAdd::str() const
 bool
 IfMgrVifRemove::execute(IfMgrIfTree& tree) const
 {
-    IfMgrIfAtom* ifa = tree.find_if(ifname());
+    IfMgrIfAtom* ifa = tree.find_interface(ifname());
     if (ifa == NULL)
 	return true;	// Not a failure if the interface doesn't exist anymore
 
@@ -488,25 +587,25 @@ IfMgrVifSetEnabled::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return false;
 
-    vifa->set_enabled(en());
+    vifa->set_enabled(enabled());
     return true;
 }
 
 bool
-IfMgrVifSetEnabled::forward(XrlSender&		  sender,
-			    const string&	  xrl_target,
-			    const IfMgrXrlSendCB& xscb) const
+IfMgrVifSetEnabled::forward(XrlSender&			sender,
+			    const string&		xrl_target,
+			    const IfMgrXrlSendCB&	xscb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    return c.send_vif_set_enabled(xt, ifname(), vifname(), en(), xscb);
+    return c.send_vif_set_enabled(xt, ifname(), vifname(), enabled(), xscb);
 }
 
 string
 IfMgrVifSetEnabled::str() const
 {
     return vif_str_begin(this, "SetEnabled")
-	+ ", " + bool2str(en()) + vif_str_end();
+	+ ", " + bool_c_str(enabled()) + vif_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -519,27 +618,26 @@ IfMgrVifSetMulticastCapable::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return false;
 
-    vifa->set_multicast_capable(capable());
+    vifa->set_multicast_capable(multicast_capable());
     return true;
 }
 
 bool
-IfMgrVifSetMulticastCapable::forward(XrlSender&		   sender,
-				     const string&	   xrl_target,
-				     const IfMgrXrlSendCB& xscb) const
+IfMgrVifSetMulticastCapable::forward(XrlSender&			sender,
+				     const string&		xrl_target,
+				     const IfMgrXrlSendCB&	xscb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& in = ifname();
-    const string& vn = vifname();
-    return c.send_vif_set_multicast_capable(xt, in, vn, capable(), xscb);
+    return c.send_vif_set_multicast_capable(xt, ifname(), vifname(),
+					    multicast_capable(), xscb);
 }
 
 string
 IfMgrVifSetMulticastCapable::str() const
 {
     return vif_str_begin(this, "SetMulticastCapable")
-	+ ", " + bool2str(capable()) + vif_str_end();
+	+ ", " + bool_c_str(multicast_capable()) + vif_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -552,27 +650,26 @@ IfMgrVifSetBroadcastCapable::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return false;
 
-    vifa->set_broadcast_capable(capable());
+    vifa->set_broadcast_capable(broadcast_capable());
     return true;
 }
 
 bool
-IfMgrVifSetBroadcastCapable::forward(XrlSender&		   sender,
-				     const string&	   xrl_target,
-				     const IfMgrXrlSendCB& xscb) const
+IfMgrVifSetBroadcastCapable::forward(XrlSender&			sender,
+				     const string&		xrl_target,
+				     const IfMgrXrlSendCB&	xscb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& in = ifname();
-    const string& vn = vifname();
-    return c.send_vif_set_broadcast_capable(xt, in, vn, capable(), xscb);
+    return c.send_vif_set_broadcast_capable(xt, ifname(), vifname(),
+					    broadcast_capable(), xscb);
 }
 
 string
 IfMgrVifSetBroadcastCapable::str() const
 {
     return vif_str_begin(this, "SetBroadcastCapable")
-	+ ", " + bool2str(capable()) + vif_str_end();
+	+ ", " + bool_c_str(broadcast_capable()) + vif_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -585,26 +682,26 @@ IfMgrVifSetP2PCapable::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return false;
 
-    vifa->set_p2p_capable(capable());
+    vifa->set_p2p_capable(p2p_capable());
     return true;
 }
 
 bool
-IfMgrVifSetP2PCapable::forward(XrlSender&	     sender,
-			       const string&	     xrl_target,
-			       const IfMgrXrlSendCB& xscb) const
+IfMgrVifSetP2PCapable::forward(XrlSender&		sender,
+			       const string&		xrl_target,
+			       const IfMgrXrlSendCB&	xscb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const bool& cap = capable();
-    return c.send_vif_set_p2p_capable(xt, ifname(), vifname(), cap, xscb);
+    return c.send_vif_set_p2p_capable(xt, ifname(), vifname(), p2p_capable(),
+				      xscb);
 }
 
 string
 IfMgrVifSetP2PCapable::str() const
 {
     return vif_str_begin(this, "SetP2PCapable")
-	+ ", " + bool2str(capable()) + vif_str_end();
+	+ ", " + bool_c_str(p2p_capable()) + vif_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -617,26 +714,58 @@ IfMgrVifSetLoopbackCapable::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return false;
 
-    vifa->set_loopback(capable());
+    vifa->set_loopback(loopback_capable());
     return true;
 }
 
 bool
-IfMgrVifSetLoopbackCapable::forward(XrlSender&		  sender,
-				    const string&	  xrl_target,
-				    const IfMgrXrlSendCB& xscb) const
+IfMgrVifSetLoopbackCapable::forward(XrlSender&			sender,
+				    const string&		xrl_target,
+				    const IfMgrXrlSendCB&	xscb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const bool& cap = capable();
-    return c.send_vif_set_loopback(xt, ifname(), vifname(), cap, xscb);
+    return c.send_vif_set_loopback(xt, ifname(), vifname(), loopback_capable(),
+				   xscb);
 }
 
 string
 IfMgrVifSetLoopbackCapable::str() const
 {
     return vif_str_begin(this, "SetLoopbackCapable")
-	+ ", " + bool2str(capable()) + vif_str_end();
+	+ ", " + bool_c_str(loopback_capable()) + vif_str_end();
+}
+
+// ----------------------------------------------------------------------------
+// IfMgrVifSetPimRegister
+
+bool
+IfMgrVifSetPimRegister::execute(IfMgrIfTree& tree) const
+{
+    IfMgrVifAtom* vifa = tree.find_vif(ifname(), vifname());
+    if (vifa == NULL)
+	return false;
+
+    vifa->set_pim_register(pim_register());
+    return true;
+}
+
+bool
+IfMgrVifSetPimRegister::forward(XrlSender&		sender,
+				const string&		xrl_target,
+				const IfMgrXrlSendCB&	xscb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_vif_set_pim_register(xt, ifname(), vifname(),
+				       pim_register(), xscb);
+}
+
+string
+IfMgrVifSetPimRegister::str() const
+{
+    return vif_str_begin(this, "SetPimRegister")
+	+ ", " + bool_c_str(pim_register()) + vif_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -654,15 +783,14 @@ IfMgrVifSetPifIndex::execute(IfMgrIfTree& tree) const
 }
 
 bool
-IfMgrVifSetPifIndex::forward(XrlSender&		   sender,
-			     const string&	   xrl_target,
-			     const IfMgrXrlSendCB& xscb) const
+IfMgrVifSetPifIndex::forward(XrlSender&			sender,
+			     const string&		xrl_target,
+			     const IfMgrXrlSendCB&	xscb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& in = ifname();
-    const string& vn = vifname();
-    return c.send_vif_set_pif_index(xt, in, vn, pif_index(), xscb);
+    return c.send_vif_set_pif_index(xt, ifname(), vifname(), pif_index(),
+				    xscb);
 }
 
 string
@@ -672,6 +800,100 @@ IfMgrVifSetPifIndex::str() const
 	+ ", " + c_format("%u", XORP_UINT_CAST(pif_index())) + vif_str_end();
 }
 
+// ----------------------------------------------------------------------------
+// IfMgrVifSetVifIndex
+
+bool
+IfMgrVifSetVifIndex::execute(IfMgrIfTree& tree) const
+{
+    IfMgrVifAtom* vifa = tree.find_vif(ifname(), vifname());
+    if (vifa == NULL)
+	return false;
+
+    vifa->set_vif_index(vif_index());
+    return true;
+}
+
+bool
+IfMgrVifSetVifIndex::forward(XrlSender&			sender,
+			     const string&		xrl_target,
+			     const IfMgrXrlSendCB&	xscb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_vif_set_vif_index(xt, ifname(), vifname(), vif_index(),
+				    xscb);
+}
+
+string
+IfMgrVifSetVifIndex::str() const
+{
+    return vif_str_begin(this, "SetVifIndex")
+	+ ", " + c_format("%u", XORP_UINT_CAST(vif_index())) + vif_str_end();
+}
+
+// ----------------------------------------------------------------------------
+// IfMgrVifSetIsVlan
+
+bool
+IfMgrVifSetIsVlan::execute(IfMgrIfTree& tree) const
+{
+    IfMgrVifAtom* vifa = tree.find_vif(ifname(), vifname());
+    if (vifa == NULL)
+	return false;
+
+    vifa->set_vlan(is_vlan());
+    return true;
+}
+
+bool
+IfMgrVifSetIsVlan::forward(XrlSender&			sender,
+			   const string&		xrl_target,
+			   const IfMgrXrlSendCB&	xscb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_vif_set_vlan(xt, ifname(), vifname(), is_vlan(), xscb);
+}
+
+string
+IfMgrVifSetIsVlan::str() const
+{
+    return vif_str_begin(this, "SetIsVlan")
+	+ ", " + bool_c_str(is_vlan()) + vif_str_end();
+}
+
+// ----------------------------------------------------------------------------
+// IfMgrVifSetVlanId
+
+bool
+IfMgrVifSetVlanId::execute(IfMgrIfTree& tree) const
+{
+    IfMgrVifAtom* vifa = tree.find_vif(ifname(), vifname());
+    if (vifa == NULL)
+	return false;
+
+    vifa->set_vlan_id(vlan_id());
+    return true;
+}
+
+bool
+IfMgrVifSetVlanId::forward(XrlSender&			sender,
+			   const string&		xrl_target,
+			   const IfMgrXrlSendCB&	xscb) const
+{
+    XrlFeaIfmgrMirrorV0p1Client c(&sender);
+    const char* xt = xrl_target.c_str();
+    return c.send_vif_set_vlan_id(xt, ifname(), vifname(), vlan_id(), xscb);
+}
+
+string
+IfMgrVifSetVlanId::str() const
+{
+    return vif_str_begin(this, "SetVlanId")
+	+ ", " + c_format("%u", XORP_UINT_CAST(vlan_id())) + vif_str_end();
+}
+
 
 // ----------------------------------------------------------------------------
 //
@@ -686,19 +908,19 @@ IfMgrIPv4Add::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return false;
 
-    IfMgrVifAtom::V4Map& addrs = vifa->ipv4addrs();
+    IfMgrVifAtom::IPv4Map& addrs = vifa->ipv4addrs();
     if (addrs.find(addr()) != addrs.end())
 	return true;	// Not a failure to add something that already exists
 
-    pair<IfMgrVifAtom::V4Map::iterator, bool> r =
+    pair<IfMgrVifAtom::IPv4Map::iterator, bool> r =
 	addrs.insert( make_pair(addr(), IfMgrIPv4Atom(addr())) );
     return r.second;
 }
 
 bool
-IfMgrIPv4Add::forward(XrlSender&	    sender,
-		      const string&	    xrl_target,
-		      const IfMgrXrlSendCB& xcb) const
+IfMgrIPv4Add::forward(XrlSender&		sender,
+		      const string&		xrl_target,
+		      const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
@@ -721,8 +943,8 @@ IfMgrIPv4Remove::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return true;	// Not a failure if the vif doesn't exist anymore
 
-    IfMgrVifAtom::V4Map& addrs = vifa->ipv4addrs();
-    IfMgrVifAtom::V4Map::iterator i = addrs.find(addr());
+    IfMgrVifAtom::IPv4Map& addrs = vifa->ipv4addrs();
+    IfMgrVifAtom::IPv4Map::iterator i = addrs.find(addr());
     if (i == addrs.end())
 	return true;	// Not a failure if the address doesn't exist anymore
 
@@ -761,15 +983,14 @@ IfMgrIPv4SetPrefix::execute(IfMgrIfTree& tree) const
 }
 
 bool
-IfMgrIPv4SetPrefix::forward(XrlSender&		  sender,
-			    const string&	  xrl_target,
-			    const IfMgrXrlSendCB& xcb) const
+IfMgrIPv4SetPrefix::forward(XrlSender&			sender,
+			    const string&		xrl_target,
+			    const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    return c.send_ipv4_set_prefix(xt, ifn, vifn, addr(), prefix_len(), xcb);
+    return c.send_ipv4_set_prefix(xt, ifname(), vifname(), addr(),
+				  prefix_len(), xcb);
 }
 
 string
@@ -789,25 +1010,26 @@ IfMgrIPv4SetEnabled::execute(IfMgrIfTree& tree) const
     if (a == NULL)
 	return false;
 
-    a->set_enabled(en());
+    a->set_enabled(enabled());
     return true;
 }
 
 bool
-IfMgrIPv4SetEnabled::forward(XrlSender&		   sender,
-			     const string&	   xrl_target,
-			     const IfMgrXrlSendCB& xcb) const
+IfMgrIPv4SetEnabled::forward(XrlSender&			sender,
+			     const string&		xrl_target,
+			     const IfMgrXrlSendCB&	xcb) const
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    return c.send_ipv4_set_enabled(xt, ifname(), vifname(), addr(), en(), xcb);
+    return c.send_ipv4_set_enabled(xt, ifname(), vifname(), addr(), enabled(),
+				   xcb);
 }
 
 string
 IfMgrIPv4SetEnabled::str() const
 {
     return ipv4_str_begin(this, "SetEnabled") + ", " +
-	bool2str(en()) + ipv4_str_end();
+	bool_c_str(enabled()) + ipv4_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -820,7 +1042,7 @@ IfMgrIPv4SetMulticastCapable::execute(IfMgrIfTree& tree) const
     if (a == NULL)
 	return false;
 
-    a->set_multicast_capable(capable());
+    a->set_multicast_capable(multicast_capable());
     return true;
 }
 
@@ -831,17 +1053,15 @@ IfMgrIPv4SetMulticastCapable::forward(XrlSender&		sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    IPv4 a = addr();
-    return c.send_ipv4_set_multicast_capable(xt, ifn, vifn, a, capable(), xcb);
+    return c.send_ipv4_set_multicast_capable(xt, ifname(), vifname(), addr(),
+					     multicast_capable(), xcb);
 }
 
 string
 IfMgrIPv4SetMulticastCapable::str() const
 {
     return ipv4_str_begin(this, "SetMulticastCapable") + ", " +
-	bool2str(capable()) + ipv4_str_end();
+	bool_c_str(multicast_capable()) + ipv4_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -865,16 +1085,15 @@ IfMgrIPv4SetLoopback::forward(XrlSender&		sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    return c.send_ipv4_set_loopback(xt, ifn, vifn, addr(), loopback(), xcb);
+    return c.send_ipv4_set_loopback(xt, ifname(), vifname(), addr(),
+				    loopback(), xcb);
 }
 
 string
 IfMgrIPv4SetLoopback::str() const
 {
     return ipv4_str_begin(this, "SetLoopback") + ", " +
-	bool2str(loopback()) + ipv4_str_end();
+	bool_c_str(loopback()) + ipv4_str_end();
 }
 
 
@@ -888,7 +1107,7 @@ IfMgrIPv4SetBroadcast::execute(IfMgrIfTree& tree) const
     if (a == NULL)
 	return false;
 
-    a->set_broadcast_addr(oaddr());
+    a->set_broadcast_addr(broadcast_addr());
     return true;
 }
 
@@ -899,16 +1118,15 @@ IfMgrIPv4SetBroadcast::forward(XrlSender&		sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    return c.send_ipv4_set_broadcast(xt, ifn, vifn, addr(), oaddr(), xcb);
+    return c.send_ipv4_set_broadcast(xt, ifname(), vifname(), addr(),
+				     broadcast_addr(), xcb);
 }
 
 string
 IfMgrIPv4SetBroadcast::str() const
 {
     return ipv4_str_begin(this, "SetBroadcast") + ", " +
-	oaddr().str() + ipv4_str_end();
+	broadcast_addr().str() + ipv4_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -921,7 +1139,7 @@ IfMgrIPv4SetEndpoint::execute(IfMgrIfTree& tree) const
     if (a == NULL)
 	return false;
 
-    a->set_endpoint_addr(oaddr());
+    a->set_endpoint_addr(endpoint_addr());
     return true;
 }
 
@@ -932,16 +1150,15 @@ IfMgrIPv4SetEndpoint::forward(XrlSender&		sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    return c.send_ipv4_set_endpoint(xt, ifn, vifn, addr(), oaddr(), xcb);
+    return c.send_ipv4_set_endpoint(xt, ifname(), vifname(), addr(),
+				    endpoint_addr(), xcb);
 }
 
 string
 IfMgrIPv4SetEndpoint::str() const
 {
     return ipv4_str_begin(this, "SetEndpoint") + ", " +
-	oaddr().str() + ipv4_str_end();
+	endpoint_addr().str() + ipv4_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -957,11 +1174,11 @@ IfMgrIPv6Add::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return false;
 
-    IfMgrVifAtom::V6Map& addrs = vifa->ipv6addrs();
+    IfMgrVifAtom::IPv6Map& addrs = vifa->ipv6addrs();
     if (addrs.find(addr()) != addrs.end())
 	return true;	// Not a failure to add something that already exists
 
-    pair<IfMgrVifAtom::V6Map::iterator, bool> r =
+    pair<IfMgrVifAtom::IPv6Map::iterator, bool> r =
 	addrs.insert( make_pair(addr(), IfMgrIPv6Atom(addr())) );
     return r.second;
 }
@@ -992,8 +1209,8 @@ IfMgrIPv6Remove::execute(IfMgrIfTree& tree) const
     if (vifa == NULL)
 	return true;	// Not a failure if the vif doesn't exist anymore
 
-    IfMgrVifAtom::V6Map& addrs = vifa->ipv6addrs();
-    IfMgrVifAtom::V6Map::iterator i = addrs.find(addr());
+    IfMgrVifAtom::IPv6Map& addrs = vifa->ipv6addrs();
+    IfMgrVifAtom::IPv6Map::iterator i = addrs.find(addr());
     if (i == addrs.end())
 	return true;	// Not a failure if the address doesn't exist anymore
 
@@ -1038,9 +1255,8 @@ IfMgrIPv6SetPrefix::forward(XrlSender&			sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    return c.send_ipv6_set_prefix(xt, ifn, vifn, addr(), prefix_len(), xcb);
+    return c.send_ipv6_set_prefix(xt, ifname(), vifname(), addr(),
+				  prefix_len(), xcb);
 }
 
 string
@@ -1060,7 +1276,7 @@ IfMgrIPv6SetEnabled::execute(IfMgrIfTree& tree) const
     if (a == NULL)
 	return false;
 
-    a->set_enabled(en());
+    a->set_enabled(enabled());
     return true;
 }
 
@@ -1071,14 +1287,15 @@ IfMgrIPv6SetEnabled::forward(XrlSender&			sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    return c.send_ipv6_set_enabled(xt, ifname(), vifname(), addr(), en(), xcb);
+    return c.send_ipv6_set_enabled(xt, ifname(), vifname(), addr(), enabled(),
+				   xcb);
 }
 
 string
 IfMgrIPv6SetEnabled::str() const
 {
     return ipv6_str_begin(this, "SetEnabled") + ", " +
-	bool2str(en()) + ipv6_str_end();
+	bool_c_str(enabled()) + ipv6_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -1091,7 +1308,7 @@ IfMgrIPv6SetMulticastCapable::execute(IfMgrIfTree& tree) const
     if (a == NULL)
 	return false;
 
-    a->set_multicast_capable(capable());
+    a->set_multicast_capable(multicast_capable());
     return true;
 }
 
@@ -1102,17 +1319,15 @@ IfMgrIPv6SetMulticastCapable::forward(XrlSender&		sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    const IPv6& a = addr();
-    return c.send_ipv6_set_multicast_capable(xt, ifn, vifn, a, capable(), xcb);
+    return c.send_ipv6_set_multicast_capable(xt, ifname(), vifname(), addr(),
+					     multicast_capable(), xcb);
 }
 
 string
 IfMgrIPv6SetMulticastCapable::str() const
 {
     return ipv6_str_begin(this, "SetMulticastCapable") + ", " +
-	bool2str(capable()) + ipv6_str_end();
+	bool_c_str(multicast_capable()) + ipv6_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -1136,16 +1351,15 @@ IfMgrIPv6SetLoopback::forward(XrlSender&		sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    return c.send_ipv6_set_loopback(xt, ifn, vifn, addr(), loopback(), xcb);
+    return c.send_ipv6_set_loopback(xt, ifname(), vifname(), addr(),
+				    loopback(), xcb);
 }
 
 string
 IfMgrIPv6SetLoopback::str() const
 {
     return ipv6_str_begin(this, "SetLoopback") + ", " +
-	bool2str(loopback()) + ipv6_str_end();
+	bool_c_str(loopback()) + ipv6_str_end();
 }
 
 // ----------------------------------------------------------------------------
@@ -1158,7 +1372,7 @@ IfMgrIPv6SetEndpoint::execute(IfMgrIfTree& tree) const
     if (a == NULL)
 	return false;
 
-    a->set_endpoint_addr(oaddr());
+    a->set_endpoint_addr(endpoint_addr());
     return true;
 }
 
@@ -1169,16 +1383,15 @@ IfMgrIPv6SetEndpoint::forward(XrlSender&		sender,
 {
     XrlFeaIfmgrMirrorV0p1Client c(&sender);
     const char* xt = xrl_target.c_str();
-    const string& ifn = ifname();
-    const string& vifn = vifname();
-    return c.send_ipv6_set_endpoint(xt, ifn, vifn, addr(), oaddr(), xcb);
+    return c.send_ipv6_set_endpoint(xt, ifname(), vifname(), addr(),
+				    endpoint_addr(), xcb);
 }
 
 string
 IfMgrIPv6SetEndpoint::str() const
 {
     return ipv6_str_begin(this, "SetEndpoint") + ", " +
-	oaddr().str() + ipv6_str_end();
+	endpoint_addr().str() + ipv6_str_end();
 }
 
 
