@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.20 2007/02/16 22:46:00 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.32 2008/07/23 05:10:37 pavlin Exp $"
 
 #include "libxorp/status_codes.h"
 #include "libxorp/eventloop.hh"
@@ -87,10 +87,20 @@ protected:
 	const string&	ifname,
 	const bool&	discard);
 
+    XrlCmdError fea_ifmgr_mirror_0_1_interface_set_unreachable(
+	// Input values,
+	const string&	ifname,
+	const bool&	unreachable);
+
+    XrlCmdError fea_ifmgr_mirror_0_1_interface_set_management(
+	// Input values,
+	const string&	ifname,
+	const bool&	management);
+
     XrlCmdError fea_ifmgr_mirror_0_1_interface_set_mtu(
 	// Input values,
 	const string&	ifname,
-	const uint32_t&	mtu_bytes);
+	const uint32_t&	mtu);
 
     XrlCmdError fea_ifmgr_mirror_0_1_interface_set_mac(
 	// Input values,
@@ -107,6 +117,11 @@ protected:
 	const string&	ifname,
 	const bool&	no_carrier);
 
+    XrlCmdError fea_ifmgr_mirror_0_1_interface_set_baudrate(
+	// Input values,
+	const string&	ifname,
+	const uint64_t&	baudrate);
+
     XrlCmdError fea_ifmgr_mirror_0_1_vif_add(
 	// Input values,
 	const string&	ifname,
@@ -147,11 +162,35 @@ protected:
 	const string&	vifname,
 	const bool&	loopback);
 
+    XrlCmdError fea_ifmgr_mirror_0_1_vif_set_pim_register(
+	// Input values,
+	const string&	ifname,
+	const string&	vifname,
+	const bool&	pim_register);
+
     XrlCmdError fea_ifmgr_mirror_0_1_vif_set_pif_index(
 	// Input values,
 	const string&	ifname,
 	const string&	vifname,
-	const uint32_t&	index);
+	const uint32_t&	pif_index);
+
+    XrlCmdError fea_ifmgr_mirror_0_1_vif_set_vif_index(
+	// Input values,
+	const string&	ifname,
+	const string&	vifname,
+	const uint32_t&	vif_index);
+
+    XrlCmdError fea_ifmgr_mirror_0_1_vif_set_vlan(
+	// Input values,
+	const string&	ifname,
+	const string&	vifname,
+	const bool&	is_vlan);
+
+    XrlCmdError fea_ifmgr_mirror_0_1_vif_set_vlan_id(
+	// Input values,
+	const string&	ifname,
+	const string&	vifname,
+	const uint32_t&	vlan_id);
 
     XrlCmdError fea_ifmgr_mirror_0_1_ipv4_add(
 	// Input values,
@@ -198,14 +237,14 @@ protected:
 	const string&	ifname,
 	const string&	vifname,
 	const IPv4&	addr,
-	const IPv4&	oaddr);
+	const IPv4&	broadcast_addr);
 
     XrlCmdError fea_ifmgr_mirror_0_1_ipv4_set_endpoint(
 	// Input values,
 	const string&	ifname,
 	const string&	vifname,
 	const IPv4&	addr,
-	const IPv4&	oaddr);
+	const IPv4&	endpoint_addr);
 
     XrlCmdError fea_ifmgr_mirror_0_1_ipv6_add(
 	// Input values,
@@ -252,7 +291,7 @@ protected:
 	const string&	ifname,
 	const string&	vifname,
 	const IPv6&	addr,
-	const IPv6&	oaddr);
+	const IPv6&	endpoint_addr);
 
     XrlCmdError fea_ifmgr_mirror_0_1_hint_tree_complete();
 
@@ -278,7 +317,7 @@ static const char* DISPATCH_FAILED = "Local dispatch error";
 IfMgrXrlMirrorTarget::IfMgrXrlMirrorTarget(XrlRouter&		 rtr,
 					   IfMgrCommandDispatcher& dispatcher)
     : XrlFeaIfmgrMirrorTargetBase(&rtr), _rtr(rtr), _dispatcher(dispatcher),
-      _hint_observer(0)
+      _hint_observer(NULL)
 {
 }
 
@@ -365,12 +404,38 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_discard(
 }
 
 XrlCmdError
-IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_mtu(
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_unreachable(
 	const string&	ifname,
-	const uint32_t&	mtu_bytes
+	const bool&	unreachable
 	)
 {
-    _dispatcher.push(new IfMgrIfSetMtu(ifname, mtu_bytes));
+    _dispatcher.push(new IfMgrIfSetUnreachable(ifname, unreachable));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_management(
+	const string&	ifname,
+	const bool&	management
+	)
+{
+    _dispatcher.push(new IfMgrIfSetManagement(ifname, management));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_mtu(
+	const string&	ifname,
+	const uint32_t&	mtu
+	)
+{
+    _dispatcher.push(new IfMgrIfSetMtu(ifname, mtu));
     if (_dispatcher.execute() == true) {
 	return XrlCmdError::OKAY();
     }
@@ -417,6 +482,19 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_no_carrier(
     return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
 }
 
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_baudrate(
+	const string&	ifname,
+	const uint64_t&	baudrate
+	)
+{
+    _dispatcher.push(new IfMgrIfSetBaudrate(ifname, baudrate));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
 XrlCmdError
 IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_add(
 	const string& ifname,
@@ -513,6 +591,20 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_set_loopback(
     return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
 }
 
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_set_pim_register(
+	const string&	ifname,
+	const string&	vifname,
+	const bool&	pim_register
+	)
+{
+    _dispatcher.push(new IfMgrVifSetPimRegister(ifname, vifname, pim_register));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
 XrlCmdError
 IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_set_pif_index(
 	const string&	ifname,
@@ -527,6 +619,48 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_set_pif_index(
     return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
 }
 
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_set_vif_index(
+	const string&	ifname,
+	const string&	vifname,
+	const uint32_t&	vif_index
+	)
+{
+    _dispatcher.push(new IfMgrVifSetVifIndex(ifname, vifname, vif_index));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_set_vlan(
+	const string&	ifname,
+	const string&	vifname,
+	const bool&	is_vlan
+	)
+{
+    _dispatcher.push(new IfMgrVifSetIsVlan(ifname, vifname, is_vlan));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_vif_set_vlan_id(
+	const string&	ifname,
+	const string&	vifname,
+	const uint32_t&	vlan_id
+	)
+{
+    _dispatcher.push(new IfMgrVifSetVlanId(ifname, vifname, vlan_id));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
 XrlCmdError
 IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_ipv4_add(
 	const string&	ifname,
@@ -620,10 +754,11 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_ipv4_set_broadcast(
 	const string&	ifn,
 	const string&	vifn,
 	const IPv4&	addr,
-	const IPv4&	oaddr
+	const IPv4&	broadcast_addr
 )
 {
-    _dispatcher.push(new IfMgrIPv4SetBroadcast(ifn, vifn, addr, oaddr));
+    _dispatcher.push(new IfMgrIPv4SetBroadcast(ifn, vifn, addr,
+					       broadcast_addr));
     if (_dispatcher.execute() == true) {
 	return XrlCmdError::OKAY();
     }
@@ -635,10 +770,10 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_ipv4_set_endpoint(
 	const string&	ifn,
 	const string&	vifn,
 	const IPv4&	addr,
-	const IPv4&	oaddr
+	const IPv4&	endpoint_addr
 )
 {
-    _dispatcher.push(new IfMgrIPv4SetEndpoint(ifn, vifn, addr, oaddr));
+    _dispatcher.push(new IfMgrIPv4SetEndpoint(ifn, vifn, addr, endpoint_addr));
     if (_dispatcher.execute() == true) {
 	return XrlCmdError::OKAY();
     }
@@ -738,10 +873,10 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_ipv6_set_endpoint(
 	const string&	ifn,
 	const string&	vifn,
 	const IPv6&	addr,
-	const IPv6&	oaddr
+	const IPv6&	endpoint_addr
 )
 {
-    _dispatcher.push(new IfMgrIPv6SetEndpoint(ifn, vifn, addr, oaddr));
+    _dispatcher.push(new IfMgrIPv6SetEndpoint(ifn, vifn, addr, endpoint_addr));
     if (_dispatcher.execute() == true) {
 	return XrlCmdError::OKAY();
     }
@@ -751,7 +886,7 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_ipv6_set_endpoint(
 bool
 IfMgrXrlMirrorTarget::attach(IfMgrHintObserver* ho)
 {
-    if (_hint_observer != 0) {
+    if (_hint_observer != NULL) {
 	return false;
     }
     _hint_observer = ho;
@@ -764,7 +899,7 @@ IfMgrXrlMirrorTarget::detach(IfMgrHintObserver* ho)
     if (_hint_observer != ho) {
 	return false;
     }
-    _hint_observer = 0;
+    _hint_observer = NULL;
     return true;
 }
 
@@ -796,27 +931,27 @@ class IfMgrXrlMirrorRouter : public XrlStdRouter
 public:
     IfMgrXrlMirrorRouter(EventLoop&	e,
 			 const char*	class_name)
-	: XrlStdRouter(e, class_name), _o(0)
+	: XrlStdRouter(e, class_name), _o(NULL)
     {}
 
     IfMgrXrlMirrorRouter(EventLoop&	e,
 			 const char*	class_name,
 			 IPv4		finder_addr)
-	: XrlStdRouter(e, class_name, finder_addr), _o(0)
+	: XrlStdRouter(e, class_name, finder_addr), _o(NULL)
     {}
 
     IfMgrXrlMirrorRouter(EventLoop&	e,
 			 const char*	class_name,
 			 IPv4		finder_addr,
 			 uint16_t	finder_port)
-	: XrlStdRouter(e, class_name, finder_addr, finder_port), _o(0)
+	: XrlStdRouter(e, class_name, finder_addr, finder_port), _o(NULL)
     {}
 
     IfMgrXrlMirrorRouter(EventLoop&	e,
 			 const char*	class_name,
 			 const char*	finder_hostname,
 			 uint16_t	finder_port)
-	: XrlStdRouter(e, class_name, finder_hostname, finder_port), _o(0)
+	: XrlStdRouter(e, class_name, finder_hostname, finder_port), _o(NULL)
     {}
 
     /**
@@ -827,7 +962,7 @@ public:
      */
     bool attach(IfMgrXrlMirrorRouterObserver* o)
     {
-	if (_o)
+	if (_o != NULL)
 	    return false;
 	_o = o;
 	return true;
@@ -843,21 +978,21 @@ public:
     {
 	if (o != _o)
 	    return false;
-	_o = 0;
+	_o = NULL;
 	return true;
     }
 
 protected:
     void finder_ready_event(const string& tgt_name)
     {
-	if (tgt_name == instance_name() && _o != 0) {
+	if (tgt_name == instance_name() && _o != NULL) {
 	    _o->finder_ready_event();
 	}
     }
 
     void finder_disconnect_event()
     {
-	if (_o != 0) {
+	if (_o != NULL) {
 	    _o->finder_disconnect_event();
 	}
     }
@@ -879,7 +1014,7 @@ IfMgrXrlMirror::IfMgrXrlMirror(EventLoop&	e,
 
     : ServiceBase("FEA Interface Mirror"),
       _e(e), _finder_addr(finder_addr), _finder_port(finder_port),
-      _dispatcher(_iftree), _rtarget(rtarget), _rtr(0), _xrl_tgt(0)
+      _dispatcher(_iftree), _rtarget(rtarget), _rtr(NULL), _xrl_tgt(NULL)
 {
 }
 
@@ -890,7 +1025,7 @@ IfMgrXrlMirror::IfMgrXrlMirror(EventLoop&	e,
 
     : ServiceBase("FEA Interface Mirror"),
       _e(e), _finder_hostname(finder_hostname), _finder_port(finder_port),
-      _dispatcher(_iftree), _rtarget(rtarget), _rtr(0), _xrl_tgt(0)
+      _dispatcher(_iftree), _rtarget(rtarget), _rtr(NULL), _xrl_tgt(NULL)
 {
 }
 
@@ -905,17 +1040,17 @@ IfMgrXrlMirror::~IfMgrXrlMirror()
 	_xrl_tgt->detach(this);
 	_rtr->detach(this);
 	delete _xrl_tgt;
-	_xrl_tgt = 0;
+	_xrl_tgt = NULL;
 	delete _rtr;
-	_rtr = 0;
+	_rtr = NULL;
     }
 }
 
-bool
+int
 IfMgrXrlMirror::startup()
 {
     if (status() != SERVICE_READY)
-	return false;
+	return (XORP_ERROR);
 
     if (_rtr == NULL) {
 	if (! _finder_hostname.empty()) {
@@ -935,19 +1070,19 @@ IfMgrXrlMirror::startup()
     }
     set_status(SERVICE_STARTING, "Initializing Xrl Router.");
 
-    return true;
+    return (XORP_OK);
 }
 
-bool
+int
 IfMgrXrlMirror::shutdown()
 {
     if (status() != SERVICE_RUNNING)
-	return false;
+	return (XORP_ERROR);
 
     set_status(SERVICE_SHUTTING_DOWN);
     unregister_with_ifmgr();
 
-    return true;
+    return (XORP_OK);
 }
 
 void
