@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/test_local_copy.cc,v 1.11 2007/02/16 22:46:00 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/test_local_copy.cc,v 1.20 2008/07/23 05:10:38 pavlin Exp $"
 
 #include "libfeaclient_module.h"
 
@@ -38,7 +38,7 @@ static const char *program_name         = "test_local_copy";
 static const char *program_description  = "Test local copying of IfMgr configuration state";
 static const char *program_version_id   = "0.1";
 static const char *program_date         = "August, 2003";
-static const char *program_copyright    = "See file LICENSE.XORP";
+static const char *program_copyright    = "See file LICENSE";
 static const char *program_return_value = "0 on success, 1 if test error, 2 if internal error";
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -78,6 +78,18 @@ test_main()
 	verbose_log("Failed to enable interface\n");
 	return 1;
     }
+    if (IfMgrIfSetDiscard("if0", true).execute(t) == false) {
+	verbose_log("Failed to set discard\n");
+	return 1;
+    }
+    if (IfMgrIfSetUnreachable("if0", true).execute(t) == false) {
+	verbose_log("Failed to set unreachable\n");
+	return 1;
+    }
+    if (IfMgrIfSetManagement("if0", true).execute(t) == false) {
+	verbose_log("Failed to set management\n");
+	return 1;
+    }
     if (IfMgrIfSetMtu("if0", 9000).execute(t) == false) {
 	verbose_log("Failed to set mtu\n");
 	return 1;
@@ -94,6 +106,10 @@ test_main()
 	verbose_log("Failed to set no_carrier\n");
 	return 1;
     }
+    if (IfMgrIfSetBaudrate("if0", 1*000*000*000).execute(t) == false) {
+	verbose_log("Failed to set baudrate\n");
+	return 1;
+    }
 
     // Create a vif on a non-existant interface and check it fails
     if (IfMgrVifAdd("iiii0", "bad0").execute(t) == true) {
@@ -110,7 +126,7 @@ test_main()
 	verbose_log("Failed to enable vif\n");
 	return 1;
     }
-    if (IfMgrVifSetMulticastCapable("if0", "vif0", true).execute (t)== false) {
+    if (IfMgrVifSetMulticastCapable("if0", "vif0", true).execute(t) == false) {
 	verbose_log("Failed to set multicast capable\n");
 	return 1;
     }
@@ -126,10 +142,22 @@ test_main()
 	verbose_log("Failed to set loopback capable\n");
 	return 1;
     }
+    if (IfMgrVifSetPimRegister("if0", "vif0", true).execute(t) == false) {
+	verbose_log("Failed to set PIM Register vif\n");
+	return 1;
+    }
     if (IfMgrVifSetPifIndex("if0", "vif0", 74).execute(t) == false) {
 	verbose_log("Failed to set pif index\n");
 	return 1;
     }
+    if (IfMgrVifSetIsVlan("if0", "vif0", true).execute(t) == false) {
+	verbose_log("Failed to set VLAN vif\n");
+	return 1;
+    }
+    if (IfMgrVifSetVlanId("if0", "vif0", 1234).execute(t) == false) {
+	verbose_log("Failed to set VLAN ID\n");
+	return 1;
+    }
 
     // Create an address on a non-existant interface and check it fails
     if (IfMgrIPv4Add("if0", "bad0", "252.17.17.17").execute(t) == true) {
@@ -202,8 +230,8 @@ test_main()
 	verbose_log("Failed to set loopback");
 	return 1;
     }
-    IPv6 oaddr("fe80::220:edff:fe5c:d0c7");
-    if (IfMgrIPv6SetEndpoint("if0", "vif0", v6a, oaddr).execute(t)
+    IPv6 endpoint_addr("fe80::220:edff:fe5c:d0c7");
+    if (IfMgrIPv6SetEndpoint("if0", "vif0", v6a, endpoint_addr).execute(t)
 	== false) {
 	verbose_log("Failed to set endpoint");
 	return 1;
