@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_main.cc,v 1.23 2007/02/16 22:46:05 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_main.cc,v 1.26 2008/07/23 05:10:41 pavlin Exp $"
 
 #include "finder_module.h"
 
@@ -207,6 +207,11 @@ finder_main(int argc, char* const argv[])
     } catch (const InvalidPort& i) {
 	fprintf(stderr, "%s: a finder may already be running.\n",
 		i.why().c_str());
+	exit(-1);
+    } catch (const InvalidAddress& i) {
+	fprintf(stderr, "Invalid finder server adddress: %s.\n",
+		i.why().c_str());
+	exit(-1);
     } catch (...) {
 	xorp_catch_standard_exceptions();
     }
