@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_tcp.cc,v 1.30 2007/02/16 22:46:06 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_tcp.cc,v 1.37 2008/07/23 05:10:42 pavlin Exp $"
 
 #include <functional>
 
@@ -173,7 +173,8 @@ FinderTcpBase::read_callback(AsyncFileOperator::Event	ev,
 	assert(buffer == &_input_buffer[0]);
 
 	// Finished reading data, notify call read_event.
-	read_event(0, buffer, buffer_bytes);
+	if (read_event(0, buffer, buffer_bytes) != true)
+	    return;
 	_reader.add_buffer(reinterpret_cast<uint8_t*>(&_isize), sizeof(_isize),
 			   callback(this, &FinderTcpBase::read_callback));
 	_reader.start();
@@ -296,6 +297,9 @@ FinderTcpListenerBase::FinderTcpListenerBase(EventLoop& e,
     if (!_lsock.is_valid()) {
 	xorp_throw(InvalidPort, comm_get_last_error_str());
     }
+    if (comm_listen(_lsock, COMM_LISTEN_DEFAULT_BACKLOG) != XORP_OK) {
+	xorp_throw(InvalidPort, comm_get_last_error_str());
+    }
 
     if (en)
 	set_enabled(en);
