@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_tcp_messenger.cc,v 1.29 2007/02/16 22:46:06 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_tcp_messenger.cc,v 1.34 2008/07/23 05:10:42 pavlin Exp $"
 
 #include "finder_module.h"
 
@@ -42,7 +42,7 @@ FinderTcpMessenger::~FinderTcpMessenger()
     drain_queue();
 }
 
-void
+bool
 FinderTcpMessenger::read_event(int	      errval,
 			       const uint8_t* data,
 			       uint32_t	      data_bytes)
@@ -53,7 +53,7 @@ FinderTcpMessenger::read_event(int	      errval,
 	 */
 	debug_msg("Got errval %d, data %p, data_bytes %u\n",
 		  errval, data, XORP_UINT_CAST(data_bytes));
-	return;
+	return true;
     }
 
     string s(data, data + data_bytes);
@@ -63,11 +63,11 @@ FinderTcpMessenger::read_event(int	      errval,
 	try {
 	    ParsedFinderXrlMessage fm(s.c_str());
 	    dispatch_xrl(fm.seqno(), fm.xrl());
-	    return;
+	    return true;
 	} catch (const WrongFinderMessageType&) {
 	    ParsedFinderXrlResponse fm(s.c_str());
 	    dispatch_xrl_response(fm.seqno(), fm.xrl_error(), fm.xrl_args());
-	    return;
+	    return true;
 	}
     } catch (const InvalidString& e) {
 	ex = e.str();
@@ -82,6 +82,7 @@ FinderTcpMessenger::read_event(int	      errval,
     }
     XLOG_ERROR("Got exception %s, closing connection", ex.c_str());
     close();
+    return false;
 }
 
 bool
@@ -221,7 +222,7 @@ FinderTcpListener::FinderTcpListener(EventLoop&		     e,
 				     IPv4		     interface,
 				     uint16_t		     port,
 				     bool		     en)
-    throw (InvalidPort)
+    throw (InvalidAddress, InvalidPort)
     : FinderTcpListenerBase(e, interface, port, en), _mm(mm), _cmds(cmds)
 {
 }
@@ -303,8 +304,8 @@ FinderTcpAutoConnector::FinderTcpAutoConnector(
 				uint32_t 		give_up_ms
 				)
     : FinderTcpConnector(e, *this, cmds, host, port),
-      _real_manager(real_manager), _connected(false), _enabled(en),
-      _once_active(false), _last_error(0), _consec_error(0)
+      _real_manager(real_manager), _connected(false), _connect_failed(false),
+      _enabled(en), _once_active(false), _last_error(0), _consec_error(0)
 {
     if (en) {
 	start_timer();
@@ -356,11 +357,19 @@ FinderTcpAutoConnector::connected() const
     return _connected;
 }
 
+bool
+FinderTcpAutoConnector::connect_failed() const
+{
+    return _connect_failed;
+}
+
 void
 FinderTcpAutoConnector::do_auto_connect()
 {
     XLOG_ASSERT(false == _connected);
 
+    _connect_failed = false;
+
     FinderTcpMessenger* fm;
     int r = connect(fm);
     if (r == 0) {
@@ -369,6 +378,7 @@ FinderTcpAutoConnector::do_auto_connect()
 	_connected = true;
     } else {
 	XLOG_ASSERT(fm == 0);
+	_connect_failed = true;
 	if (r != _last_error) {
 	    XLOG_ERROR("Failed to connect to %s/%u: %s",
 		       _host.str().c_str(), _port, strerror(r));
