@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,8 +12,9 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_xrl_target.cc,v 1.25 2007/02/16 22:46:06 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_xrl_target.cc,v 1.31 2008/07/23 05:10:42 pavlin Exp $"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/status_codes.h"
 #include "libxorp/random.h"
@@ -29,10 +30,10 @@ public:
     TraceFinder() {
 	_do_trace = !(getenv("FINDERTRACE") == 0);
     }
-    inline bool on() const { return _do_trace; }
+    bool on() const { return _do_trace; }
     operator bool() { return _do_trace; }
-    inline void set_context(const string& s) { _context = s; }
-    inline const string& context() const { return _context; }
+    void set_context(const string& s) { _context = s; }
+    const string& context() const { return _context; }
 protected:
     bool _do_trace;
     string _context;
@@ -40,15 +41,16 @@ protected:
 
 #define finder_trace_init(x...) 					      \
 do {									      \
-    if (finder_tracer.on()) finder_tracer.set_context(c_format(x));	      \
+    if (finder_tracer.on()) {						      \
+	finder_tracer.set_context(c_format(x));				      \
+    }									      \
 } while (0)
 
 #define finder_trace_result(x...)					      \
 do {									      \
     if (finder_tracer.on()) {						      \
 	string r = c_format(x);						      \
-	XLOG_INFO(c_format("%s -> %s\n",				      \
-		  finder_tracer.context().c_str(), r.c_str()).c_str());	      \
+	XLOG_INFO("%s -> %s", finder_tracer.context().c_str(), r.c_str());    \
     }									      \
 } while (0)
 
@@ -80,11 +82,11 @@ make_cookie()
     static uint32_t invoked = 0;
     static uint32_t hash_base;
     if (invoked == 0) {
-	srandom(((unsigned long)(getpid())) ^ ((unsigned long)(&hash_base)));
-	invoked = random() ^ random();
-	hash_base = random();
+	xorp_srandom(((unsigned long)(getpid())) ^ ((unsigned long)(&hash_base)));
+	invoked = xorp_random() ^ xorp_random();
+	hash_base = xorp_random();
     }
-    uint32_t r = random();
+    uint32_t r = xorp_random();
     invoked++;
     return c_format("%08x%08x", XORP_UINT_CAST(invoked),
 		    XORP_UINT_CAST(r ^ hash_base));
@@ -177,7 +179,7 @@ FinderXrlTarget::finder_0_2_set_finder_client_enabled(const string& tgt_name,
 						      const bool&   en)
 {
     finder_trace_init("set_finder_client_enabled(\"%s\", %s)",
-		      tgt_name.c_str(), (en) ? "true" : "false");
+		      tgt_name.c_str(), bool_c_str(en));
 
     if (_finder.active_messenger_represents_target(tgt_name)) {
 	_finder.set_target_enabled(tgt_name, en);
