@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder.cc,v 1.22 2007/02/16 22:46:07 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_finder.cc,v 1.26 2008/07/23 05:10:43 pavlin Exp $"
 
 #include "finder_module.h"
 
@@ -45,7 +45,7 @@ static const char *program_description  = "Test carrying and execution of "
 					  "Finder and Finder client";
 static const char *program_version_id   = "0.1";
 static const char *program_date         = "February, 2003";
-static const char *program_copyright    = "See file LICENSE.XORP";
+static const char *program_copyright    = "See file LICENSE";
 static const char *program_return_value = "0 on success, 1 if test error, 2 if internal error";
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -199,13 +199,24 @@ test_main(void)
 {
     EventLoop e;
 
-    IPv4 test_host(FinderConstants::FINDER_DEFAULT_HOST());
-    uint16_t test_port = 16666;
+    IPv4 init_test_host(FinderConstants::FINDER_DEFAULT_HOST());
+    uint16_t init_test_port = 16666;
+    IPv4 test_host = init_test_host;
+    uint16_t test_port = init_test_port;
+    
 
     //
     // Construct finder and messenger source for finder
     //
-    FinderServer* finder_box = new FinderServer(e, test_host, test_port);
+    FinderServer* finder_box = new FinderServer(e, init_test_host,
+						init_test_port);
+
+    //
+    // Set the finder's real host address and port which might have been
+    // overwritten by environmental variables.
+    //
+    test_host = finder_box->addr();
+    test_port = finder_box->port();
 
     //
     // Construct first finder client and messenger source for it.
