@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder_events.cc,v 1.24 2007/02/16 22:46:08 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_finder_events.cc,v 1.29 2008/07/23 05:10:43 pavlin Exp $"
 
 #include "finder_module.h"
 
@@ -53,7 +53,7 @@ static const char *program_description  = "Test Finder events are "
 					  "properly reported";
 static const char *program_version_id   = "0.1";
 static const char *program_date         = "May, 2003";
-static const char *program_copyright    = "See file LICENSE.XORP";
+static const char *program_copyright    = "See file LICENSE";
 static const char *program_return_value = "0 on success, 1 if test error, 2 if internal error";
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -276,7 +276,7 @@ public:
 	_feo.router()->finalize();
     }
 
-    inline FinderEventObserver& observer()		{ return _feo; }
+    FinderEventObserver& observer()	{ return _feo; }
 
     bool xrl_router_connected() const	{ return _feo.router()->connected(); }
     bool xrl_router_ready() const	{ return _feo.router()->ready(); }
@@ -302,14 +302,8 @@ public:
     {
 	_router.finalize();
     }
-    inline const string& instance_name() const
-    {
-	return _router.instance_name();
-    }
-    inline const string& class_name() const
-    {
-	return _router.class_name();
-    }
+    const string& instance_name() const { return _router.instance_name(); }
+    const string& class_name() const { return _router.class_name(); }
 private:
     XrlStdRouter _router;
 };
@@ -447,7 +441,7 @@ remove_any_xrl_target(list<AnXrlTarget*>* store)
     uint32_t n = store->size();
     XLOG_ASSERT(n > 0);
 
-    n = (uint32_t)random() % n;
+    n = (uint32_t)xorp_random() % n;
     list<AnXrlTarget*>::iterator i = store->begin();
     while (n > 0) {
 	i++;
