@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder_tcp.cc,v 1.22 2007/02/16 22:46:08 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_finder_tcp.cc,v 1.27 2008/07/23 05:10:43 pavlin Exp $"
 
 #include "finder_module.h"
 
@@ -39,7 +39,7 @@ static const char *program_name         = "test_finder_tcp";
 static const char *program_description  = "Test Finder TCP transport";
 static const char *program_version_id   = "0.1";
 static const char *program_date         = "January, 2003";
-static const char *program_copyright    = "See file LICENSE.XORP";
+static const char *program_copyright    = "See file LICENSE";
 static const char *program_return_value = "0 on success, 1 if test error, 2 if internal error";
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -68,13 +68,14 @@ public:
 	: FinderTcpBase(e, fd), _name(name), _reads(0), _writes(0)
     {}
     
-    void read_event(int		   errval,
+    bool read_event(int		   errval,
 		    const uint8_t* data,
 		    uint32_t	   data_bytes)
     {
 	verbose_log("- %s %d %p %u\n", _name, errval, data,
 		    XORP_UINT_CAST(data_bytes));
 	_reads++;
+	return true;
     }
 
     void write_event(int	    errval,
@@ -96,10 +97,10 @@ public:
 	write_data(reinterpret_cast<const uint8_t*>(_buf), strlen(_buf));
     }
     
-    inline uint32_t read_events() const { return _reads; }
-    inline uint32_t write_events() const { return _writes; }
+    uint32_t read_events() const { return _reads; }
+    uint32_t write_events() const { return _writes; }
 
-    inline bool can_write() const { return !_writer.running(); }
+    bool can_write() const { return !_writer.running(); }
 
 protected:
     const char*	_name;
