@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_lemming.cc,v 1.19 2007/02/16 22:46:08 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_lemming.cc,v 1.22 2008/07/23 05:10:43 pavlin Exp $"
 
 #include "ipc_module.h"
 
@@ -216,10 +216,8 @@ lemming_main()
 
     while (run && pfs) {
 	bool life = true;
-	XorpTimer snuffer = e.new_oneoff_after_ms((random() % 7777) + 1000,
-						  callback(&snuff_flag,
-							   &life));
-
+	XorpTimer snuffer = e.new_oneoff_after_ms(
+	    (xorp_random() % 7777) + 1000, callback(&snuff_flag, &life));
 	{
 	    Lemming l(e, life);
 	    while (life) e.run();
