@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_xrl.cc,v 1.13 2007/02/16 22:46:09 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_xrl.cc,v 1.16 2008/07/23 05:10:44 pavlin Exp $"
 
 // test_xrl: String Serialization Tests
 
@@ -94,6 +94,14 @@ run_test()
 	    XrlAtom("foo-string",
 		    string("ABCabc DEFdef 1234 !@#$%^&*(){}[]:;'\"<>"))
 	},
+	{
+	    "Xrl named int64",
+	    XrlAtom("foo-i64", int64_t(876543210123456789LL))
+	},
+	{
+	    "Xrl named uint32",
+	    XrlAtom("foo-u64", uint64_t(0xf00fdeadbabecafeULL))
+	},
     };
 
     uint32_t ntests = sizeof(tests) / sizeof(tests[0]);
