@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_xrl_args.cc,v 1.14 2007/02/16 22:46:09 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_xrl_args.cc,v 1.19 2008/07/23 05:10:44 pavlin Exp $"
 
 // test_xrl_args: String Serialization Tests
 
@@ -37,7 +37,7 @@ static const char *program_name         = "test_xrl_args";
 static const char *program_description  = "Test Xrl argument class";
 static const char *program_version_id   = "0.2";
 static const char *program_date         = "October, 2003";
-static const char *program_copyright    = "See file LICENSE.XORP";
+static const char *program_copyright    = "See file LICENSE";
 static const char *program_return_value = "0 on success, 1 if test error, "
 					  "2 if internal error";
 
@@ -159,7 +159,9 @@ run_serialization_test()
 	XrlAtom("a_mac_addr",	 EtherMac("00:ab:10:11:12:13")),
 	XrlAtom("string",	 string("hello world, kippers, yum")),
 	XrlAtom("binary_data",	 test_binary),
-	XrlAtom("a_list",	 test_list)
+	XrlAtom("a_list",	 test_list),
+	XrlAtom("integer64",	 int64_t(-1234567890123456789LL)),
+	XrlAtom("uinteger64",	 uint64_t(0xabadc0ffee123456ULL))
     };
     uint32_t n_test_args = sizeof(test_args) / sizeof(test_args[0]);
 
@@ -200,6 +202,8 @@ run_test()
     al.add_mac("a_named_mac", Mac("01:02:03:04:05:06"));
     al.add_string(0, "potentially hazardous null pointer named string object");
     al.add_string("bad_karma", "");
+    al.add_int64("a_named_int64", int64_t(-98765432101234LL));
+    al.add_uint64("a_named_uint64", uint64_t(123456789012345ULL));
 
     XrlAtomList xal;
     xal.append(XrlAtom("first", string("fooo")));
@@ -226,8 +230,8 @@ run_test()
 
     try {
 	al.get_uint32("only_uint");
-    } catch (const XrlArgs::XrlAtomNotFound &e) {
-	verbose_log("Atom not Found\n");
+    } catch (const XrlArgs::BadArgs &e) {
+	verbose_log("Error decoding the argument: %s\n", e.str().c_str());
 	return 1;
     } catch (...) {
 	verbose_log("Unanticipated exception.");
@@ -247,8 +251,10 @@ run_test()
 	al.get_string(0);
 	al.get_string("");
 	al.get_string("bad_karma");
-    } catch (XrlArgs::XrlAtomNotFound& e) {
-	verbose_log("Failed to find atom.\n");
+	al.get_int64("a_named_int64");
+	al.get_uint64("a_named_uint64");
+    } catch (XrlArgs::BadArgs& e) {
+	verbose_log("Error decoding the argument: %s\n", e.str().c_str());
 	return 1;
     }
     return 0;
