@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_xrl_atom.cc,v 1.14 2007/02/16 22:46:09 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_xrl_atom.cc,v 1.18 2008/07/23 05:10:44 pavlin Exp $"
 
 // test_xrl_atom: String Serialization Tests
 
@@ -218,7 +218,7 @@ test()
 		    tracef("Binary data size = %u\n", XORP_UINT_CAST(sz));
 		    vector<uint8_t> t(sz);
 		    for (size_t i = 0; i < sz; i++) {
-			t[i] = random();
+			t[i] = xorp_random();
 		    }
 		    assert(t.size() == sz);
 		    XrlAtom a("binary_data", t);
@@ -227,6 +227,14 @@ test()
 		}
 	    }
 	    break;
+	case xrlatom_int64:
+	    test_atom(XrlAtom("test_int64_value",
+		int64_t(-1234567890123456789LL)));
+	    break;
+	case xrlatom_uint64:
+	    test_atom(XrlAtom("test_uint64_value",
+		uint64_t(0xabadc0ffee123456ULL)));
+	    break;
 	}
     }
 }
