@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/xrl_dispatcher.cc,v 1.10 2007/02/16 22:46:10 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_dispatcher.cc,v 1.13 2008/07/23 05:10:45 pavlin Exp $"
 
 #include "ipc_module.h"
 
@@ -29,7 +29,7 @@ public:
     TraceXrl() {
 	_do_trace = !(getenv("XRLDISPATCHTRACE") == 0);
     }
-    inline bool on() const { return _do_trace; }
+    bool on() const { return _do_trace; }
     operator bool() { return _do_trace; }
 
 protected:
