@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/xrl_parser.cc,v 1.13 2007/02/16 22:46:11 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_parser.cc,v 1.16 2008/07/23 05:10:45 pavlin Exp $"
 
 #include <stdio.h>
 
@@ -129,7 +129,9 @@ static inline size_t
 skip_xrl_plain_chars(const string& input, string::const_iterator& sci)
 {
     string::const_iterator start = sci;
-    for (; sci != input.end() && isxrlplain(*sci); sci++);
+    for ( ; sci != input.end() && isxrlplain(*sci); sci++) {
+	// XXX: Empty body
+    }
     return sci - start;
 }
 
