@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -14,7 +14,7 @@
 
 //#define DEBUG_LOGGING
 
-#ident "$XORP: xorp/libxipc/xrl_pf_stcp.cc,v 1.53 2007/02/16 22:46:12 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_pf_stcp.cc,v 1.62 2008/07/23 05:10:46 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -201,10 +201,11 @@ STCPRequestHandler::read_event(BufferedAsyncReader*		/* source */,
 	    buffer += sph.frame_bytes();
 	    buffer_bytes -= sph.frame_bytes();
 	} else {
-	    if (sph.frame_bytes() > _reader.reserve_bytes()) {
-		_reader.set_reserve_bytes(sph.frame_bytes());
+	    size_t frame_bytes = sph.frame_bytes();
+	    if (frame_bytes > _reader.reserve_bytes()) {
+		_reader.set_reserve_bytes(frame_bytes);
 	    }
-	    _reader.set_trigger_bytes(sph.frame_bytes());
+	    _reader.set_trigger_bytes(frame_bytes);
 	    return;
 	}
     }
@@ -363,6 +364,10 @@ XrlPFSTCPListener::XrlPFSTCPListener(EventLoop&	    e,
 	xorp_throw(XrlPFConstructorError,
 		   comm_get_last_error_str());
     }
+    if (comm_listen(_sock, COMM_LISTEN_DEFAULT_BACKLOG) != XORP_OK) {
+	xorp_throw(XrlPFConstructorError,
+		   comm_get_last_error_str());
+    }
 
     string addr;
     if (get_local_socket_details(_sock, addr, port) == false) {
@@ -479,14 +484,13 @@ public:
 	sph.initialize(_sn, STCP_PT_HELO, XrlError::OKAY(), 0);
     }
 
-    inline bool		    has_seqno(uint32_t n) const { return _sn == n; }
-    inline XrlPFSTCPSender* parent() const		{ return _p; }
-    inline uint32_t	    seqno() const		{ return _sn; }
-    inline vector<uint8_t>& buffer() 			{ return _b; }
-    inline Callback&	    cb() 			{ return _cb; }
+    bool		has_seqno(uint32_t n) const { return _sn == n; }
+    XrlPFSTCPSender*	parent() const		{ return _p; }
+    uint32_t		seqno() const		{ return _sn; }
+    vector<uint8_t>&	buffer() 		{ return _b; }
+    Callback&		cb() 			{ return _cb; }
 
-    inline bool is_keepalive()
-    {
+    bool is_keepalive() {
 	if (_b.size() < STCPPacketHeader::header_size())
 	    return false;
 	const STCPPacketHeader sph(&_b[0]);
@@ -920,6 +924,15 @@ XrlPFSTCPSender::defer_keepalives()
 bool
 XrlPFSTCPSender::send_keepalive()
 {
+    TimeVal now;
+
+    _eventloop.current_time(now);
+    if ((now - _keepalive_last_fired).to_ms() <
+	 static_cast<int32_t>(_keepalive_ms)) {
+	// This keepalive timer has fired too soon. Rate-limit it.
+	debug_msg("Dropping keepalive due to rate-limiting\n");
+	return true;
+    }
     if (_keepalive_sent == true) {
 	// There's an unack'ed keepalive message.
 	die("Keepalive timeout");
@@ -929,5 +942,9 @@ XrlPFSTCPSender::send_keepalive()
     _keepalive_sent = true;
     send_request(new RequestState(this, _current_seqno++));
 
+    // Record the relative timestamp of this keepalive timer firing
+    // so that we may rate-limit it above.
+    _keepalive_last_fired = now;
+
     return true;
 }
