@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/xrl_router.cc,v 1.53 2007/02/16 22:46:13 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_router.cc,v 1.58 2008/07/23 05:10:46 pavlin Exp $"
 
 #include "xrl_module.h"
 #include "libxorp/debug.h"
@@ -46,7 +46,7 @@ public:
     TraceXrl() {
 	_do_trace = !(getenv("XRLTRACE") == 0);
     }
-    inline bool on() const { return _do_trace; }
+    bool on() const { return _do_trace; }
     operator bool() { return _do_trace; }
 
 protected:
@@ -74,8 +74,8 @@ public:
 	: _xrl(x), _xcb(xcb)
     {}
 
-    inline const Xrl& xrl() const		{ return _xrl; }
-    inline XrlCallback& cb()			{ return _xcb; }
+    const Xrl& xrl() const		{ return _xrl; }
+    XrlCallback& cb()			{ return _xcb; }
 
 protected:
     Xrl				_xrl;
@@ -289,6 +289,12 @@ XrlRouter::connected() const
     return _fc && _fc->connected();
 }
 
+bool
+XrlRouter::connect_failed() const
+{
+    return _fac && _fac->connect_failed();
+}
+
 bool
 XrlRouter::ready() const
 {
@@ -635,7 +641,7 @@ wait_until_xrl_router_is_ready(EventLoop& eventloop, XrlRouter& xrl_router)
 	xlog_stop();
 	xlog_exit();
     } else {
-	fprintf(stderr, msg);
+	fprintf(stderr, "%s", msg);
     }
     exit(-1);
 }
