@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/asyncio.cc,v 1.31 2007/02/16 22:46:15 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/asyncio.cc,v 1.42 2008/07/23 05:10:49 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -85,11 +85,22 @@ AsyncFileOperator::~AsyncFileOperator()
 // ----------------------------------------------------------------------------
 // AsyncFileReader read method and entry hook
 
+AsyncFileReader::AsyncFileReader(EventLoop& e, XorpFd fd, int priority)
+    : AsyncFileOperator(e, fd, priority)
+{
+}
+
+AsyncFileReader::~AsyncFileReader()
+{
+    stop();
+    delete_pointers_list(_buffers);
+}
+
 void
 AsyncFileReader::add_buffer(uint8_t* b, size_t b_bytes, const Callback& cb)
 {
     assert(b_bytes != 0);
-    _buffers.push_back(BufferInfo(b, b_bytes, cb));
+    _buffers.push_back(new BufferInfo(b, b_bytes, cb));
 }
 
 void
@@ -99,7 +110,7 @@ AsyncFileReader::add_buffer_with_offset(uint8_t*	b,
 					const Callback&	cb) 
 {
     assert(off < b_bytes);
-    _buffers.push_back(BufferInfo(b, b_bytes, off, cb));
+    _buffers.push_back(new BufferInfo(b, b_bytes, off, cb));
 }
 
 #ifdef HOST_OS_WINDOWS
@@ -111,8 +122,8 @@ AsyncFileReader::disconnect(XorpFd fd, IoEventType type)
     assert(fd.is_valid()); 
     assert(fd.is_socket()); 
     debug_msg("IOT_DISCONNECT close detected (reader side)\n");
-    BufferInfo& head = _buffers.front();
-    head.dispatch_callback(END_OF_FILE);
+    BufferInfo* head = _buffers.front();
+    head->dispatch_callback(END_OF_FILE);
 }
 #endif // HOST_OS_WINDOWS
 
@@ -129,21 +140,27 @@ AsyncFileReader::read(XorpFd fd, IoEventType type)
 
 #if 0
     // XXX: comm_sock_is_connected() is cross-reference to libcomm
-    if (_fd.is_socket() && (XORP_OK != comm_sock_is_connected(_fd))) {
-	debug_msg("Warning: socket %p may have been closed\n", (HANDLE)_fd);
+    int is_connected = 0;
+    if (_fd.is_socket()) {
+	if ((comm_sock_is_connected(_fd, &is_connected) != XORP_OK)
+	    || (is_connected == 0)) {
+	    debug_msg("Warning: socket %p may have been closed\n",
+		      (HANDLE)_fd);
+	}
     }
-#endif
+#endif // 0
+
     debug_msg("Buffer count %u\n", XORP_UINT_CAST(_buffers.size()));
 
-    BufferInfo& head = _buffers.front();
+    BufferInfo* head = _buffers.front();
     ssize_t done = 0;
 #ifdef HOST_OS_WINDOWS
     BOOL result = FALSE;
 
     switch (fd.type()) {
     case XorpFd::FDTYPE_SOCKET:
-	done = recv((SOCKET)_fd, (char *)(head._buffer + head._offset),
-		    head._buffer_bytes - head._offset, 0);
+	done = recv((SOCKET)_fd, (char *)(head->buffer() + head->offset()),
+		    head->buffer_bytes() - head->offset(), 0);
 	if (done == SOCKET_ERROR) {
 	    _last_error = WSAGetLastError();
 	    done = -1;
@@ -157,21 +174,21 @@ AsyncFileReader::read(XorpFd fd, IoEventType type)
 
     case XorpFd::FDTYPE_PIPE:
 	// XXX: Return values need review.
-	done = win_pipe_read(_fd, head._buffer + head._offset,
-			     head._buffer_bytes - head._offset);
+	done = win_pipe_read(_fd, head->buffer() + head->offset(),
+			     head->buffer_bytes() - head->offset());
 	_last_error = GetLastError();
 	break;
 
     case XorpFd::FDTYPE_CONSOLE:
 	// XXX: Return values need review.
-	done = win_con_read(_fd, head._buffer + head._offset,
-			     head._buffer_bytes - head._offset);
+	done = win_con_read(_fd, head->buffer() + head->offset(),
+			     head->buffer_bytes() - head->offset());
 	_last_error = GetLastError();
 	break;
 
     case XorpFd::FDTYPE_FILE:
-	result = ReadFile(_fd, (LPVOID)(head._buffer + head._offset),
-			       (DWORD)(head._buffer_bytes - head._offset),
+	result = ReadFile(_fd, (LPVOID)(head->buffer() + head->offset()),
+			       (DWORD)(head->buffer_bytes() - head->offset()),
 			       (LPDWORD)&done, NULL);
 	if (result == FALSE) {
 	    _last_error = GetLastError();
@@ -186,8 +203,8 @@ AsyncFileReader::read(XorpFd fd, IoEventType type)
 #else // ! HOST_OS_WINDOWS
     errno = 0;
     _last_error = 0;
-    done = ::read(_fd, head._buffer + head._offset,
-		  head._buffer_bytes - head._offset);
+    done = ::read(_fd, head->buffer() + head->offset(),
+		  head->buffer_bytes() - head->offset());
     if (done < 0)
 	_last_error = errno;
     errno = 0;
@@ -229,27 +246,27 @@ AsyncFileReader::complete_transfer(int err, ssize_t done)
     // not reference any object state after callback.
 
     if (done > 0) {
-	BufferInfo& head = _buffers.front();
-	head._offset += done;
-	if (head._offset == head._buffer_bytes) {
-	    BufferInfo copy = head; 		// copy head
-	    _buffers.erase(_buffers.begin());	// remove head
+	BufferInfo* head = _buffers.front();
+	head->incr_offset(done);
+	if (head->offset() == head->buffer_bytes()) {
+	    _buffers.pop_front();
 	    if (_buffers.empty()) {
 		stop();
 	    }
-	    copy.dispatch_callback(DATA);
+	    head->dispatch_callback(DATA);
+	    delete head;
 	} else {
-	    head.dispatch_callback(DATA);
+	    head->dispatch_callback(DATA);
 	}
 	return;
     }
 
-    BufferInfo& head = _buffers.front();
+    BufferInfo* head = _buffers.front();
     if (err != 0 || done < 0) {
 	stop();
-	head.dispatch_callback(OS_ERROR);
+	head->dispatch_callback(OS_ERROR);
     } else {
-	head.dispatch_callback(END_OF_FILE);
+	head->dispatch_callback(END_OF_FILE);
     }
 }
 
@@ -266,8 +283,13 @@ AsyncFileReader::start()
 
 #if 0
     // XXX: comm_sock_is_connected() is cross-reference to libcomm
-    if (_fd.is_socket() && (XORP_OK != comm_sock_is_connected(_fd))) {
-	debug_msg("Warning: socket %p may have been closed\n", (HANDLE)_fd);
+    int is_connected = 0;
+    if (_fd.is_socket()) {
+	if ((comm_sock_is_connected(_fd, &is_connected) != XORP_OK)
+	    || (is_connected == 0)) {
+	    debug_msg("Warning: socket %p may have been closed\n",
+		      (HANDLE)_fd);
+	}
     }
 #endif // 0
 
@@ -329,11 +351,10 @@ AsyncFileReader::flush_buffers()
 {
     stop();
     while (_buffers.empty() == false) {
-	// Copy out buffer so flush buffers can be called re-entrantly (even
-	// if we happen to think this is bad coding style :-).
-	BufferInfo head = _buffers.front();
-	_buffers.erase(_buffers.begin());
-	head.dispatch_callback(FLUSHING);
+	BufferInfo* head = _buffers.front();
+	_buffers.pop_front();
+	head->dispatch_callback(FLUSHING);
+	delete head;
     }
 }
 
@@ -362,6 +383,7 @@ AsyncFileWriter::~AsyncFileWriter()
     stop();
 
     delete[] _iov;
+    delete_pointers_list(_buffers);
 }
 
 void
@@ -370,7 +392,25 @@ AsyncFileWriter::add_buffer(const uint8_t*	b,
 			    const Callback&	cb)
 {
     assert(b_bytes != 0);
-    _buffers.push_back(BufferInfo(b, b_bytes, cb));
+    _buffers.push_back(new BufferInfo(b, b_bytes, cb));
+#ifdef EDGE_TRIGGERED_WRITES
+    if (_running == true) {
+	_deferred_io_task = _eventloop.new_oneoff_task(
+	    callback(this, &AsyncFileWriter::write, _fd, IOT_WRITE));
+	XLOG_ASSERT(_deferred_io_task.scheduled());
+    }
+#endif // EDGE_TRIGGERED_WRITES
+}
+
+void
+AsyncFileWriter::add_buffer_sendto(const uint8_t*	b,
+				   size_t		b_bytes,
+				   const IPvX&		dst_addr,
+				   uint16_t		dst_port,
+				   const Callback&	cb)
+{
+    assert(b_bytes != 0);
+    _buffers.push_back(new BufferInfo(b, b_bytes, dst_addr, dst_port, cb));
 #ifdef EDGE_TRIGGERED_WRITES
     if (_running == true) {
 	_deferred_io_task = _eventloop.new_oneoff_task(
@@ -387,7 +427,39 @@ AsyncFileWriter::add_buffer_with_offset(const uint8_t*	b,
 					const Callback&	cb)
 {
     assert(off < b_bytes);
-    _buffers.push_back(BufferInfo(b, b_bytes, off, cb));
+    _buffers.push_back(new BufferInfo(b, b_bytes, off, cb));
+#ifdef EDGE_TRIGGERED_WRITES
+    if (_running == true) {
+	_deferred_io_task = _eventloop.new_oneoff_task(
+	    callback(this, &AsyncFileWriter::write, _fd, IOT_WRITE));
+	XLOG_ASSERT(_deferred_io_task.scheduled());
+    }
+#endif // EDGE_TRIGGERED_WRITES
+}
+
+void
+AsyncFileWriter::add_data(const vector<uint8_t>&	data,
+			  const Callback&		cb)
+{
+    assert(data.size() != 0);
+    _buffers.push_back(new BufferInfo(data, cb));
+#ifdef EDGE_TRIGGERED_WRITES
+    if (_running == true) {
+	_deferred_io_task = _eventloop.new_oneoff_task(
+	    callback(this, &AsyncFileWriter::write, _fd, IOT_WRITE));
+	XLOG_ASSERT(_deferred_io_task.scheduled());
+    }
+#endif // EDGE_TRIGGERED_WRITES
+}
+
+void
+AsyncFileWriter::add_data_sendto(const vector<uint8_t>&	data,
+				 const IPvX&		dst_addr,
+				 uint16_t		dst_port,
+				 const Callback&	cb)
+{
+    assert(data.size() != 0);
+    _buffers.push_back(new BufferInfo(data, dst_addr, dst_port, cb));
 #ifdef EDGE_TRIGGERED_WRITES
     if (_running == true) {
 	_deferred_io_task = _eventloop.new_oneoff_task(
@@ -421,8 +493,8 @@ AsyncFileWriter::disconnect(XorpFd fd, IoEventType type)
     assert(fd.is_socket()); 
     debug_msg("IOT_DISCONNECT close detected (writer side)\n");
 #if 0
-    BufferInfo& head = _buffers.front();
-    head.dispatch_callback(END_OF_FILE);
+    BufferInfo* head = _buffers.front();
+    head->dispatch_callback(END_OF_FILE);
 #endif // 0
 }
 #endif // HOST_OS_WINDOWS
@@ -430,14 +502,37 @@ AsyncFileWriter::disconnect(XorpFd fd, IoEventType type)
 void
 AsyncFileWriter::write(XorpFd fd, IoEventType type)
 {
+    bool is_sendto = false;
+    IPvX dst_addr;
+    uint16_t dst_port = 0;
+    uint32_t iov_cnt = 0;
+    size_t total_bytes = 0;
+    ssize_t done = 0;
+    int flags = 0;
+    bool mod_signals = true;
+#ifndef HOST_OS_WINDOWS
+    sig_t saved_sigpipe = SIG_ERR;
+#endif
+
+#ifdef MSG_NOSIGNAL
+    flags |= MSG_NOSIGNAL;
+    mod_signals = false;
+#endif
+
 #ifdef EDGE_TRIGGERED_WRITES
     if (_running == false)
 	return;
 #endif
+
 #if 0
     // XXX: comm_sock_is_connected() is cross-reference to libcomm
-    if (_fd.is_socket() && (XORP_OK != comm_sock_is_connected(_fd))) {
-	debug_msg("Warning: socket %p may have been closed\n", (HANDLE)_fd);
+    int is_connected = 0;
+    if (_fd.is_socket()) {
+	if ((comm_sock_is_connected(_fd, &is_connected) != XORP_OK)
+	    || (is_connected == 0)) {
+	    debug_msg("Warning: socket %p may have been closed\n",
+		      (HANDLE)_fd);
+	}
     }
 #endif // 0
 
@@ -445,65 +540,154 @@ AsyncFileWriter::write(XorpFd fd, IoEventType type)
     assert(fd == _fd);
     assert(_buffers.empty() == false);
 
-    // Coalesce buffers into a group
-    uint32_t iov_cnt = 0;
-    size_t   total_bytes = 0;
-    ssize_t  done = 0;
-
-    list<BufferInfo>::iterator i = _buffers.begin();
-
+    //
+    // Group together a number of buffers.
+    // If the buffer is sendto()-type, then send that buffer on its own.
+    //
+    list<BufferInfo *>::const_iterator i = _buffers.begin();
     while (i != _buffers.end()) {
-	BufferInfo bi = *i;
+	const BufferInfo* bi = *i;
+	is_sendto = bi->is_sendto();
 
-	uint8_t* u = const_cast<uint8_t*>(bi._buffer + bi._offset);
-	size_t   u_bytes = bi._buffer_bytes - bi._offset;
+	if (is_sendto && (iov_cnt > 0)) {
+	    // XXX: Send first all buffers before this sendto()-type
+	    break;
+	}
+
+	uint8_t* u = const_cast<uint8_t*>(bi->buffer() + bi->offset());
+	size_t   u_bytes = bi->buffer_bytes() - bi->offset();
 	iov_place(_iov[iov_cnt].iov_base, _iov[iov_cnt].iov_len, u, u_bytes);
 
 	total_bytes += u_bytes;
 	assert(total_bytes != 0);
 	iov_cnt++;
+	if (is_sendto) {
+	    dst_addr = bi->dst_addr();
+	    dst_port = bi->dst_port();
+	    break;
+	}
 	if (iov_cnt == _coalesce)
 	    break;
 	++i;
     }
 
+    if (is_sendto) {
+	//
+	// Use sendto(2) to send the data from the first buffer only
+	//
+	XLOG_ASSERT(! dst_addr.is_zero());
+
+	if (mod_signals) {
+#ifndef HOST_OS_WINDOWS
+	    saved_sigpipe = signal(SIGPIPE, SIG_IGN);
+#endif
+	}
+
+	switch (dst_addr.af()) {
+	case AF_INET:
+	{
+	    struct sockaddr_in sin;
+
+	    dst_addr.copy_out(sin);
+	    sin.sin_port = htons(dst_port);
+
+	    done = ::sendto(_fd, XORP_CONST_BUF_CAST(_iov[0].iov_base),
+			    _iov[0].iov_len,
+			    flags,
+			    reinterpret_cast<const sockaddr*>(&sin),
+			    sizeof(sin));
+	    break;
+	}
+#ifdef HAVE_IPV6
+	case AF_INET6:
+	{
+	    struct sockaddr_in6 sin6;
+
+	    dst_addr.copy_out(sin6);
+	    sin6.sin6_port = htons(dst_port);
+
+	    done = ::sendto(_fd, XORP_CONST_BUF_CAST(_iov[0].iov_base),
+			    _iov[0].iov_len,
+			    flags,
+			    reinterpret_cast<const sockaddr*>(&sin6),
+			    sizeof(sin6));
+	    break;
+	}
+#endif // HAVE_IPV6
+	default:
+	    XLOG_ERROR("Address family %d is not supported", dst_addr.af());
+	    done = _iov[0].iov_len;	// XXX: Pretend that the data was sent
+	    break;
+	}
+
+	if (done < 0) {
 #ifdef HOST_OS_WINDOWS
-    if (fd.is_socket()) {
-	// Socket handles take non-blocking writes.
-	// WSASend() approximates writev().
-	int result = WSASend((SOCKET)_fd, (LPWSABUF)_iov, iov_cnt,
-			     (LPDWORD)&done, 0, NULL, NULL);
-	_last_error = (result == SOCKET_ERROR) ? WSAGetLastError() : 0;
+	    _last_error = WSAGetLastError();
+#else
+	    _last_error = errno;
+#endif
+	}
+
+	if (mod_signals) {
+#ifndef HOST_OS_WINDOWS
+	    signal(SIGPIPE, saved_sigpipe);
+#endif
+	}
+
+    } else {
+	//
+	// Write the data to the socket/file descriptor
+	//
+#ifdef HOST_OS_WINDOWS
+	if (fd.is_socket()) {
+	    // Socket handles take non-blocking writes.
+	    // WSASend() approximates writev().
+	    int result = WSASend((SOCKET)_fd, (LPWSABUF)_iov, iov_cnt,
+				 (LPDWORD)&done, 0, NULL, NULL);
+	    _last_error = (result == SOCKET_ERROR) ? WSAGetLastError() : 0;
 #if 1
-	if (_last_error != 0)
-	    debug_msg("writer: winsock error %d\n", _last_error);
+	    if (_last_error != 0)
+		debug_msg("writer: winsock error %d\n", _last_error);
 #endif // 1
-    } else {
-	// Non-socket handles take blocking writes.
-	// There is no writev() equivalent, so emulate it.
-	BOOL result;
-	DWORD done2;
-	for (uint32_t j = 0; j < iov_cnt; j++) {
-	    result = WriteFile(_fd, (LPVOID)_iov[j].iov_base,
-			       (DWORD)_iov[j].iov_len, (LPDWORD)&done2, NULL);
-	    done += done2;
-	    if (result == FALSE)
-		 break;
+	} else {
+	    // Non-socket handles take blocking writes.
+	    // There is no writev() equivalent, so emulate it.
+	    BOOL result;
+	    DWORD done2;
+	    for (uint32_t j = 0; j < iov_cnt; j++) {
+		result = WriteFile(_fd, (LPVOID)_iov[j].iov_base,
+				   (DWORD)_iov[j].iov_len, (LPDWORD)&done2,
+				   NULL);
+		done += done2;
+		if (result == FALSE)
+		    break;
+	    }
+	    _last_error = (result == FALSE) ? GetLastError() : 0;
 	}
-	_last_error = (result == FALSE) ? GetLastError() : 0;
-    }	
 #else // ! HOST_OS_WINDOWS
-    sig_t saved_sigpipe = signal(SIGPIPE, SIG_IGN);
 
-    errno = 0;
-    _last_error = 0;
-    done = ::writev(_fd, _iov, (int)iov_cnt);
-    if (done < 0)
-	_last_error = errno;
-    errno = 0;
+	errno = 0;
+	_last_error = 0;
 
-    signal(SIGPIPE, saved_sigpipe);
+	if ((iov_cnt == 1) && (! mod_signals)) {
+	    //
+	    // No need for coalesce, so use send(2). This saves us
+	    // two sigaction calls since we can pass the MSG_NOSIGNAL flag.
+	    //
+	    done = ::send(_fd, XORP_CONST_BUF_CAST(_iov[0].iov_base),
+			  _iov[0].iov_len, flags);
+	    if (done < 0)
+		_last_error = errno;
+	} else {
+	    saved_sigpipe = signal(SIGPIPE, SIG_IGN);
+	    done = ::writev(_fd, _iov, (int)iov_cnt);
+	    if (done < 0)
+		_last_error = errno;
+	    signal(SIGPIPE, saved_sigpipe);
+	}
+	errno = 0;
 #endif // ! HOST_OS_WINDOWS
+    }
 
     debug_msg("Wrote %d of %u bytes\n",
 	      XORP_INT_CAST(done), XORP_UINT_CAST(total_bytes));
@@ -532,8 +716,8 @@ AsyncFileWriter::complete_transfer(ssize_t sdone)
     if (sdone < 0) {
 	XLOG_ERROR("Write error %d\n", _last_error);
 	stop();
-	BufferInfo& head = _buffers.front();
-	head.dispatch_callback(OS_ERROR);
+	BufferInfo* head = _buffers.front();
+	head->dispatch_callback(OS_ERROR);
 	return;
     }
 
@@ -558,20 +742,19 @@ AsyncFileWriter::complete_transfer(ssize_t sdone)
 	assert(notified <= done);
 	assert(_buffers.empty() == false);
 
-	BufferInfo& head = _buffers.front();
-	assert(head._buffer_bytes >= head._offset);
+	BufferInfo* head = _buffers.front();
+	assert(head->buffer_bytes() >= head->offset());
 
-	size_t bytes_needed = head._buffer_bytes - head._offset;
+	size_t bytes_needed = head->buffer_bytes() - head->offset();
 
 	if (done - notified >= bytes_needed) {
 	    //
 	    // All data in this buffer has been written
 	    //
-	    head._offset += bytes_needed;
-	    assert(head._offset == head._buffer_bytes);
+	    head->incr_offset(bytes_needed);
+	    assert(head->offset() == head->buffer_bytes());
 
-	    // Copy, then detach head buffer and update state
-	    BufferInfo copy = head;
+	    // Detach head buffer and update state
 	    _buffers.pop_front();
 	    if (_buffers.empty()) {
 		stop();
@@ -579,7 +762,8 @@ AsyncFileWriter::complete_transfer(ssize_t sdone)
 
 	    assert(stack_token.is_only() == false);
 
-	    copy.dispatch_callback(DATA);
+	    head->dispatch_callback(DATA);
+	    delete head;
 	    if (stack_token.is_only() == true) {
 		// "this" instance of AsyncFileWriter was deleted by the
 		// calback, return immediately.
@@ -591,8 +775,8 @@ AsyncFileWriter::complete_transfer(ssize_t sdone)
 	    //
 	    // Not enough data has been written
 	    //
-	    head._offset += (done - notified);
-	    assert(head._offset < head._buffer_bytes);
+	    head->incr_offset(done - notified);
+	    assert(head->offset() < head->buffer_bytes());
 
 	    return;
 	}
@@ -612,8 +796,13 @@ AsyncFileWriter::start()
 
 #if 0
     // XXX: comm_sock_is_connected() is cross-reference to libcomm
-    if (_fd.is_socket() && (XORP_OK != comm_sock_is_connected(_fd))) {
-	debug_msg("Warning: socket %p may have been closed\n", (HANDLE)_fd);
+    int is_connected = 0;
+    if (_fd.is_socket()) {
+	if ((comm_sock_is_connected(_fd, &is_connected) != XORP_OK)
+	    || (is_connected == 0)) {
+	    debug_msg("Warning: socket %p may have been closed\n",
+		      (HANDLE)_fd);
+	}
     }
 #endif // 0
 
@@ -682,10 +871,9 @@ AsyncFileWriter::flush_buffers()
 {
     stop();
     while (_buffers.empty() == false) {
-	// Copy out buffer so flush buffers can be called re-entrantly (even
-	// if we happen to think this is bad coding style :-).
-	BufferInfo head = _buffers.front();
-	_buffers.erase(_buffers.begin());
-	head.dispatch_callback(FLUSHING);
+	BufferInfo* head = _buffers.front();
+	_buffers.pop_front();
+	head->dispatch_callback(FLUSHING);
+	delete head;
     }
 }
