@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
-/* Copyright (c) 2001-2007 International Computer Science Institute
+/* Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libxorp/debug.c,v 1.10 2007/02/16 22:46:18 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/debug.c,v 1.13 2008/07/23 05:10:51 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 #include "libxorp/debug.h"
@@ -57,9 +57,10 @@ _xdebug_preamble(const char*	file,
      * arguments to emacs, vi, vim, nedit, etc, but not ed :-( 
      */
     if (func) {
-	sprintf(sbuf, "[ %d %+5d %s %s ] ", spid, line, file, func);
+	snprintf(sbuf, sbuf_bytes, "[ %d %+5d %s %s ] ", spid, line, file,
+		 func);
     } else {
-	sprintf(sbuf, "[ %d %+5d %s ] ", spid, line, file);
+	snprintf(sbuf, sbuf_bytes, "[ %d %+5d %s ] ", spid, line, file);
     }
     return sbuf;
 }
