@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/eventloop.cc,v 1.22 2007/03/08 00:03:55 atanu Exp $"
+#ident "$XORP: xorp/libxorp/eventloop.cc,v 1.27 2008/07/23 05:10:51 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -26,16 +26,13 @@
 
 //
 // Number of EventLoop instances.
-// XXX: Warning: static instance. Not suitable for shared library use.
 //
-static int instance_count = 0;
+int eventloop_instance_count;
 
 //
 // Last call to EventLoop::run.  0 is a special value that indicates
 // EventLoop::run has not been called.
 //
-// XXX: Warning: static instance. Not suitable for shared library use.
-//
 static time_t last_ev_run;
 
 EventLoop::EventLoop()
@@ -46,16 +43,21 @@ EventLoop::EventLoop()
       _selector_list(_clock)
 #endif
 {
-    instance_count++;
-    XLOG_ASSERT(instance_count == 1);
+    XLOG_ASSERT(eventloop_instance_count == 0);
+    eventloop_instance_count++;
     last_ev_run = 0;
 }
 
 EventLoop::~EventLoop()
 {
-    instance_count--;
-    XLOG_ASSERT(instance_count == 0);
+    eventloop_instance_count--;
+    XLOG_ASSERT(eventloop_instance_count == 0);
+    //
+    // XXX: The _clock pointer is invalidated for other EventLoop fields
+    // that might be using it.
+    //
     delete _clock;
+    _clock = NULL;
 }
 
 void
@@ -116,9 +118,9 @@ EventLoop::run()
 
 	// the most important thing to run next is a selector
 #ifdef HOST_OS_WINDOWS
-	_win_dispatcher.wait_and_dispatch(&t);
+	_win_dispatcher.wait_and_dispatch(t);
 #else
-	_selector_list.wait_and_dispatch(&t);
+	_selector_list.wait_and_dispatch(t);
 #endif
 
     } else if (task_priority != XorpTask::PRIORITY_INFINITY) {
@@ -130,9 +132,9 @@ EventLoop::run()
 	// there's nothing immediate to run, so go to sleep until the
 	// next selector or timer goes off
 #ifdef HOST_OS_WINDOWS
-	_win_dispatcher.wait_and_dispatch(&t);
+	_win_dispatcher.wait_and_dispatch(t);
 #else
-	_selector_list.wait_and_dispatch(&t);
+	_selector_list.wait_and_dispatch(t);
 #endif
     }
 
