@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
 // Portions of this code originally derived from:
 // 	FreeBSD dummynet code, (C) 2001 Luigi Rizzo.
 
-#ident "$XORP: xorp/libxorp/heap.cc,v 1.19 2007/02/16 22:46:19 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/heap.cc,v 1.22 2008/07/23 05:10:51 pavlin Exp $"
 
 #include "libxorp_module.h"
 #include "libxorp/xorp.h"
@@ -258,12 +258,10 @@ Heap::Heap(bool intrude)
     debug_msg("++ constructor for 0x%p\n", this);
 }
 
-// cleanup the heap and free data structure
 Heap::~Heap()
 {
-    if (_size >0)
+    if (_p != NULL)
         delete[] _p;
-    memset(this, 0, sizeof(*this));
 }
 
 void
