@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/ipv4.cc,v 1.29 2007/02/16 22:46:19 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/ipv4.cc,v 1.34 2008/07/23 05:10:52 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -45,8 +45,17 @@ IPv4::IPv4(const sockaddr& sa) throw (InvalidFamily)
 {
     if (sa.sa_family != AF_INET)
 	xorp_throw(InvalidFamily, sa.sa_family);
-    const sockaddr_in& sin = reinterpret_cast<const sockaddr_in&>(sa);
-    _addr = sin.sin_addr.s_addr;
+    const sockaddr_in* sin = sockaddr2sockaddr_in(&sa);
+    _addr = sin->sin_addr.s_addr;
+}
+
+IPv4::IPv4(const sockaddr_storage& ss) throw (InvalidFamily)
+{
+    if (ss.ss_family != AF_INET)
+	xorp_throw(InvalidFamily, ss.ss_family);
+    const sockaddr* sa = sockaddr_storage2sockaddr(&ss);
+    const sockaddr_in* sin = sockaddr2sockaddr_in(sa);
+    _addr = sin->sin_addr.s_addr;
 }
 
 IPv4::IPv4(const sockaddr_in& sin) throw(InvalidFamily)
@@ -93,7 +102,18 @@ IPv4::copy_out(in_addr& to_in_addr) const
 size_t
 IPv4::copy_out(struct sockaddr& to_sockaddr) const
 {
-    return (copy_out(reinterpret_cast<sockaddr_in&>(to_sockaddr)));
+    return (copy_out(*sockaddr2sockaddr_in(&to_sockaddr)));
+}
+
+/**
+ * Copy the raw address to @to_sockaddr_storage, and assign appropriately
+ * the rest of the fields in @to_sockaddr_storage.
+ * @return the number of copied octets.
+ */
+size_t
+IPv4::copy_out(struct sockaddr_storage& to_sockaddr_storage) const
+{
+    return (copy_out(*sockaddr_storage2sockaddr(&to_sockaddr_storage)));
 }
 
 /**
@@ -105,7 +125,7 @@ size_t
 IPv4::copy_out(struct sockaddr_in& to_sockaddr_in) const
 {
     memset(&to_sockaddr_in, 0, sizeof(to_sockaddr_in));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     to_sockaddr_in.sin_len = sizeof(to_sockaddr_in);
 #endif
     to_sockaddr_in.sin_family = AF_INET;
@@ -141,7 +161,18 @@ IPv4::copy_in(const in_addr& from_in_addr)
 size_t
 IPv4::copy_in(const sockaddr& from_sockaddr) throw (InvalidFamily)
 {
-    return (copy_in(reinterpret_cast<const sockaddr_in&>(from_sockaddr)));
+    return (copy_in(*sockaddr2sockaddr_in(&from_sockaddr)));
+}
+
+/**
+ * Copy a raw address from @from_sockaddr_storage.
+ * @return the number of copied octets.
+ */
+size_t
+IPv4::copy_in(const sockaddr_storage& from_sockaddr_storage)
+    throw (InvalidFamily)
+{
+    return (copy_in(*sockaddr_storage2sockaddr(&from_sockaddr_storage)));
 }
 
 /**
