@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/ipv6.cc,v 1.29 2007/02/16 22:46:19 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/ipv6.cc,v 1.34 2008/07/23 05:10:52 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -53,15 +53,24 @@ IPv6::IPv6(const sockaddr& sa) throw (InvalidFamily)
 {
     if (sa.sa_family != AF_INET6)
 	xorp_throw(InvalidFamily, sa.sa_family);
-    const sockaddr_in6& sin = reinterpret_cast<const sockaddr_in6&>(sa);
-    memcpy(_addr, sin.sin6_addr.s6_addr, sizeof(_addr));
+    const sockaddr_in6* sin6 = sockaddr2sockaddr_in6(&sa);
+    memcpy(_addr, sin6->sin6_addr.s6_addr, sizeof(_addr));
 }
 
-IPv6::IPv6(const sockaddr_in6& sin) throw (InvalidFamily)
+IPv6::IPv6(const sockaddr_storage& ss) throw (InvalidFamily)
 {
-    if (sin.sin6_family != AF_INET6)
-	xorp_throw(InvalidFamily, sin.sin6_family);
-    memcpy(_addr, sin.sin6_addr.s6_addr, sizeof(_addr));
+    if (ss.ss_family != AF_INET6)
+	xorp_throw(InvalidFamily, ss.ss_family);
+    const sockaddr* sa = sockaddr_storage2sockaddr(&ss);
+    const sockaddr_in6* sin6 = sockaddr2sockaddr_in6(sa);
+    memcpy(_addr, sin6->sin6_addr.s6_addr, sizeof(_addr));
+}
+
+IPv6::IPv6(const sockaddr_in6& sin6) throw (InvalidFamily)
+{
+    if (sin6.sin6_family != AF_INET6)
+	xorp_throw(InvalidFamily, sin6.sin6_family);
+    memcpy(_addr, sin6.sin6_addr.s6_addr, sizeof(_addr));
 }
 
 IPv6::IPv6(const char* from_cstring) throw (InvalidString)
@@ -101,7 +110,18 @@ IPv6::copy_out(struct in6_addr& to_in6_addr) const
 size_t
 IPv6::copy_out(struct sockaddr& to_sockaddr) const
 {
-    return (copy_out(reinterpret_cast<sockaddr_in6&>(to_sockaddr)));
+    return (copy_out(*sockaddr2sockaddr_in6(&to_sockaddr)));
+}
+
+/**
+ * Copy the raw address to @to_sockaddr_storage, and assign appropriately
+ * the rest of the fields in @to_sockaddr_storage.
+ * @return the number of copied octets.
+ */
+size_t
+IPv6::copy_out(struct sockaddr_storage& to_sockaddr_storage) const
+{
+    return (copy_out(*sockaddr_storage2sockaddr(&to_sockaddr_storage)));
 }
 
 /**
@@ -113,12 +133,12 @@ size_t
 IPv6::copy_out(struct sockaddr_in6& to_sockaddr_in6) const
 {
     memset(&to_sockaddr_in6, 0, sizeof(to_sockaddr_in6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     to_sockaddr_in6.sin6_len = sizeof(to_sockaddr_in6);
 #endif
     to_sockaddr_in6.sin6_family = AF_INET6;
 
-#ifdef HAVE_SIN6_SCOPE_ID
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
 #ifdef IPV6_STACK_KAME
     //
     // XXX: In case of KAME the local interface index (also the link-local
@@ -134,7 +154,7 @@ IPv6::copy_out(struct sockaddr_in6& to_sockaddr_in6) const
 	to_sockaddr_in6.sin6_scope_id = zoneid;
     }
 #endif // IPV6_STACK_KAME
-#endif // HAVE_SIN6_SCOPE_ID
+#endif // HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
 
     return (copy_out(to_sockaddr_in6.sin6_addr));
 }
@@ -167,7 +187,18 @@ IPv6::copy_in(const in6_addr& from_in6_addr)
 size_t
 IPv6::copy_in(const sockaddr& from_sockaddr) throw (InvalidFamily)
 {
-    return (copy_in(reinterpret_cast<const sockaddr_in6&>(from_sockaddr)));
+    return (copy_in(*sockaddr2sockaddr_in6(&from_sockaddr)));
+}
+
+/**
+ * Copy a raw address from @from_sockaddr_storage.
+ * @return the number of copied octets.
+ */
+size_t
+IPv6::copy_in(const sockaddr_storage& from_sockaddr_storage)
+    throw (InvalidFamily)
+{
+    return (copy_in(*sockaddr_storage2sockaddr(&from_sockaddr_storage)));
 }
 
 /**
