@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/ipvx.cc,v 1.28 2007/02/16 22:46:20 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/ipvx.cc,v 1.35 2008/07/23 05:10:52 pavlin Exp $"
 
 #include "xorp.h"
 #include "ipvx.hh"
@@ -34,6 +34,13 @@
 // Static class members
 //
 
+// Construct an IN_ADDR_ANY address of family AF_INET.
+IPvX::IPvX()
+    : _af(AF_INET)
+{
+    memset(_addr, 0, sizeof(_addr));
+}
+
 // Construct an IN_ADDR_ANY address of @family.
 IPvX::IPvX(int family) throw (InvalidFamily)
 {
@@ -89,6 +96,11 @@ IPvX::IPvX(const sockaddr& from_sockaddr) throw (InvalidFamily)
     copy_in(from_sockaddr);
 }
 
+IPvX::IPvX(const sockaddr_storage& from_sockaddr_storage) throw (InvalidFamily)
+{
+    copy_in(from_sockaddr_storage);
+}
+
 IPvX::IPvX(const sockaddr_in& from_sockaddr_in) throw (InvalidFamily)
 {
     copy_in(from_sockaddr_in);
@@ -315,7 +327,19 @@ IPvX::copy_out(struct in6_addr& to_in6_addr) const throw (InvalidFamily)
 size_t
 IPvX::copy_out(struct sockaddr& to_sockaddr) const throw (InvalidFamily)
 {
-    return (copy_out(reinterpret_cast<sockaddr_in&>(to_sockaddr)));
+    return (copy_out(*sockaddr2sockaddr_in(&to_sockaddr)));
+}
+
+/**
+ * Copy the raw address to @to_sockaddr_storage, and assign appropriately
+ * the rest of the fields in @to_sockaddr_storage.
+ * @return the number of copied octets.
+ */
+size_t
+IPvX::copy_out(struct sockaddr_storage& to_sockaddr_storage)
+    const throw (InvalidFamily)
+{
+    return (copy_out(*sockaddr_storage2sockaddr(&to_sockaddr_storage)));
 }
 
 /**
@@ -329,7 +353,7 @@ IPvX::copy_out(struct sockaddr_in& to_sockaddr_in) const throw (InvalidFamily)
     switch (_af) {
     case AF_INET:
 	memset(&to_sockaddr_in, 0, sizeof(to_sockaddr_in));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
 	to_sockaddr_in.sin_len = sizeof(to_sockaddr_in);
 #endif
 	to_sockaddr_in.sin_family = _af;
@@ -343,7 +367,7 @@ IPvX::copy_out(struct sockaddr_in& to_sockaddr_in) const throw (InvalidFamily)
 	xorp_throw(InvalidFamily, _af);
     }
 
-    return((size_t)-1);
+    return ((size_t)-1);
 }
 
 /**
@@ -361,12 +385,12 @@ IPvX::copy_out(struct sockaddr_in6& to_sockaddr_in6) const
 
     case AF_INET6:
 	memset(&to_sockaddr_in6, 0, sizeof(to_sockaddr_in6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
 	to_sockaddr_in6.sin6_len = sizeof(to_sockaddr_in6);
 #endif
 	to_sockaddr_in6.sin6_family = _af;
 
-#ifdef HAVE_SIN6_SCOPE_ID
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
 #ifdef IPV6_STACK_KAME
 	//
 	// XXX: In case of KAME the local interface index (also the link-local
@@ -382,7 +406,7 @@ IPvX::copy_out(struct sockaddr_in6& to_sockaddr_in6) const
 	    to_sockaddr_in6.sin6_scope_id = zoneid;
 	}
 #endif // IPV6_STACK_KAME
-#endif // HAVE_SIN6_SCOPE_ID
+#endif // HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
 
 	return (copy_out(to_sockaddr_in6.sin6_addr));
 
@@ -390,7 +414,7 @@ IPvX::copy_out(struct sockaddr_in6& to_sockaddr_in6) const
 	xorp_throw(InvalidFamily, _af);
     }
 
-    return((size_t)-1);
+    return ((size_t)-1);
 }
 
 /**
@@ -441,7 +465,18 @@ IPvX::copy_in(const in6_addr& from_in6_addr)
 size_t
 IPvX::copy_in(const sockaddr& from_sockaddr) throw (InvalidFamily)
 {
-    return (copy_in(reinterpret_cast<const sockaddr_in&>(from_sockaddr)));
+    return (copy_in(*sockaddr2sockaddr_in(&from_sockaddr)));
+}
+
+/**
+ * Copy a raw address from @from_sockaddr_storage.
+ * @return the number of copied octets.
+ */
+size_t
+IPvX::copy_in(const sockaddr_storage& from_sockaddr_storage)
+    throw (InvalidFamily)
+{
+    return (copy_in(*sockaddr_storage2sockaddr(&from_sockaddr_storage)));
 }
 
 /**
@@ -461,7 +496,7 @@ IPvX::copy_in(const sockaddr_in& from_sockaddr_in) throw (InvalidFamily)
     default:
 	xorp_throw(InvalidFamily, _af);
     }
-    return((size_t)-1);
+    return ((size_t)-1);
 }
 
 /**
@@ -482,7 +517,7 @@ IPvX::copy_in(const sockaddr_in6& from_sockaddr_in6) throw (InvalidFamily)
 	xorp_throw(InvalidFamily, _af);
     }
 
-    return((size_t)-1);
+    return ((size_t)-1);
 }
 
 bool
