@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
  * vim:set sts=4 ts=8:
  *
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -14,11 +14,30 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libxorp/random.c,v 1.8 2007/02/16 22:46:21 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/random.c,v 1.18 2008/07/23 05:10:53 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 
-#ifndef HAVE_RANDOM	/* This entire file is not needed if we have random() */
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
+/*
+ * Local implementation of random(3).
+ *
+ * It is always used instead of the system random(3) implementation so it
+ * will be easier to obtain repeatable results across different systems
+ * (when we have support for simulations).
+ */
 
 /*
  * Copyright (c) 1983, 1993
@@ -53,9 +72,15 @@
  * SUCH DAMAGE.
  */
 
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>          /* for srandomdev() */
+#endif
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>             /* for srandomdev() */
+#endif
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -286,7 +311,7 @@ static inline uint32_t good_rand (x)
  * for default usage relies on values produced by this routine.
  */
 void
-srandom(x)
+xorp_srandom(x)
 	unsigned long x;
 {
 	int i, lim;
@@ -302,7 +327,7 @@ srandom(x)
 		lim = 10 * rand_deg;
 	}
 	for (i = 0; i < lim; i++)
-		(void)random();
+		(void)xorp_random();
 }
 
 /*
@@ -317,7 +342,7 @@ srandom(x)
  * a fixed seed.
  */
 void
-srandomdev()
+xorp_srandomdev()
 {
 	int fd, done;
 	size_t len;
@@ -332,11 +357,11 @@ srandomdev()
 	/* XXX: We need to use CAPI to get hardware randomness. */
 	UNUSED(fd);
 #else
-	fd = _open("/dev/random", O_RDONLY, 0);
+	fd = open("/dev/random", O_RDONLY, 0);
 	if (fd >= 0) {
-		if (_read(fd, (void *) state, len) == (ssize_t) len)
+		if (read(fd, (void *) state, len) == (ssize_t) len)
 			done = 1;
-		_close(fd);
+		close(fd);
 	}
 #endif
 
@@ -348,10 +373,17 @@ srandomdev()
 		 */
 #else
 		struct timeval tv;
-		unsigned long junk;
+		unsigned long junk = 0;
+		unsigned long junk_seed[2];
 
+		/*
+		 * XXX: We need "junk_seed" to set the "junk" value to avoid
+		 * compiler warning about using an uninitialized variable.
+		 */
+		memset(&junk_seed[0], 0, sizeof(junk_seed[0]));
+		junk = junk_seed[1];
 		gettimeofday(&tv, NULL);
-		srandom((getpid() << 16) ^ tv.tv_sec ^ tv.tv_usec ^ junk);
+		xorp_srandom((getpid() << 16) ^ tv.tv_sec ^ tv.tv_usec ^ junk);
 #endif
 		return;
 	}
@@ -386,13 +418,13 @@ srandomdev()
  * complain about mis-alignment, but you should disregard these messages.
  */
 char *
-initstate(seed, arg_state, n)
+xorp_initstate(seed, arg_state, n)
 	unsigned long seed;		/* seed for R.N.G. */
 	char *arg_state;		/* pointer to state array */
 	long n;				/* # bytes of state info */
 {
 	char *ostate = (char *)(&state[-1]);
-	uint32_t *int_arg_state = (uint32_t *)arg_state;
+	uint32_t *int_arg_state = (uint32_t *)(void *)arg_state;
 
 	if (rand_type == TYPE_0)
 		state[-1] = rand_type;
@@ -401,7 +433,7 @@ initstate(seed, arg_state, n)
 	if (n < BREAK_0) {
 		(void)fprintf(stderr,
 		    "random: not enough state (%ld bytes); ignored.\n", n);
-		return(0);
+		return (0);
 	}
 	if (n < BREAK_1) {
 		rand_type = TYPE_0;
@@ -426,12 +458,12 @@ initstate(seed, arg_state, n)
 	}
 	state = int_arg_state + 1; /* first location */
 	end_ptr = &state[rand_deg];	/* must set end_ptr before srandom */
-	srandom(seed);
+	xorp_srandom(seed);
 	if (rand_type == TYPE_0)
 		int_arg_state[0] = rand_type;
 	else
 		int_arg_state[0] = MAX_TYPES * (rptr - state) + rand_type;
-	return(ostate);
+	return (ostate);
 }
 
 /*
@@ -454,10 +486,10 @@ initstate(seed, arg_state, n)
  * complain about mis-alignment, but you should disregard these messages.
  */
 char *
-setstate(arg_state)
+xorp_setstate(arg_state)
 	char *arg_state;		/* pointer to state array */
 {
-	uint32_t *new_state = (uint32_t *)arg_state;
+	uint32_t *new_state = (uint32_t *)(void *)arg_state;
 	uint32_t type = new_state[0] % MAX_TYPES;
 	uint32_t rear = new_state[0] / MAX_TYPES;
 	char *ostate = (char *)(&state[-1]);
@@ -486,7 +518,7 @@ setstate(arg_state)
 		fptr = &state[(rear + rand_sep) % rand_deg];
 	}
 	end_ptr = &state[rand_deg];		/* set end_ptr too */
-	return(ostate);
+	return (ostate);
 }
 
 /*
@@ -507,7 +539,7 @@ setstate(arg_state)
  * Returns a 31-bit random number.
  */
 long
-random()
+xorp_random()
 {
 	uint32_t i;
 	uint32_t *f, *r;
@@ -532,7 +564,5 @@ random()
 
 		fptr = f; rptr = r;
 	}
-	return((long)i);
+	return ((long)i);
 }
-
-#endif /* !HAVE_RANDOM */
