@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-// $XORP: xorp/libxorp/run_command.cc,v 1.31 2007/02/16 22:46:22 pavlin Exp $
+// $XORP: xorp/libxorp/run_command.cc,v 1.34 2008/07/23 05:10:54 pavlin Exp $
 
 #include "libxorp_module.h"
 
@@ -48,6 +48,11 @@
 #define _PATH_BSHELL "C:\\MSYS\\bin\\sh.exe"
 #endif
 
+
+#ifdef fileno
+#undef fileno
+#endif
+
 #define	fileno(stream) (_get_osfhandle(_fileno(stream)))
 
 #else // ! HOST_OS_WINDOWS
