@@ -2,7 +2,7 @@
 /* vim:set sts=4 ts=8: */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libxorp/strptime.c,v 1.14 2007/02/16 22:46:23 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/strptime.c,v 1.18 2008/07/23 05:10:54 pavlin Exp $"
 
 
 /*
@@ -45,6 +45,8 @@
  *	  with "unsigned int".
  */
 
+#include "config.h"
+
 /*
  * XXX: Linux's glibc2 and NetBSD need to define _XOPEN_SOURCE for strptime(3).
  *
@@ -55,7 +57,9 @@
  */
 #define _XOPEN_SOURCE	500
 #include <time.h>
+#ifdef HAVE_STRINGS_H
 #include <strings.h>
+#endif
 #undef _XOPEN_SOURCE
 
 #include <string.h>
@@ -69,7 +73,6 @@
  */
 /* #include "libxorp/xorp.h" */
 
-#include "config.h"
 #include "libxorp/utility.h"
 
 #ifndef TM_YEAR_BASE
@@ -221,6 +224,8 @@ __RCSID("$NetBSD: strptime.c,v 1.25 2005/11/29 03:12:00 christos Exp $");
 #define	LEGAL_ALT(x)		{ if (alt_format & ~(x)) return NULL; }
 
 
+#ifndef HAVE_STRPTIME
+
 static const unsigned char *conv_num(const unsigned char *, int *,
 	unsigned int, unsigned int);
 static const unsigned char *find_string(const unsigned char *, int *,
@@ -522,6 +527,9 @@ find_string(const unsigned char *bp, int *tgt, const char * const *n1,
 	return NULL;
 }
 
+#endif /* HAVE_STRPTIME */
+
+
 char *
 xorp_strptime(const char *buf, const char *fmt, struct tm *tm)
 {
