@@ -1,5 +1,5 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -11,7 +11,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_asyncio.cc,v 1.17 2007/02/16 22:46:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_asyncio.cc,v 1.21 2008/07/23 05:10:54 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -197,7 +197,7 @@ local_comm_sock_pair(int domain, int type, int protocol, xsock_t sv[2])
 
     numtries = 3;
     do {
-	port = htons((rand() % (CSP_LOWPORT - CSP_HIGHPORT)) + CSP_LOWPORT);
+	port = htons((xorp_random() % (CSP_LOWPORT - CSP_HIGHPORT)) + CSP_LOWPORT);
 #ifdef HAVE_IPV6
 	if (intdomain == AF_INET6)
 	    psin6->sin6_port = port;
@@ -452,11 +452,11 @@ run_test()
 	UNUSED(was_started);
 	assert(was_started == false); // can't start no buffer
 	// Choose number of buffers to use
-	int n = 1 + (random() % MAX_BUFFERS);
+	int n = 1 + (xorp_random() % MAX_BUFFERS);
 	printf("%d ", n); fflush(stdout);
 	while (n >= 0) {
 	    // Size of buffer add must be at least 1
-	    size_t b_bytes = 1 + (rand() % (MAX_BUFFER_BYTES - 1)); 
+	    size_t b_bytes = 1 + (xorp_random() % (MAX_BUFFER_BYTES - 1)); 
 	    afw.add_buffer(msg, b_bytes, 
 			   callback(&writer_check, msg, &t));
 	    afr.add_buffer(msg, b_bytes,
