@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_config_param.cc,v 1.9 2007/02/16 22:46:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_config_param.cc,v 1.14 2008/07/23 05:10:54 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -34,7 +34,7 @@ static const char *program_name		= "test_config_param";
 static const char *program_description	= "Test ConfigParam class";
 static const char *program_version_id	= "0.1";
 static const char *program_date		= "December 4, 2002";
-static const char *program_copyright	= "See file LICENSE.XORP";
+static const char *program_copyright	= "See file LICENSE";
 static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";
 
 static bool s_verbose = false;
@@ -194,13 +194,39 @@ test_config_param()
 		   "config parameter update callback value");
 
     //
-    // Increment and decrement operators
+    // Increment and decrement operators (prefix)
     //
     config_param2 = 111;
-    verbose_assert(++config_param2 == 112, "++");
+    verbose_assert(++config_param2 == 112, "++ (prefix)");
     verbose_assert(config_param_update_callback_value == 112,
 		   "config parameter update callback value");
-    verbose_assert(--config_param2 == 111, "--");
+    verbose_assert(--config_param2 == 111, "-- (prefix)");
+    verbose_assert(config_param_update_callback_value == 111,
+		   "config parameter update callback value");
+
+    //
+    // Increment and decrement operators (postfix)
+    //
+    config_param2 = 111;
+    verbose_assert(config_param2++ == 111, "++ (postfix)");
+    verbose_assert(config_param2.get() == 112, "++ (postfix)");
+    verbose_assert(config_param_update_callback_value == 112,
+		   "config parameter update callback value");
+    verbose_assert(config_param2-- == 112, "-- (postfix)");
+    verbose_assert(config_param2.get() == 111, "-- (postfix)");
+    verbose_assert(config_param_update_callback_value == 111,
+		   "config parameter update callback value");
+
+    //
+    // Methods incr() and decr()
+    //
+    config_param2 = 111;
+    config_param2.incr();
+    verbose_assert(config_param2.get() == 112, "incr()");
+    verbose_assert(config_param_update_callback_value == 112,
+		   "config parameter update callback value");
+    config_param2.decr();
+    verbose_assert(config_param2.get() == 111, "decr()");
     verbose_assert(config_param_update_callback_value == 111,
 		   "config parameter update callback value");
     
