@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipnet.cc,v 1.16 2007/02/16 22:46:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_ipnet.cc,v 1.19 2008/07/23 05:10:55 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -32,8 +32,8 @@
 
 static bool s_verbose = false;
 
-inline bool verbose() 		{ return s_verbose; }
-inline void set_verbose(bool v)	{ s_verbose = v; }
+bool verbose()			{ return s_verbose; }
+void set_verbose(bool v)	{ s_verbose = v; }
 
 #define verbose_log(x...) 						      \
 do {									      \
