@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipv4.cc,v 1.27 2007/02/16 22:46:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_ipv4.cc,v 1.32 2008/07/23 05:10:55 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -34,7 +34,7 @@ static const char *program_name		= "test_ipv4";
 static const char *program_description	= "Test IPv4 address class";
 static const char *program_version_id	= "0.1";
 static const char *program_date		= "December 2, 2002";
-static const char *program_copyright	= "See file LICENSE.XORP";
+static const char *program_copyright	= "See file LICENSE";
 static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";
 
 static bool s_verbose = false;
@@ -146,12 +146,13 @@ test_ipv4_valid_constructors()
     in_addr.s_addr = ui;
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_INET;
     sin.sin_addr.s_addr = ui;
     struct sockaddr *sap = (struct sockaddr *)&sin;
+    struct sockaddr_storage *ssp = (struct sockaddr_storage *)&sin;
 
     //
     // Default constructor.
@@ -195,11 +196,17 @@ test_ipv4_valid_constructors()
     IPv4 ip7(*sap);
     verbose_match(ip7.str(), addr_string);
 
+    //
+    // Constructor from sockaddr_storage structure.
+    //
+    IPv4 ip8(*ssp);
+    verbose_match(ip8.str(), addr_string);
+
     //
     // Constructor from sockaddr_in structure.
     //
-    IPv4 ip8(sin);
-    verbose_match(ip8.str(), addr_string);
+    IPv4 ip9(sin);
+    verbose_match(ip9.str(), addr_string);
 }
 
 /**
@@ -211,12 +218,13 @@ test_ipv4_invalid_constructors()
     // Test values for IPv4 address: "12.34.56.78"
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_UNSPEC;		// Note: invalid IP address family
     sin.sin_addr.s_addr = htonl((12 << 24) | (34 << 16) | (56 << 8) | 78);
     struct sockaddr *sap = (struct sockaddr *)&sin;
+    struct sockaddr_storage *ssp = (struct sockaddr_storage *)&sin;
 
     //
     // Constructor from an invalid address string.
@@ -245,6 +253,19 @@ test_ipv4_invalid_constructors()
 	verbose_log("%s : OK\n", e.str().c_str());
     }
 
+    //
+    // Constructor from an invalid sockaddr_storage structure.
+    //
+    try {
+	IPv4 ip(*ssp);
+	verbose_log("Cannot catch invalid IP address family AF_UNSPEC : FAIL\n");
+	incr_failures();
+	UNUSED(ip);
+    } catch (const InvalidFamily& e) {
+	// The problem was caught
+	verbose_log("%s : OK\n", e.str().c_str());
+    }
+
     //
     // Constructor from an invalid sockaddr_in structure.
     //
@@ -272,13 +293,14 @@ test_ipv4_valid_copy_in_out()
     in_addr.s_addr = ui;
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_INET;
     sin.sin_addr.s_addr = ui;
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
 
     //
@@ -312,6 +334,17 @@ test_ipv4_valid_copy_in_out()
     verbose_assert(memcmp(&sin, &ip5_sockaddr_in, sizeof(sin)) == 0,
 		   "compare copy_out(sockaddr&) for IPv4 address");
 
+    //
+    // Copy the IPv4 raw address to a sockaddr_storage structure.
+    //
+    IPv4 ip5_2(addr_string4);
+    struct sockaddr_in ip5_2_sockaddr_in;
+    ssp = (struct sockaddr_storage *)&ip5_2_sockaddr_in;
+    verbose_assert(ip5_2.copy_out(*ssp) == 4,
+		   "copy_out(sockaddr_storage&) for IPv4 address");
+    verbose_assert(memcmp(&sin, &ip5_2_sockaddr_in, sizeof(sin)) == 0,
+		   "compare copy_out(sockaddr_storage&) for IPv4 address");
+
     //
     // Copy the IPv4 raw address to a sockaddr_in structure.
     //
@@ -347,6 +380,16 @@ test_ipv4_valid_copy_in_out()
 		   "copy_in(sockaddr&) for IPv4 address");
     verbose_match(ip15.str(), addr_string4);
 
+    //
+    // Copy a raw address from a sockaddr_storage structure into IPv4
+    // structure.
+    //
+    IPv4 ip15_2;
+    ssp = (struct sockaddr_storage *)&sin;
+    verbose_assert(ip15_2.copy_in(*ssp) == 4,
+		   "copy_in(sockaddr_storage&) for IPv4 address");
+    verbose_match(ip15_2.str(), addr_string4);
+
     //
     // Copy a raw address from a sockaddr_in structure into IPv4 structure.
     //
@@ -366,13 +409,14 @@ test_ipv4_invalid_copy_in_out()
     // const char *addr_string4 = "12.34.56.78";
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_UNSPEC;		// Note: invalid IP address family
     sin.sin_addr.s_addr = htonl((12 << 24) | (34 << 16) | (56 << 8) | 78);
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
     //
     // Copy-in from a sockaddr structure for invalid address family.
@@ -388,6 +432,20 @@ test_ipv4_invalid_copy_in_out()
 	verbose_log("%s : OK\n", e.str().c_str());
     }
 
+    //
+    // Copy-in from a sockaddr_storage structure for invalid address family.
+    //
+    try {
+	IPv4 ip;
+	ssp = (struct sockaddr_storage *)&sin;
+	ip.copy_in(*ssp);
+	verbose_log("Cannot catch invalid IP address family AF_UNSPEC : FAIL\n");
+	incr_failures();
+    } catch (const InvalidFamily& e) {
+	// The problem was caught
+	verbose_log("%s : OK\n", e.str().c_str());
+    }
+
     //
     // Copy-in from a sockaddr_in structure for invalid address family.
     //
