@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipv4net.cc,v 1.21 2007/02/16 22:46:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_ipv4net.cc,v 1.25 2008/07/23 05:10:55 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -40,7 +40,7 @@ static const char *program_name		= "test_ipv4net";
 static const char *program_description	= "Test IPv4Net address class";
 static const char *program_version_id	= "0.1";
 static const char *program_date		= "December 2, 2002";
-static const char *program_copyright	= "See file LICENSE.XORP";
+static const char *program_copyright	= "See file LICENSE";
 static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";
 #endif // 0
 
@@ -160,7 +160,7 @@ test_ipv4net_valid_constructors(TestInfo& test_info)
     in_addr.s_addr = ui;
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_INET;
