@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipv6.cc,v 1.26 2007/02/16 22:46:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_ipv6.cc,v 1.32 2008/07/23 05:10:55 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -34,7 +34,7 @@ static const char *program_name		= "test_ipv6";
 static const char *program_description	= "Test IPv6 address class";
 static const char *program_version_id	= "0.1";
 static const char *program_date		= "December 2, 2002";
-static const char *program_copyright	= "See file LICENSE.XORP";
+static const char *program_copyright	= "See file LICENSE";
 static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";
 
 static bool s_verbose = false;
@@ -152,12 +152,13 @@ test_ipv6_valid_constructors()
     memcpy(&ui32[0], &in6_addr, sizeof(in6_addr));
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_INET6;
     sin6.sin6_addr = in6_addr;
     struct sockaddr *sap = (struct sockaddr *)&sin6;
+    struct sockaddr_storage *ssp = (struct sockaddr_storage *)&sin6;
 
     //
     // Default constructor.
@@ -201,11 +202,17 @@ test_ipv6_valid_constructors()
     IPv6 ip7(*sap);
     verbose_match(ip7.str(), addr_string);
 
+    //
+    // Constructor from sockaddr_storage structure.
+    //
+    IPv6 ip8(*ssp);
+    verbose_match(ip8.str(), addr_string);
+
     //
     // Constructor from sockaddr_in6 structure.
     //
-    IPv6 ip8(sin6);
-    verbose_match(ip8.str(), addr_string);
+    IPv6 ip9(sin6);
+    verbose_match(ip9.str(), addr_string);
 }
 
 /**
@@ -221,12 +228,13 @@ test_ipv6_invalid_constructors()
 				     0x87, 0x65, 0x43, 0x21 } } };
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_UNSPEC;	// Note: invalid IP address family
     sin6.sin6_addr = in6_addr;
     struct sockaddr *sap = (struct sockaddr *)&sin6;
+    struct sockaddr_storage *ssp = (struct sockaddr_storage *)&sin6;
 
     //
     // Constructor from an invalid address string.
@@ -256,6 +264,19 @@ test_ipv6_invalid_constructors()
 	verbose_log("%s : OK\n", e.str().c_str());
     }
 
+    //
+    // Constructor from an invalid sockaddr_storage structure.
+    //
+    try {
+	IPv6 ip(*ssp);
+	verbose_log("Cannot catch invalid IP address family AF_UNSPEC : FAIL\n");
+	incr_failures();
+	UNUSED(ip);
+    } catch (const InvalidFamily& e) {
+	// The problem was caught
+	verbose_log("%s : OK\n", e.str().c_str());
+    }
+
     //
     // Constructor from an invalid sockaddr_in6 structure.
     //
@@ -288,13 +309,14 @@ test_ipv6_valid_copy_in_out()
     memcpy(&ui32[0], &in6_addr, sizeof(in6_addr));
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_INET6;
     sin6.sin6_addr = in6_addr;
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
 
     //
@@ -328,6 +350,17 @@ test_ipv6_valid_copy_in_out()
     verbose_assert(memcmp(&sin6, &ip6_sockaddr_in6, sizeof(sin6)) == 0,
 		   "compare copy_out(sockaddr&) for IPv6 address");
 
+    //
+    // Copy the IPv6 raw address to a sockaddr_storage structure.
+    //
+    IPv6 ip6_2(addr_string6);
+    struct sockaddr_in6 ip6_2_sockaddr_in6;
+    ssp = (struct sockaddr_storage *)&ip6_2_sockaddr_in6;
+    verbose_assert(ip6_2.copy_out(*ssp) == 16,
+		   "copy_out(sockaddr_storage&) for IPv6 address");
+    verbose_assert(memcmp(&sin6, &ip6_2_sockaddr_in6, sizeof(sin6)) == 0,
+		   "compare copy_out(sockaddr_storage&) for IPv6 address");
+
     //
     // Copy the IPv6 raw address to a sockaddr_in6 structure.
     //
@@ -363,6 +396,16 @@ test_ipv6_valid_copy_in_out()
 		   "copy_in(sockaddr&) for IPv6 address");
     verbose_match(ip16.str(), addr_string6);
 
+    //
+    // Copy a raw address from a sockaddr_storage structure into IPv6
+    // structure.
+    //
+    IPv6 ip16_2;
+    ssp = (struct sockaddr_storage *)&sin6;
+    verbose_assert(ip16_2.copy_in(*ssp) == 16,
+		   "copy_in(sockaddr_storage&) for IPv6 address");
+    verbose_match(ip16_2.str(), addr_string6);
+
     //
     // Copy a raw address from a sockaddr_in6 structure into IPv6 structure.
     //
@@ -390,13 +433,14 @@ test_ipv6_invalid_copy_in_out()
     memcpy(&ui32[0], &in6_addr, sizeof(in6_addr));
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_UNSPEC;	// Note: invalid IP address family
     sin6.sin6_addr = in6_addr;
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
     //
     // Copy-in from a sockaddr structure for invalid address family.
@@ -412,6 +456,20 @@ test_ipv6_invalid_copy_in_out()
 	verbose_log("%s : OK\n", e.str().c_str());
     }
 
+    //
+    // Copy-in from a sockaddr_storage structure for invalid address family.
+    //
+    try {
+	IPv6 ip;
+	ssp = (struct sockaddr_storage *)&sin6;
+	ip.copy_in(*ssp);
+	verbose_log("Cannot catch invalid IP address family AF_UNSPEC : FAIL\n");
+	incr_failures();
+    } catch (const InvalidFamily& e) {
+	// The problem was caught
+	verbose_log("%s : OK\n", e.str().c_str());
+    }
+
     //
     // Copy-in from a sockaddr_in6 structure for invalid address family.
     //
