@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipvx.cc,v 1.27 2007/02/16 22:46:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_ipvx.cc,v 1.33 2008/07/23 05:10:55 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -34,7 +34,7 @@ static const char *program_name		= "test_ipvx";
 static const char *program_description	= "Test IPvX address class";
 static const char *program_version_id	= "0.1";
 static const char *program_date		= "December 2, 2002";
-static const char *program_copyright	= "See file LICENSE.XORP";
+static const char *program_copyright	= "See file LICENSE";
 static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";
 
 static bool s_verbose = false;
@@ -147,7 +147,7 @@ test_ipvx_valid_constructors()
     in_addr.s_addr = ui;
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_INET;
@@ -165,13 +165,14 @@ test_ipvx_valid_constructors()
     memcpy(&ui32[0], &in6_addr, sizeof(in6_addr));
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_INET6;
     sin6.sin6_addr = in6_addr;
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
 
     //
@@ -266,6 +267,13 @@ test_ipvx_valid_constructors()
     IPvX ip14(*sap);
     verbose_match(ip14.str(), addr_string4);
 
+    //
+    // Constructor from sockaddr_storage structure: IPv4
+    //
+    ssp = (struct sockaddr_storage *)&sin;
+    IPvX ip14_2(*sap);
+    verbose_match(ip14_2.str(), addr_string4);
+
     //
     // Constructor from sockaddr structure: IPv6
     //
@@ -273,6 +281,13 @@ test_ipvx_valid_constructors()
     IPvX ip15(*sap);
     verbose_match(ip15.str(), addr_string6);
 
+    //
+    // Constructor from sockaddr_storage structure: IPv6
+    //
+    ssp = (struct sockaddr_storage *)&sin6;
+    IPvX ip15_2(*ssp);
+    verbose_match(ip15_2.str(), addr_string6);
+
     //
     // Constructor from sockaddr_in structure: IPv4
     //
@@ -309,7 +324,7 @@ test_ipvx_invalid_constructors()
     // Test values for IPv4 address: "12.34.56.78"
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_UNSPEC;		// Note: invalid IP address family
@@ -326,13 +341,14 @@ test_ipvx_invalid_constructors()
     memcpy(&ui32[0], &in6_addr, sizeof(in6_addr));
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_UNSPEC;	// Note: invalid IP address family
     sin6.sin6_addr = in6_addr;
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
     //
     // Constructor for invalid address family.
@@ -403,6 +419,20 @@ test_ipvx_invalid_constructors()
 	verbose_log("%s : OK\n", e.str().c_str());
     }
 
+    //
+    // Constructor from an invalid sockaddr_storage structure.
+    //
+    try {
+	ssp = (struct sockaddr_storage *)&sin;
+	IPvX ip(*ssp);
+	verbose_log("Cannot catch invalid IP address family AF_UNSPEC : FAIL\n");
+	incr_failures();
+	UNUSED(ip);
+    } catch (const InvalidFamily& e) {
+	// The problem was caught
+	verbose_log("%s : OK\n", e.str().c_str());
+    }
+
     //
     // Constructor from an invalid sockaddr_in structure.
     //
@@ -443,7 +473,7 @@ test_ipvx_valid_copy_in_out()
     in_addr.s_addr = ui;
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_INET;
@@ -461,13 +491,14 @@ test_ipvx_valid_copy_in_out()
     memcpy(&ui32[0], &in6_addr, sizeof(in6_addr));
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_INET6;
     sin6.sin6_addr = in6_addr;
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
 
     //
@@ -521,6 +552,17 @@ test_ipvx_valid_copy_in_out()
     verbose_assert(memcmp(&sin, &ip5_sockaddr_in, sizeof(sin)) == 0,
 		   "compare copy_out(sockaddr&) for IPv4 address");
 
+    //
+    // Copy the IPvX raw address to a sockaddr_storage structure: IPv4.
+    //
+    IPvX ip5_2(addr_string4);
+    struct sockaddr_in ip5_2_sockaddr_in;
+    ssp = (struct sockaddr_storage *)&ip5_2_sockaddr_in;
+    verbose_assert(ip5_2.copy_out(*ssp) == 4,
+		   "copy_out(sockaddr_storage&) for IPv4 address");
+    verbose_assert(memcmp(&sin, &ip5_2_sockaddr_in, sizeof(sin)) == 0,
+		   "compare copy_out(sockaddr_storage&) for IPv4 address");
+
     //
     // Copy the IPvX raw address to a sockaddr structure: IPv6.
     //
@@ -532,6 +574,17 @@ test_ipvx_valid_copy_in_out()
     verbose_assert(memcmp(&sin6, &ip6_sockaddr_in6, sizeof(sin6)) == 0,
 		   "compare copy_out(sockaddr&) for IPv6 address");
 
+    //
+    // Copy the IPvX raw address to a sockaddr_storage structure: IPv6.
+    //
+    IPvX ip6_2(addr_string6);
+    struct sockaddr_in6 ip6_2_sockaddr_in6;
+    ssp = (struct sockaddr_storage *)&ip6_2_sockaddr_in6;
+    verbose_assert(ip6_2.copy_out(*ssp) == 16,
+		   "copy_out(sockaddr_storage&) for IPv6 address");
+    verbose_assert(memcmp(&sin6, &ip6_2_sockaddr_in6, sizeof(sin6)) == 0,
+		   "compare copy_out(sockaddr_storage&) for IPv6 address");
+
     //
     // Copy the IPvX raw address to a sockaddr_in structure: IPv4.
     //
@@ -615,6 +668,16 @@ test_ipvx_valid_copy_in_out()
 		   "copy_in(sockaddr&) for IPv4 address");
     verbose_match(ip15.str(), addr_string4);
 
+    //
+    // Copy a raw address from a sockaddr_storage structure into IPvX
+    // structure: IPv4.
+    //
+    IPvX ip15_2(AF_INET);
+    ssp = (struct sockaddr_storage *)&sin;
+    verbose_assert(ip15_2.copy_in(*ssp) == 4,
+		   "copy_in(sockaddr_storage&) for IPv4 address");
+    verbose_match(ip15_2.str(), addr_string4);
+
     //
     // Copy a raw address from a sockaddr structure into IPvX structure: IPv6.
     //
@@ -624,6 +687,16 @@ test_ipvx_valid_copy_in_out()
 		   "copy_in(sockaddr&) for IPv6 address");
     verbose_match(ip16.str(), addr_string6);
 
+    //
+    // Copy a raw address from a sockaddr_storage structure into IPvX
+    // structure: IPv6.
+    //
+    IPvX ip16_2(AF_INET6);
+    ssp = (struct sockaddr_storage *)&sin6;
+    verbose_assert(ip16_2.copy_in(*ssp) == 16,
+		   "copy_in(sockaddr_storage&) for IPv6 address");
+    verbose_match(ip16_2.str(), addr_string6);
+
     //
     // Copy a raw address from a sockaddr_in structure into IPvX structure: IPv4.
     //
@@ -669,7 +742,7 @@ test_ipvx_invalid_copy_in_out()
     const char *addr_string4 = "12.34.56.78";
     struct sockaddr_in sin;
     memset(&sin, 0, sizeof(sin));
-#ifdef HAVE_SIN_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
     sin.sin_len = sizeof(sin);
 #endif
     sin.sin_family = AF_UNSPEC;		// Note: invalid IP address family
@@ -687,13 +760,14 @@ test_ipvx_invalid_copy_in_out()
     memcpy(&ui32[0], &in6_addr, sizeof(in6_addr));
     struct sockaddr_in6 sin6;
     memset(&sin6, 0, sizeof(sin6));
-#ifdef HAVE_SIN6_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
     sin6.sin6_len = sizeof(sin6);
 #endif
     sin6.sin6_family = AF_UNSPEC;	// Note: invalid IP address family
     sin6.sin6_addr = in6_addr;
 
     struct sockaddr *sap;
+    struct sockaddr_storage *ssp;
 
     //
     // Mismatch copy-out: copy-out IPv6 address to in_addr structure.
@@ -723,8 +797,8 @@ test_ipvx_invalid_copy_in_out()
 	verbose_log("%s : OK\n", e.str().c_str());
     }
 
-    // XXX: we should test for copy_out() to sockaddr, sockaddr_in,
-    // sockaddr_in6 structures that throw InvalidFamily.
+    // XXX: we should test for copy_out() to sockaddr, sockaddr_storage,
+    // sockaddr_in, sockaddr_in6 structures that throw InvalidFamily.
     // To do so we must creast first IPvX with invalid address family.
     // However, this doesn't seem possible, hence we skip those checks.
 
@@ -755,6 +829,20 @@ test_ipvx_invalid_copy_in_out()
 	verbose_log("%s : OK\n", e.str().c_str());
     }
 
+    //
+    // Copy-in from a sockaddr_storage structure for invalid address family.
+    //
+    try {
+	IPvX ip(AF_INET);
+	ssp = (struct sockaddr_storage *)&sin;
+	ip.copy_in(*ssp);
+	verbose_log("Cannot catch invalid IP address family AF_UNSPEC : FAIL\n");
+	incr_failures();
+    } catch (const InvalidFamily& e) {
+	// The problem was caught
+	verbose_log("%s : OK\n", e.str().c_str());
+    }
+
     //
     // Copy-in from a sockaddr_in structure for invalid address family.
     //
