@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_run_command.cc,v 1.17 2007/02/16 22:46:25 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_run_command.cc,v 1.21 2008/07/23 05:10:55 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -35,8 +36,8 @@
 #define AWK_VIA_SHELL_PATH	"/bin/gawk"
 #else
 #define SLEEP_PATH		"/bin/sleep"
-#define AWK_PATH		"/usr/bin/awk"
-#define AWK_VIA_SHELL_PATH	AWK_PATH
+string  AWK_PATH("/usr/bin/awk");
+string& AWK_VIA_SHELL_PATH = AWK_PATH;
 #endif
 
 static string cmdroot = "";
@@ -50,7 +51,7 @@ static const char *program_name		= "test_run_command";
 static const char *program_description	= "Test a class for running an external program";
 static const char *program_version_id	= "0.1";
 static const char *program_date		= "November 22, 2004";
-static const char *program_copyright	= "See file LICENSE.XORP";
+static const char *program_copyright	= "See file LICENSE";
 static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";
 
 static bool s_verbose = false;
@@ -917,6 +918,13 @@ main(int argc, char * const argv[])
 	    _cmdroot = const_cast<char *>(MSYS_DEFAULT_ROOT);
 	cmdroot = string(_cmdroot);
     }
+#else
+    struct stat st;
+
+    if (stat(AWK_PATH.c_str(), &st) == -1)
+	AWK_PATH = "/bin/awk";
+
+    XLOG_ASSERT(stat(AWK_PATH.c_str(), &st) == 0);
 #endif
 
     int ch;
