@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_vif.cc,v 1.14 2007/02/16 22:46:26 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_vif.cc,v 1.21 2008/07/23 05:10:56 pavlin Exp $"
 
 #include "libxorp_module.h"
 
@@ -34,7 +34,7 @@ static const char *program_name		= "test_vif";
 static const char *program_description	= "Test Vif address class";
 static const char *program_version_id	= "0.1";
 static const char *program_date		= "December 4, 2002";
-static const char *program_copyright	= "See file LICENSE.XORP";
+static const char *program_copyright	= "See file LICENSE";
 static const char *program_return_value	= "0 on success, 1 if test error, 2 if internal error";
 
 static bool s_verbose = false;
@@ -392,6 +392,22 @@ test_vif_methods()
     vif1.set_discard(false);
     verbose_assert(! vif1.is_discard(), "is_discard()");
 
+    //
+    // Test if this vif is an unreachable interface.
+    //
+    vif1.set_unreachable(true);
+    verbose_assert(vif1.is_unreachable(), "is_unreachable()");
+    vif1.set_unreachable(false);
+    verbose_assert(! vif1.is_unreachable(), "is_unreachable()");
+
+    //
+    // Test if this vif is a management interface.
+    //
+    vif1.set_management(true);
+    verbose_assert(vif1.is_management(), "is_management()");
+    vif1.set_management(false);
+    verbose_assert(! vif1.is_management(), "is_management()");
+
     //
     // Test if this vif is multicast capable.
     //
