@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8 sw=4:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -28,7 +29,7 @@
 // notice is a summary of the Click LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/timer.cc,v 1.38 2007/02/16 22:46:27 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/timer.cc,v 1.42 2008/07/23 05:10:56 pavlin Exp $"
 
 
 #include "libxorp_module.h"
@@ -206,27 +207,38 @@ private:
 // ----------------------------------------------------------------------------
 // TimerList implemention
 
-static TimerList* the_timerlist = NULL;
+TimerList* the_timerlist = NULL;
+int timerlist_instance_count;
 
 TimerList::TimerList(ClockBase* clock)
     : _clock(clock), _observer(NULL)
 {
     assert(the_timerlist == NULL);
-    the_timerlist = this;
+    assert(timerlist_instance_count == 0);
 #ifdef HOST_OS_WINDOWS
     // timeBeginPeriod(1);	// requires WINMM.DLL
     _hirestimer = CreateWaitableTimer(NULL, TRUE, NULL);
     assert(_hirestimer != NULL);
 #endif // HOST_OS_WINDOWS
+    the_timerlist = this;
+    timerlist_instance_count++;
 }
 
 TimerList::~TimerList()
 {
+#ifdef notyet
+    // Attempting to plug the leak causes problems elsewhere.
+    map<int, Heap*>::iterator ii;
+    for (ii = _heaplist.begin(); ii != _heaplist.end(); ii++)
+	delete (*ii).second;
+#endif
+
 #ifdef HOST_OS_WINDOWS
     if (_hirestimer != NULL)
 	CloseHandle(_hirestimer);
     //timeEndPeriod(1);
 #endif // HOST_OS_WINDOWS
+    timerlist_instance_count--;
     the_timerlist = NULL;
 }
 
