@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/transaction.cc,v 1.9 2007/02/16 22:46:27 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/transaction.cc,v 1.12 2008/07/23 05:10:57 pavlin Exp $"
 
 #include <assert.h>
 
@@ -100,7 +100,7 @@ TransactionManager::crank_tid()
     //
     _next_tid++;
     do {
-	_next_tid += (random() & 0xfffff);
+	_next_tid += (xorp_random() & 0xfffff);
     } while (_transactions.find(_next_tid) != _transactions.end());
 }
 
