@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/libxorp/utility.h,v 1.14 2007/02/16 22:46:28 pavlin Exp $
+ * $XORP: xorp/libxorp/utility.h,v 1.19 2008/07/23 05:10:57 pavlin Exp $
  */
 
 #ifndef __LIBXORP_UTILITY_H__
@@ -63,6 +63,68 @@
 #define	XORP_INT_CAST(x)	static_cast<int>(x)
 #endif
 
+/*
+ * On some architectures casting a "(struct sockaddr *)" pointer to
+ * "(struct sockaddr_in *)" or "(struct sockaddr_in6 *)" pointer generates
+ * a warning like:
+ *     warning: cast from 'sockaddr*' to 'sockaddr_in*' increases required
+ *     alignment of target type
+ *
+ * In general such casting shouldn't create any alignment issues and
+ * shouldn't generate such warning.
+ * To get around the problem we use the help of a "void" pointer.
+ *
+ * If the casting actually creates an alignment problem, then we need
+ * to copy the "struct sockaddr" content to "struct sockaddr_in" or
+ * "struct sockaddr_in6" placeholder.
+ * Doing this (without using local static storage) might requite changing
+ * the semantics hence we don't provide the implementation.
+ */
+#ifdef __cplusplus
+inline const struct sockaddr_in *
+sockaddr2sockaddr_in(const struct sockaddr* sa)
+{
+    const void* v = sa;
+    return (reinterpret_cast<const struct sockaddr_in*>(v));
+}
+
+inline struct sockaddr_in *
+sockaddr2sockaddr_in(struct sockaddr* sa)
+{
+    void* v = sa;
+    return (reinterpret_cast<struct sockaddr_in*>(v));
+}
+
+inline const struct sockaddr_in6 *
+sockaddr2sockaddr_in6(const struct sockaddr* sa)
+{
+    const void* v = sa;
+    return (reinterpret_cast<const struct sockaddr_in6*>(v));
+}
+
+inline struct sockaddr_in6 *
+sockaddr2sockaddr_in6(struct sockaddr* sa)
+{
+    void* v = sa;
+    return (reinterpret_cast<struct sockaddr_in6*>(v));
+}
+
+inline const struct sockaddr *
+sockaddr_storage2sockaddr(const struct sockaddr_storage* ss)
+{
+    const void* v = ss;
+    return (reinterpret_cast<const struct sockaddr*>(v));
+}
+
+inline struct sockaddr *
+sockaddr_storage2sockaddr(struct sockaddr_storage* ss)
+{
+    void* v = ss;
+    return (reinterpret_cast<struct sockaddr*>(v));
+}
+
+#endif /* __cplusplus */
+
 #define ADD_POINTER(pointer, size, type)				\
 	((type)(void *)(((uint8_t *)(pointer)) + (size)))
 
@@ -116,6 +178,11 @@ extern int xorp_toupper(int c);
  */
 extern char *xorp_strptime(const char *buf, const char *fmt, struct tm *tm);
 
+/*
+ * Function to return C-string representation of a boolean: "true" of "false".
+ */
+extern const char *bool_c_str(int v);
+
 #ifdef __cplusplus
 }
 #endif
