@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/vif.cc,v 1.19 2007/02/16 22:46:28 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/vif.cc,v 1.26 2008/07/23 05:10:57 pavlin Exp $"
 
 #include <functional>
 #include <string>
@@ -89,6 +89,8 @@ Vif::Vif(const string& vifname, const string& ifname)
     set_p2p(false);
     set_loopback(false);
     set_discard(false);
+    set_unreachable(false);
+    set_management(false);
     set_multicast_capable(false);
     set_broadcast_capable(false);
     set_underlying_vif_up(false);
@@ -109,6 +111,8 @@ Vif::Vif(const Vif& vif)
     set_p2p(vif.is_p2p());
     set_loopback(vif.is_loopback());
     set_discard(vif.is_discard());
+    set_unreachable(vif.is_unreachable());
+    set_management(vif.is_management());
     set_multicast_capable(vif.is_multicast_capable());
     set_broadcast_capable(vif.is_broadcast_capable());
     set_underlying_vif_up(vif.is_underlying_vif_up());
@@ -163,6 +167,10 @@ Vif::str() const
 	r += " LOOPBACK";
     if (is_discard())
 	r += " DISCARD";
+    if (is_unreachable())
+	r += " UNREACHABLE";
+    if (is_management())
+	r += " MANAGEMENT";
     if (is_underlying_vif_up())
 	r += " UNDERLYING_VIF_UP";
     r += c_format(" MTU: %u", XORP_UINT_CAST(mtu()));
@@ -181,6 +189,8 @@ Vif::operator==(const Vif& other) const
 	    && (is_p2p() == other.is_p2p())
 	    && (is_loopback() == other.is_loopback())
 	    && (is_discard() == other.is_discard())
+	    && (is_unreachable() == other.is_unreachable())
+	    && (is_management() == other.is_management())
 	    && (is_multicast_capable() == other.is_multicast_capable())
 	    && (is_broadcast_capable() == other.is_broadcast_capable())
 	    && (is_underlying_vif_up() == other.is_underlying_vif_up())
@@ -256,6 +266,20 @@ Vif::find_address(const IPvX& ipvx_addr)
     return (NULL);
 }
 
+const VifAddr *
+Vif::find_address(const IPvX& ipvx_addr) const
+{
+    list<VifAddr>::const_iterator iter;
+    
+    for (iter = _addr_list.begin(); iter != _addr_list.end(); ++iter) {
+	if ((iter)->is_my_addr(ipvx_addr)) {
+	    return &(*iter);
+	}
+    }
+    
+    return (NULL);
+}
+
 bool
 Vif::is_my_addr(const IPvX& ipvx_addr) const
 {
