@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
+/* vim:set sts=4 ts=8: */
 
-/* Copyright (c) 2001-2007 International Computer Science Institute
+/* Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -13,7 +14,7 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libxorp/xlog.c,v 1.22 2007/02/16 22:46:29 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/xlog.c,v 1.28 2008/07/23 05:10:58 pavlin Exp $"
 
 /*
  * Message logging utility.
@@ -26,6 +27,9 @@
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#ifdef HAVE_SYSLOG_H
+#include <syslog.h>
+#endif
 
 #include "xlog.h"
 
@@ -63,6 +67,7 @@ static size_t		xlog_output_func_count = 0;
 static FILE		*fp_default = NULL;
 static int		xlog_level_enabled[XLOG_LEVEL_MAX];
 static xlog_verbose_t	xlog_verbose_level[XLOG_LEVEL_MAX];
+
 /*
  * XXX: the log level names below has to be consistent with the XLOG_LEVEL_*
  * values.
@@ -100,7 +105,6 @@ static int	xlog_write_va(FILE* fp, const char* fmt, va_list ap);
 static int	xlog_flush(FILE* fp);
 static const char* xlog_localtime2string_short(void);
 
-
 /*
  * ****************************************************************************
  * Control functions to init/exit/start/stop/etc the log utility.
@@ -404,7 +408,7 @@ xlog_level_set_verbose(xlog_level_t log_level, xlog_verbose_t verbose_level)
  * Macro function to generate xlog_info, xlog_warning, etc...
  */
 #define xlog_fn(fn, log_level)					\
-void									\
+void								\
 xlog_##fn (const char *module_name, const char *where, const char *fmt, ...) \
 {									\
     va_list ap;								\
@@ -674,7 +678,7 @@ xlog_record_va(xlog_level_t log_level, const char *module_name,
     for (i = 0; i < xlog_output_func_count; ) {
 	xlog_output_func_t func = xlog_outputs_func[i];
 	void *obj = xlog_outputs_obj[i];
-	if (func(obj, buf_output_ptr) < 0) {
+	if (func(obj, log_level, buf_output_ptr) < 0) {
 	    xlog_remove_output_func(func, obj);
 	    continue;
 	}
@@ -785,8 +789,8 @@ _xcond_trace_msg_long(const char	*module_name,
 	char xlog_where_buf[8000];
 
 	/* TODO: use _xcond_trace_entry and _xcond_trace_msg_short instead */
-	sprintf(xlog_where_buf, "+%d %s %s", line, file,
-		(func) ? func : "(unknown_func)");
+	snprintf(xlog_where_buf, sizeof(xlog_where_buf), "+%d %s %s", line,
+		 file, (func) ? func : "(unknown_func)");
 	va_start(ap, fmt);
 	xlog_record_va(XLOG_LEVEL_TRACE, module_name, xlog_where_buf, fmt, ap);
 	va_end(ap);
@@ -831,8 +835,8 @@ _xcond_trace_msg_short(int flag, const char* fmt, ...)
 	va_list ap;
 	char xlog_where_buf[8000];
 
-	sprintf(xlog_where_buf, "+%d %s %s", the_line, the_file,
-		(the_func) ? the_func : "(unknown_func)");
+	snprintf(xlog_where_buf, sizeof(xlog_where_buf), "+%d %s %s",
+		 the_line, the_file, (the_func) ? the_func : "(unknown_func)");
 	va_start(ap, fmt);
 	xlog_record_va(XLOG_LEVEL_TRACE, the_module, xlog_where_buf, fmt, ap);
 	va_end(ap);
@@ -1079,7 +1083,6 @@ xlog_localtime2string_short(void)
  * Return value: A statically allocated string with the local time using
  * the format described above.
  * XXX: This function uses a BSD-specific function, gettimeofday().
- * XXX: This function calls sprintf() without bounds checking.
  **/
 const char *
 xlog_localtime2string(void)
@@ -1107,11 +1110,12 @@ xlog_localtime2string(void)
     clock = tv.tv_sec;
     tm = localtime(&clock);
     if (strftime(buf, sizeof(buf), "%Y/%m/%d %H:%M:%S", tm) == 0) {
-	sprintf(ret_buf, "strftime ERROR");
+	snprintf(ret_buf, sizeof(ret_buf), "strftime ERROR");
 	return (ret_buf);
     }
 
-    sprintf(ret_buf, "%s.%lu", buf, (unsigned long)tv.tv_usec);
+    snprintf(ret_buf, sizeof(ret_buf), "%s.%lu", buf,
+	     (unsigned long)tv.tv_usec);
 
     return (ret_buf);
 #endif /* HOST_OS_WINDOWS */
@@ -1228,3 +1232,160 @@ x_asprintf(char **ret, const char *format, ...)
     return (ret_size);
 }
 
+/*
+ * ****************************************************************************
+ * Syslog interface functions, conforming to X/Open.
+ * ****************************************************************************
+ */
+
+#if defined(HAVE_SYSLOG_H) && defined(HAVE_SYSLOG)
+typedef struct _code {
+	const char	*c_name;
+	int		c_val;
+} SYSLOG_CODE;
+
+static SYSLOG_CODE prioritynames[] = {
+	{ "alert",	LOG_ALERT,	},
+	{ "crit",	LOG_CRIT,	},
+	{ "debug",	LOG_DEBUG,	},
+	{ "emerg",	LOG_EMERG,	},
+	{ "err",	LOG_ERR,	},
+	{ "info",	LOG_INFO,	},
+	{ "notice",	LOG_NOTICE,	},
+	{ "warning",	LOG_WARNING,	},
+	{ NULL,		-1,		}
+};
+
+static SYSLOG_CODE facilitynames[] = {
+	{ "auth",	LOG_AUTH,	},
+	{ "cron", 	LOG_CRON,	},
+	{ "daemon",	LOG_DAEMON,	},
+	{ "kern",	LOG_KERN,	},
+	{ "lpr",	LOG_LPR,	},
+	{ "mail",	LOG_MAIL,	},
+	{ "news",	LOG_NEWS,	},
+	{ "user",	LOG_USER,	},
+	{ "uucp",	LOG_UUCP,	},
+	{ "local0",	LOG_LOCAL0,	},
+	{ "local1",	LOG_LOCAL1,	},
+	{ "local2",	LOG_LOCAL2,	},
+	{ "local3",	LOG_LOCAL3,	},
+	{ "local4",	LOG_LOCAL4,	},
+	{ "local5",	LOG_LOCAL5,	},
+	{ "local6",	LOG_LOCAL6,	},
+	{ "local7",	LOG_LOCAL7,	},
+	{ NULL,		-1,		}
+};
+
+static int
+xlog_level_to_syslog_priority(xlog_level_t xloglevel)
+{
+
+    switch (xloglevel) {
+    case XLOG_LEVEL_FATAL:
+	return (LOG_CRIT);
+	break;
+    case XLOG_LEVEL_ERROR:
+	return (LOG_ERR);
+	break;
+    case XLOG_LEVEL_WARNING:
+	return (LOG_WARNING);
+	break;
+    case XLOG_LEVEL_INFO:
+	return (LOG_INFO);
+	break;
+    default:
+	XLOG_UNREACHABLE();
+    }
+    return (-1);
+}
+
+static int
+xlog_syslog_output_func(void *obj, xlog_level_t level, const char *msg)
+{
+    int priority = xlog_level_to_syslog_priority(level);
+
+    syslog(priority, "%s", msg);
+
+    return (0);
+    UNUSED(obj);
+}
+
+/*
+ * Parse <facility.priority>.
+ */
+static int
+xlog_parse_syslog_spec(const char *syslogspec, int *facility, int *priority)
+{
+    int i, retval, xfacility, xpriority;
+    char *facname, *priname, *tmpspec;
+    SYSLOG_CODE* sc;
+
+    retval = -1;
+    facname = priname = tmpspec = NULL;
+
+    tmpspec = strdup(syslogspec);
+    if (tmpspec == NULL)
+	return (retval);
+
+    priname = strchr(tmpspec, '.');
+    if (priname != NULL)
+	*priname = '\0';
+
+    facname = tmpspec;
+    xfacility = -1;
+    for (i = 0, sc = &facilitynames[0]; sc->c_val != -1;  ++sc, ++i) {
+	if (0 == strcasecmp(sc->c_name, facname)) {
+	    xfacility = i;
+	    break;
+	}
+    }
+    if (xfacility == -1)
+	goto out;
+
+    *facility = xfacility;
+
+    if (priname != NULL && ++priname != '\0') {
+	    xpriority = -1;
+	    for (i = 0, sc = &prioritynames[0]; sc->c_val != -1; ++sc, ++i) {
+		if (0 == strcasecmp(sc->c_name, priname)) {
+		    xpriority = i;
+		    break;
+		}
+	    }
+	    if (xpriority == -1) {
+		goto out;
+	    }
+	    *priority = xpriority;
+    } else
+	*priority = LOG_WARNING;
+
+    retval = 0;
+out:
+    free(tmpspec);
+    return (retval);
+}
+
+int
+xlog_add_syslog_output(const char *syslogspec)
+{
+    int facility = -1;
+    int priority = -1;
+
+    if (-1 == xlog_parse_syslog_spec(syslogspec, &facility, &priority))
+	return (-1);
+
+    openlog("xorp", LOG_PID | LOG_NDELAY | LOG_CONS, facility);
+    xlog_add_output_func(xlog_syslog_output_func, NULL);
+
+    return (0);
+}
+
+#else /* !(HAVE_SYSLOG_H && HAVE_SYSLOG) */
+int
+xlog_add_syslog_output(const char *syslogspec)
+{
+    return (-1);
+    UNUSED(syslogspec);
+}
+#endif /* HAVE_SYSLOG_H && HAVE_SYSLOG */
