@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/libxorp/xlog.h,v 1.15 2007/02/16 22:46:29 pavlin Exp $
+ * $XORP: xorp/libxorp/xlog.h,v 1.19 2008/07/23 05:10:58 pavlin Exp $
  */
 
 
@@ -79,7 +79,8 @@ typedef enum {
 /**
  * The type of add-on functions to process the log messages.
  */
-typedef int (*xlog_output_func_t)(void *obj, const char *msg);
+typedef int (*xlog_output_func_t)(void *obj, xlog_level_t level,
+    const char *msg);
 
 /**
  * A macro used for internal purpose to generate the appropriate xlog code
@@ -98,10 +99,11 @@ typedef int (*xlog_output_func_t)(void *obj, const char *msg);
 #else
 #  define _XLOG_MODULE_NAME XORP_MODULE_NAME
 #endif
-#define XLOG_FN(fn, fmt...)						 \
-do {									 \
-	char xlog_where_buf[8000];					 \
-	sprintf(xlog_where_buf, "+%d %s %s", __LINE__, __FILE__, __FUNCTION__);\
+#define XLOG_FN(fn, fmt...)						\
+do {									\
+	char xlog_where_buf[8000];					\
+	snprintf(xlog_where_buf, sizeof(xlog_where_buf), "+%d %s %s",	\
+		 __LINE__, __FILE__, __FUNCTION__);			\
 	xlog_##fn(_XLOG_MODULE_NAME, xlog_where_buf, fmt);		\
 } while (0)
 
@@ -238,6 +240,15 @@ int	xlog_remove_output(FILE *fp);
  */
 int 	xlog_add_output_func(xlog_output_func_t func, void *obj);
 
+/**
+ * Add a channel which goes to the X/Open compatible system log aka syslog.
+ *
+ * @param syslogspec the syslog facility and priority as a string
+ *                   "facility.priority"
+ * @return 0 on success, otherwise -1.
+ */
+int	xlog_add_syslog_output(const char *syslogspec);
+
 /**
  * Remove a processing function and an object from the set of output streams.
  *
