@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/libxorp/xorp.h,v 1.15 2007/02/16 22:46:29 pavlin Exp $
+ * $XORP: xorp/libxorp/xorp.h,v 1.18 2008/07/23 05:10:58 pavlin Exp $
  */
 
 
@@ -24,6 +24,39 @@
 
 #include "config.h"
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
+#if defined (__cplusplus) && !defined(__STL_NO_NAMESPACES)
+using namespace std;
+#endif
+
+/*
+ * Include sys/cdefs.h to define __BEGIN_DECLS and __END_DECLS.  Even if
+ * this file exists, not all platforms define these macros.
+ */
+#ifdef HAVE_SYS_CDEFS_H
+#include <sys/cdefs.h>
+#endif
+
+/*
+ * Define C++ decls wrappers if not previously defined.
+ */
+#ifndef __BEGIN_DECLS
+#  if defined(__cplusplus)
+#    define __BEGIN_DECLS	extern "C" {
+#    define __END_DECLS		};    
+#  else /* __BEGIN_DECLS */
+#    define __BEGIN_DECLS
+#    define __END_DECLS
+#  endif /* __BEGIN_DECLS */
+#endif /* __BEGIN_DECLS */
+
 #include "xorp_osdep_begin.h"
 
 #ifdef __cplusplus
