@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/libxorp/xorp_osdep_mid.h,v 1.7 2007/02/16 22:46:29 pavlin Exp $
+ * $XORP: xorp/libxorp/xorp_osdep_mid.h,v 1.13 2008/07/23 05:10:58 pavlin Exp $
  */
 
 #ifndef __LIBXORP_XORP_OSDEP_MID_H__
@@ -100,8 +100,8 @@ struct iovec {
 typedef HANDLE xfd_t;
 typedef SOCKET xsock_t;
 
-typedef uint32_t gid_t;		/* XXX: This is a hack. */
-typedef uint32_t uid_t;		/* XXX: This is a hack. */
+typedef unsigned long gid_t;		/* XXX: This is a hack. */
+typedef unsigned long uid_t;		/* XXX: This is a hack. */
 
 #ifdef _NO_OLDNAMES
 /*
@@ -115,6 +115,7 @@ typedef uint32_t uid_t;		/* XXX: This is a hack. */
 #ifdef pid_t
 #undef pid_t
 #endif
+typedef _off_t off_t;
 typedef _pid_t pid_t;
 typedef long ssize_t;		/* XXX: This is a hack. */
 typedef _sigset_t sigset_t;	/* XXX: Appease libtecla. */
@@ -134,9 +135,12 @@ typedef _sigset_t sigset_t;	/* XXX: Appease libtecla. */
 #define XORP_BAD_FD		INVALID_HANDLE_VALUE
 #define XORP_BAD_SOCKET		INVALID_SOCKET
 
-/* Windows expects char * for socket option data. */
+/*
+ * Windows expects (char *) and (const char *) pointers for transmitted
+ * data and socket option data.
+ */
 #define XORP_BUF_CAST(x) ((char *)(x))
-#define XORP_BUF_CONST_CAST(x) (const_cast<char *>(x))
+#define XORP_CONST_BUF_CAST(x) ((const char *)(x))
 #define XORP_SOCKOPT_CAST(x) ((char *)(x))
 
 /* I/O errors pertaining to non-blocking sockets. */
@@ -155,11 +159,11 @@ typedef int xsock_t;
 #define XORP_BAD_SOCKET		XORP_BAD_FD
 
 /*
- * The rest of the world now expects void pointers to socket options.
- *
+ * The rest of the world expects void pointers to transmitted
+ * data and socket option data.
  */
 #define XORP_BUF_CAST(x) ((void *)(x))
-#define XORP_BUF_CONST_CAST(x) (const_cast<void *>(x))
+#define XORP_CONST_BUF_CAST(x) ((const void *)(x))
 #define XORP_SOCKOPT_CAST(x) (x)
 
 #endif /* HOST_OS_WINDOWS */
@@ -174,6 +178,10 @@ typedef int xsock_t;
 extern "C" {
 #endif
 
+#ifndef HAVE_STRLCPY
+size_t strlcpy(char *dst, const char *src, size_t size);
+#endif
+
 /*
  * IP address presentation routines.
  */
@@ -181,7 +189,7 @@ extern "C" {
 int inet_pton(int af, const char *src, void *dst);
 #endif
 #ifndef HAVE_INET_NTOP
-const char *inet_ntop(int af, const void *src, char *dst, socklen_t size);
+const char *inet_ntop(int af, const void *src, char *dst, size_t size);
 #endif
 
 /*
