@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/mld6igmp/igmp_proto.h,v 1.16 2007/02/16 22:46:35 pavlin Exp $
+ * $XORP: xorp/mld6igmp/igmp_proto.h,v 1.19 2008/07/23 05:11:03 pavlin Exp $
  */
 
 #ifndef __MLD6IGMP_IGMP_PROTO_H__
@@ -27,6 +27,9 @@
  * IGMPv1 and IGMPv2 (RFC 2236), and IGMPv3 (RFC 3376).
  */
 
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
 #ifdef HAVE_NETINET_IGMP_H
 #include <netinet/igmp.h>
 #endif
@@ -35,6 +38,10 @@
 /*
  * Constants definitions
  */
+#ifndef IPPROTO_IGMP
+#define IPPROTO_IGMP				2
+#endif
+
 /* IGMP versions definition */
 #define IGMP_V1					1
 #define IGMP_V2					2
