@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/mld6igmp/mld6_proto.h,v 1.18 2007/02/16 22:46:35 pavlin Exp $
+ * $XORP: xorp/mld6igmp/mld6_proto.h,v 1.22 2008/07/23 05:11:03 pavlin Exp $
  */
 
 #ifndef __MLD6IGMP_MLD6_PROTO_H__
@@ -27,6 +27,9 @@
  * MLDv1 (RFC 2710), and MLDv2 (RFC 3810).
  */
 
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
 #ifdef HAVE_NETINET_ICMP6_H
 #include <netinet/icmp6.h>
 #endif
@@ -35,6 +38,10 @@
 /*
  * Constants definitions
  */
+#ifndef IPPROTO_ICMPV6
+#define IPPROTO_ICMPV6				58
+#endif
+
 /* MLD versions definition */
 #define MLD_V1					1
 #define MLD_V2					2
@@ -116,7 +123,7 @@
 #endif
 
 #ifndef MLD_MINLEN
-#  ifdef HAVE_MLD_HDR
+#  ifdef HAVE_STRUCT_MLD_HDR
 #    define MLD_MINLEN			(sizeof(struct mld_hdr))
 #  else
 #    define MLD_MINLEN			24
