@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mld6igmp/mld6igmp_config.cc,v 1.17 2007/02/16 22:46:36 pavlin Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6igmp_config.cc,v 1.21 2008/07/23 05:11:03 pavlin Exp $"
 
 
 //
@@ -132,9 +132,6 @@ Mld6igmpNode::set_config_all_vifs_done(string& error_msg)
 	}
     }
 
-    // TODO: XXX: PAVPAVPAV: remove it!!
-    Mld6igmpNode::set_vif_setup_completed(true);
-    
     if (end_config(error_msg) != XORP_OK)
 	return (XORP_ERROR);
     
@@ -177,7 +174,7 @@ Mld6igmpNode::set_vif_proto_version(const string& vif_name, int proto_version,
 	return (XORP_ERROR);
     }
     
-    if (mld6igmp_vif->set_proto_version(proto_version) < 0) {
+    if (mld6igmp_vif->set_proto_version(proto_version) != XORP_OK) {
 	end_config(error_msg);
         error_msg = c_format("Cannot set protocol version for vif %s: "
 			     "invalid protocol version %d",
