@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.84 2007/02/16 22:46:37 pavlin Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.89 2008/07/23 05:11:04 pavlin Exp $"
 
 
 //
@@ -235,10 +235,10 @@ Mld6igmpVif::start(string& error_msg)
 	return (XORP_ERROR);
     }
 
-    if (update_primary_address(error_msg) < 0)
+    if (update_primary_address(error_msg) != XORP_OK)
 	return (XORP_ERROR);
 
-    if (ProtoUnit::start() < 0) {
+    if (ProtoUnit::start() != XORP_OK) {
 	error_msg = "internal error";
 	return (XORP_ERROR);
     }
@@ -248,10 +248,15 @@ Mld6igmpVif::start(string& error_msg)
     set_i_am_querier(true);
     
     //
-    // Start the vif with the kernel
+    // Register as a receiver with the kernel
     //
-    if (mld6igmp_node().start_protocol_kernel_vif(vif_index()) != XORP_OK) {
-	error_msg = c_format("cannot start protocol vif %s with the kernel",
+    if (mld6igmp_node().register_receiver(name(),
+					  name(),
+					  mld6igmp_node().ip_protocol_number(),
+					  true)
+	!= XORP_OK) {
+	error_msg = c_format("cannot register as a receiver on vif %s "
+			     "with the kernel",
 			     name().c_str());
 	return (XORP_ERROR);
     }
@@ -269,7 +274,10 @@ Mld6igmpVif::start(string& error_msg)
 	 groups_iter != groups.end();
 	 ++groups_iter) {
 	const IPvX& group = *groups_iter;
-	if (mld6igmp_node().join_multicast_group(vif_index(), group)
+	if (mld6igmp_node().join_multicast_group(name(),
+						 name(),
+						 mld6igmp_node().ip_protocol_number(),
+						 group)
 	    != XORP_OK) {
 	    error_msg = c_format("cannot join group %s on vif %s",
 				 cstring(group), name().c_str());
@@ -324,7 +332,7 @@ Mld6igmpVif::stop(string& error_msg)
 	return (XORP_ERROR);
     }
 
-    if (ProtoUnit::pending_stop() < 0) {
+    if (ProtoUnit::pending_stop() != XORP_OK) {
 	error_msg = "internal error";
 	ret_value = XORP_ERROR;
     }
@@ -335,7 +343,7 @@ Mld6igmpVif::stop(string& error_msg)
     // automatically when we stop the vif through the MFEA.
     //
 
-    if (ProtoUnit::stop() < 0) {
+    if (ProtoUnit::stop() != XORP_OK) {
 	error_msg = "internal error";
 	ret_value = XORP_ERROR;
     }
@@ -378,10 +386,13 @@ Mld6igmpVif::stop(string& error_msg)
     _group_records.delete_payload_and_clear();
     
     //
-    // Stop the vif with the kernel
+    // Unregister as a receiver with the kernel
     //
-    if (mld6igmp_node().stop_protocol_kernel_vif(vif_index()) != XORP_OK) {
-	XLOG_ERROR("Cannot stop protocol vif %s with the kernel",
+    if (mld6igmp_node().unregister_receiver(name(),
+					    name(),
+					    mld6igmp_node().ip_protocol_number())
+	!= XORP_OK) {
+	XLOG_ERROR("Cannot unregister as a receiver on vif %s with the kernel",
 		   name().c_str());
 	ret_value = XORP_ERROR;
     }
@@ -458,6 +469,8 @@ Mld6igmpVif::mld6igmp_send(const IPvX& src,
     uint16_t cksum;
     int ret_value;
     size_t datalen;
+    bool ip_router_alert = true;	// XXX: always use Router Alert option
+    bool ip_internet_control = true;	// XXX: always true
     
     if (! (is_up() || is_pending_down())) {
 	error_msg = c_format("vif %s is not UP", name().c_str());
@@ -518,9 +531,13 @@ Mld6igmpVif::mld6igmp_send(const IPvX& src,
     //
     // Send the message
     //
-    ret_value = mld6igmp_node().mld6igmp_send(vif_index(), src, dst,
-					      MINTTL, -1, true, buffer,
-					      error_msg);
+    ret_value = mld6igmp_node().mld6igmp_send(name(), name(),
+					      src, dst,
+					      mld6igmp_node().ip_protocol_number(),
+					      MINTTL, -1,
+					      ip_router_alert,
+					      ip_internet_control,
+					      buffer, error_msg);
     
     return (ret_value);
     
@@ -856,8 +873,9 @@ Mld6igmpVif::mld6igmp_query_send(const IPvX& src,
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @is_router_alert: True if the received IP packet had the Router Alert
+ * @ip_router_alert: True if the received IP packet had the Router Alert
  * IP option set.
+ * @ip_internet_control: If true, then this is IP control traffic.
  * @buffer: The buffer with the received message.
  * @error_msg: The error message (if error).
  * 
@@ -870,7 +888,8 @@ Mld6igmpVif::mld6igmp_recv(const IPvX& src,
 			   const IPvX& dst,
 			   int ip_ttl,
 			   int ip_tos,
-			   bool is_router_alert,
+			   bool ip_router_alert,
+			   bool ip_internet_control,
 			   buffer_t *buffer,
 			   string& error_msg)
 {
@@ -881,8 +900,8 @@ Mld6igmpVif::mld6igmp_recv(const IPvX& src,
 	return (XORP_ERROR);
     }
     
-    ret_value = mld6igmp_process(src, dst, ip_ttl, ip_tos, is_router_alert,
-				 buffer, error_msg);
+    ret_value = mld6igmp_process(src, dst, ip_ttl, ip_tos, ip_router_alert,
+				 ip_internet_control, buffer, error_msg);
     
     return (ret_value);
 }
@@ -895,8 +914,9 @@ Mld6igmpVif::mld6igmp_recv(const IPvX& src,
  * it should be ignored.
  * @ip_tos: The IP TOS of the message. If it has a negative value,
  * it should be ignored.
- * @is_router_alert: True if the received IP packet had the Router Alert
+ * @ip_router_alert: True if the received IP packet had the Router Alert
  * IP option set.
+ * @ip_internet_control: If true, then this is IP control traffic.
  * @buffer: The buffer with the message.
  * @error_msg: The error message (if error).
  * 
@@ -910,7 +930,8 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 			      const IPvX& dst,
 			      int ip_ttl,
 			      int ip_tos,
-			      bool is_router_alert,
+			      bool ip_router_alert,
+			      bool ip_internet_control,
 			      buffer_t *buffer,
 			      string& error_msg)
 {
@@ -1139,7 +1160,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
     //
     // IP Router Alert option check.
     //
-    if (check_router_alert_option && (! is_router_alert)) {
+    if (check_router_alert_option && (! ip_router_alert)) {
 	error_msg = c_format("RX %s from %s to %s on vif %s: "
 			     "missing IP Router Alert option",
 			     proto_message_type2ascii(message_type),
@@ -1149,10 +1170,12 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 	return (XORP_ERROR);
     }
     //
-    // TODO: check the TTL and TOS if we are running in secure mode
+    // TODO: check the TTL, TOS and ip_internet_control flag if we are
+    // running in secure mode.
     //
     UNUSED(ip_ttl);
     UNUSED(ip_tos);
+    UNUSED(ip_internet_control);
 #if 0
     if (ip_ttl != MINTTL) {
 	error_msg = c_format("RX %s from %s to %s on vif %s: "
@@ -1756,7 +1779,8 @@ Mld6igmpVif::join_prune_notify_routing(const IPvX& source,
 						      vif_index(),
 						      source,
 						      group,
-						      action_jp) < 0) {
+						      action_jp)
+	    != XORP_OK) {
 	    //
 	    // TODO: remove <module_id, module_instance_name> ??
 	    //
