@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mrt/mrib_table.cc,v 1.20 2007/02/16 22:46:38 pavlin Exp $"
+#ident "$XORP: xorp/mrt/mrib_table.cc,v 1.23 2008/07/23 05:11:06 pavlin Exp $"
 
 
 //
@@ -546,6 +546,14 @@ MribTableIterator::operator++()
     return (*this);
 }
 
+MribTableIterator
+MribTableIterator::operator++(int)
+{
+    MribTableIterator old_value = *this;
+    _mrib_lookup = _mrib_lookup->get_next();
+    return (old_value);
+}
+
 Mrib *
 MribTableIterator::operator*() const
 {
