@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/lsa.cc,v 1.103 2007/03/19 05:47:07 atanu Exp $"
+#ident "$XORP: xorp/ospf/lsa.cc,v 1.110 2008/07/23 05:11:08 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -383,9 +383,11 @@ Ls_request::str() const
 {
     string output;
 
-    output = c_format(" LS type %u", get_ls_type());
-    output += c_format(" Link State ID %#x", get_link_state_id());
-    output += c_format(" Advertising Router %#x", get_advertising_router());
+    output = c_format(" LS type %#x", get_ls_type());
+    output += c_format(" Link State ID %s", 
+		       pr_id(get_link_state_id()).c_str());
+    output += c_format(" Advertising Router %s",
+		       pr_id(get_advertising_router()).c_str());
     
     return output;
 }
@@ -968,19 +970,19 @@ RouterLsa::str() const
 
     output += "\n";
 
-    output += c_format("\tbit Nt %s\n", pb(get_nt_bit()));
+    output += c_format("\tbit Nt %s\n", bool_c_str(get_nt_bit()));
 
     switch(version) {
     case OspfTypes::V2:
 	break;
     case OspfTypes::V3:
-	output += c_format("\tbit W %s\n", pb(get_w_bit()));
+	output += c_format("\tbit W %s\n", bool_c_str(get_w_bit()));
 	break;
     }
 
-    output += c_format("\tbit V %s\n", pb(get_v_bit()));
-    output += c_format("\tbit E %s\n", pb(get_e_bit()));
-    output += c_format("\tbit B %s", pb(get_b_bit()));
+    output += c_format("\tbit V %s\n", bool_c_str(get_v_bit()));
+    output += c_format("\tbit E %s\n", bool_c_str(get_e_bit()));
+    output += c_format("\tbit B %s", bool_c_str(get_b_bit()));
 
     switch(version) {
     case OspfTypes::V2:
@@ -1615,6 +1617,68 @@ ASExternalLsa::encode()
     return true;
 }
 
+template <>
+void
+ASExternalLsa::set_network<IPv4>(IPNet<IPv4> net)
+{
+    XLOG_ASSERT(OspfTypes::V2 == get_version());
+    set_network_mask(ntohl(net.netmask().addr()));
+    get_header().set_link_state_id(ntohl(net.masked_addr().addr()));
+}
+
+template <>
+void 
+ASExternalLsa::set_network<IPv6>(IPNet<IPv6> net)
+{
+    XLOG_ASSERT(OspfTypes::V3 == get_version());
+    IPv6Prefix prefix(get_version());
+    prefix.set_network(net);
+    set_ipv6prefix(prefix);
+}
+
+template <>
+IPNet<IPv4>
+ASExternalLsa::get_network<IPv4>(IPv4) const
+{
+    return IPNet<IPv4>(IPv4(htonl(get_header().get_link_state_id())),
+		       IPv4(htonl(get_network_mask())).mask_len());
+}
+
+template <>
+IPNet<IPv6>
+ASExternalLsa::get_network<IPv6>(IPv6) const 
+{
+    return get_ipv6prefix().get_network();
+}
+
+template <>
+void
+ASExternalLsa::set_forwarding_address<IPv4>(IPv4 forwarding_address_ipv4)
+{
+    set_forwarding_address_ipv4(forwarding_address_ipv4);
+}
+	
+template <>
+void 
+ASExternalLsa::set_forwarding_address<IPv6>(IPv6 forwarding_address_ipv6)
+{
+    set_forwarding_address_ipv6(forwarding_address_ipv6);
+}
+
+template <>
+IPv4 
+ASExternalLsa::get_forwarding_address<IPv4>(IPv4) const
+{
+    return get_forwarding_address_ipv4();
+}
+	
+template <>
+IPv6
+ASExternalLsa::get_forwarding_address<IPv6>(IPv6) const 
+{
+    return get_forwarding_address_ipv6();
+}
+
 string
 ASExternalLsa::str() const
 {
@@ -1631,7 +1695,7 @@ ASExternalLsa::str() const
     switch(version) {
     case OspfTypes::V2:
 	output += c_format("\n\tNetwork Mask %#x", get_network_mask());
-	output += c_format("\n\tbit E %s", pb(get_e_bit()));
+	output += c_format("\n\tbit E %s", bool_c_str(get_e_bit()));
 	output += c_format("\n\tMetric %d %#x", get_metric(), get_metric());
 	if (get_metric() == OspfTypes::LSInfinity)
 	    output += c_format(" LSInfinity");
@@ -1641,9 +1705,9 @@ ASExternalLsa::str() const
 			   get_external_route_tag());
 	break;
     case OspfTypes::V3:
-	output += c_format("\n\tbit E %s", pb(get_e_bit()));
-	output += c_format("\n\tbit F %s", pb(get_f_bit()));
-	output += c_format("\n\tbit T %s", pb(get_t_bit()));
+	output += c_format("\n\tbit E %s", bool_c_str(get_e_bit()));
+	output += c_format("\n\tbit F %s", bool_c_str(get_f_bit()));
+	output += c_format("\n\tbit T %s", bool_c_str(get_t_bit()));
 	output += c_format("\n\tMetric %d %#x", get_metric(), get_metric());
 	if (get_metric() == OspfTypes::LSInfinity)
 	    output += c_format(" LSInfinity");
