@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/ospf.cc,v 1.91 2007/03/12 10:16:03 atanu Exp $"
+#ident "$XORP: xorp/ospf/ospf.cc,v 1.99 2008/07/23 05:11:08 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -73,6 +73,11 @@ void
 Ospf<A>::receive(const string& interface, const string& vif,
 		 A dst, A src, uint8_t* data, uint32_t len)
 {
+    XLOG_TRACE(trace()._packets, 
+	       "Interface %s Vif %s dst %s src %s data %p len %u\n",
+	      interface.c_str(), vif.c_str(),
+	      dst.str().c_str(), src.str().c_str(),
+	      data, len);
     debug_msg("Interface %s Vif %s dst %s src %s data %p len %u\n",
 	      interface.c_str(), vif.c_str(),
 	      dst.str().c_str(), src.str().c_str(),
@@ -91,7 +96,8 @@ Ospf<A>::receive(const string& interface, const string& vif,
 	return;
     }
 
-    debug_msg("%s\n", packet->str().c_str());
+    XLOG_TRACE(trace()._packets, "%s\n", cstring(*packet));
+    debug_msg("%s\n", cstring(*packet));
     // We have a packet and its good.
 
     bool packet_accepted = false;
@@ -130,6 +136,15 @@ Ospf<A>::disable_interface_vif(const string& interface, const string& vif)
     return _io->disable_interface_vif(interface, vif);
 }
 
+template <typename A>
+bool
+Ospf<A>::enabled(const string& interface, const string& vif)
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    return _io->is_vif_enabled(interface, vif);
+}
+
 template <typename A>
 bool
 Ospf<A>::enabled(const string& interface, const string& vif, A address)
@@ -278,6 +293,8 @@ Ospf<A>::transmit(const string& interface, const string& vif,
 		  A dst, A src,
 		  uint8_t* data, uint32_t len)
 {
+    XLOG_TRACE(trace()._packets, "Interface %s Vif %s data %p len %u\n",
+	      interface.c_str(), vif.c_str(), data, len);
     debug_msg("Interface %s Vif %s data %p len %u\n",
 	      interface.c_str(), vif.c_str(), data, len);
 
@@ -285,6 +302,18 @@ Ospf<A>::transmit(const string& interface, const string& vif,
     // pseudo header. In the IPv4 case this function is a noop.
     ipv6_checksum_apply<A>(src, dst, data, len, Packet::CHECKSUM_OFFSET,
 			   _io->get_ip_protocol_number());
+
+    if (trace()._packets) {
+	try {
+	    // Decode the packet in order to pretty print it.
+	    Packet *packet = _packet_decoder.decode(data, len);
+	    XLOG_TRACE(trace()._packets, "Transmit: %s\n", cstring(*packet));
+	    delete packet;
+	} catch(InvalidPacket& e) {
+	    XLOG_TRACE(trace()._packets, "Unable to decode packet\n");
+	}
+    }
+
 #ifdef	DEBUG_LOGGING
     try {
 	// Decode the packet in order to pretty print it.
@@ -589,12 +618,12 @@ Ospf<A>::delete_md5_authentication_key(const string&		interface,
 template <typename A>
 bool
 Ospf<A>::set_passive(const string& interface, const string& vif,
-		     OspfTypes::AreaID area, bool passive)
+		     OspfTypes::AreaID area, bool passive, bool host)
 {
     try {
 	_peer_manager.set_passive(_peer_manager.
 				  get_peerid(interface,vif),
-				  area, passive);
+				  area, passive, host);
     } catch(BadPeer& e) {
 	XLOG_ERROR("%s", cstring(e));
 	return false;
@@ -606,7 +635,7 @@ template <typename A>
 bool
 Ospf<A>::originate_default_route(OspfTypes::AreaID area, bool enable)
 {
-    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     return _peer_manager.originate_default_route(area, enable);
 }
@@ -624,7 +653,7 @@ template <typename A>
 bool
 Ospf<A>::summaries(OspfTypes::AreaID area, bool enable)
 {
-    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     return _peer_manager.summaries(area, enable);
 }
@@ -641,7 +670,7 @@ bool
 Ospf<A>::area_range_add(OspfTypes::AreaID area, IPNet<A> net, bool advertise)
 {
     debug_msg("Area %s Net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     return _peer_manager.area_range_add(area, net, advertise);
 }
@@ -661,7 +690,7 @@ Ospf<A>::area_range_change_state(OspfTypes::AreaID area, IPNet<A> net,
 				 bool advertise)
 {
     debug_msg("Area %s Net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     return _peer_manager.area_range_change_state(area, net, advertise);
 }
@@ -702,6 +731,13 @@ Ospf<A>::get_neighbour_info(OspfTypes::NeighbourID nid,
     return _peer_manager.get_neighbour_info(nid, ninfo);
 }
 
+template <typename A>
+bool
+Ospf<A>::clear_database()
+{
+    return _peer_manager.clear_database();
+}
+
 template <typename A>
 bool
 Ospf<A>::add_route(IPNet<A> net, A nexthop, uint32_t nexthop_id,
@@ -709,14 +745,14 @@ Ospf<A>::add_route(IPNet<A> net, A nexthop, uint32_t nexthop_id,
 		   const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
-	      cstring(net), cstring(nexthop), metric, pb(equal),
-	      pb(discard), cstring(policytags));
+	      cstring(net), cstring(nexthop), metric, bool_c_str(equal),
+	      bool_c_str(discard), cstring(policytags));
 
     XLOG_TRACE(trace()._routes,
 	       "Add route "
 	       "Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
-	      cstring(net), cstring(nexthop), metric, pb(equal),
-	      pb(discard), cstring(policytags));
+	       cstring(net), cstring(nexthop), metric, bool_c_str(equal),
+	       bool_c_str(discard), cstring(policytags));
 
     return _io->add_route(net, nexthop, nexthop_id, metric, equal, discard,
 			  policytags);
@@ -729,14 +765,14 @@ Ospf<A>::replace_route(IPNet<A> net, A nexthop, uint32_t nexthop_id,
 		       const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
-	      cstring(net), cstring(nexthop), metric, pb(equal),
-	      pb(discard), cstring(policytags));
+	      cstring(net), cstring(nexthop), metric, bool_c_str(equal),
+	      bool_c_str(discard), cstring(policytags));
 
     XLOG_TRACE(trace()._routes,
 	       "Replace route "
 	       "Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
-	      cstring(net), cstring(nexthop), metric, pb(equal),
-	      pb(discard), cstring(policytags));
+	       cstring(net), cstring(nexthop), metric, bool_c_str(equal),
+	       bool_c_str(discard), cstring(policytags));
 
     return _io->replace_route(net, nexthop, nexthop_id, metric, equal, discard,
 			      policytags);
