@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/packet.cc,v 1.43 2007/02/16 22:46:41 pavlin Exp $"
+#ident "$XORP: xorp/ospf/packet.cc,v 1.47 2008/07/23 05:11:08 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -459,26 +459,24 @@ PacketDecoder::decode(uint8_t *ptr, size_t len) throw(InvalidPacket)
 
     map<OspfTypes::Type , Packet *>::iterator i;
     uint8_t type = ptr[1];
-    bool bad = false;
+    Packet *packet = NULL;
     switch(version) {
     case OspfTypes::V2:
 	i = _ospfv2.find(type);
-	if (i == _ospfv2.end())
-	    bad = true;
+	if (i != _ospfv2.end())
+	    packet = i->second;
 	break;
     case OspfTypes::V3:
 	i = _ospfv3.find(type);
-	if (i == _ospfv3.end())
-	    bad = true;
+	if (i != _ospfv3.end())
+	    packet = i->second;
 	break;
     }
     
-    if (bad)
+    if (packet == NULL)
 	xorp_throw(InvalidPacket,
 		   c_format("OSPF Version %u Unknown Type %u", version, type));
     
-    Packet *packet = i->second;
-
     return packet->decode(ptr, len);
 }
 
@@ -773,9 +771,9 @@ DataDescriptionPacket::str() const
     output += c_format("\tInterface MTU %u\n", get_interface_mtu());
     output += c_format("\tOptions %#x %s\n", get_options(),
 		       cstring(Options(get_version(), get_options())));
-    output += c_format("\tI-bit %s\n", get_i_bit() ? "true" : "false");
-    output += c_format("\tM-bit %s\n", get_m_bit() ? "true" : "false");
-    output += c_format("\tMS-bit %s\n", get_ms_bit() ? "true" : "false");
+    output += c_format("\tI-bit %s\n", bool_c_str(get_i_bit()));
+    output += c_format("\tM-bit %s\n", bool_c_str(get_m_bit()));
+    output += c_format("\tMS-bit %s\n", bool_c_str(get_ms_bit()));
     output += c_format("\tDD sequence number %u", get_dd_seqno());
 
     list<Lsa_header> li = _lsa_headers;
