@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/peer_manager.cc,v 1.139 2007/03/13 18:25:51 atanu Exp $"
+#ident "$XORP: xorp/ospf/peer_manager.cc,v 1.158 2008/07/23 05:11:09 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -126,8 +126,10 @@ PeerManager<A>::create_area_router(OspfTypes::AreaID area,
     // XXX Should subsume the refreshing of the Router-LSA into the
     // generic area border router transition method.
     if (area_border_router_p() != old_border_router_state) {
-	refresh_router_lsas();
-	area_border_router_transition(true /* up */);
+	if (!_ospf.get_testing()) {
+	    refresh_router_lsas();
+	    area_border_router_transition(true /* up */);
+	}
     }
 
     // Inform this area if any virtual links are configured.
@@ -232,8 +234,10 @@ PeerManager<A>::destroy_area_router(OspfTypes::AreaID area)
     // XXX Should subsume the refreshing of the Router-LSA into the
     // generic area border router transition method.
     if (area_border_router_p() != old_border_router_state) {
-	refresh_router_lsas();
-	area_border_router_transition(false /* down */);
+	if (!_ospf.get_testing()) {
+	    refresh_router_lsas();
+	    area_border_router_transition(false /* down */);
+	}
     }
 
     // Flag to the virtual link code that this area is going away.
@@ -248,7 +252,7 @@ PeerManager<A>::area_range_add(OspfTypes::AreaID area, IPNet<A> net,
 			       bool advertise)
 {
     debug_msg("Area %s Net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     AreaRouter<A> *area_router = get_area_router(area);
 
@@ -284,7 +288,7 @@ PeerManager<A>::area_range_change_state(OspfTypes::AreaID area, IPNet<A> net,
 					bool advertise)
 {
     debug_msg("Area %s Net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     AreaRouter<A> *area_router = get_area_router(area);
 
@@ -430,6 +434,30 @@ PeerManager<A>::destroy_peerid(const string& interface, const string& vif)
     _pmap.erase(_pmap.find(concat));
 }
 
+template <typename A>
+bool
+PeerManager<A>::enabled(const string& interface, const string& vif, A address)
+{
+    debug_msg("Interface %s Vif %s Address %s\n", interface.c_str(),
+	      vif.c_str(), cstring(address));
+
+    switch (_ospf.get_version()) {
+    case OspfTypes::V2:
+	break;
+    case OspfTypes::V3:
+	// If the address is set to zero then try and get the
+	// link-local address.
+	if (A::ZERO() == address) {
+	    if (!_ospf.get_link_local_address(interface, vif, address))
+		if (_ospf.enabled(interface, vif))
+		    XLOG_WARNING("link-local address must be configured "
+				 "on %s/%s", interface.c_str(), vif.c_str());
+	}
+	break;
+    }
+    return _ospf.enabled(interface, vif, address);
+}
+
 template <typename A>
 OspfTypes::PeerID
 PeerManager<A>::create_peer(const string& interface, const string& vif,
@@ -451,96 +479,30 @@ PeerManager<A>::create_peer(const string& interface, const string& vif,
 
     OspfTypes::PeerID peerid = create_peerid(interface, vif);
 
-    switch (_ospf.get_version()) {
-    case OspfTypes::V2:
-	break;
-    case OspfTypes::V3:
-	if (OspfTypes::VirtualLink != linktype) {
-	    // Note that the source address is going to be replaced with
-	    // the link local address, unless this is a virtual link.
-	    if (!_ospf.get_link_local_address(interface, vif, source)) {
-		destroy_peerid(interface, vif);
-		xorp_throw(BadPeer, 
-			   c_format("Unable to get link local address for "
-				    "%s/%s",
-				    interface.c_str(), vif.c_str()));
-	    }
-	}
-	break;
-    }
-
-    // Get the prefix length.
-    uint16_t interface_prefix_length;
-    if (!_ospf.get_prefix_length(interface, vif, source,
-				 interface_prefix_length)) {
-	destroy_peerid(interface, vif);
-	xorp_throw(BadPeer, 
-		   c_format("Unable to get prefix length for %s/%s/%s",
-			    interface.c_str(), vif.c_str(), cstring(source)));
-    }
-
-    // Get the MTU.
-    uint16_t interface_mtu = _ospf.get_mtu(interface);
-    if (0 == interface_mtu) {
-	destroy_peerid(interface, vif);
-	xorp_throw(BadPeer, 
-		   c_format("Unable to get MTU for %s", interface.c_str()));
-    }
-
     // If we got this far create_peerid did not throw an exception so
     // this interface/vif is unique.
 
     _peers[peerid] = new PeerOut<A>(_ospf, interface, vif, peerid,
-				    source, interface_prefix_length,
-				    interface_mtu, linktype,
-				    area, area_router->get_area_type());
-
-    switch (_ospf.get_version()) {
-    case OspfTypes::V2:
-	break;
-    case OspfTypes::V3: {
-	uint32_t interface_id;
-	if (!_ospf.get_interface_id(interface, vif, interface_id)) {
-	    delete_peer(peerid);
-	    xorp_throw(BadPeer, 
-		       c_format("Unable to get interface ID for %s",
-				interface.c_str()));
-	}
-	_peers[peerid]->set_interface_id(interface_id);
-    }
-	break;
-    }
+				    source, linktype, area,
+				    area_router->get_area_type());
 
     // Pass in the option to be sent by the hello packet.
     _peers[peerid]->set_options(area,
 				compute_options(area_router->get_area_type()));
 
-    switch (_ospf.get_version()) {
-    case OspfTypes::V2:
-	_peers[peerid]->set_link_status(_ospf.enabled(interface, vif, source));
-	break;
-    case OspfTypes::V3:
-#if	0
-	// If this is a virtual link bring it up now, otherwise
-	// wait for the call to activate.
-	if (OspfTypes::VirtualLink == linktype) {
-	    _peers[peerid]->set_link_status(_ospf.enabled(interface, vif,
-							  source));
-	}
-#endif
-	break;
-    }
-
-    // This call needs to be made only once per invocation of OSPF but
-    // at this point we know that the interface mirror is up and running.
-    _ospf.register_address_status(callback(this,
-					   &PeerManager<A>::
+    // These two registrations need to be made only once per
+    // invocation of OSPF, however, at this point we know that the interface
+    // mirror is up and running.
+    _ospf.register_vif_status(callback(this,
+				       &PeerManager<A>::
+				       vif_status_change));
+    _ospf.register_address_status(callback(this, &PeerManager<A>::
 					   address_status_change));
 
     area_router->add_peer(peerid);
 
-    // The peer has now been fully configured so initialise it.
-    _peers[peerid]->go(area);
+    // If the interface, vif and source are up the peer will start running.
+    _peers[peerid]->set_link_status(enabled(interface, vif, source));
 
     return peerid;
 }
@@ -665,7 +627,7 @@ PeerManager<A>::set_address_state_peer(const OspfTypes::PeerID peerid,
 				       bool enable)
 {
     debug_msg("PeerID %u, area %s address %s enable %s\n",
-	      peerid, pr_id(area).c_str(), cstring(addr), pb(enable));
+	      peerid, pr_id(area).c_str(), cstring(addr), bool_c_str(enable));
 
     if (0 == _peers.count(peerid)) {
 	XLOG_ERROR("Unknown PeerID %u", peerid);
@@ -705,7 +667,7 @@ PeerManager<A>::activate_peer(const string& interface, const string& vif,
     recompute_addresses_peer(peerid, area);
 
     A source = _peers[peerid]->get_interface_address();
-    _peers[peerid]->set_link_status(_ospf.enabled(interface, vif, source));
+    _peers[peerid]->set_link_status(enabled(interface, vif, source));
 
     return true;
 }
@@ -750,6 +712,10 @@ PeerManager<A>::recompute_addresses_peer(const OspfTypes::PeerID peerid,
 		       "PeerID %u", peerid);
 	    return false;
 	}
+	// Before trying to get the addresses verify that this
+	// interface/vif exists and has a usable address configured.
+	if (!enabled(interface, vif, _peers[peerid]->get_interface_address()))
+	    return false;
 	list<A> addresses;
 	if (!_ospf.get_addresses(interface, vif, addresses)) {
 	    XLOG_ERROR("Unable to find addresses on %s/%s ", interface.c_str(),
@@ -791,12 +757,11 @@ PeerManager<A>::recompute_addresses_peer(const OspfTypes::PeerID peerid,
 
 template <typename A>
 void
-PeerManager<A>::address_status_change(const string& interface,
-				      const string& vif, A source,
-				      bool state)
+PeerManager<A>::vif_status_change(const string& interface, const string& vif,
+				  bool state)
 {
-    debug_msg("interface %s vif %s address %s state %s\n",
-	      interface.c_str(), vif.c_str(), cstring(source), pb(state));
+    debug_msg("interface %s vif %s state %s\n",
+	      interface.c_str(), vif.c_str(), bool_c_str(state));
 
     OspfTypes::PeerID peerid;
 
@@ -813,6 +778,41 @@ PeerManager<A>::address_status_change(const string& interface,
 	return;
     }
 
+    _peers[peerid]->set_link_status(state);
+
+    return;
+}
+
+template <typename A>
+void
+PeerManager<A>::address_status_change(const string& interface,
+				      const string& vif, A source,
+				      bool state)
+{
+    debug_msg("interface %s vif %s address %s state %s\n",
+	      interface.c_str(), vif.c_str(), cstring(source),
+	      bool_c_str(state));
+
+    OspfTypes::PeerID peerid;
+
+    // All interface/vif/address changes on the host come through
+    // here, ignore the changes that are not for OSPF.
+    try {
+	peerid = get_peerid(interface, vif);
+    } catch(...) {
+	return;
+    }
+
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return;
+    }
+
+    _peers[peerid]->set_link_status(enabled(interface,
+					    vif,
+					    _peers[peerid]->
+					    get_interface_address()));
+
     switch(_ospf.get_version()) {
     case OspfTypes::V2:
 	break;
@@ -825,8 +825,6 @@ PeerManager<A>::address_status_change(const string& interface,
 	break;
     }
 
-    _peers[peerid]->set_link_status(state);
-
     return;
 }
 
@@ -892,6 +890,46 @@ PeerManager<A>::receive(const string& interface, const string& vif,
     return _peers[peerid]->receive(dst, src, packet);
 }
 
+template <typename A>
+bool
+PeerManager<A>::clear_database()
+{
+    // Drop all adjacencies.
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator p;
+    for(p = _peers.begin(); p != _peers.end(); p++) {
+	// If this peer is up then taking it down and bringing it up
+	// should drop all adjacencies.
+	if( (*p).second->get_state()) {
+	    (*p).second->set_state(false);
+	    (*p).second->set_state(true);
+	}
+    }
+
+    // Clear the AS-External-LSA database.
+    _external.clear_database();
+
+    // Clear the area databases.
+    typename map<OspfTypes::AreaID, AreaRouter<A> *>::const_iterator a;
+    for (a = _areas.begin(); a != _areas.end(); a++) {
+	// Changing the area type of the router will delete the
+	// Router-LSA generated by this router and revive it.
+	(*a).second->change_area_router_type((*a).second->get_area_type());
+    }
+
+    // Recompute the routes in all areas to withdraw the routes and
+    // remove virtual links. This will also cause the summary routes
+    // to be withdrawn. Should not be required as dropping the
+    // adjacencies should prompt a route recomputation.
+    routing_recompute_all_areas();
+
+    typename map<IPNet<A>, Summary>::iterator i;
+    for(i = _summaries.begin(); i != _summaries.end(); i++)
+	XLOG_WARNING("Summary not removed %s %s", cstring(i->first),
+		     cstring(i->second._rtentry));
+
+    return true;
+}
+
 template <typename A>
 bool
 PeerManager<A>::queue_lsa(const OspfTypes::PeerID peerid,
@@ -931,6 +969,20 @@ PeerManager<A>::get_interface_id(const OspfTypes::PeerID peerid)
     return _peers[peerid]->get_interface_id();
 }
 
+template <typename A>
+bool
+PeerManager<A>::get_attached_routers(const OspfTypes::PeerID peerid,
+				     OspfTypes::AreaID area,
+				     list<RouterInfo>& routers)
+{
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return false;
+    }
+
+    return _peers[peerid]->get_attached_routers(area, routers);
+}
+
 template <typename A>
 bool
 PeerManager<A>::configured_network(const A address) const
@@ -1633,21 +1685,21 @@ template <typename A>
 bool
 PeerManager<A>::set_passive(const OspfTypes::PeerID peerid,
 			    OspfTypes::AreaID area,
-			    bool passive)
+			    bool passive, bool host)
 {
     if (0 == _peers.count(peerid)) {
 	XLOG_ERROR("Unknown PeerID %u", peerid);
 	return false;
     }
 
-    return _peers[peerid]->set_passive(area, passive);
+    return _peers[peerid]->set_passive(area, passive, host);
 }
 
 template <typename A>
 bool
 PeerManager<A>::originate_default_route(OspfTypes::AreaID area, bool enable)
 {
-    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     AreaRouter<A> *area_router = get_area_router(area);
 
@@ -1681,7 +1733,7 @@ template <typename A>
 bool
 PeerManager<A>::summaries(OspfTypes::AreaID area, bool enable)
 {
-    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("Area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     AreaRouter<A> *area_router = get_area_router(area);
 
@@ -1777,6 +1829,13 @@ PeerManager<A>::external_push_routes()
     _external.push_routes();
 }
 
+template <typename A>
+void
+PeerManager<A>::external_suppress_lsas(OspfTypes::AreaID area)
+{
+    _external.suppress_lsas(area);
+}
+
 template <typename A>
 void
 PeerManager<A>::routing_recompute_all_areas()
@@ -1817,6 +1876,9 @@ PeerManager<A>::summary_candidate(OspfTypes::AreaID area, IPNet<A> net,
     debug_msg("Area %s net %s rentry %s\n", pr_id(area).c_str(),
 	      cstring(net), cstring(rt));
 
+    if (_ospf.get_testing())
+	return false;
+
     // RFC 2328 Section 12.4.3. Summary-LSAs
     // Select routes that are candidate for summarisation.
 
@@ -1870,10 +1932,12 @@ PeerManager<A>::summary_announce(OspfTypes::AreaID area, IPNet<A> net,
     if (!summary_candidate(area, net, rt))
 	return;
 
+    _external.suppress_route_announce(area, net, rt);
+
     // Save this route for possible later replay.
     XLOG_ASSERT(0 == _summaries.count(net));
     Summary s(area, rt);
-    _summaries[net] = s;
+    _summaries.insert(make_pair(net, s));
 
     if (!area_border_router_p())
 	return;
@@ -1894,6 +1958,8 @@ PeerManager<A>::summary_withdraw(OspfTypes::AreaID area, IPNet<A> net,
     if (!summary_candidate(area, net, rt))
 	return;
 
+    _external.suppress_route_withdraw(area, net, rt);
+
     // Remove this saved route.
     XLOG_ASSERT(1 == _summaries.count(net));
     _summaries.erase(_summaries.find(net));
@@ -1912,6 +1978,61 @@ PeerManager<A>::summary_withdraw(OspfTypes::AreaID area, IPNet<A> net,
 	    (*i).second->summary_withdraw(area, net, rt);
 }
 
+template <typename A>
+void
+PeerManager<A>::summary_replace(OspfTypes::AreaID area, IPNet<A> net,
+				RouteEntry<A>& rt,
+				RouteEntry<A>& previous_rt,
+				OspfTypes::AreaID previous_area)
+{
+    debug_msg("Area %s net %s rentry %s\n", pr_id(area).c_str(),
+	      cstring(net), cstring(rt));
+
+    bool previous = summary_candidate(previous_area, net, previous_rt);
+    bool current = summary_candidate(area, net, rt);
+
+    if (current != previous) {
+	if (previous)
+	    summary_withdraw(previous_area, net, previous_rt);
+	if (current)
+	    summary_announce(area, net, rt);
+	return;
+    }
+
+    XLOG_ASSERT(current == previous);
+
+    // Don't need to check for !previous as it is the same as current.
+    if (!current)
+	return;
+
+    XLOG_ASSERT(current);
+    XLOG_ASSERT(previous);
+
+    _external.suppress_route_withdraw(previous_area, net, previous_rt);
+    _external.suppress_route_announce(area, net, rt);
+    
+    XLOG_ASSERT(1 == _summaries.count(net));
+    _summaries.erase(_summaries.find(net));
+    Summary s(area, rt);
+    _summaries.insert(make_pair(net, s));
+
+    typename map<OspfTypes::AreaID, AreaRouter<A> *>::const_iterator i;
+    for (i = _areas.begin(); i != _areas.end(); i++) {
+	if ((*i).first == area) {
+	    if (area != previous_area)
+		(*i).second->summary_withdraw(previous_area, net,
+					      previous_rt);
+	    continue;
+	}
+	if ((*i).first == previous_area) {
+	    if (area != previous_area)
+		(*i).second->summary_announce(area, net, rt, false);
+	    continue;
+	}
+ 	(*i).second->summary_replace(area, net, rt, previous_rt,previous_area);
+    }
+}
+
 template <typename A>
 void
 PeerManager<A>::summary_push(OspfTypes::AreaID area)
@@ -2011,11 +2132,11 @@ PeerManager<A>::external_withdraw(const IPNet<A>& net)
 
 template <typename A>
 bool
-PeerManager<A>::external_shove(OspfTypes::AreaID area)
+PeerManager<A>::external_announce_complete(OspfTypes::AreaID area)
 {
     debug_msg("Area %s\n", pr_id(area).c_str());
 
-    return _external.shove(area);
+    return _external.announce_complete(area);
 }
 
 template <typename A>
