@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/routing_table.cc,v 1.60 2007/02/27 18:33:13 atanu Exp $"
+#ident "$XORP: xorp/ospf/routing_table.cc,v 1.68 2008/07/23 05:11:09 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -101,18 +101,18 @@ RoutingTable<A>::add_entry(OspfTypes::AreaID area, IPNet<A> net,
 
     if (rt.get_destination_type() == OspfTypes::Router) {
 	status = _adv.add_entry(area, rt.get_router_id(), rt);
-	switch(_ospf.get_version()) {
-	case OspfTypes::V2:
-	    break;
-	case OspfTypes::V3:
+// 	switch(_ospf.get_version()) {
+// 	case OspfTypes::V2:
+// 	    break;
+// 	case OspfTypes::V3:
 // 	    XLOG_ASSERT(!net.is_valid());
-	    if (net.is_valid()) {
-		XLOG_WARNING("Net should be zero %s", cstring(net));
-		status = false;
-	    }
-	    return status;
-	    break;
-	}
+// 	    if (net.is_valid()) {
+// 		XLOG_WARNING("Net should be zero %s", cstring(net));
+// 		status = false;
+// 	    }
+// 	    return status;
+// 	    break;
+// 	}
     }
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
@@ -140,18 +140,18 @@ RoutingTable<A>::replace_entry(OspfTypes::AreaID area, IPNet<A> net,
 
     if (rt.get_destination_type() == OspfTypes::Router) {
 	status = _adv.replace_entry(area, rt.get_router_id(), rt);
-	switch(_ospf.get_version()) {
-	case OspfTypes::V2:
-	    break;
-	case OspfTypes::V3:
+// 	switch(_ospf.get_version()) {
+// 	case OspfTypes::V2:
+// 	    break;
+// 	case OspfTypes::V3:
 // 	    XLOG_ASSERT(!net.is_valid());
-	    if (net.is_valid()) {
-		XLOG_WARNING("Net should be zero %s", cstring(net));
-		status = false;
-	    }
-	    return status;
-	    break;
-	}
+// 	    if (net.is_valid()) {
+// 		XLOG_WARNING("Net should be zero %s", cstring(net));
+// 		status = false;
+// 	    }
+// 	    return status;
+// 	    break;
+// 	}
     }
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
@@ -172,6 +172,9 @@ RoutingTable<A>::lookup_entry(A router, RouteEntry<A>& rt)
 {
     debug_msg("%s\n", cstring(router));
 
+    if (0 == _current)
+	return false;
+
     IPNet<A> net(router, A::ADDR_BITLEN);
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
@@ -193,6 +196,9 @@ RoutingTable<A>::lookup_entry(OspfTypes::AreaID area, A router,
 {
     debug_msg("area %s %s\n",  pr_id(area).c_str(), cstring(router));
 
+    if (0 == _current)
+	return false;
+
     IPNet<A> net(router, A::ADDR_BITLEN);
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
@@ -211,6 +217,9 @@ RoutingTable<A>::lookup_entry(IPNet<A> net, RouteEntry<A>& rt)
 {
     debug_msg("%s\n", cstring(net));
 
+    if (0 == _current)
+	return false;
+
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
     i = _current->lookup_node(net);
     if (_current->end() == i)
@@ -230,6 +239,9 @@ RoutingTable<A>::lookup_entry(OspfTypes::AreaID area, IPNet<A> net,
 {
     debug_msg("%s\n", cstring(net));
 
+    if (0 == _current)
+	return false;
+
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
     i = _current->lookup_node(net);
     if (_current->end() == i)
@@ -248,6 +260,9 @@ RoutingTable<A>::lookup_entry_by_advertising_router(OspfTypes::AreaID area,
 {
     debug_msg("area %s %s\n",  pr_id(area).c_str(), pr_id(adv).c_str());
 
+    if (0 == _current)
+	return false;
+
     return _adv.lookup_entry(area, adv, rt);
 }
 
@@ -257,6 +272,9 @@ RoutingTable<A>::longest_match_entry(A nexthop, RouteEntry<A>& rt)
 {
     debug_msg("%s\n", cstring(nexthop));
 
+    if (0 == _current)
+	return false;
+
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
     i = _current->find(nexthop);
     if (_current->end() == i)
@@ -287,7 +305,7 @@ RoutingTable<A>::end()
 	for (tic = _current->begin(); tic != _current->end(); tic++) {
 	    RouteEntry<A>& rt = tic.payload().get_entry();
 	    if (!add_route(rt.get_area(), tic.key(),
-			   rt.get_nexthop(), rt.get_cost(), rt)) {
+			   rt.get_nexthop(), rt.get_cost(), rt, true)) {
 		XLOG_WARNING("Add of %s failed", cstring(tic.key()));
 	    }
 	}
@@ -307,7 +325,7 @@ RoutingTable<A>::end()
     for (tip = _previous->begin(); tip != _previous->end(); tip++) {
 	if (_current->end() == _current->lookup_node(tip.key())) {
 	    RouteEntry<A>& rt = tip.payload().get_entry();
-	    if (!delete_route(rt.get_area(), tip.key(), rt)) {
+	    if (!delete_route(rt.get_area(), tip.key(), rt, true)) {
 		XLOG_WARNING("Delete of %s failed", cstring(tip.key()));
 	    }
 	}
@@ -318,7 +336,7 @@ RoutingTable<A>::end()
  	RouteEntry<A>& rt = tic.payload().get_entry();
 	if (_previous->end() == tip) {
 	    if (!add_route(rt.get_area(), tic.key(),
-			   rt.get_nexthop(), rt.get_cost(), rt)) {
+			   rt.get_nexthop(), rt.get_cost(), rt, true)) {
 		XLOG_WARNING("Add of %s failed", cstring(tic.key()));
 	    }
 	} else {
@@ -359,7 +377,7 @@ RoutingTable<A>::remove_area(OspfTypes::AreaID area)
 	// If the winning entry is for this area delete it from the
 	// routing table.
 	if (rt.get_area() == area)
-	    delete_route(area, tic.key(), rt);
+	    delete_route(area, tic.key(), rt, true);
 	    
 	// Unconditionally remove the area, it may be a losing route.
 	bool winner_changed;
@@ -375,7 +393,7 @@ RoutingTable<A>::remove_area(OspfTypes::AreaID area)
 	// If a new winner has emerged add it to the routing table.
 	if (winner_changed) {
 	    add_route(area, tic.key(), rt.get_nexthop(), rt.get_cost(),
-		      ire.get_entry());
+		      ire.get_entry(), true);
 	}
     }
 }
@@ -383,7 +401,7 @@ RoutingTable<A>::remove_area(OspfTypes::AreaID area)
 template <typename A>
 bool
 RoutingTable<A>::add_route(OspfTypes::AreaID area, IPNet<A> net, A nexthop,
-			   uint32_t metric, RouteEntry<A>& rt)
+			   uint32_t metric, RouteEntry<A>& rt, bool summaries)
 {
     debug_msg("ADD ROUTE area %s net %s nexthop %s metric %u\n",
 	      pr_id(area).c_str(), cstring(net), cstring(nexthop), metric);
@@ -403,7 +421,8 @@ RoutingTable<A>::add_route(OspfTypes::AreaID area, IPNet<A> net, A nexthop,
 	result = false;
     }
 
-    _ospf.get_peer_manager().summary_announce(area, net, rt);
+    if (summaries)
+	_ospf.get_peer_manager().summary_announce(area, net, rt);
 
     return result;
 }
@@ -411,10 +430,10 @@ RoutingTable<A>::add_route(OspfTypes::AreaID area, IPNet<A> net, A nexthop,
 template <typename A>
 bool
 RoutingTable<A>::delete_route(OspfTypes::AreaID area, IPNet<A> net,
-			      RouteEntry<A>& rt)
+			      RouteEntry<A>& rt, bool summaries)
 {
     debug_msg("DELETE ROUTE area %s %s filtered %s\n", pr_id(area).c_str(), 
-	      cstring(net), pb(rt.get_filtered()));
+	      cstring(net), bool_c_str(rt.get_filtered()));
 
     bool result;
     if (!rt.get_discard()) {
@@ -427,7 +446,8 @@ RoutingTable<A>::delete_route(OspfTypes::AreaID area, IPNet<A> net,
 	result = false;
     }
 
-    _ospf.get_peer_manager().summary_withdraw(area, net, rt);
+    if (summaries)
+	_ospf.get_peer_manager().summary_withdraw(area, net, rt);
 
     return result;
 }
@@ -441,10 +461,13 @@ RoutingTable<A>::replace_route(OspfTypes::AreaID area, IPNet<A> net, A nexthop,
 {
     debug_msg("REPLACE ROUTE area %s %s\n", pr_id(area).c_str(), cstring(net));
 
-    bool result = delete_route(previous_area, net, previous_rt);
+    bool result = delete_route(previous_area, net, previous_rt, false);
     if (!result)
 	XLOG_WARNING("Failed to delete: %s", cstring(net));
-    result = add_route(area, net, nexthop, metric, rt);
+    result = add_route(area, net, nexthop, metric, rt, false);
+
+    _ospf.get_peer_manager().summary_replace(area, net, rt, previous_rt,
+					     previous_area);
 
     return result;
 }
@@ -637,7 +660,11 @@ InternalRouteEntry<A>::reset_winner()
 	    continue;
 	}
 	RouteEntry<A>& comp = i->second;
-	if (comp.get_path_type() <= _winner->get_path_type())
+	if (comp.get_path_type() < _winner->get_path_type()) {
+	    _winner = &comp;
+	    continue;
+	}
+	if (comp.get_path_type() == _winner->get_path_type()) {
 	    if (comp.get_cost() < _winner->get_cost()) {
 		_winner = &comp;
 		continue;
@@ -647,6 +674,7 @@ InternalRouteEntry<A>::reset_winner()
 		    _winner = &comp;
 		continue;
 	    }
+	}
     }
 
     return _winner != old_winner;
