@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001-2007 International Computer Science Institute
+# Copyright (c) 2001-2008 XORP, Inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 # notice is a summary of the XORP LICENSE file; the license in that file is
 # legally binding.
 
-# $XORP: xorp/ospf/test_routing1.py,v 1.25 2007/03/19 09:02:21 atanu Exp $
+# $XORP: xorp/ospf/test_routing1.py,v 1.29 2008/07/23 05:11:10 pavlin Exp $
 
 import getopt
 import sys
@@ -35,6 +35,12 @@ TESTS=[
     ['r5V3', True, 'v3'],
     ['r6V3', True, 'v3'],
     ['r7V3', True, 'v3'],
+    ['r8V2', True, 'v2'],
+    ['r9V2', True, 'v2'],
+    ['r10V2', True, 'v2'],
+    ['r11V3', True, 'v3'],
+    ['r12V3', True, 'v3'],
+    ['r13V3', True, 'v3'],
     ]
 
 def start_routing_interactive(verbose, protocol):
@@ -797,6 +803,463 @@ verify_routing_entry  5f00:0000:c001:0200::/56 5f00::0002 2 false false
     else:
         return False
 
+def r8V2(verbose, protocol):
+    """
+    Simple two router case the next two tests will add another link between
+    RT1 and RT2.
+    """
+
+    # RT1 is this router
+    # RT2 is its direct neighbour
+    # A single route should be installed for 10.1.1.0/24 via 10.1.2.1
+			       
+    #            +-----+                     +-----+
+    #   	 | RT1 |                     | RT2 |
+    #   	 |     |                     |     |
+    #   	 |     |                     |     |
+    # 10.2.2.0/24|     |                     |     |10.1.1.0/24
+    # -----------|     |                     |     |------------
+    #   	 |     |     10.1.2.0/24     |     |         
+    #   	 |     |---------------------|	   |
+    #   	 |     |10.1.2.2     10.1.2.1|     |
+    #   	 |     |     metric 1        |	   |
+    #   	 +-----+        	     +-----+
+
+    fp = start_routing_interactive(verbose, protocol)
+
+    RT1 = "RouterLsa E-bit lsid 127.1.0.2 adv 127.1.0.2 \
+    stub lsid 10.2.2.0 ldata  255.255.255.0 metric 1 \
+    transit lsid 10.1.2.2 ldata 10.1.2.2 metric 1 \
+    "
+
+    RT2 = "RouterLsa E-bit lsid 127.1.0.1 adv 127.1.0.1 \
+    stub lsid 10.1.1.0 ldata  255.255.255.0 metric 1 \
+    transit lsid 10.1.2.2 ldata 10.1.2.1 metric 1 \
+    "
+
+    RT1_NETWORK1 = "NetworkLsa E-bit lsid 10.1.2.2 adv 127.1.0.2 \
+    netmask 0xffffff00 \
+    add-router 127.1.0.2 \
+    add-router 127.1.0.1 \
+    "
+
+    RT1_NETWORK2 = "NetworkLsa E-bit lsid  99.1.1.2 adv 127.1.0.2 \
+    netmask 0xffffff00 \
+    add-router 127.1.0.2 \
+    add-router 127.1.0.1 \
+    "
+
+    command = """
+set_router_id 127.1.0.2
+create 0.0.0.0 normal
+select 0.0.0.0
+replace %s
+add %s
+add %s
+add %s
+compute 0.0.0.0
+verify_routing_table_size 1
+verify_routing_entry 10.1.1.0/24 10.1.2.1 2 false false
+""" % (RT1, RT2, RT1_NETWORK1, RT1_NETWORK2)
+
+    print >>fp, command
+
+    if not fp.close():
+        return True
+    else:
+        return False
+
+def r9V2(verbose, protocol):
+    """
+    Test that when there are two paths between two routers the one
+    with the lowest metric is choosen. This test is the same as the
+    previous test apart from the addition of the more expensive path.
+    """
+
+    # RT1 is this router
+    # RT2 is its direct neighbour via two interfaces.
+    # A single route should be installed for 10.1.1.0/24 via 10.1.2.1
+
+    #            +-----+                     +-----+
+    #   	 | RT1 |     99.1.2.0/24     | RT2 |
+    #   	 |     |---------------------|     |
+    #   	 |     |99.1.2.2     99.1.2.1|     |
+    # 10.2.2.0/24|     |     metric 10       |     |10.1.1.0/24
+    # -----------|     |                     |     |------------
+    #   	 |     |     10.1.2.0/24     |     |         
+    #   	 |     |---------------------|	   |
+    #   	 |     |10.1.2.2     10.1.2.1|     |
+    #   	 |     |     metric 1        |	   |
+    #   	 +-----+                     +-----+
+        	       	       
+    fp = start_routing_interactive(verbose, protocol)
+
+    RT1 = "RouterLsa E-bit lsid 127.1.0.2 adv 127.1.0.2 \
+    age 1074 seqno 0x8000001a cksum 0x9976 \
+    transit lsid 99.1.1.2 ldata 99.1.1.2 metric 10 \
+    stub lsid 10.2.2.0 ldata  255.255.255.0 metric 1 \
+    transit lsid 10.1.2.2 ldata 10.1.2.2 metric 1 \
+    "
+
+    RT2 = "RouterLsa E-bit lsid 127.1.0.1 adv 127.1.0.1 \
+    age 1076 seqno 0x8000001a cksum 0x4cc9 \
+    transit lsid 99.1.1.2 ldata 99.1.1.1 metric 10 \
+    stub lsid 10.1.1.0 ldata  255.255.255.0 metric 1 \
+    transit lsid 10.1.2.2 ldata 10.1.2.1 metric 1 \
+    "
+ 
+    RT1_NETWORK1 = "NetworkLsa E-bit lsid 10.1.2.2 adv 127.1.0.2 \
+    age 1075 seqno 0x80000019 cksum 0x3676 \
+    netmask 0xffffff00 \
+    add-router 127.1.0.2 \
+    add-router 127.1.0.1 \
+    "
+
+    RT1_NETWORK2 = "NetworkLsa E-bit lsid  99.1.1.2 adv 127.1.0.2 \
+    age 1074 seqno 0x80000019 cksum 0xb79c \
+    netmask 0xffffff00 \
+    add-router 127.1.0.2 \
+    add-router 127.1.0.1 \
+    "
+
+    command = """
+set_router_id 127.1.0.2
+create 0.0.0.0 normal
+select 0.0.0.0
+replace %s
+add %s
+add %s
+add %s
+compute 0.0.0.0
+verify_routing_table_size 1
+verify_routing_entry 10.1.1.0/24 10.1.2.1 2 false false
+""" % (RT1, RT2, RT1_NETWORK1, RT1_NETWORK2)
+
+    print >>fp, command
+
+    if not fp.close():
+        return True
+    else:
+        return False
+
+def r10V2(verbose, protocol):
+    """
+    Test that when there are two paths between two routers the one
+    with the lowest metric is choosen. This test is identical to the
+    previous test apart from the order of the transit links in the
+    Router-LSAs.
+    """
+
+    # RT1 is this router
+    # RT2 is its direct neighbour via two interfaces.
+    # A single route should be installed for 10.1.1.0/24 via 10.1.2.1
+
+    #            +-----+                     +-----+
+    #   	 | RT1 |     99.1.2.0/24     | RT2 |
+    #   	 |     |---------------------|     |
+    #   	 |     |99.1.2.2     99.1.2.1|     |
+    # 10.2.2.0/24|     |     metric 10       |     |10.1.1.0/24
+    # -----------|     |                     |     |------------
+    #   	 |     |     10.1.2.0/24     |     |         
+    #   	 |     |---------------------|	   |
+    #   	 |     |10.1.2.2     10.1.2.1|     |
+    #   	 |     |     metric 1        |	   |
+    #   	 +-----+                     +-----+
+
+    fp = start_routing_interactive(verbose, protocol)
+
+    RT1 = "RouterLsa E-bit lsid 127.1.0.2 adv 127.1.0.2 \
+    age 1074 seqno 0x8000001a cksum 0x9976 \
+    transit lsid 10.1.2.2 ldata 10.1.2.2 metric 1 \
+    stub lsid 10.2.2.0 ldata  255.255.255.0 metric 1 \
+    transit lsid 99.1.1.2 ldata 99.1.1.2 metric 10 \
+    "
+
+    RT2 = "RouterLsa E-bit lsid 127.1.0.1 adv 127.1.0.1 \
+    age 1076 seqno 0x8000001a cksum 0x4cc9 \
+    transit lsid 10.1.2.2 ldata 10.1.2.1 metric 1 \
+    stub lsid 10.1.1.0 ldata  255.255.255.0 metric 1 \
+    transit lsid 99.1.1.2 ldata 99.1.1.1 metric 10 \
+    "
+ 
+    RT1_NETWORK1 = "NetworkLsa E-bit lsid 10.1.2.2 adv 127.1.0.2 \
+    age 1075 seqno 0x80000019 cksum 0x3676 \
+    netmask 0xffffff00 \
+    add-router 127.1.0.2 \
+    add-router 127.1.0.1 \
+    "
+
+    RT1_NETWORK2 = "NetworkLsa E-bit lsid  99.1.1.2 adv 127.1.0.2 \
+    age 1074 seqno 0x80000019 cksum 0xb79c \
+    netmask 0xffffff00 \
+    add-router 127.1.0.2 \
+    add-router 127.1.0.1 \
+    "
+
+    command = """
+set_router_id 127.1.0.2
+create 0.0.0.0 normal
+select 0.0.0.0
+replace %s
+add %s
+add %s
+add %s
+compute 0.0.0.0
+verify_routing_table_size 1
+verify_routing_entry 10.1.1.0/24 10.1.2.1 2 false false
+""" % (RT1, RT2, RT1_NETWORK1, RT1_NETWORK2)
+
+    print >>fp, command
+
+    if not fp.close():
+        return True
+    else:
+        return False
+
+def r11database():
+    """
+    Return the database for test r11V3
+    """
+
+    # RT1 is this router.
+    # RTC is an area border router that generates an Inter-Area-Prefix-LSA.
+    # RT1 is connected to RTC via transit.
+
+    #                            +
+    #     +---+1       1+---+1   |
+    #     |RT1|---------|RTC|----| Other Area
+    #     +---+         +---+    |
+    #                            +
+
+    # Network           IPv6 prefix
+    # -----------------------------------
+    # Other Area	3ffe:4725:c404::/64
+
+    # Router	Interface   Interface ID   link-local address
+    # -------------------------------------------------------
+    # RT1	to RTC	    4		   fe80::8:800:200c:4300
+    # RTC	to RT1	    3		   fe80::8:800:200c:4350
+
+    RT1 = "RouterLsa V6-bit E-bit R-bit lsid 0.0.0.0 adv 10.80.52.103 \
+    transit iid 4 nid 4 nrid 10.80.52.103 metric 1 \
+    "
+    RT1_LINK = "LinkLsa lsid 0.0.0.4 adv 10.80.52.103 \
+    link-local-address fe80::8:800:200c:4300"
+
+    RT1_NETWORK = "NetworkLsa lsid 0.0.0.4 adv 10.80.52.103 \
+    add-router 10.80.52.103 \
+    add-router 10.80.52.107 \
+    "
+
+    RTC = "RouterLsa bit-B V6-bit E-bit R-bit lsid 0.0.0.1 adv 10.80.52.107 \
+    transit iid 3 nid 4 nrid 10.80.52.103 metric 1 \
+    "
+
+    RTC_LINK = "LinkLsa lsid 0.0.0.3 adv 10.80.52.107 \
+    link-local-address fe80::8:800:200c:4350"
+
+    RTC_INTER = "SummaryNetworkLsa lsid 0.0.0.2 adv 10.80.52.107 \
+    metric 1 \
+    IPv6Prefix 3ffe:4725:c404::/64 \
+    "
+
+    database = """
+replace %s
+add %s
+add %s
+add %s
+add %s
+add %s
+""" % (RT1,RT1_LINK,RT1_NETWORK,
+       RTC,RTC_LINK,RTC_INTER)
+
+    return database
+
+def r11V3(verbose, protocol):
+    """
+    Verify the correct processing of Inter-Area-Prefix-LSAs, when this router
+    is connected to the generating router by a Network-LSA.
+    """
+
+    fp = start_routing_interactive(verbose, protocol)
+
+    command = """
+set_router_id 10.80.52.103
+create 0.0.0.0 normal
+select 0.0.0.0
+%s
+compute 0.0.0.0
+verify_routing_table_size 1
+verify_routing_entry 3ffe:4725:c404::/64 fe80::8:800:200c:4350 2 false false
+""" % r11database()
+
+    print >>fp, command
+
+    if not fp.close():
+        return True
+    else:
+        return False
+
+def r12database():
+    """
+    Return the database for test r12V3
+    """
+
+    # RT1 is this router.
+    # RTB is an area border router that generates an Intra-Area-Prefix-LSA.
+    # RT1 is connected to RTB via transit.
+
+    #                            +
+    #     +---+1       1+---+1   |
+    #     |RT1|---------|RTB|----| Other Area
+    #     +---+         +---+    |
+    #                            +
+
+    # Network           IPv6 prefix
+    # -----------------------------------
+    # Other Area	3ffe:4725:c404::/64
+
+    # Router	Interface   Interface ID   link-local address
+    # -------------------------------------------------------
+    # RT1	to RTB	    5		   fe80::8:800:200c:4200
+    # RTB	to RT1	    2		   fe80::8:800:200c:4250
+
+    RT1 = "RouterLsa V6-bit E-bit R-bit lsid 0.0.0.0 adv 10.80.52.103 \
+    transit iid 5 nid 5 nrid 10.80.52.103 metric 1 \
+    "
+    RT1_LINK = "LinkLsa lsid 0.0.0.5 adv 10.80.52.103 \
+    link-local-address fe80::8:800:200c:4200"
+
+    RT1_NETWORK = "NetworkLsa lsid 0.0.0.5 adv 10.80.52.103 \
+    add-router 10.80.52.103 \
+    add-router 10.80.52.106 \
+    "
+
+    RTB = "RouterLsa bit-B V6-bit E-bit R-bit lsid 0.0.0.1 adv 10.80.52.106 \
+    transit iid 2 nid 5 nrid 10.80.52.103 metric 1 \
+    "
+
+    RTB_LINK = "LinkLsa lsid 0.0.0.2 adv 10.80.52.106 \
+    link-local-address fe80::8:800:200c:4250"
+
+    RTB_INTRA = "IntraAreaPrefixLsa lsid 0.0.0.2 adv 10.80.52.106 \
+    rlstype 0x2001 rlsid 0.0.0.0 radv 10.80.52.106 \
+    IPv6Prefix 3ffe:4725:c404::/64 metric 3 \
+    "
+
+    database = """
+replace %s
+add %s
+add %s
+add %s
+add %s
+add %s
+""" % (RT1,RT1_LINK,RT1_NETWORK,
+       RTB,RTB_LINK,RTB_INTRA)
+
+    return database
+
+def r12V3(verbose, protocol):
+    """
+    Verify the correct processing of Intra-Area-Prefix-LSAs.
+    """
+
+    fp = start_routing_interactive(verbose, protocol)
+
+    command = """
+set_router_id 10.80.52.103
+create 36.0.0.0 normal
+select 36.0.0.0
+%s
+compute 36.0.0.0
+verify_routing_table_size 1
+verify_routing_entry 3ffe:4725:c404::/64 fe80::8:800:200c:4250 4 false false
+""" % r12database()
+
+    print >>fp, command
+
+    if not fp.close():
+        return True
+    else:
+        return False
+
+def r13V3(verbose, protocol):
+    """
+    Verify that Intra-Area-Prefix-LSAs always win over Inter-Area-Prefix-LSAs.
+    The same route is introduced into thebackbone 0.0.0.0 and area 36.0.0.0,
+    In the backbone 0.0.0.0 it is an Inter-Area-Prefix-LSA, in area 36.0.0.0
+    it is an Intra-Area-Prefix-LSA.
+
+    The two area databases are introduced in various orders and the
+    computations run to make sure that however the LSAs are introduced
+    the IntraAreaPrefixLsa always wins.
+    """
+
+    init = """
+set_router_id 10.80.52.103
+create 0.0.0.0 normal
+create 36.0.0.0 normal
+"""
+
+    area0 = """
+select 0.0.0.0
+%s
+""" % (r11database())
+
+    verify0 = """
+verify_routing_table_size 1
+verify_routing_entry 3ffe:4725:c404::/64 fe80::8:800:200c:4350 2 false false
+"""
+
+    area36 = """
+select 36.0.0.0
+%s
+""" % (r12database())
+
+    verify36 = """
+verify_routing_table_size 1
+verify_routing_entry 3ffe:4725:c404::/64 fe80::8:800:200c:4250 4 false false
+"""
+
+    fp = start_routing_interactive(verbose, protocol)
+
+    print >>fp, init
+    print >>fp, area0
+    print >>fp, "compute 0.0.0.0"
+    print >>fp, verify0
+    print >>fp, area36
+    print >>fp, "compute 36.0.0.0"
+    print >>fp, verify36
+
+    if fp.close():
+        return False
+
+    fp = start_routing_interactive(verbose, protocol)
+
+    print >>fp, init
+    print >>fp, area36
+    print >>fp, "compute 36.0.0.0"
+    print >>fp, verify36
+    print >>fp, area0
+    print >>fp, "compute 0.0.0.0"
+    print >>fp, verify36
+
+    if fp.close():
+        return False
+
+    fp = start_routing_interactive(verbose, protocol)
+
+    print >>fp, init
+    print >>fp, area0
+    print >>fp, area36
+    print >>fp, "compute 0.0.0.0"
+    print >>fp, verify0
+
+    if fp.close():
+        return False
+
+    return True
+
 def main():
     def usage():
         us = \
