@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/test_routing_table.cc,v 1.10 2007/03/14 09:14:06 atanu Exp $"
+#ident "$XORP: xorp/ospf/test_routing_table.cc,v 1.13 2008/07/23 05:11:10 pavlin Exp $"
 
 #define DEBUG_LOGGING
 #define DEBUG_PRINT_FUNCTION_NAME
@@ -623,7 +623,7 @@ main(int argc, char **argv)
 	{"r1v2", callback(routing1<IPv4>, OspfTypes::V2)},
 // 	{"r1v3", callback(routing1<IPv6>, OspfTypes::V3)},
 //	{"r2v2", callback(routing1<IPv4>, OspfTypes::V2)},
- 	{"r2v3", callback(routing2<IPv6>, OspfTypes::V3)},
+//  	{"r2v3", callback(routing2<IPv6>, OspfTypes::V3)},
     };
 
     try {
