@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/xrl_io.cc,v 1.40 2007/03/19 11:03:12 atanu Exp $"
+#ident "$XORP: xorp/ospf/xrl_io.cc,v 1.50 2008/07/23 05:11:10 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -48,21 +48,23 @@ XrlIO<A>::recv(const string& interface,
 	       const string& vif,
 	       A src,
 	       A dst,
-	       uint32_t ip_protocol,
+	       uint8_t ip_protocol,
 	       int32_t ip_ttl,
 	       int32_t ip_tos,
 	       bool ip_router_alert,
+	       bool ip_internet_control,
 	       const vector<uint8_t>& payload)
 {
     debug_msg("recv(interface = %s, vif = %s, src = %s, dst = %s, "
 	      "protocol = %u, ttl = %d tos = 0x%x router_alert = %s, "
-	      "payload_size = %u\n",
+	      "internet_control = %s payload_size = %u\n",
 	      interface.c_str(), vif.c_str(),
 	      src.str().c_str(), dst.str().c_str(),
 	      XORP_UINT_CAST(ip_protocol),
 	      XORP_INT_CAST(ip_ttl),
 	      ip_tos,
-	      (ip_router_alert)? "true" : "false",
+	      bool_c_str(ip_router_alert),
+	      bool_c_str(ip_internet_control),
 	      XORP_UINT_CAST(payload.size()));
 
     if (IO<A>::_receive_cb.is_empty())
@@ -106,6 +108,7 @@ XrlIO<IPv4>::send(const string& interface, const string& vif,
 	-1,		// XXX: let the FEA set it
 	-1,		// XXX: let the FEA set it
 	get_ip_router_alert(),
+	true,		// ip_internet_control
 	payload,
 	callback(this, &XrlIO::send_cb, interface, vif));
 
@@ -142,6 +145,7 @@ XrlIO<IPv6>::send(const string& interface, const string& vif,
 	dst.is_multicast() ? 1 : -1,		// XXX: let the FEA set it
 	-1,					// XXX: let the FEA set it
 	get_ip_router_alert(),
+	true,					// ip_internet_control
 	ext_headers_type,
 	ext_headers_payload,
 	payload,
@@ -197,7 +201,7 @@ XrlIO<IPv4>::enable_interface_vif(const string& interface, const string& vif)
     XrlRawPacket4V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_register_receiver(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -218,7 +222,7 @@ XrlIO<IPv6>::enable_interface_vif(const string& interface, const string& vif)
     XrlRawPacket6V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_register_receiver(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -276,7 +280,7 @@ XrlIO<IPv4>::disable_interface_vif(const string& interface, const string& vif)
     XrlRawPacket4V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_unregister_receiver(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -296,7 +300,7 @@ XrlIO<IPv6>::disable_interface_vif(const string& interface, const string& vif)
     XrlRawPacket6V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_unregister_receiver(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -348,7 +352,7 @@ XrlIO<A>::is_interface_enabled(const string& interface) const
 {
     debug_msg("Interface %s\n", interface.c_str());
 
-    const IfMgrIfAtom* fi = ifmgr_iftree().find_if(interface);
+    const IfMgrIfAtom* fi = ifmgr_iftree().find_interface(interface);
     if (fi == NULL)
 	return false;
 
@@ -422,7 +426,7 @@ XrlIO<IPv4>::get_addresses(const string& interface, const string& vif,
     if (fv == NULL)
 	return false;
 
-    IfMgrVifAtom::V4Map::const_iterator i;
+    IfMgrVifAtom::IPv4Map::const_iterator i;
     for (i = fv->ipv4addrs().begin(); i != fv->ipv4addrs().end(); i++)
 	addresses.push_back(i->second.addr());
 
@@ -440,7 +444,7 @@ XrlIO<IPv6>::get_addresses(const string& interface, const string& vif,
     if (fv == NULL)
 	return false;
 
-    IfMgrVifAtom::V6Map::const_iterator i;
+    IfMgrVifAtom::IPv6Map::const_iterator i;
     for (i = fv->ipv6addrs().begin(); i != fv->ipv6addrs().end(); i++)
 	addresses.push_back(i->second.addr());
 
@@ -458,7 +462,7 @@ XrlIO<IPv4>::get_link_local_address(const string& interface, const string& vif,
     if (fv == NULL)
 	return false;
 
-    IfMgrVifAtom::V4Map::const_iterator i;
+    IfMgrVifAtom::IPv4Map::const_iterator i;
     for (i = fv->ipv4addrs().begin(); i != fv->ipv4addrs().end(); i++) {
 	if (i->second.addr().is_linklocal_unicast()) {
 	    address = i->second.addr();
@@ -480,7 +484,7 @@ XrlIO<IPv6>::get_link_local_address(const string& interface, const string& vif,
     if (fv == NULL)
 	return false;
 
-    IfMgrVifAtom::V6Map::const_iterator i;
+    IfMgrVifAtom::IPv6Map::const_iterator i;
     for (i = fv->ipv6addrs().begin(); i != fv->ipv6addrs().end(); i++) {
 	if (i->second.addr().is_linklocal_unicast()) {
 	    address = i->second.addr();
@@ -497,7 +501,7 @@ XrlIO<A>::get_interface_id(const string& interface, uint32_t& interface_id)
 {
     debug_msg("Interface %s\n", interface.c_str());
 
-    const IfMgrIfAtom* fi = ifmgr_iftree().find_if(interface);
+    const IfMgrIfAtom* fi = ifmgr_iftree().find_interface(interface);
     if (fi == NULL)
 	return false;
 
@@ -546,11 +550,11 @@ XrlIO<A>::get_mtu(const string& interface)
 {
     debug_msg("Interface %s\n", interface.c_str());
 
-    const IfMgrIfAtom* fi = ifmgr_iftree().find_if(interface);
+    const IfMgrIfAtom* fi = ifmgr_iftree().find_interface(interface);
     if (fi == NULL)
 	return 0;
 
-    return (fi->mtu_bytes());
+    return (fi->mtu());
 }
 
 template <>
@@ -566,7 +570,7 @@ XrlIO<IPv4>::join_multicast_group(const string& interface, const string& vif,
     XrlRawPacket4V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_join_multicast_group(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -589,7 +593,7 @@ XrlIO<IPv6>::join_multicast_group(const string& interface, const string& vif,
     XrlRawPacket6V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_join_multicast_group(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -623,14 +627,14 @@ XrlIO<A>::join_multicast_group_cb(const XrlError& xrl_error, string interface,
 	break;
 
     case NO_FINDER:
-	// XXX - Temporarily code dump if this condition occurs.
+	// XXX - Temporarily core dump if this condition occurs.
 	XLOG_FATAL("NO FINDER");
 	break;
 
     case BAD_ARGS:
     case COMMAND_FAILED:
     case INTERNAL_ERROR:
-	XLOG_FATAL("Cannot join a multicast group on interface %s vif %s: %s",
+	XLOG_ERROR("Cannot join a multicast group on interface %s vif %s: %s",
 		   interface.c_str(), vif.c_str(), xrl_error.str().c_str());
 	break;
     }
@@ -649,7 +653,7 @@ XrlIO<IPv4>::leave_multicast_group(const string& interface, const string& vif,
     XrlRawPacket4V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_leave_multicast_group(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -672,7 +676,7 @@ XrlIO<IPv6>::leave_multicast_group(const string& interface, const string& vif,
     XrlRawPacket6V0p1Client fea_client(&_xrl_router);
     success = fea_client.send_leave_multicast_group(
 	_feaname.c_str(),
-	_class_name,		// XXX: maybe we should use _instance_name?
+	_xrl_router.instance_name(),
 	interface,
 	vif,
 	get_ip_protocol_number(),
@@ -829,8 +833,8 @@ XrlIO<A>::add_route(IPNet<A> net, A nexthop, uint32_t nexthop_id,
 		    const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
-	      cstring(net), cstring(nexthop), metric, equal ? "true" : "false",
-	      discard ? "true" : "false", cstring(policytags));
+	      cstring(net), cstring(nexthop), metric,
+	      bool_c_str(equal), bool_c_str(discard), cstring(policytags));
 
     _rib_queue.queue_add_route(_ribname, net, nexthop, nexthop_id, metric,
 			       policytags);
@@ -845,8 +849,8 @@ XrlIO<A>::replace_route(IPNet<A> net, A nexthop, uint32_t nexthop_id,
 			const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
-	      cstring(net), cstring(nexthop), metric, equal ? "true" : "false",
-	      discard ? "true" : "false", cstring(policytags));
+	      cstring(net), cstring(nexthop), metric,
+	      bool_c_str(equal), bool_c_str(discard), cstring(policytags));
 
     // XXX - The queue should support replace see TODO 36.
     _rib_queue.queue_delete_route(_ribname, net);
@@ -1184,7 +1188,7 @@ XrlIO<IPv4>::updates_made()
 {
     IfMgrIfTree::IfMap::const_iterator ii;
     IfMgrIfAtom::VifMap::const_iterator vi;
-    IfMgrVifAtom::V4Map::const_iterator ai;
+    IfMgrVifAtom::IPv4Map::const_iterator ai;
     const IfMgrIfAtom* if_atom;
     const IfMgrIfAtom* other_if_atom;
     const IfMgrVifAtom* vif_atom;
@@ -1195,7 +1199,7 @@ XrlIO<IPv4>::updates_made()
     //
     // Check whether the old interfaces, vifs and addresses are still there
     //
-    for (ii = _iftree.ifs().begin(); ii != _iftree.ifs().end(); ++ii) {
+    for (ii = _iftree.interfaces().begin(); ii != _iftree.interfaces().end(); ++ii) {
 	bool is_old_interface_enabled = false;
 	bool is_new_interface_enabled = false;
 	bool is_old_vif_enabled = false;
@@ -1208,7 +1212,7 @@ XrlIO<IPv4>::updates_made()
 	is_old_interface_enabled &= (! if_atom->no_carrier());
 
 	// Check the interface
-	other_if_atom = ifmgr_iftree().find_if(if_atom->name());
+	other_if_atom = ifmgr_iftree().find_interface(if_atom->name());
 	if (other_if_atom == NULL) {
 	    // The interface has disappeared
 	    is_new_interface_enabled = false;
@@ -1282,13 +1286,13 @@ XrlIO<IPv4>::updates_made()
     //
     // Check for new interfaces, vifs and addresses
     //
-    for (ii = ifmgr_iftree().ifs().begin();
-	 ii != ifmgr_iftree().ifs().end();
+    for (ii = ifmgr_iftree().interfaces().begin();
+	 ii != ifmgr_iftree().interfaces().end();
 	 ++ii) {
 	if_atom = &ii->second;
 
 	// Check the interface
-	other_if_atom = _iftree.find_if(if_atom->name());
+	other_if_atom = _iftree.find_interface(if_atom->name());
 	if (other_if_atom == NULL) {
 	    // A new interface
 	    if (if_atom->enabled()
@@ -1353,7 +1357,7 @@ XrlIO<IPv6>::updates_made()
 {
     IfMgrIfTree::IfMap::const_iterator ii;
     IfMgrIfAtom::VifMap::const_iterator vi;
-    IfMgrVifAtom::V6Map::const_iterator ai;
+    IfMgrVifAtom::IPv6Map::const_iterator ai;
     const IfMgrIfAtom* if_atom;
     const IfMgrIfAtom* other_if_atom;
     const IfMgrVifAtom* vif_atom;
@@ -1364,7 +1368,7 @@ XrlIO<IPv6>::updates_made()
     //
     // Check whether the old interfaces, vifs and addresses are still there
     //
-    for (ii = _iftree.ifs().begin(); ii != _iftree.ifs().end(); ++ii) {
+    for (ii = _iftree.interfaces().begin(); ii != _iftree.interfaces().end(); ++ii) {
 	bool is_old_interface_enabled = false;
 	bool is_new_interface_enabled = false;
 	bool is_old_vif_enabled = false;
@@ -1377,7 +1381,7 @@ XrlIO<IPv6>::updates_made()
 	is_old_interface_enabled &= (! if_atom->no_carrier());
 
 	// Check the interface
-	other_if_atom = ifmgr_iftree().find_if(if_atom->name());
+	other_if_atom = ifmgr_iftree().find_interface(if_atom->name());
 	if (other_if_atom == NULL) {
 	    // The interface has disappeared
 	    is_new_interface_enabled = false;
@@ -1451,13 +1455,13 @@ XrlIO<IPv6>::updates_made()
     //
     // Check for new interfaces, vifs and addresses
     //
-    for (ii = ifmgr_iftree().ifs().begin();
-	 ii != ifmgr_iftree().ifs().end();
+    for (ii = ifmgr_iftree().interfaces().begin();
+	 ii != ifmgr_iftree().interfaces().end();
 	 ++ii) {
 	if_atom = &ii->second;
 
 	// Check the interface
-	other_if_atom = _iftree.find_if(if_atom->name());
+	other_if_atom = _iftree.find_interface(if_atom->name());
 	if (other_if_atom == NULL) {
 	    // A new interface
 	    if (if_atom->enabled()
