@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/xrl_target.cc,v 1.54 2007/03/12 10:16:04 atanu Exp $"
+#ident "$XORP: xorp/ospf/xrl_target.cc,v 1.62 2008/07/23 05:11:11 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -167,6 +167,7 @@ XrlOspfV2Target::raw_packet4_client_0_1_recv(
     const int32_t&	ip_ttl,
     const int32_t&	ip_tos,
     const bool&		ip_router_alert,
+    const bool&		ip_internet_control,
     const vector<uint8_t>& payload)
 {
     _xrl_io.recv(if_name,
@@ -177,6 +178,7 @@ XrlOspfV2Target::raw_packet4_client_0_1_recv(
 		 ip_ttl,
 		 ip_tos,
 		 ip_router_alert,
+		 ip_internet_control,
 		 payload);
 
     return XrlCmdError::OKAY();
@@ -193,6 +195,7 @@ XrlOspfV3Target::raw_packet4_client_0_1_recv(
     const int32_t&	/*ip_ttl*/,
     const int32_t&	/*ip_tos*/,
     const bool&		/*ip_router_alert*/,
+    const bool&		/*ip_internet_control*/,
     const vector<uint8_t>& /*payload*/)
 {
 #if	0
@@ -204,6 +207,7 @@ XrlOspfV3Target::raw_packet4_client_0_1_recv(
 		      ip_ttl,
 		      ip_tos,
 		      ip_router_alert,
+		      ip_internet_control,
 		      payload);
 #else
     XLOG_UNFINISHED();
@@ -223,6 +227,7 @@ XrlOspfV3Target::raw_packet6_client_0_1_recv(
     const int32_t&	ip_ttl,
     const int32_t&	ip_tos,
     const bool&		ip_router_alert,
+    const bool&		ip_internet_control,
     const XrlAtomList&	ext_headers_type,
     const XrlAtomList&	ext_headers_payload,
     const vector<uint8_t>& payload)
@@ -238,6 +243,7 @@ XrlOspfV3Target::raw_packet6_client_0_1_recv(
 		      ip_ttl,
 		      ip_tos,
 		      ip_router_alert,
+		      ip_internet_control,
 		      payload);
 
     return XrlCmdError::OKAY();
@@ -334,8 +340,8 @@ XrlOspfV2Target::policy_redist4_0_1_add_route4(const IPv4Net& network,
 					       const XrlAtomList& policytags)
 {
     debug_msg("Net: %s Nexthop: %s Unicast: %s Multicast %s metric %d\n",
-	      cstring(network), cstring(nexthop), pb(unicast), pb(multicast),
-	      metric);
+	      cstring(network), cstring(nexthop), bool_c_str(unicast),
+	      bool_c_str(multicast), metric);
 
     if (!unicast)
 	return XrlCmdError::OKAY();
@@ -356,8 +362,8 @@ XrlOspfV3Target::policy_redist6_0_1_add_route6(const IPv6Net& network,
 					       const XrlAtomList& policytags)
 {
     debug_msg("Net: %s Nexthop: %s Unicast: %s Multicast %s metric %d\n",
-	      cstring(network), cstring(nexthop), pb(unicast), pb(multicast),
-	      metric);
+	      cstring(network), cstring(nexthop), bool_c_str(unicast),
+	      bool_c_str(multicast), metric);
 
     if (!unicast)
 	return XrlCmdError::OKAY();
@@ -375,7 +381,7 @@ XrlOspfV2Target::policy_redist4_0_1_delete_route4(const IPv4Net& network,
 						  const bool& multicast)
 {
     debug_msg("Net: %s Unicast: %s Multicast %s\n",
-	      cstring(network), pb(unicast), pb(multicast));
+	      cstring(network), bool_c_str(unicast), bool_c_str(multicast));
 
     if (!unicast)
 	return XrlCmdError::OKAY();
@@ -393,7 +399,7 @@ XrlOspfV3Target::policy_redist6_0_1_delete_route6(const IPv6Net& network,
 						  const bool& multicast)
 {
     debug_msg("Net: %s Unicast: %s Multicast %s\n",
-	      cstring(network), pb(unicast), pb(multicast));
+	      cstring(network), bool_c_str(unicast), bool_c_str(multicast));
 
     if (!unicast)
 	return XrlCmdError::OKAY();
@@ -585,7 +591,8 @@ XrlOspfV3Target::ospfv3_0_1_create_peer(const string& ifname,
 	return XrlCmdError::COMMAND_FAILED("Unrecognised type " + type);
 
     try {
-	_ospf_ipv6.get_peer_manager().create_peer(ifname, vifname, IPv6(),
+	_ospf_ipv6.get_peer_manager().create_peer(ifname, vifname,
+						  IPv6::ZERO(),
 						  linktype, area);
     } catch(XorpException& e) {
 	return XrlCmdError::COMMAND_FAILED(e.str());
@@ -636,7 +643,7 @@ XrlOspfV2Target::ospfv2_0_1_set_peer_state(const string& ifname,
 					   const bool& enable)
 {
     debug_msg("interface %s vif %s enable %s\n", ifname.c_str(),
-	      vifname.c_str(), pb(enable));
+	      vifname.c_str(), bool_c_str(enable));
 
     OspfTypes::PeerID peerid;
     try {
@@ -656,7 +663,7 @@ XrlOspfV3Target::ospfv3_0_1_set_peer_state(const string& ifname,
 					   const bool& enable)
 {
     debug_msg("interface %s vif %s enable %s\n", ifname.c_str(),
-	      vifname.c_str(), pb(enable));
+	      vifname.c_str(), bool_c_str(enable));
 
     OspfTypes::PeerID peerid;
     try {
@@ -719,7 +726,7 @@ XrlOspfV3Target::ospfv3_0_1_set_address_state_peer(const string& ifname,
     OspfTypes::AreaID area = ntohl(a.addr());
     debug_msg("interface %s vif %s area %s address %s enable %s\n",
 	      ifname.c_str(), vifname.c_str(), pr_id(area).c_str(),
-	      cstring(addr), pb(enable));
+	      cstring(addr), bool_c_str(enable));
 
     OspfTypes::PeerID peerid;
     try {
@@ -1328,14 +1335,15 @@ XrlCmdError
 XrlOspfV2Target::ospfv2_0_1_set_passive(const string& ifname,
 					const string& vifname,
 					const IPv4& a,
-					const bool& passive)
+					const bool& passive,
+					const bool& host)
 {
     OspfTypes::AreaID area = ntohl(a.addr());
-    debug_msg("interface %s vif %s area %s passive %s\n",
+    debug_msg("interface %s vif %s area %s passive %s host %s\n",
 	      ifname.c_str(), vifname.c_str(), pr_id(area).c_str(),
-	      pb(passive));
+	      bool_c_str(passive), bool_c_str(host));
 
-    if (!_ospf.set_passive(ifname, vifname, area, passive))
+    if (!_ospf.set_passive(ifname, vifname, area, passive, host))
 	return XrlCmdError::COMMAND_FAILED("Failed to configure make passive");
 
     return XrlCmdError::OKAY();
@@ -1350,9 +1358,9 @@ XrlOspfV3Target::ospfv3_0_1_set_passive(const string& ifname,
     OspfTypes::AreaID area = ntohl(a.addr());
     debug_msg("interface %s vif %s area %s passive %s\n",
 	      ifname.c_str(), vifname.c_str(), pr_id(area).c_str(),
-	      pb(passive));
+	      bool_c_str(passive));
 
-    if (!_ospf_ipv6.set_passive(ifname, vifname, area, passive))
+    if (!_ospf_ipv6.set_passive(ifname, vifname, area, passive, false))
 	return XrlCmdError::COMMAND_FAILED("Failed to configure make passive");
 
     return XrlCmdError::OKAY();
@@ -1363,7 +1371,7 @@ XrlOspfV2Target::ospfv2_0_1_originate_default_route(const IPv4&	a,
 						    const bool&	enable)
 {
     OspfTypes::AreaID area = ntohl(a.addr());
-    debug_msg("area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     if (!_ospf.originate_default_route(area, enable))
 	return XrlCmdError::
@@ -1377,7 +1385,7 @@ XrlOspfV3Target::ospfv3_0_1_originate_default_route(const IPv4&	a,
 						    const bool&	enable)
 {
     OspfTypes::AreaID area = ntohl(a.addr());
-    debug_msg("area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     if (!_ospf_ipv6.originate_default_route(area, enable))
 	return XrlCmdError::
@@ -1419,7 +1427,7 @@ XrlOspfV2Target::ospfv2_0_1_summaries(const IPv4& a,
 				      const bool& enable)
 {
     OspfTypes::AreaID area = ntohl(a.addr());
-    debug_msg("area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     if (!_ospf.summaries(area, enable))
 	return XrlCmdError::
@@ -1433,7 +1441,7 @@ XrlOspfV3Target::ospfv3_0_1_summaries(const IPv4& a,
 				      const bool& enable)
 {
     OspfTypes::AreaID area = ntohl(a.addr());
-    debug_msg("area %s enable %s\n", pr_id(area).c_str(), pb(enable));
+    debug_msg("area %s enable %s\n", pr_id(area).c_str(), bool_c_str(enable));
 
     if (!_ospf_ipv6.summaries(area, enable))
 	return XrlCmdError::
@@ -1449,14 +1457,14 @@ XrlOspfV2Target::ospfv2_0_1_area_range_add(const IPv4& a,
 {
     OspfTypes::AreaID area = ntohl(a.addr());
     debug_msg("area %s net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     if (!_ospf.area_range_add(area, net, advertise))
 	return XrlCmdError::
 	    COMMAND_FAILED(c_format("Failed to add area range "
 				    "area %s net %s advertise %s\n",
 				    pr_id(area).c_str(), cstring(net),
-				    pb(advertise)));
+				    bool_c_str(advertise)));
 
     return XrlCmdError::OKAY();
 }
@@ -1468,14 +1476,14 @@ XrlOspfV3Target::ospfv3_0_1_area_range_add(const IPv4& a,
 {
     OspfTypes::AreaID area = ntohl(a.addr());
     debug_msg("area %s net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     if (!_ospf_ipv6.area_range_add(area, net, advertise))
 	return XrlCmdError::
 	    COMMAND_FAILED(c_format("Failed to add area range "
 				    "area %s net %s advertise %s\n",
 				    pr_id(area).c_str(), cstring(net),
-				    pb(advertise)));
+				    bool_c_str(advertise)));
 
     return XrlCmdError::OKAY();
 }
@@ -1521,14 +1529,14 @@ XrlOspfV2Target::ospfv2_0_1_area_range_change_state(const IPv4& a,
 {
     OspfTypes::AreaID area = ntohl(a.addr());
     debug_msg("area %s net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     if (!_ospf.area_range_change_state(area, net, advertise))
 	return XrlCmdError::
 	    COMMAND_FAILED(c_format("Failed to change area range "
 				    "area %s net %s advertise %s\n",
 				    pr_id(area).c_str(), cstring(net),
-				    pb(advertise)));
+				    bool_c_str(advertise)));
 
     return XrlCmdError::OKAY();
 }
@@ -1540,14 +1548,14 @@ XrlOspfV3Target::ospfv3_0_1_area_range_change_state(const IPv4& a,
 {
     OspfTypes::AreaID area = ntohl(a.addr());
     debug_msg("area %s net %s advertise %s\n", pr_id(area).c_str(),
-	      cstring(net), pb(advertise));
+	      cstring(net), bool_c_str(advertise));
 
     if (!_ospf_ipv6.area_range_change_state(area, net, advertise))
 	return XrlCmdError::
 	    COMMAND_FAILED(c_format("Failed to change area range "
 				    "area %s net %s advertise %s\n",
 				    pr_id(area).c_str(), cstring(net),
-				    pb(advertise)));
+				    bool_c_str(advertise)));
 
     return XrlCmdError::OKAY();
 }
@@ -1555,7 +1563,8 @@ XrlOspfV3Target::ospfv3_0_1_area_range_change_state(const IPv4& a,
 XrlCmdError
 XrlOspfV2Target::ospfv2_0_1_trace(const string&	tvar, const bool& enable)
 {
-    debug_msg("trace variable %s enable %s\n", tvar.c_str(), pb(enable));
+    debug_msg("trace variable %s enable %s\n", tvar.c_str(),
+	      bool_c_str(enable));
 
     if (tvar == "all") {
 	_ospf.trace().all(enable);
@@ -1570,7 +1579,8 @@ XrlOspfV2Target::ospfv2_0_1_trace(const string&	tvar, const bool& enable)
 XrlCmdError
 XrlOspfV3Target::ospfv3_0_1_trace(const string&	tvar, const bool& enable)
 {
-    debug_msg("trace variable %s enable %s\n", tvar.c_str(), pb(enable));
+    debug_msg("trace variable %s enable %s\n", tvar.c_str(),
+	      bool_c_str(enable));
 
     if (tvar == "all") {
 	_ospf_ipv6.trace().all(enable);
@@ -1597,7 +1607,8 @@ XrlOspfV2Target::XrlOspfV2Target::ospfv2_0_1_get_lsa(const IPv4& a,
 	return XrlCmdError::COMMAND_FAILED("Unable to get LSA");
 
     debug_msg("area %s index %u valid %s toohigh %s self %s\n",
-	      pr_id(area).c_str(), index, pb(valid), pb(toohigh), pb(self));
+	      pr_id(area).c_str(), index, bool_c_str(valid),
+	      bool_c_str(toohigh), bool_c_str(self));
 
     return XrlCmdError::OKAY();
 }
@@ -1617,7 +1628,8 @@ XrlOspfV3Target::XrlOspfV3Target::ospfv3_0_1_get_lsa(const IPv4& a,
 	return XrlCmdError::COMMAND_FAILED("Unable to get LSA");
 
     debug_msg("area %s index %u valid %s toohigh %s self %s\n",
-	      pr_id(area).c_str(), index, pb(valid), pb(toohigh), pb(self));
+	      pr_id(area).c_str(), index, bool_c_str(valid),
+	      bool_c_str(toohigh), bool_c_str(self));
 
     return XrlCmdError::OKAY();
 }
@@ -1757,3 +1769,22 @@ XrlOspfV3Target::ospfv3_0_1_get_neighbour_info(const uint32_t& nid,
 
     return XrlCmdError::OKAY();
 }
+
+XrlCmdError
+XrlOspfV2Target::ospfv2_0_1_clear_database()
+{
+    if (!_ospf.clear_database())
+	return XrlCmdError::COMMAND_FAILED("Unable clear database");
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlOspfV3Target::ospfv3_0_1_clear_database()
+{
+    if (!_ospf_ipv6.clear_database())
+	return XrlCmdError::COMMAND_FAILED("Unable clear database");
+
+   return XrlCmdError::OKAY();
+}
+
