@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_bsr.cc,v 1.49 2007/02/16 22:46:46 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_bsr.cc,v 1.53 2008/07/23 05:11:12 pavlin Exp $"
 
 
 //
@@ -27,11 +27,10 @@
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/random.h"
 #include "libxorp/ipvx.hh"
 #include "libxorp/utils.hh"
 
-#include "mrt/random.h"
-
 #include "pim_proto.h"
 #include "pim_bsr.hh"
 #include "pim_node.hh"
@@ -117,7 +116,7 @@ PimBsr::start()
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
-    if (ProtoUnit::start() < 0)
+    if (ProtoUnit::start() != XORP_OK)
 	return (XORP_ERROR);
     
     //
@@ -164,7 +163,7 @@ PimBsr::stop()
     if (is_down())
 	return (XORP_OK);
 
-    if (ProtoUnit::stop() < 0)
+    if (ProtoUnit::stop() != XORP_OK)
 	return (XORP_ERROR);
     
     //
@@ -1209,7 +1208,7 @@ PimBsr::send_test_bootstrap_by_dest(const string& vif_name,
 	 ++iter_zone) {
 	BsrZone *bsr_zone = *iter_zone;
 	if (pim_vif->pim_bootstrap_send(dest_addr, *bsr_zone, error_msg)
-	    < 0) {
+	    != XORP_OK) {
 	    ret_value = XORP_ERROR;
 	    goto ret_label;
 	}
@@ -1274,7 +1273,7 @@ PimBsr::send_test_cand_rp_adv()
 	}
 
 	if (pim_vif->pim_cand_rp_adv_send(bsr_zone->bsr_addr(), *bsr_zone)
-	    < 0) {
+	    != XORP_OK) {
 	    ret_value = XORP_ERROR;
 	    goto ret_label;
 	}
@@ -1361,7 +1360,7 @@ BsrZone::BsrZone(PimBsr& pim_bsr, const PimScopeZoneId& zone_id)
       _bsr_addr(IPvX::ZERO(_pim_bsr.family())),
       _bsr_priority(0),
       _hash_mask_len(PIM_BOOTSTRAP_HASH_MASK_LEN_DEFAULT(_pim_bsr.family())),
-      _fragment_tag(RANDOM(0xffff)),
+      _fragment_tag(xorp_random() % 0xffff),
       _is_accepted_message(false),
       _is_unicast_message(false),
       _unicast_message_src(IPvX::ZERO(_pim_bsr.family())),
