@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_config.cc,v 1.50 2007/02/16 22:46:46 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_config.cc,v 1.56 2008/07/23 05:11:13 pavlin Exp $"
 
 
 //
@@ -25,10 +25,9 @@
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/random.h"
 #include "libxorp/ipvx.hh"
 
-#include "mrt/random.h"
-
 #include "pim_node.hh"
 #include "pim_vif.hh"
 
@@ -160,9 +159,6 @@ PimNode::set_config_all_vifs_done(string& error_msg)
 	}
     }
     
-    // TODO: XXX: PAVPAVPAV: remove it!!
-    PimNode::set_vif_setup_completed(true);
-    
     if (end_config(error_msg) != XORP_OK)
 	return (XORP_ERROR);
     
@@ -205,7 +201,7 @@ PimNode::set_vif_proto_version(const string& vif_name, int proto_version,
 	return (XORP_ERROR);
     }
     
-    if (pim_vif->set_proto_version(proto_version) < 0) {
+    if (pim_vif->set_proto_version(proto_version) != XORP_OK) {
 	end_config(error_msg);
 	error_msg = c_format("Cannot set protocol version for vif %s: "
 			     "invalid protocol version %d",
@@ -245,81 +241,6 @@ PimNode::reset_vif_proto_version(const string& vif_name, string& error_msg)
     return (XORP_OK);
 }
 
-int
-PimNode::get_vif_ip_router_alert_option_check(const string& vif_name,
-					      bool& enabled,
-					      string& error_msg)
-{
-    PimVif *pim_vif = vif_find_by_name(vif_name);
-    
-    if (pim_vif == NULL) {
-	error_msg = c_format("Cannot get 'IP Router Alert option check' "
-			     "flag for vif %s: "
-			     "no such vif",
-			     vif_name.c_str());
-	return (XORP_ERROR);
-    }
-    
-    enabled = pim_vif->ip_router_alert_option_check().get();
-    
-    return (XORP_OK);
-}
-
-int
-PimNode::set_vif_ip_router_alert_option_check(const string& vif_name,
-					      bool enable,
-					      string& error_msg)
-{
-    PimVif *pim_vif = vif_find_by_name(vif_name);
-
-    if (start_config(error_msg) != XORP_OK)
-	return (XORP_ERROR);
-
-    if (pim_vif == NULL) {
-	end_config(error_msg);
-	error_msg = c_format("Cannot set 'IP Router Alert option check' "
-			     "flag for vif %s: "
-			     "no such vif",
-			     vif_name.c_str());
-	XLOG_ERROR("%s", error_msg.c_str());
-	return (XORP_ERROR);
-    }
-    
-    pim_vif->ip_router_alert_option_check().set(enable);
-    
-    if (end_config(error_msg) != XORP_OK)
-	return (XORP_ERROR);
-    
-    return (XORP_OK);
-}
-
-int
-PimNode::reset_vif_ip_router_alert_option_check(const string& vif_name,
-						string& error_msg)
-{
-    PimVif *pim_vif = vif_find_by_name(vif_name);
-    
-    if (start_config(error_msg) != XORP_OK)
-	return (XORP_ERROR);
-    
-    if (pim_vif == NULL) {
-	end_config(error_msg);
-	error_msg = c_format("Cannot reset 'IP Router Alert option check' "
-			     "flag for vif %s: "
-			     "no such vif",
-			     vif_name.c_str());
-	XLOG_ERROR("%s", error_msg.c_str());
-	return (XORP_ERROR);
-    }
-    
-    pim_vif->ip_router_alert_option_check().reset();
-    
-    if (end_config(error_msg) != XORP_OK)
-	return (XORP_ERROR);
-    
-    return (XORP_OK);
-}
-
 int
 PimNode::get_vif_hello_triggered_delay(const string& vif_name,
 				       uint16_t& hello_triggered_delay,
@@ -1254,7 +1175,7 @@ PimNode::add_config_cand_bsr(const IPvXNet& scope_zone_id,
 {
     PimVif *pim_vif = vif_find_by_name(vif_name);
     IPvX my_cand_bsr_addr = vif_addr;
-    uint16_t fragment_tag = RANDOM(0xffff);
+    uint16_t fragment_tag = xorp_random() % 0xffff;
     string local_error_msg = "";
     PimScopeZoneId zone_id(scope_zone_id, is_scope_zone);
     
@@ -1937,7 +1858,8 @@ PimNode::send_test_assert(const string& vif_name,
 				 rpt_bit,
 				 metric_preference,
 				 metric,
-				 error_msg) < 0) {
+				 error_msg)
+	!= XORP_OK) {
 	return (XORP_ERROR);
     }
     
@@ -1994,7 +1916,7 @@ PimNode::add_test_bsr_rp(const PimScopeZoneId& zone_id,
 int
 PimNode::send_test_bootstrap(const string& vif_name, string& error_msg)
 {
-    if (pim_bsr().send_test_bootstrap(vif_name, error_msg) < 0) {
+    if (pim_bsr().send_test_bootstrap(vif_name, error_msg) != XORP_OK) {
 	return (XORP_ERROR);
     }
     
@@ -2008,7 +1930,7 @@ PimNode::send_test_bootstrap_by_dest(const string& vif_name,
 {
     if (pim_bsr().send_test_bootstrap_by_dest(vif_name, dest_addr,
 					      error_msg)
-	< 0) {
+	!= XORP_OK) {
 	return (XORP_ERROR);
     }
     
@@ -2018,7 +1940,7 @@ PimNode::send_test_bootstrap_by_dest(const string& vif_name,
 int
 PimNode::send_test_cand_rp_adv()
 {
-    if (pim_bsr().send_test_cand_rp_adv() < 0) {
+    if (pim_bsr().send_test_cand_rp_adv() != XORP_OK) {
 	return (XORP_ERROR);
     }
     
