@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_node.cc,v 1.82 2007/02/16 22:46:48 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_node.cc,v 1.90 2008/07/23 05:11:14 pavlin Exp $"
 
 
 //
@@ -174,12 +174,13 @@ PimNode::start()
 	return (XORP_ERROR);
     }
 
-    if (ProtoNode<PimVif>::pending_start() < 0)
+    if (ProtoNode<PimVif>::pending_start() != XORP_OK)
 	return (XORP_ERROR);
 
     //
-    // Register with the MFEA
+    // Register with the FEA and MFEA
     //
+    fea_register_startup();
     mfea_register_startup();
 
     //
@@ -216,7 +217,7 @@ PimNode::final_start()
 	return (XORP_ERROR);
 #endif
 
-    if (ProtoNode<PimVif>::start() < 0) {
+    if (ProtoNode<PimVif>::start() != XORP_OK) {
 	ProtoNode<PimVif>::stop();
 	return (XORP_ERROR);
     }
@@ -225,7 +226,7 @@ PimNode::final_start()
     start_all_vifs();
     
     // Start the BSR module
-    if (_pim_bsr.start() < 0)
+    if (_pim_bsr.start() != XORP_OK)
 	return (XORP_ERROR);
 
     XLOG_INFO("Protocol started");
@@ -267,7 +268,7 @@ PimNode::stop()
 	return (XORP_ERROR);
     }
 
-    if (ProtoNode<PimVif>::pending_stop() < 0)
+    if (ProtoNode<PimVif>::pending_stop() != XORP_OK)
 	return (XORP_ERROR);
 
     //
@@ -305,7 +306,7 @@ PimNode::final_stop()
     if (! (is_up() || is_pending_up() || is_pending_down()))
 	return (XORP_ERROR);
 
-    if (ProtoNode<PimVif>::stop() < 0)
+    if (ProtoNode<PimVif>::stop() != XORP_OK)
 	return (XORP_ERROR);
 
     XLOG_INFO("Protocol stopped");
@@ -341,38 +342,379 @@ PimNode::disable()
     XLOG_INFO("Protocol disabled");
 }
 
+/**
+ * Get the IP protocol number.
+ *
+ * @return the IP protocol number.
+ */
+uint8_t
+PimNode::ip_protocol_number() const
+{
+    return (IPPROTO_PIM);
+}
+
 void
 PimNode::status_change(ServiceBase*  service,
 		       ServiceStatus old_status,
 		       ServiceStatus new_status)
 {
-    XLOG_ASSERT(this == service);
-
-    if ((old_status == SERVICE_STARTING)
-	&& (new_status == SERVICE_RUNNING)) {
-	// The startup process has completed
-	if (final_start() < 0) {
-	    XLOG_ERROR("Cannot complete the startup process; "
-		       "current state is %s",
-		       ProtoNode<PimVif>::state_str().c_str());
+    if (service == this) {
+	if ((old_status == SERVICE_STARTING)
+	    && (new_status == SERVICE_RUNNING)) {
+	    // The startup process has completed
+	    if (final_start() != XORP_OK) {
+		XLOG_ERROR("Cannot complete the startup process; "
+			   "current state is %s",
+			   ProtoNode<PimVif>::state_str().c_str());
+		return;
+	    }
+	    ProtoNode<PimVif>::set_node_status(PROC_READY);
 	    return;
 	}
-	ProtoNode<PimVif>::set_node_status(PROC_READY);
+
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
+	    // The shutdown process has completed
+	    final_stop();
+	    // Set the node status
+	    ProtoNode<PimVif>::set_node_status(PROC_DONE);
+	    return;
+	}
+	//
+	// TODO: check if there was an error
+	//
 	return;
     }
 
-    if ((old_status == SERVICE_SHUTTING_DOWN)
-	&& (new_status == SERVICE_SHUTDOWN)) {
-	// The shutdown process has completed
-	final_stop();
-	// Set the node status
-	ProtoNode<PimVif>::set_node_status(PROC_DONE);
-	return;
+    if (service == ifmgr_mirror_service_base()) {
+	if ((old_status == SERVICE_SHUTTING_DOWN)
+	    && (new_status == SERVICE_SHUTDOWN)) {
+	    decr_shutdown_requests_n();		// XXX: for the ifmgr
+	}
+    }
+}
+
+void
+PimNode::tree_complete()
+{
+    decr_startup_requests_n();			// XXX: for the ifmgr
+
+    //
+    // XXX: we use same actions when the tree is completed or updates are made
+    //
+    updates_made();
+}
+
+void
+PimNode::updates_made()
+{
+    map<string, Vif>::iterator pim_vif_iter;
+    string error_msg;
+
+    //
+    // Update the local copy of the interface tree
+    //
+    _iftree = ifmgr_iftree();
+
+    //
+    // Add new vifs and update existing ones
+    //
+    IfMgrIfTree::IfMap::const_iterator ifmgr_iface_iter;
+    for (ifmgr_iface_iter = _iftree.interfaces().begin();
+	 ifmgr_iface_iter != _iftree.interfaces().end();
+	 ++ifmgr_iface_iter) {
+	const IfMgrIfAtom& ifmgr_iface = ifmgr_iface_iter->second;
+
+	IfMgrIfAtom::VifMap::const_iterator ifmgr_vif_iter;
+	for (ifmgr_vif_iter = ifmgr_iface.vifs().begin();
+	     ifmgr_vif_iter != ifmgr_iface.vifs().end();
+	     ++ifmgr_vif_iter) {
+	    const IfMgrVifAtom& ifmgr_vif = ifmgr_vif_iter->second;
+	    const string& ifmgr_vif_name = ifmgr_vif.name();
+	    Vif* node_vif = NULL;
+	
+	    pim_vif_iter = configured_vifs().find(ifmgr_vif_name);
+	    if (pim_vif_iter != configured_vifs().end()) {
+		node_vif = &(pim_vif_iter->second);
+	    }
+
+	    //
+	    // Add a new vif
+	    //
+	    if (node_vif == NULL) {
+		uint32_t vif_index = ifmgr_vif.vif_index();
+		XLOG_ASSERT(vif_index != Vif::VIF_INDEX_INVALID);
+		if (add_config_vif(ifmgr_vif_name, vif_index, error_msg)
+		    != XORP_OK) {
+		    XLOG_ERROR("Cannot add vif %s to the set of configured "
+			       "vifs: %s",
+			       ifmgr_vif_name.c_str(), error_msg.c_str());
+		    continue;
+		}
+		pim_vif_iter = configured_vifs().find(ifmgr_vif_name);
+		XLOG_ASSERT(pim_vif_iter != configured_vifs().end());
+		node_vif = &(pim_vif_iter->second);
+		// FALLTHROUGH
+	    }
+
+	    //
+	    // Update the pif_index
+	    //
+	    set_config_pif_index(ifmgr_vif_name,
+				 ifmgr_vif.pif_index(),
+				 error_msg);
+	
+	    //
+	    // Update the vif flags
+	    //
+	    bool is_up = ifmgr_iface.enabled();
+	    is_up &= (! ifmgr_iface.no_carrier());
+	    is_up &= ifmgr_vif.enabled();
+	    set_config_vif_flags(ifmgr_vif_name,
+				 ifmgr_vif.pim_register(),
+				 ifmgr_vif.p2p_capable(),
+				 ifmgr_vif.loopback(),
+				 ifmgr_vif.multicast_capable(),
+				 ifmgr_vif.broadcast_capable(),
+				 is_up,
+				 ifmgr_iface.mtu(),
+				 error_msg);
+	
+	}
     }
 
     //
-    // TODO: check if there was an error
+    // Add new vif addresses, update existing ones, and remove old addresses
     //
+    for (ifmgr_iface_iter = _iftree.interfaces().begin();
+	 ifmgr_iface_iter != _iftree.interfaces().end();
+	 ++ifmgr_iface_iter) {
+	const IfMgrIfAtom& ifmgr_iface = ifmgr_iface_iter->second;
+	const string& ifmgr_iface_name = ifmgr_iface.name();
+	IfMgrIfAtom::VifMap::const_iterator ifmgr_vif_iter;
+
+	for (ifmgr_vif_iter = ifmgr_iface.vifs().begin();
+	     ifmgr_vif_iter != ifmgr_iface.vifs().end();
+	     ++ifmgr_vif_iter) {
+	    const IfMgrVifAtom& ifmgr_vif = ifmgr_vif_iter->second;
+	    const string& ifmgr_vif_name = ifmgr_vif.name();
+	    Vif* node_vif = NULL;
+
+	    //
+	    // Add new vif addresses and update existing ones
+	    //
+	    pim_vif_iter = configured_vifs().find(ifmgr_vif_name);
+	    if (pim_vif_iter != configured_vifs().end()) {
+		node_vif = &(pim_vif_iter->second);
+	    }
+
+	    if (is_ipv4()) {
+		IfMgrVifAtom::IPv4Map::const_iterator a4_iter;
+
+		for (a4_iter = ifmgr_vif.ipv4addrs().begin();
+		     a4_iter != ifmgr_vif.ipv4addrs().end();
+		     ++a4_iter) {
+		    const IfMgrIPv4Atom& a4 = a4_iter->second;
+		    VifAddr* node_vif_addr = node_vif->find_address(IPvX(a4.addr()));
+		    IPvX addr(a4.addr());
+		    IPvXNet subnet_addr(addr, a4.prefix_len());
+		    IPvX broadcast_addr(IPvX::ZERO(family()));
+		    IPvX peer_addr(IPvX::ZERO(family()));
+		    if (a4.has_broadcast())
+			broadcast_addr = IPvX(a4.broadcast_addr());
+		    if (a4.has_endpoint())
+			peer_addr = IPvX(a4.endpoint_addr());
+
+		    if (node_vif_addr == NULL) {
+			if (add_config_vif_addr(
+				ifmgr_vif_name,
+				addr,
+				subnet_addr,
+				broadcast_addr,
+				peer_addr,
+				error_msg)
+			    != XORP_OK) {
+			    XLOG_ERROR("Cannot add address %s to vif %s from "
+				       "the set of configured vifs: %s",
+				       cstring(addr), ifmgr_vif_name.c_str(),
+				       error_msg.c_str());
+			}
+			continue;
+		    }
+		    if ((addr == node_vif_addr->addr())
+			&& (subnet_addr == node_vif_addr->subnet_addr())
+			&& (broadcast_addr == node_vif_addr->broadcast_addr())
+			&& (peer_addr == node_vif_addr->peer_addr())) {
+			continue;	// Nothing changed
+		    }
+
+		    // Update the address
+		    if (delete_config_vif_addr(ifmgr_vif_name,
+					       addr,
+					       error_msg)
+			!= XORP_OK) {
+			XLOG_ERROR("Cannot delete address %s from vif %s "
+				   "from the set of configured vifs: %s",
+				   cstring(addr),
+				   ifmgr_vif_name.c_str(),
+				   error_msg.c_str());
+		    }
+		    if (add_config_vif_addr(
+			    ifmgr_vif_name,
+			    addr,
+			    subnet_addr,
+			    broadcast_addr,
+			    peer_addr,
+			    error_msg)
+			!= XORP_OK) {
+			XLOG_ERROR("Cannot add address %s to vif %s from "
+				   "the set of configured vifs: %s",
+				   cstring(addr), ifmgr_vif_name.c_str(),
+				   error_msg.c_str());
+		    }
+		}
+	    }
+
+	    if (is_ipv6()) {
+		IfMgrVifAtom::IPv6Map::const_iterator a6_iter;
+
+		for (a6_iter = ifmgr_vif.ipv6addrs().begin();
+		     a6_iter != ifmgr_vif.ipv6addrs().end();
+		     ++a6_iter) {
+		    const IfMgrIPv6Atom& a6 = a6_iter->second;
+		    VifAddr* node_vif_addr = node_vif->find_address(IPvX(a6.addr()));
+		    IPvX addr(a6.addr());
+		    IPvXNet subnet_addr(addr, a6.prefix_len());
+		    IPvX broadcast_addr(IPvX::ZERO(family()));
+		    IPvX peer_addr(IPvX::ZERO(family()));
+		    if (a6.has_endpoint())
+			peer_addr = IPvX(a6.endpoint_addr());
+
+		    if (node_vif_addr == NULL) {
+			if (add_config_vif_addr(
+				ifmgr_vif_name,
+				addr,
+				subnet_addr,
+				broadcast_addr,
+				peer_addr,
+				error_msg)
+			    != XORP_OK) {
+			    XLOG_ERROR("Cannot add address %s to vif %s from "
+				       "the set of configured vifs: %s",
+				       cstring(addr), ifmgr_vif_name.c_str(),
+				       error_msg.c_str());
+			}
+			continue;
+		    }
+		    if ((addr == node_vif_addr->addr())
+			&& (subnet_addr == node_vif_addr->subnet_addr())
+			&& (peer_addr == node_vif_addr->peer_addr())) {
+			continue;	// Nothing changed
+		    }
+
+		    // Update the address
+		    if (delete_config_vif_addr(ifmgr_vif_name,
+					       addr,
+					       error_msg)
+			!= XORP_OK) {
+			XLOG_ERROR("Cannot delete address %s from vif %s "
+				   "from the set of configured vifs: %s",
+				   cstring(addr),
+				   ifmgr_vif_name.c_str(),
+				   error_msg.c_str());
+		    }
+		    if (add_config_vif_addr(
+			    ifmgr_vif_name,
+			    addr,
+			    subnet_addr,
+			    broadcast_addr,
+			    peer_addr,
+			    error_msg)
+			!= XORP_OK) {
+			XLOG_ERROR("Cannot add address %s to vif %s from "
+				   "the set of configured vifs: %s",
+				   cstring(addr), ifmgr_vif_name.c_str(),
+				   error_msg.c_str());
+		    }
+		}
+	    }
+
+	    //
+	    // Delete vif addresses that don't exist anymore
+	    //
+	    {
+		list<IPvX> delete_addresses_list;
+		list<VifAddr>::const_iterator vif_addr_iter;
+		for (vif_addr_iter = node_vif->addr_list().begin();
+		     vif_addr_iter != node_vif->addr_list().end();
+		     ++vif_addr_iter) {
+		    const VifAddr& vif_addr = *vif_addr_iter;
+		    if (vif_addr.addr().is_ipv4()
+			&& (_iftree.find_addr(ifmgr_iface_name,
+					      ifmgr_vif_name,
+					      vif_addr.addr().get_ipv4()))
+			    == NULL) {
+			    delete_addresses_list.push_back(vif_addr.addr());
+		    }
+		    if (vif_addr.addr().is_ipv6()
+			&& (_iftree.find_addr(ifmgr_iface_name,
+					      ifmgr_vif_name,
+					      vif_addr.addr().get_ipv6()))
+			    == NULL) {
+			    delete_addresses_list.push_back(vif_addr.addr());
+		    }
+		}
+
+		// Delete the addresses
+		list<IPvX>::iterator ipvx_iter;
+		for (ipvx_iter = delete_addresses_list.begin();
+		     ipvx_iter != delete_addresses_list.end();
+		     ++ipvx_iter) {
+		    const IPvX& ipvx = *ipvx_iter;
+		    if (delete_config_vif_addr(ifmgr_vif_name, ipvx, error_msg)
+			!= XORP_OK) {
+			XLOG_ERROR("Cannot delete address %s from vif %s from "
+				   "the set of configured vifs: %s",
+				   cstring(ipvx), ifmgr_vif_name.c_str(),
+				   error_msg.c_str());
+		    }
+		}
+	    }
+	}
+    }
+
+    //
+    // Remove vifs that don't exist anymore
+    //
+    list<string> delete_vifs_list;
+    for (pim_vif_iter = configured_vifs().begin();
+	 pim_vif_iter != configured_vifs().end();
+	 ++pim_vif_iter) {
+	Vif* node_vif = &pim_vif_iter->second;
+#if 0
+	if (node_vif->is_pim_register())
+	    continue;		// XXX: don't delete the PIM Register vif
+#endif
+	if (_iftree.find_vif(node_vif->name(), node_vif->name()) == NULL) {
+	    // Add the vif to the list of old interfaces
+	    delete_vifs_list.push_back(node_vif->name());
+	}
+    }
+    // Delete the old vifs
+    list<string>::iterator vif_name_iter;
+    for (vif_name_iter = delete_vifs_list.begin();
+	 vif_name_iter != delete_vifs_list.end();
+	 ++vif_name_iter) {
+	const string& vif_name = *vif_name_iter;
+	if (delete_config_vif(vif_name, error_msg) != XORP_OK) {
+	    XLOG_ERROR("Cannot delete vif %s from the set of configured "
+		       "vifs: %s",
+		       vif_name.c_str(), error_msg.c_str());
+	}
+    }
+    
+    // Done
+    set_config_all_vifs_done(error_msg);
 }
 
 /**
@@ -709,9 +1051,14 @@ PimNode::delete_vif_addr(const string& vif_name,
 	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
-
     VifAddr vif_addr = *tmp_vif_addr;	// Get a copy
 
+    //
+    // Get the vif's old primary address and whether the vif is UP
+    //
+    bool old_vif_is_up = pim_vif->is_up() || pim_vif->is_pending_up();
+    IPvX old_primary_addr = pim_vif->primary_addr();
+
     //
     // Spec:
     // "Before an interface goes down or changes primary IP address, a Hello
@@ -724,6 +1071,16 @@ PimNode::delete_vif_addr(const string& vif_name,
 	}
     }
 
+    //
+    // If an interface's primary address is deleted, first stop the vif.
+    //
+    if (old_vif_is_up) {
+	if (pim_vif->primary_addr() == addr) {
+	    string dummy_error_msg;
+	    pim_vif->stop(dummy_error_msg);
+	}
+    }
+
     if (pim_vif->delete_address(addr) != XORP_OK) {
 	XLOG_UNREACHABLE();
 	return (XORP_ERROR);
@@ -733,26 +1090,32 @@ PimNode::delete_vif_addr(const string& vif_name,
 	      pim_vif->name().c_str(), vif_addr.str().c_str());
 
     //
-    // Update the primary and domain-wide addresses.
-    // If the vif has no more primary or a domain-wide address, then stop it.
+    // Update and check the primary and domain-wide addresses.
+    // If the vif has no primary or a domain-wide address, then stop it.
+    // If the vif's primary address was changed, then restart the vif.
     //
     do {
+	string dummy_error_msg;
+
 	if (pim_vif->update_primary_and_domain_wide_address(error_msg)
-	    == XORP_OK) {
+	    != XORP_OK) {
+	    XLOG_ERROR("Error updating primary and domain-wide addresses "
+		       "for vif %s: %s",
+		       pim_vif->name().c_str(), error_msg.c_str());
+	}
+	if (pim_vif->primary_addr().is_zero()
+	    || pim_vif->domain_wide_addr().is_zero()) {
+	    pim_vif->stop(dummy_error_msg);
 	    break;
 	}
-	if (! (pim_vif->is_up() || pim_vif->is_pending_up())) {
-	    // XXX: don't do anything if the interface is not UP or PENDING_UP
-	    break;
-	}
-	if (pim_vif->is_loopback()) {
-	    // XXX: don't do anything if this is a loopback interface
-	    break;
-	}
-	XLOG_ERROR("Error updating primary and domain-wide addresses "
-		   "for vif %s: %s",
-		   pim_vif->name().c_str(), error_msg.c_str());
-	pim_vif->stop(error_msg);
+	if (old_primary_addr == pim_vif->primary_addr())
+	    break;		// Nothing changed
+
+	// Conditionally restart the interface
+	pim_vif->stop(dummy_error_msg);
+	if (old_vif_is_up)
+	    pim_vif->start(dummy_error_msg);
+	break;
     } while (false);
 
     //
@@ -1061,53 +1424,50 @@ PimNode::vif_shutdown_completed(const string& vif_name)
 	rib_register_shutdown();
 
 	//
-	// De-register with the MFEA
+	// De-register with the FEA and MFEA
 	//
 	mfea_register_shutdown();
+	fea_register_shutdown();
     }
 
     UNUSED(vif_name);
 }
 
-/**
- * PimNode::proto_recv:
- * @src_module_instance_name: The module instance name of the module-origin
- * of the message.
- * @src_module_id: The #xorp_module_id of the module-origin of the message.
- * @vif_index: The vif index of the interface used to receive this message.
- * @src: The source address of the message.
- * @dst: The destination address of the message.
- * @ip_ttl: The IP TTL of the message. If it has a negative value,
- * it should be ignored.
- * @ip_tos: The IP TOS of the message. If it has a negative value,
- * it should be ignored.
- * @is_router_alert: If true, the Router Alert IP option for the IP
- * packet of the incoming message was set.
- * @rcvbuf: The data buffer with the received message.
- * @rcvlen: The data length in @rcvbuf.
- * @error_msg: The error message (if error).
- * 
- * Receive a protocol message from the kernel.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
-PimNode::proto_recv(const string&	, // src_module_instance_name,
-		    xorp_module_id src_module_id,
-		    uint32_t vif_index,
-		    const IPvX& src, const IPvX& dst,
-		    int ip_ttl, int ip_tos, bool is_router_alert,
-		    const uint8_t *rcvbuf, size_t rcvlen,
+PimNode::proto_recv(const string& if_name,
+		    const string& vif_name,
+		    const IPvX& src_address,
+		    const IPvX& dst_address,
+		    uint8_t ip_protocol,
+		    int32_t ip_ttl,
+		    int32_t ip_tos,
+		    bool ip_router_alert,
+		    bool ip_internet_control,
+		    const vector<uint8_t>& payload,
 		    string& error_msg)
 {
     PimVif *pim_vif = NULL;
     int ret_value = XORP_ERROR;
-    
-    debug_msg("Received message from %s to %s on vif_index %d: "
-	      "ip_ttl = %d ip_tos = %#x router_alert = %d rcvlen = %u\n",
-	      cstring(src), cstring(dst), vif_index,
-	      ip_ttl, ip_tos, is_router_alert, XORP_UINT_CAST(rcvlen));
-    
+
+    debug_msg("Received message on %s/%s from %s to %s: "
+	      "ip_ttl = %d ip_tos = %#x ip_router_alert = %d "
+	      "ip_internet_control = %d rcvlen = %u\n",
+	      if_name.c_str(), vif_name.c_str(),
+	      cstring(src_address), cstring(dst_address),
+	      ip_ttl, ip_tos, ip_router_alert, ip_internet_control,
+	      XORP_UINT_CAST(payload.size()));
+
+    UNUSED(if_name);
+    UNUSED(ip_ttl);
+    UNUSED(ip_tos);
+    UNUSED(ip_router_alert);
+    UNUSED(ip_internet_control);
+    //
+    // XXX: We registered to receive only one protocol, hence we ignore
+    // the ip_protocol value.
+    //
+    UNUSED(ip_protocol);
+ 
     //
     // Check whether the node is up.
     //
@@ -1119,67 +1479,52 @@ PimNode::proto_recv(const string&	, // src_module_instance_name,
     //
     // Find the vif for that packet
     //
-    pim_vif = vif_find_by_vif_index(vif_index);
+    pim_vif = vif_find_by_name(vif_name);
     if (pim_vif == NULL) {
-	error_msg = c_format("Cannot find vif with vif_index = %u", vif_index);
+	error_msg = c_format("Cannot find vif with vif_name = %s",
+			     vif_name.c_str());
 	return (XORP_ERROR);
     }
     
     // Copy the data to the receiving #buffer_t
     BUFFER_RESET(_buffer_recv);
-    BUFFER_PUT_DATA(rcvbuf, _buffer_recv, rcvlen);
+    BUFFER_PUT_DATA(&payload[0], _buffer_recv, payload.size());
     
     // Process the data by the vif
-    ret_value = pim_vif->pim_recv(src, dst, ip_ttl, ip_tos,
-				  is_router_alert,
-				  _buffer_recv);
+    ret_value = pim_vif->pim_recv(src_address, dst_address, _buffer_recv);
     
     return (ret_value);
     
  buflen_error:
     XLOG_UNREACHABLE();
     return (XORP_ERROR);
-    
-    UNUSED(src_module_id);
 }
 
-/**
- * PimNode::pim_send:
- * @vif_index: The vif index of the interface to send this message.
- * @src: The source address of the message.
- * @dst: The destination address of the message.
- * @ip_ttl: The IP TTL of the message. If it has a negative value,
- * the TTL will be set by the lower layers.
- * @ip_tos: The IP TOS of the message. If it has a negative value,
- * the TOS will be set by the lower layers.
- * @is_router_alert: If true, set the Router Alert IP option for the IP
- * packet of the outgoung message.
- * @buffer: The #buffer_t data buffer with the message to send.
- * @error_msg: The error message (if error).
- * 
- * Send a PIM message.
- * 
- * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
- **/
 int
-PimNode::pim_send(uint32_t vif_index,
-		  const IPvX& src, const IPvX& dst,
-		  int ip_ttl, int ip_tos, bool is_router_alert,
-		  buffer_t *buffer, string& error_msg)
+PimNode::pim_send(const string& if_name,
+		  const string& vif_name,
+		  const IPvX& src_address,
+		  const IPvX& dst_address,
+		  uint8_t ip_protocol,
+		  int32_t ip_ttl,
+		  int32_t ip_tos,
+		  bool ip_router_alert,
+		  bool ip_internet_control,
+		  buffer_t *buffer,
+		  string& error_msg)
 {
     if (! (is_up() || is_pending_down())) {
 	error_msg = c_format("MLD/IGMP node is not UP");
 	return (XORP_ERROR);
     }
     
-    // TODO: the target name of the MFEA must be configurable.
-    if (proto_send(xorp_module_name(family(), XORP_MODULE_MFEA),
-		   XORP_MODULE_MFEA,
-		   vif_index, src, dst,
-		   ip_ttl, ip_tos, is_router_alert,
+    if (proto_send(if_name, vif_name, src_address, dst_address,
+		   ip_protocol, ip_ttl, ip_tos, ip_router_alert,
+		   ip_internet_control,
 		   BUFFER_DATA_HEAD(buffer),
 		   BUFFER_DATA_SIZE(buffer),
-		   error_msg) < 0) {
+		   error_msg)
+	!= XORP_OK) {
 	return (XORP_ERROR);
     }
     
@@ -1190,7 +1535,6 @@ PimNode::pim_send(uint32_t vif_index,
  * PimNode::signal_message_recv:
  * @src_module_instance_name: The module instance name of the module-origin
  * of the message.
- * @src_module_id: The #xorp_module_id of the module-origin of the message.
  * @message_type: The message type of the kernel signal.
  * At this moment, one of the following:
  * %MFEA_KERNEL_MESSAGE_NOCACHE (if a cache-miss in the kernel)
@@ -1210,7 +1554,6 @@ PimNode::pim_send(uint32_t vif_index,
  **/
 int
 PimNode::signal_message_recv(const string& src_module_instance_name,
-			     xorp_module_id src_module_id,
 			     int message_type,
 			     uint32_t vif_index,
 			     const IPvX& src,
@@ -1224,7 +1567,6 @@ PimNode::signal_message_recv(const string& src_module_instance_name,
 	if (message_type == MFEA_KERNEL_MESSAGE_NOCACHE) {
 	    ret_value = pim_mrt().signal_message_nocache_recv(
 		src_module_instance_name,
-		src_module_id,
 		vif_index,
 		src,
 		dst);
@@ -1233,7 +1575,6 @@ PimNode::signal_message_recv(const string& src_module_instance_name,
 	if (message_type == MFEA_KERNEL_MESSAGE_WRONGVIF) {
 	    ret_value = pim_mrt().signal_message_wrongvif_recv(
 		src_module_instance_name,
-		src_module_id,
 		vif_index,
 		src,
 		dst);
@@ -1242,7 +1583,6 @@ PimNode::signal_message_recv(const string& src_module_instance_name,
 	if (message_type == MFEA_KERNEL_MESSAGE_WHOLEPKT) {
 	    ret_value = pim_mrt().signal_message_wholepkt_recv(
 		src_module_instance_name,
-		src_module_id,
 		vif_index,
 		src,
 		dst,
