@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_node_cli.cc,v 1.41 2007/02/16 22:46:49 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_node_cli.cc,v 1.45 2008/07/23 05:11:15 pavlin Exp $"
 
 
 //
@@ -82,10 +82,10 @@ PimNodeCli::start()
     if (is_up() || is_pending_up())
 	return (XORP_OK);
 
-    if (ProtoUnit::start() < 0)
+    if (ProtoUnit::start() != XORP_OK)
 	return (XORP_ERROR);
 
-    if (add_all_cli_commands() < 0)
+    if (add_all_cli_commands() != XORP_OK)
 	return (XORP_ERROR);
 
     XLOG_INFO("CLI started");
@@ -101,10 +101,10 @@ PimNodeCli::stop()
     if (is_down())
 	return (XORP_OK);
 
-    if (ProtoUnit::stop() < 0)
+    if (ProtoUnit::stop() != XORP_OK)
 	return (XORP_ERROR);
     
-    if (delete_all_cli_commands() < 0)
+    if (delete_all_cli_commands() != XORP_OK)
 	ret_code = XORP_ERROR;
 
     XLOG_INFO("CLI stopped");
@@ -1116,8 +1116,7 @@ PimNodeCli::cli_print_pim_mre(const PimMre *pim_mre)
     }
     if (pim_mre->is_sg()) {
 	cli_print(c_format("    KAT(S,G) running:          %s\n",
-			   pim_mre->is_keepalive_timer_running()?
-			   "true" : "false"));
+			   bool_c_str(pim_mre->is_keepalive_timer_running())));
     }
     
     
