@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2007 International Computer Science Institute
+ * Copyright (c) 2001-2008 XORP, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,7 +15,7 @@
  */
 
 /*
- * $XORP: xorp/pim/pim_proto.h,v 1.23 2007/02/16 22:46:49 pavlin Exp $
+ * $XORP: xorp/pim/pim_proto.h,v 1.26 2008/07/23 05:11:15 pavlin Exp $
  */
 
 
@@ -33,6 +33,9 @@
 /* XXX: _PIM_VT is needed if we want the extra features of <netinet/pim.h> */
 #define _PIM_VT 1
 
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
 #ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
 #endif
@@ -68,6 +71,10 @@
 /*
  * Constants definitions
  */
+#ifndef IPPROTO_PIM
+#define IPPROTO_PIM				103
+#endif
+
 /* PIM versions definition */
 #define PIMSM_V1				1
 #define PIMSM_V2				2
