@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_vif.cc,v 1.65 2007/02/16 22:46:51 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_vif.cc,v 1.72 2008/07/23 05:11:16 pavlin Exp $"
 
 
 //
@@ -24,12 +24,11 @@
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/random.h"
 #include "libxorp/ipvx.hh"
 
 #include "libproto/checksum.h"
 
-#include "mrt/random.h"
-
 #include "pim_node.hh"
 #include "pim_vif.hh"
 
@@ -69,7 +68,6 @@ PimVif::PimVif(PimNode& pim_node, const Vif& vif)
       _dr_addr(pim_node.family()),
       _pim_nbr_me(*this, IPvX::ZERO(pim_node.family()), PIM_VERSION_DEFAULT),
       _domain_wide_addr(IPvX::ZERO(pim_node.family())),
-      _ip_router_alert_option_check(false),
       _hello_triggered_delay(PIM_HELLO_HELLO_TRIGGERED_DELAY_DEFAULT),
       _hello_period(PIM_HELLO_HELLO_PERIOD_DEFAULT,
 		    callback(this, &PimVif::set_hello_period_callback)),
@@ -87,7 +85,7 @@ PimVif::PimVif(PimNode& pim_node, const Vif& vif)
 				    callback(this,
 					     &PimVif::set_is_tracking_support_disabled_callback)),
       _accept_nohello_neighbors(false),
-      _genid(RANDOM(0xffffffffU),
+      _genid(xorp_random() % 0xffffffffU,
 	     callback(this, &PimVif::set_genid_callback)),
       _join_prune_period(PIM_JOIN_PRUNE_PERIOD_DEFAULT,
 			 callback(this,
@@ -222,7 +220,7 @@ PimVif::set_default_config()
     accept_nohello_neighbors().reset();
     
     // Hello-related non-configurable parameters
-    genid().set(RANDOM(0xffffffffU));
+    genid().set(xorp_random() % 0xffffffffU);
     
     // Join/Prune-related parameters
     _join_prune_period.reset();
@@ -294,19 +292,37 @@ PimVif::start(string& error_msg)
 	return (XORP_ERROR);
     }
 
-    if (update_primary_and_domain_wide_address(error_msg) < 0)
+    if (update_primary_and_domain_wide_address(error_msg) != XORP_OK)
 	return (XORP_ERROR);
 
-    if (ProtoUnit::start() < 0) {
+    if (ProtoUnit::start() != XORP_OK) {
 	error_msg = "internal error";
 	return (XORP_ERROR);
     }
     
     //
-    // Start the vif with the kernel
+    // Register as a receiver with the kernel
     //
-    if (pim_node().start_protocol_kernel_vif(vif_index()) != XORP_OK) {
-	error_msg = c_format("cannot start protocol vif %s with the kernel",
+    if (pim_node().register_receiver(name(),
+				     name(),
+				     pim_node().ip_protocol_number(),
+				     false)
+	!= XORP_OK) {
+	error_msg = c_format("cannot register as a receiver on vif %s "
+			     "with the kernel",
+			     name().c_str());
+	return (XORP_ERROR);
+    }
+
+    //
+    // Register as a protocol with the MFEA
+    //
+    if (pim_node().register_protocol(name(),
+				     name(),
+				     pim_node().ip_protocol_number())
+	!= XORP_OK) {
+	error_msg = c_format("cannot register as a protocol on vif %s "
+			     "with the MFEA",
 			     name().c_str());
 	return (XORP_ERROR);
     }
@@ -315,10 +331,14 @@ PimVif::start(string& error_msg)
 	//
 	// Join the appropriate multicast groups: ALL-PIM-ROUTERS
 	//
-	const IPvX group1 = IPvX::PIM_ROUTERS(family());
-	if (pim_node().join_multicast_group(vif_index(), group1) != XORP_OK) {
+	const IPvX group = IPvX::PIM_ROUTERS(family());
+	if (pim_node().join_multicast_group(name(),
+					    name(),
+					    pim_node().ip_protocol_number(),
+					    group)
+	    != XORP_OK) {
 	    error_msg = c_format("cannot join group %s on vif %s",
-				 cstring(group1), name().c_str());
+				 cstring(group), name().c_str());
 	    return (XORP_ERROR);
 	}
 	
@@ -369,7 +389,7 @@ PimVif::stop(string& error_msg)
 	return (XORP_ERROR);
     }
 
-    if (ProtoUnit::pending_stop() < 0) {
+    if (ProtoUnit::pending_stop() != XORP_OK) {
 	error_msg = "internal error";
 	ret_value = XORP_ERROR;
     }
@@ -385,7 +405,7 @@ PimVif::stop(string& error_msg)
     // Add the shutdown operation of this vif as a shutdown task
     // for the node.
     //
-    pim_node().incr_shutdown_requests_n();
+    pim_node().incr_shutdown_requests_n();	// XXX: for PIM-stop-vif
 
     if (! is_pim_register()) {
 	//
@@ -437,7 +457,7 @@ PimVif::final_stop(string& error_msg)
     // automatically when we stop the vif through the MFEA.
     //
     
-    if (ProtoUnit::stop() < 0) {
+    if (ProtoUnit::stop() != XORP_OK) {
 	error_msg = "internal error";
 	ret_value = XORP_ERROR;
     }
@@ -455,14 +475,26 @@ PimVif::final_stop(string& error_msg)
     }
     
     //
-    // Stop the vif with the kernel
+    // Unregister as a protocol with the MFEA
     //
-    if (pim_node().stop_protocol_kernel_vif(vif_index()) != XORP_OK) {
-	XLOG_ERROR("Cannot stop protocol vif %s with the kernel",
+    if (pim_node().unregister_protocol(name(), name()) != XORP_OK) {
+	XLOG_ERROR("Cannot unregister as a protocol on vif %s with the MFEA",
 		   name().c_str());
 	ret_value = XORP_ERROR;
     }
-    
+
+    //
+    // Unregister as a receiver with the kernel
+    //
+    if (pim_node().unregister_receiver(name(),
+				       name(),
+				       pim_node().ip_protocol_number())
+	!= XORP_OK) {
+	XLOG_ERROR("Cannot unregister as a receiver on vif %s with the kernel",
+		   name().c_str());
+	ret_value = XORP_ERROR;
+    }
+
     XLOG_INFO("Interface stopped: %s%s",
 	      this->str().c_str(), flags_string().c_str());
 
@@ -475,7 +507,7 @@ PimVif::final_stop(string& error_msg)
     // Remove the shutdown operation of this vif as a shutdown task
     // for the node.
     //
-    pim_node().decr_shutdown_requests_n();
+    pim_node().decr_shutdown_requests_n();	// XXX: for PIM-stop-vif
 
     return (ret_value);
 }
@@ -538,11 +570,7 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     int ret_value;
     size_t datalen;
     int ttl = MINTTL;
-    //
-    // XXX: According to newer revisions of the PIM-SM spec, the PIM-SM control
-    // messages don't include the IP Router Alert option.
-    //
-    bool is_router_alert = false;
+    bool ip_internet_control = true;	// XXX: might be overwritten below
 
     if (! (is_up() || is_pending_down()))
 	return (XORP_ERROR);
@@ -575,10 +603,12 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     if (dst.is_unicast()) {
 	switch (message_type) {
 	case PIM_REGISTER:
+	    // XXX: The ip_tos is copied from the inner IP header
+	    ip_internet_control = false;
+	    // FALLTHROUGH
 	case PIM_REGISTER_STOP:
 	case PIM_CAND_RP_ADV:
 	    ttl = IPDEFTTL;
-	    is_router_alert = false;
 	    break;
 	default:
 	    break;
@@ -704,13 +734,18 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     //
     // Send the message
     //
-    ret_value = pim_node().pim_send(vif_index(), src, dst, ttl, ip_tos,
-				    is_router_alert, buffer, error_msg);
+    ret_value = pim_node().pim_send(name(), name(),
+				    src, dst,
+				    pim_node().ip_protocol_number(),
+				    ttl, ip_tos,
+				    false, // router alert is deprecated
+				    ip_internet_control,
+				    buffer, error_msg);
     
     //
     // Actions after the message is sent
     //
-    if (ret_value >= 0) {
+    if (ret_value == XORP_OK) {
 	switch (message_type) {
 	case PIM_HELLO:
 	    set_should_send_pim_hello(false);
@@ -723,7 +758,7 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     //
     // Keep statistics per message type
     //
-    if (ret_value >= 0) {
+    if (ret_value == XORP_OK) {
 	switch (message_type) {
 	case PIM_HELLO:
 	    ++_pimstat_hello_messages_sent;
@@ -782,12 +817,6 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
  * PimVif::pim_recv:
  * @src: The message source address.
  * @dst: The message destination address.
- * @ip_ttl: The IP TTL of the message. If it has a negative value,
- * it should be ignored.
- * @ip_tos: The IP TOS of the message. If it has a negative value,
- * it should be ignored.
- * @is_router_alert: True if the received IP packet had the Router Alert
- * IP option set.
  * @buffer: The buffer with the received message.
  * 
  * Receive PIM message and pass it for processing.
@@ -795,12 +824,7 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-PimVif::pim_recv(const IPvX& src,
-		 const IPvX& dst,
-		 int ip_ttl,
-		 int ip_tos,
-		 bool is_router_alert,
-		 buffer_t *buffer)
+PimVif::pim_recv(const IPvX& src, const IPvX& dst, buffer_t *buffer)
 {
     int ret_value = XORP_ERROR;
     
@@ -809,8 +833,7 @@ PimVif::pim_recv(const IPvX& src,
 	return (XORP_ERROR);
     }
     
-    ret_value = pim_process(src, dst, ip_ttl, ip_tos, is_router_alert,
-			    buffer);
+    ret_value = pim_process(src, dst, buffer);
     
     return (ret_value);
 }
@@ -819,12 +842,6 @@ PimVif::pim_recv(const IPvX& src,
  * PimVif::pim_process:
  * @src: The message source address.
  * @dst: The message destination address.
- * @ip_ttl: The IP TTL of the message. If it has a negative value,
- * it should be ignored.
- * @ip_tos: The IP TOS of the message. If it has a negative value,
- * it should be ignored.
- * @is_router_alert: True if the received IP packet had the Router Alert
- * IP option set.
  * @buffer: The buffer with the message.
  * 
  * Process PIM message and pass the control to the type-specific functions.
@@ -832,11 +849,7 @@ PimVif::pim_recv(const IPvX& src,
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-PimVif::pim_process(const IPvX& src, const IPvX& dst,
-		    int ip_ttl,
-		    int ip_tos,
-		    bool is_router_alert,
-		    buffer_t *buffer)
+PimVif::pim_process(const IPvX& src, const IPvX& dst, buffer_t *buffer)
 {
     uint8_t pim_vt;
     uint16_t cksum;
@@ -995,55 +1008,7 @@ PimVif::pim_process(const IPvX& src, const IPvX& dst,
 	++_pimstat_unknown_version_messages;
 	return (XORP_ERROR);
     }
-    
-    //
-    // IP Router Alert option check
-    //
-    if (_ip_router_alert_option_check.get()) {
-	switch (message_type) {
-	case PIM_HELLO:
-	case PIM_JOIN_PRUNE:
-	case PIM_ASSERT:
-	case PIM_GRAFT:
-	case PIM_GRAFT_ACK:
-	case PIM_BOOTSTRAP:
-	    if (! is_router_alert) {
-		XLOG_WARNING("RX %s from %s to %s on vif %s: "
-			     "missing IP Router Alert option",
-			     PIMTYPE2ASCII(message_type),
-			     cstring(src), cstring(dst),
-			     name().c_str());
-		ret_value = XORP_ERROR;
-		goto ret_label;
-	    }
-	    //
-	    // TODO: check the TTL and TOS if we are running in secure mode
-	    //
-	    UNUSED(ip_ttl);
-	    UNUSED(ip_tos);
-#if 0
-	    if (ip_ttl != MINTTL) {
-		XLOG_WARNING("RX %s from %s to %s on vif %s: "
-			     "ip_ttl = %d instead of %d",
-			     PIMTYPE2ASCII(message_type),
-			     cstring(src), cstring(dst),
-			     name().c_str(),
-			     ip_ttl, MINTTL);
-		ret_value = XORP_ERROR;
-		goto ret_label;
-	    }
-#endif // 0
-	    break;
-	case PIM_REGISTER:
-	case PIM_REGISTER_STOP:
-	case PIM_CAND_RP_ADV:
-	    // Destination should be unicast. No TTL and RA check needed.
-	    break;
-	default:
-	    break;
-	}
-    }
-    
+
     //
     // Source address check.
     //
@@ -1361,7 +1326,7 @@ PimVif::pim_process(const IPvX& src, const IPvX& dst,
     //
     // Keep statistics per message type
     //
-    if (ret_value >= 0) {
+    if (ret_value == XORP_OK) {
 	switch (message_type) {
 	case PIM_HELLO:
 	    ++_pimstat_hello_messages_received;
