@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/xorp_pimsm4.cc,v 1.12 2007/02/16 22:46:51 pavlin Exp $"
+#ident "$XORP: xorp/pim/xorp_pimsm4.cc,v 1.16 2008/07/23 05:11:16 pavlin Exp $"
 
 
 //
@@ -25,6 +25,7 @@
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/random.h"
 #include "libxorp/callback.hh"
 #include "libxorp/eventloop.hh"
 #include "libxorp/exceptions.hh"
@@ -91,6 +92,15 @@ pim_main(const string& finder_hostname, uint16_t finder_port)
     //
     EventLoop eventloop;
 
+    //
+    // Initialize the random generator
+    //
+    {
+	TimeVal now;
+	TimerList::system_gettimeofday(&now);
+	xorp_srandom(now.sec());
+    }
+
     //
     // PIMSM node
     //
@@ -101,6 +111,7 @@ pim_main(const string& finder_hostname, uint16_t finder_port)
 			       finder_hostname,
 			       finder_port,
 			       "finder",
+			       xorp_module_name(AF_INET, XORP_MODULE_FEA),
 			       xorp_module_name(AF_INET, XORP_MODULE_MFEA),
 			       xorp_module_name(AF_INET, XORP_MODULE_RIB),
 			       xorp_module_name(AF_INET, XORP_MODULE_MLD6IGMP));
