@@ -1,7 +1,7 @@
 #!/bin/sh
 
 #
-# $XORP: xorp/pim/xrl_pim_shell_funcs.sh,v 1.30 2005/06/01 00:36:59 pavlin Exp $
+# $XORP: xorp/pim/xrl_pim_shell_funcs.sh,v 1.32 2008/05/02 00:00:54 pavlin Exp $
 #
 
 #
@@ -103,14 +103,6 @@ pim_stop_all_vifs()
     call_xrl_wrapper -r 0 $XRL$XRL_ARGS
 }
 
-pim_is_vif_setup_completed()
-{
-    echo "pim_is_vif_setup_completed" $*
-    XRL="finder://$PIM_TARGET/mfea_client/0.1/is_vif_setup_completed"
-    XRL_ARGS=""
-    call_xrl_wrapper -r 0 $XRL$XRL_ARGS is_completed:bool = true
-}
-
 pim_enable_pim()
 {
     if [ $# -lt 1 ] ; then
@@ -617,49 +609,6 @@ pim_reset_vif_proto_version()
     call_xrl_wrapper $XRL$XRL_ARGS
 }
 
-pim_get_vif_ip_router_alert_option_check()
-{
-    if [ $# -lt 1 ] ; then
-	echo "Usage: pim_get_vif_ip_router_alert_option_check <vif_name:txt>"
-	exit 1
-    fi
-    vif_name=$1
-
-    echo "pim_get_vif_ip_router_alert_option_check" $*
-    XRL="finder://$PIM_TARGET/pim/0.1/get_vif_ip_router_alert_option_check"
-    XRL_ARGS="?vif_name:txt=$vif_name"
-    call_xrl_wrapper -p enabled:bool $XRL$XRL_ARGS
-}
-
-pim_set_vif_ip_router_alert_option_check()
-{
-    if [ $# -lt 2 ] ; then
-	echo "Usage: pim_set_vif_ip_router_alert_option_check <vif_name:txt> <enable:bool>"
-	exit 1
-    fi
-    vif_name=$1
-    enable=$2
-    
-    echo "pim_set_vif_ip_router_alert_option_check" $*
-    XRL="finder://$PIM_TARGET/pim/0.1/set_vif_ip_router_alert_option_check"
-    XRL_ARGS="?vif_name:txt=$vif_name&enable:bool=$enable"
-    call_xrl_wrapper $XRL$XRL_ARGS
-}
-
-pim_reset_vif_ip_router_alert_option_check()
-{
-    if [ $# -lt 1 ] ; then
-	echo "Usage: pim_reset_vif_ip_router_alert_option_check <vif_name:txt>"
-	exit 1
-    fi
-    vif_name=$1
-    
-    echo "pim_reset_vif_ip_router_alert_option_check" $*
-    XRL="finder://$PIM_TARGET/pim/0.1/reset_vif_ip_router_alert_option_check"
-    XRL_ARGS="?vif_name:txt=$vif_name"
-    call_xrl_wrapper $XRL$XRL_ARGS
-}
-
 pim_get_vif_hello_triggered_delay()
 {
     if [ $# -lt 1 ] ; then
