@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/common/element.cc,v 1.9 2007/02/16 22:47:02 pavlin Exp $"
+#ident "$XORP: xorp/policy/common/element.cc,v 1.13 2008/07/23 05:11:26 pavlin Exp $"
 
 #include "element.hh"
 #include "elem_null.hh"
@@ -59,8 +59,8 @@ template<> Element::Hash ElemIPv6Net::_hash = 0;
 template<> const char* ElemU32Range::id = "u32range";
 template<> Element::Hash ElemU32Range::_hash = 0;
 
-template<> const char* ElemAsPath::id = "aspath";
-template<> Element::Hash ElemAsPath::_hash = 0;
+template<> const char* ElemASPath::id = "aspath";
+template<> Element::Hash ElemASPath::_hash = 0;
 
 const char* ElemNull::id = "null";
 Element::Hash ElemNull::_hash = 1;
@@ -121,5 +121,5 @@ ElemCom32::str() const {
 	    return com_aliases[i].text;
     ostringstream oss;
     oss << (_val >> 16) << ":" << (_val & 0xffff);
-    return(oss.str());
+    return (oss.str());
 }
