@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/redist_xrl.cc,v 1.30 2007/02/16 22:47:07 pavlin Exp $"
+#ident "$XORP: xorp/rib/redist_xrl.cc,v 1.34 2008/07/23 05:11:30 pavlin Exp $"
 
 #include <list>
 #include <string>
@@ -31,6 +31,7 @@
 #include "xrl/interfaces/redist_transaction4_xif.hh"
 #include "xrl/interfaces/redist_transaction6_xif.hh"
 
+#include "rib.hh"
 #include "route.hh"
 #include "redist_xrl.hh"
 #include "profile_vars.hh"
@@ -58,15 +59,15 @@ public:
     /**
      * Get number of times dispatch() invoked on instance.
      */
-    inline uint32_t dispatch_attempts() const		{ return _attempts; }
+    uint32_t dispatch_attempts() const			{ return _attempts; }
 
 protected:
-    inline void incr_dispatch_attempts()		{ _attempts++; }
-    inline RedistXrlOutput<A>* parent()			{ return _parent; }
-    inline const RedistXrlOutput<A>* parent() const	{ return _parent; }
+    void incr_dispatch_attempts()			{ _attempts++; }
+    RedistXrlOutput<A>* parent()			{ return _parent; }
+    const RedistXrlOutput<A>* parent() const		{ return _parent; }
 
-    inline void signal_complete_ok()	{ _parent->task_completed(this); }
-    inline void signal_fatal_failure()	{ _parent->task_failed_fatally(this); }
+    void signal_complete_ok()		{ _parent->task_completed(this); }
+    void signal_fatal_failure()		{ _parent->task_failed_fatally(this); }
 
 private:
     RedistXrlOutput<A>* _parent;
