@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rib.cc,v 1.63 2007/02/16 22:47:08 pavlin Exp $"
+#ident "$XORP: xorp/rib/rib.cc,v 1.73 2008/07/23 05:11:30 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -32,8 +32,8 @@
 
 template <typename A>
 struct table_has_name {
-    inline table_has_name(const string& name) : _n(name) {}
-    inline bool operator() (const RouteTable<A>* rt) const {
+    table_has_name(const string& name) : _n(name) {}
+    bool operator() (const RouteTable<A>* rt) const {
 	return rt->tablename() == _n;
     }
 private:
@@ -42,8 +42,8 @@ private:
 
 template <typename A, typename T>
 struct table_has_name_and_type {
-    inline table_has_name_and_type(const string& name) : _n(name) {}
-    inline bool operator() (const RouteTable<A>* rt) const {
+    table_has_name_and_type(const string& name) : _n(name) {}
+    bool operator() (const RouteTable<A>* rt) const {
 	const T* t = dynamic_cast<const T*>(rt);
 	return (t != 0) && (rt->tablename() == _n);
     }
@@ -122,35 +122,55 @@ RIB<A>::remove_table(const string& tablename)
     return XORP_OK;
 }
 
+template <typename A>
+int
+RIB<A>::set_protocol_admin_distance(const string& protocol_name,
+				    const uint32_t& admin_distance)
+{
+    map<string, uint32_t>::iterator mi = _admin_distances.find(protocol_name);
+    if (mi != _admin_distances.end()) {
+	OriginTable<A>* ot =
+	    dynamic_cast<OriginTable<A>* >(find_table(protocol_name));
+	if (NULL != ot) {
+	    XLOG_ERROR("May not set an admin distance for protocol \"%s\", "
+		       "which has already instantiated an origin table.",
+		       protocol_name.c_str());
+	    return XORP_ERROR;
+	}
+    }
+    _admin_distances[protocol_name] = admin_distance;
+    return XORP_OK;
+}
+
 template <typename A>
 inline uint32_t
-RIB<A>::admin_distance(const string& tablename)
+RIB<A>::get_protocol_admin_distance(const string& protocol_name)
 {
     map<string, uint32_t>::iterator mi;
 
-    mi = _admin_distances.find(tablename);
+    mi = _admin_distances.find(protocol_name);
     if (mi == _admin_distances.end()) {
-	XLOG_ERROR("Administrative distance of \"%s\" unknown.",
-		   tablename.c_str());
-	return _admin_distances["unknown"];
+	XLOG_WARNING("Administrative distance of \"%s\" unknown.",
+		     protocol_name.c_str());
+	return UNKNOWN_ADMIN_DISTANCE;
     }
     return mi->second;
 }
 
 template <typename A>
-inline Vif*
+inline RibVif*
 RIB<A>::find_vif(const A& addr)
 {
-    map<string, Vif>::iterator iter;
+    map<string, RibVif*>::iterator iter;
 
     for (iter = _vifs.begin(); iter != _vifs.end(); ++iter) {
-	Vif& vif = iter->second;
-	if (! vif.is_underlying_vif_up())
+	RibVif* vif = iter->second;
+	if (! vif->is_underlying_vif_up())
 	    continue;		// XXX: ignore vifs that are not up
-	if (vif.is_my_addr(addr))
-	    return &vif;
-	if (vif.is_p2p() && vif.is_same_p2p(addr))
-	    return &vif;
+	if (vif->is_my_addr(addr))
+	    return vif;
+	if (vif->is_p2p() && vif->is_same_p2p(addr))
+	    return vif;
     }
     return NULL;
 }
@@ -239,7 +259,7 @@ RIB<A>::RIB(RibTransportType t, RibManager& rib_manager, EventLoop& eventloop)
     }
 
     // TODO: XXX: don't use hard-coded values below!
-    _admin_distances["connected"] =        0;
+    _admin_distances["connected"] =        CONNECTED_ADMIN_DISTANCE;
     _admin_distances["static"] =           1;
     _admin_distances["eigrp-summary"] =    5;
     _admin_distances["ebgp"] =            20;
@@ -251,7 +271,7 @@ RIB<A>::RIB(RibTransportType t, RibManager& rib_manager, EventLoop& eventloop)
     _admin_distances["eigrp-external"] = 170;
     _admin_distances["ibgp"] =           200;
     _admin_distances["fib2mrib"] =	 254;
-    _admin_distances["unknown"] =        255;
+    _admin_distances["unknown"] =        UNKNOWN_ADMIN_DISTANCE;
 }
 
 template <typename A>
@@ -261,6 +281,14 @@ RIB<A>::~RIB()
 	delete _tables.front();
 	_tables.pop_front();
     }
+    while (_vifs.empty() == false) {
+	delete _vifs.begin()->second;
+	_vifs.erase(_vifs.begin());
+    }
+    while (_deleted_vifs.empty() == false) {
+	delete _deleted_vifs.begin()->second;
+	_deleted_vifs.erase(_deleted_vifs.begin());
+    }
 }
 
 template <typename A>
@@ -300,6 +328,7 @@ RIB<A>::initialize(RegisterServer& register_server)
 		   "table for %s",
 		   name().c_str());
     }
+
     if (add_igp_table("connected", "", "") != XORP_OK) {
 	XLOG_FATAL("Could not add igp table \"connected\" for %s",
 		   name().c_str());
@@ -426,7 +455,7 @@ RIB<A>::new_origin_table(const string&	tablename,
 
 template <typename A>
 int
-RIB<A>::add_connected_route(const Vif& vif,
+RIB<A>::add_connected_route(const RibVif& vif,
 			    const IPNet<A>& net,
 			    const A& nexthop_addr,
 			    const A& peer_addr)
@@ -447,7 +476,7 @@ RIB<A>::add_connected_route(const Vif& vif,
 
 template <typename A>
 int
-RIB<A>::delete_connected_route(const Vif& vif, const IPNet<A>& net,
+RIB<A>::delete_connected_route(const RibVif& vif, const IPNet<A>& net,
 			       const A& peer_addr)
 {
     delete_route("connected", net);
@@ -463,27 +492,37 @@ template <typename A>
 int
 RIB<A>::new_vif(const string& vifname, const Vif& vif)
 {
+    map<string, RibVif*>::iterator vi;
+    RibVif* new_rib_vif = NULL;
+
     debug_msg("RIB::new_vif: %s\n", vifname.c_str());
     if (_vifs.find(vifname) != _vifs.end())
 	return XORP_ERROR;
 
-    // Can't use _vifs[vifname] = vif because no Vif() constructor
-    map<string, Vif>::value_type v(vifname, vif);
-    _vifs.insert(_vifs.end(), v);
+    //
+    // If the vif is pending deletion, then reuse it instead
+    //
+    vi = _deleted_vifs.find(vifname);
+    if (vi != _deleted_vifs.end()) {
+	// Reuse previously deleted vif
+	new_rib_vif = vi->second;
+	new_rib_vif->set_deleted(false);
+	_deleted_vifs.erase(vi);
+	new_rib_vif->copy_in(vif);
+    } else {
+	// Create a new vif
+	new_rib_vif = new RibVif(this, vif);
+    }
+    XLOG_ASSERT(new_rib_vif != NULL);
+    _vifs[vifname] = new_rib_vif;
 
-    // We need to add the routes from the VIF to the connected table
-    map<string, Vif>::iterator iter = _vifs.find(vifname);
-    XLOG_ASSERT(iter != _vifs.end());
-    Vif* new_vif = &(iter->second);
-    XLOG_ASSERT(new_vif != NULL);
-
-    if (vif.is_underlying_vif_up()) {
+    if (new_rib_vif->is_underlying_vif_up()) {
 	//
 	// Add the directly connected routes associated with this vif
 	//
 	list<VifAddr>::const_iterator ai;
-	for (ai = new_vif->addr_list().begin();
-	     ai != new_vif->addr_list().end();
+	for (ai = new_rib_vif->addr_list().begin();
+	     ai != new_rib_vif->addr_list().end();
 	     ++ai) {
 	    if (ai->addr().af() != A::af())
 		continue;
@@ -493,7 +532,7 @@ RIB<A>::new_vif(const string& vifname, const Vif& vif)
 	    ai->subnet_addr().get(subnet_addr);
 	    ai->addr().get(addr);
 	    ai->peer_addr().get(peer_addr);
-	    add_connected_route(*new_vif, subnet_addr, addr, peer_addr);
+	    add_connected_route(*new_rib_vif, subnet_addr, addr, peer_addr);
 	}
     }
 
@@ -505,18 +544,20 @@ int
 RIB<A>::delete_vif(const string& vifname)
 {
     debug_msg("RIB::delete_vif: %s\n", vifname.c_str());
-    map<string, Vif>::iterator vi = _vifs.find(vifname);
+    map<string, RibVif*>::iterator vi = _vifs.find(vifname);
     if (vi == _vifs.end()) {
 	return XORP_ERROR;
     }
-    Vif& vif = vi->second;
+    RibVif* rib_vif = vi->second;
 
-    if (vif.is_underlying_vif_up()) {
+    if (rib_vif->is_underlying_vif_up()) {
 	//
 	// Delete the directly connected routes associated with this vif
 	//
 	list<VifAddr>::const_iterator ai;
-	for (ai = vif.addr_list().begin(); ai != vif.addr_list().end(); ++ai) {
+	for (ai = rib_vif->addr_list().begin();
+	     ai != rib_vif->addr_list().end();
+	     ++ai) {
 	    if (ai->addr().af() != A::af())
 		continue;
 
@@ -524,14 +565,41 @@ RIB<A>::delete_vif(const string& vifname)
 	    A peer_addr;
 	    ai->subnet_addr().get(subnet_addr);
 	    ai->peer_addr().get(peer_addr);
-	    delete_connected_route(vif, subnet_addr, peer_addr);
+	    delete_connected_route(*rib_vif, subnet_addr, peer_addr);
 	}
     }
 
     _vifs.erase(vi);
+
+    //
+    // If the vif is still used by some routes, then add it to the
+    // container with vifs pending deletion.
+    // Otherwise just delete it.
+    //
+    if (rib_vif->usage_counter() > 0) {
+	XLOG_ASSERT(_deleted_vifs.find(vifname) == _deleted_vifs.end());
+	_deleted_vifs[vifname] = rib_vif;
+	rib_vif->set_deleted(true);
+    } else {
+	delete rib_vif;
+    }
+    
     return XORP_OK;
 }
 
+template <typename A>
+void
+RIB<A>::destroy_deleted_vif(RibVif* rib_vif)
+{
+    map<string, RibVif*>::iterator vi = _deleted_vifs.find(rib_vif->name());
+
+    XLOG_ASSERT(vi != _deleted_vifs.end());
+    XLOG_ASSERT(vi->second == rib_vif);
+
+    _deleted_vifs.erase(vi);
+    delete rib_vif;
+}
+
 template <typename A>
 int
 RIB<A>::set_vif_flags(const string& vifname,
@@ -542,22 +610,22 @@ RIB<A>::set_vif_flags(const string& vifname,
 		      bool is_up,
 		      uint32_t mtu)
 {
-    map<string, Vif>::iterator vi = _vifs.find(vifname);
+    map<string, RibVif*>::iterator vi = _vifs.find(vifname);
     if (vi == _vifs.end()) {
 	XLOG_ERROR("Attempting to set flags to non-existant Vif \"%s\"",
 		   vifname.c_str());
 	return XORP_ERROR;
     }
-    Vif& vif = vi->second;
+    RibVif* vif = vi->second;
 
-    bool old_is_up = vif.is_underlying_vif_up();
+    bool old_is_up = vif->is_underlying_vif_up();
 
-    vif.set_p2p(is_p2p);
-    vif.set_loopback(is_loopback);
-    vif.set_multicast_capable(is_multicast);
-    vif.set_broadcast_capable(is_broadcast);
-    vif.set_underlying_vif_up(is_up);
-    vif.set_mtu(mtu);
+    vif->set_p2p(is_p2p);
+    vif->set_loopback(is_loopback);
+    vif->set_multicast_capable(is_multicast);
+    vif->set_broadcast_capable(is_broadcast);
+    vif->set_underlying_vif_up(is_up);
+    vif->set_mtu(mtu);
 
     if (old_is_up == is_up)
 	return XORP_OK;
@@ -568,7 +636,9 @@ RIB<A>::set_vif_flags(const string& vifname,
 	//
 	// Add all connected routes
 	//
-	for (ai = vif.addr_list().begin(); ai != vif.addr_list().end(); ++ai) {
+	for (ai = vif->addr_list().begin();
+	     ai != vif->addr_list().end();
+	     ++ai) {
 	    if (ai->addr().af() != A::af())
 		continue;
 
@@ -577,7 +647,7 @@ RIB<A>::set_vif_flags(const string& vifname,
 	    ai->subnet_addr().get(subnet_addr);
 	    ai->addr().get(addr);
 	    ai->peer_addr().get(peer_addr);
-	    add_connected_route(vif, subnet_addr, addr, peer_addr);
+	    add_connected_route(*vif, subnet_addr, addr, peer_addr);
 	}
     }
 
@@ -586,7 +656,9 @@ RIB<A>::set_vif_flags(const string& vifname,
 	//
 	// Delete all connected routes
 	//
-	for (ai = vif.addr_list().begin(); ai != vif.addr_list().end(); ++ai) {
+	for (ai = vif->addr_list().begin();
+	     ai != vif->addr_list().end();
+	     ++ai) {
 	    if (ai->addr().af() != A::af())
 		continue;
 
@@ -594,7 +666,7 @@ RIB<A>::set_vif_flags(const string& vifname,
 	    A peer_addr;
 	    ai->subnet_addr().get(subnet_addr);
 	    ai->peer_addr().get(peer_addr);
-	    delete_connected_route(vif, subnet_addr, peer_addr);
+	    delete_connected_route(*vif, subnet_addr, peer_addr);
 	}
     }
 
@@ -609,18 +681,18 @@ RIB<A>::add_vif_address(const string&	vifname,
 			const A&	broadcast_addr,
 			const A&	peer_addr)
 {
-    map<string, Vif>::iterator vi = _vifs.find(vifname);
+    map<string, RibVif*>::iterator vi = _vifs.find(vifname);
     if (vi == _vifs.end()) {
 	XLOG_ERROR("Attempting to add address to non-existant Vif \"%s\"",
 		   vifname.c_str());
 	return XORP_ERROR;
     }
-    Vif& vif = vi->second;
+    RibVif* vif = vi->second;
 
-    vif.add_address(VifAddr(addr, subnet, broadcast_addr, peer_addr));
+    vif->add_address(VifAddr(addr, subnet, broadcast_addr, peer_addr));
 
-    if (vif.is_underlying_vif_up())
-	add_connected_route(vif, subnet, addr, peer_addr);
+    if (vif->is_underlying_vif_up())
+	add_connected_route(*vif, subnet, addr, peer_addr);
 
     return XORP_OK;
 }
@@ -630,16 +702,16 @@ int
 RIB<A>::delete_vif_address(const string& vifname,
 			   const A& addr)
 {
-    map<string, Vif>::iterator vi = _vifs.find(vifname);
+    map<string, RibVif*>::iterator vi = _vifs.find(vifname);
     if (vi == _vifs.end()) {
 	XLOG_ERROR("Attempting to delete address from non-existant Vif \"%s\"",
 		   vifname.c_str());
 	return XORP_ERROR;
     }
-    Vif& vif = vi->second;
+    RibVif* vif = vi->second;
     
     list<VifAddr>::const_iterator ai;
-    for (ai = vif.addr_list().begin(); ai != vif.addr_list().end(); ++ai) {
+    for (ai = vif->addr_list().begin(); ai != vif->addr_list().end(); ++ai) {
 	const IPvX& ipvx = ai->addr();
 	if (ipvx.af() != A::af())
 	    continue;
@@ -651,10 +723,10 @@ RIB<A>::delete_vif_address(const string& vifname,
 	ai->subnet_addr().get(subnet_addr);
 	ai->peer_addr().get(peer_addr);
 
-	vif.delete_address(ipvx);
+	vif->delete_address(ipvx);
 
-	if (vif.is_underlying_vif_up())
-	    delete_connected_route(vif, subnet_addr, peer_addr);
+	if (vif->is_underlying_vif_up())
+	    delete_connected_route(*vif, subnet_addr, peer_addr);
 
 	return XORP_OK;
     }
@@ -711,7 +783,7 @@ RIB<A>::add_route(const string&		tablename,
 	//
 	// Add a route with explicitly specified network interface
 	//
-	map<string, Vif>::iterator iter = _vifs.find(vifname);
+	map<string, RibVif*>::iterator iter = _vifs.find(vifname);
 	if (iter == _vifs.end()) {
 	    XLOG_ERROR("Attempting to add route to table \"%s\" "
 		       "(prefix %s next-hop %s ifname %s vifname %s): "
@@ -721,7 +793,7 @@ RIB<A>::add_route(const string&		tablename,
 		       ifname.c_str(), vifname.c_str());
 	    return XORP_ERROR;
 	}
-	Vif* vif = &iter->second;
+	RibVif* vif = iter->second;
 	IPNextHop<A>* nexthop = find_or_create_peer_nexthop(nexthop_addr);
 	ot->add_route(IPRouteEntry<A>(net, vif, nexthop, *protocol,
 		      metric, policytags));
@@ -732,7 +804,7 @@ RIB<A>::add_route(const string&		tablename,
     //
     // Find the vif so we can see if the nexthop is directly connected
     //
-    Vif* vif = NULL;
+    RibVif* vif = NULL;
     IPNextHop<A>* nexthop = NULL;
     do {
 	//
@@ -785,21 +857,11 @@ RIB<A>::add_route(const string&		tablename,
     }
     XLOG_ASSERT(nexthop->addr() == nexthop_addr);
 
-    //
-    // Only accept the least significant 16 bits of metric.
-    //
-    if (metric > 0xffff) {
-	XLOG_WARNING("Protocol metric value %u is greater than 0xffff from "
-		     "table \"%s\"",  XORP_UINT_CAST(metric),
-		     tablename.c_str());
-	metric &= 0xffff;
-    }
-
     //
     // Add the route
     //
-    ot->add_route(IPRouteEntry<A>(net, vif, nexthop,
-				  *protocol, metric, policytags));
+    ot->add_route(IPRouteEntry<A>(net, vif, nexthop, *protocol, metric,
+				  policytags));
     flush();
     return XORP_OK;
 }
@@ -889,17 +951,29 @@ RIB<A>::verify_route(const A& lookup_addr,
     }
 
 #ifdef notyet
-    // 3. Check for discard (blackhole) routes.
+    // 3a. Check for discard (blackhole) routes.
     // XXX: re->vif() must be non-NULL and valid. Revisit this in future.
     DiscardNextHop* dnh = dynamic_cast<DiscardNextHop*>(re->nexthop());
     if (matchtype == RibVerifyType(DISCARD)) {
-	    if (dnh == NULL) {
-		    debug_msg("Next hop is not a DiscardNextHop");
-		    return XORP_ERROR;
-	    } else {
-		debug_msg("****DISCARD ROUTE SUCCESSFULLY VERIFIED****\n");
-		return XORP_OK;
-	    }
+	if (dnh == NULL) {
+	    debug_msg("Next hop is not a DiscardNextHop");
+	    return XORP_ERROR;
+	} else {
+	    debug_msg("****DISCARD ROUTE SUCCESSFULLY VERIFIED****\n");
+	    return XORP_OK;
+	}
+    }
+    // 3b. Check for unreachable routes.
+    // XXX: re->vif() must be non-NULL and valid. Revisit this in future.
+    UnreachableNextHop* unh = dynamic_cast<UnreachableNextHop*>(re->nexthop());
+    if (matchtype == RibVerifyType(UNREACHABLE)) {
+	if (unh == NULL) {
+	    debug_msg("Next hop is not an UnreachableNextHop");
+	    return XORP_ERROR;
+	} else {
+	    debug_msg("****UNREACHABLE ROUTE SUCCESSFULLY VERIFIED****\n");
+	    return XORP_OK;
+	}
     }
 #endif
 
@@ -955,10 +1029,16 @@ RIB<A>::lookup_route(const A& lookupaddr)
 #ifdef notyet
     DiscardNextHop* discard_nexthop =
 	dynamic_cast<DiscardNextHop* >(re->nexthop());
-    // Case 2: Discard route. Return the loopback address.
+    // Case 2a: Discard route. Return the loopback address.
     if (discard_nexthop != NULL)
 	return A::LOOPBACK();
 
+    UnreachableNextHop* unreachable_nexthop =
+	dynamic_cast<UnreachableNextHop* >(re->nexthop());
+    // Case 2b: Unreachable route. Return the loopback address.
+    if (unreachable_nexthop != NULL)
+	return A::LOOPBACK();
+
     IPNextHop<A>* ip_nexthop = dynamic_cast<IPNextHop<A>* >(re->nexthop());
     // Case 3: IP protocol route. Return the nexthop address.
     if (ip_nexthop != NULL)
@@ -1133,7 +1213,7 @@ RIB<A>::add_origin_table(const string& tablename,
 			 const string& target_instance,
 			 ProtocolType protocol_type)
 {
-    debug_msg("add_origin_table %s type: %dn\n",
+    debug_msg("add_origin_table %s type: %d\n",
 	      tablename.c_str(), protocol_type);
 
     Protocol* protocol = find_protocol(tablename);
@@ -1172,7 +1252,8 @@ RIB<A>::add_origin_table(const string& tablename,
     }
 
     if (new_origin_table(tablename, target_class, target_instance,
-			 admin_distance(tablename), protocol_type)
+			 get_protocol_admin_distance(tablename),
+			 protocol_type)
 	!= XORP_OK) {
 	debug_msg("new_origin_table failed\n");
 	return XORP_ERROR;
