@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rt_tab_extint.cc,v 1.31 2007/02/16 22:47:09 pavlin Exp $"
+#ident "$XORP: xorp/rib/rt_tab_extint.cc,v 1.36 2008/07/23 05:11:31 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -20,6 +20,7 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#include "rib.hh"
 #include "rt_tab_extint.hh"
 
 template <typename A>
@@ -99,7 +100,7 @@ ExtIntTable<A>::add_route(const IPRouteEntry<A>& route, RouteTable<A>* caller)
 	    const IPRouteEntry<A>* nexthop_route;
 	    nexthop_route = lookup_route_in_igp_parent(nexthop_addr);
 	    if (nexthop_route != NULL) {
-		Vif* vif = nexthop_route->vif();
+		RibVif* vif = nexthop_route->vif();
 		if ((vif != NULL)
 		    && (vif->is_same_subnet(IPvXNet(nexthop_route->net()))
 			|| vif->is_same_p2p(IPvX(nexthop_addr)))) {
@@ -160,7 +161,7 @@ ExtIntTable<A>::add_route(const IPRouteEntry<A>& route, RouteTable<A>* caller)
 		    this->next_table()->delete_route(found, this);
 	    }
 
-	    Vif* vif = nexthop_route->vif();
+	    RibVif* vif = nexthop_route->vif();
 	    if ((vif != NULL)
 		&& (vif->is_same_subnet(IPvXNet(nexthop_route->net()))
 		    || vif->is_same_p2p(IPvX(nexthop_addr)))) {
@@ -513,6 +514,7 @@ ExtIntTable<A>::recalculate_nexthops(const IPRouteEntry<A>& new_route)
     while (found != NULL) {
 	egp_parent = found->egp_parent();
 	XLOG_ASSERT(egp_parent->nexthop()->type() != DISCARD_NEXTHOP);
+	XLOG_ASSERT(egp_parent->nexthop()->type() != UNREACHABLE_NEXTHOP);
 	A nexthop = (reinterpret_cast<IPNextHop<A>* >(egp_parent->nexthop()))->addr();
 
 	if (new_route.net().contains(nexthop)) {
@@ -646,7 +648,7 @@ ExtIntTable<A>::lookup_route(const A& addr) const
     }
 
     // Retain only the routes with the lowest admin_distance
-    uint32_t lowest_admin_distance = 0xffff;
+    uint32_t lowest_admin_distance = MAX_ADMIN_DISTANCE;
     for (iter = found.begin(); iter != found.end(); ++iter) {
 	if ((*iter)->admin_distance() < lowest_admin_distance) {
 	    lowest_admin_distance = (*iter)->admin_distance();
