@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/test_redist.cc,v 1.13 2007/02/16 22:47:11 pavlin Exp $"
+#ident "$XORP: xorp/rib/test_redist.cc,v 1.17 2008/07/23 05:11:32 pavlin Exp $"
 
 #include <set>
 
@@ -25,6 +25,7 @@
 #include "libxorp/eventloop.hh"
 
 #include "route.hh"
+#include "rib.hh"
 #include "rt_tab_redist.hh"
 #include "rt_tab_origin.hh"
 #include "rt_tab_deletion.hh"
@@ -42,7 +43,7 @@ static const char* program_name         = "test_redist";
 static const char* program_description  = "Test Route Redistribution Components";
 static const char* program_version_id   = "0.1";
 static const char* program_date         = "April, 2004";
-static const char* program_copyright    = "See file LICENSE.XORP";
+static const char* program_copyright    = "See file LICENSE";
 static const char* program_return_value = "0 on success, 1 if test error, "
 					  "2 if internal error";
 
@@ -270,7 +271,8 @@ test_deterministic()
     OriginTable<IPv4> 	origin("static", 1, IGP, e);
     IPPeerNextHop<IPv4> nh("22.0.0.1");
     Protocol		protocol("static", IGP, 1);
-    Vif			vif("vif0");
+    Vif			tmp_vif("vif0");
+    RibVif		vif((RIB<IPv4>*)NULL, tmp_vif);
 
     // Attach redist table
     RedistTable<IPv4> redist_table("StaticRedistTable", &origin);
