@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/test_register.cc,v 1.23 2007/02/16 22:47:11 pavlin Exp $"
+#ident "$XORP: xorp/rib/test_register.cc,v 1.26 2008/07/23 05:11:32 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -63,6 +63,8 @@ test_route_range(const string& ipv4,
 	verifytype = RibVerifyType(MISS);
     else if (verifytypestr == "discard")
 	verifytype = RibVerifyType(DISCARD);
+    else if (verifytypestr == "unreachable")
+	verifytype = RibVerifyType(UNREACHABLE);
     else if (verifytypestr == "ip")
 	verifytype = RibVerifyType(IP);
     else {
@@ -81,6 +83,13 @@ test_route_range(const string& ipv4,
 	    printf("Expected discard route\n");
 	    abort();
 	}
+	UnreachableNextHop* unh;
+	unh = reinterpret_cast<UnreachableNextHop* >(rr->route()->nexthop());
+	if (verifytype != RibVerifyType(UNREACHABLE) && unh == NULL) {
+	    printf("**RouteRange for %s\n", ipv4.c_str());
+	    printf("Expected unreachable route\n");
+	    abort();
+	}
 	if (rr->route()->vif() == NULL) {
 	    printf("BAD Vif NULL != \"%s\"\n", vifname.c_str());
 	    abort();
