@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/test_rib_direct.cc,v 1.21 2007/02/16 22:47:11 pavlin Exp $"
+#ident "$XORP: xorp/rib/test_rib_direct.cc,v 1.25 2008/07/23 05:11:32 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -45,6 +45,8 @@ RibParser::RibParser(RIB<IPv4>& rib)
     : _rib(rib)
 {
     add_command(new DirectDiscardVifCommand(_rib));
+    add_command(new DirectUnreachableVifCommand(_rib));
+    add_command(new DirectManagementVifCommand(_rib));
     add_command(new DirectEtherVifCommand(_rib));
     add_command(new DirectLoopbackVifCommand(_rib));
 
