@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/test_rib_xrls.cc,v 1.42 2007/02/16 22:47:11 pavlin Exp $"
+#ident "$XORP: xorp/rib/test_rib_xrls.cc,v 1.46 2008/07/23 05:11:32 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -55,6 +55,8 @@ public:
 	// XXX The following should probably use XRL's but punting for
 	// time being.
 	add_command(new DirectDiscardVifCommand(rib));
+	add_command(new DirectUnreachableVifCommand(rib));
+	add_command(new DirectManagementVifCommand(rib));
 	add_command(new DirectEtherVifCommand(rib));
 	add_command(new DirectLoopbackVifCommand(rib));
     }
