@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/vifmanager.cc,v 1.44 2007/02/16 22:47:11 pavlin Exp $"
+#ident "$XORP: xorp/rib/vifmanager.cc,v 1.52 2008/07/23 05:11:33 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -82,13 +82,13 @@ VifManager::start()
 
     enable();	// XXX: by default the VifManager is always enabled
 
-    if (ProtoState::pending_start() < 0)
+    if (ProtoState::pending_start() != XORP_OK)
 	return (XORP_ERROR);
 
     //
     // Startup the interface manager
     //
-    if (ifmgr_startup() != true) {
+    if (ifmgr_startup() != XORP_OK) {
 	ServiceBase::set_status(SERVICE_FAILED);
 	return (XORP_ERROR);
     }
@@ -112,7 +112,7 @@ VifManager::final_start()
 	return (XORP_ERROR);
 #endif
 
-    if (ProtoState::start() < 0) {
+    if (ProtoState::start() != XORP_OK) {
 	ProtoState::stop();
 	return (XORP_ERROR);
     }
@@ -139,13 +139,13 @@ VifManager::stop()
     if (! (is_up() || is_pending_up() || is_pending_down()))
 	return (XORP_ERROR);
 
-    if (ProtoState::pending_stop() < 0)
+    if (ProtoState::pending_stop() != XORP_OK)
 	return (XORP_ERROR);
 
     //
     // Shutdown the interface manager
     //
-    if (ifmgr_shutdown() != true) {
+    if (ifmgr_shutdown() != XORP_OK) {
 	ServiceBase::set_status(SERVICE_FAILED);
 	return (XORP_ERROR);
     }
@@ -168,7 +168,7 @@ VifManager::final_stop()
     if (! (is_up() || is_pending_up() || is_pending_down()))
 	return (XORP_ERROR);
 #endif
-    if (ProtoState::stop() < 0)
+    if (ProtoState::stop() != XORP_OK)
 	return (XORP_ERROR);
 
     // Clear the old state
@@ -188,7 +188,7 @@ VifManager::status_change(ServiceBase*  service,
 	if ((old_status == SERVICE_STARTING)
 	    && (new_status == SERVICE_RUNNING)) {
 	    // The startup process has completed
-	    if (final_start() < 0) {
+	    if (final_start() != XORP_OK) {
 		XLOG_ERROR("Cannot complete the startup process; "
 			   "current state is %s",
 			   ProtoState::state_str().c_str());
@@ -253,10 +253,10 @@ VifManager::update_status()
     }
 }
 
-bool
+int
 VifManager::ifmgr_startup()
 {
-    bool ret_value;
+    int ret_value;
 
     // TODO: XXX: we should startup the ifmgr only if it hasn't started yet
     incr_startup_requests_n();
@@ -268,13 +268,13 @@ VifManager::ifmgr_startup()
     // will be called.
     //
 
-    return ret_value;
+    return (ret_value);
 }
 
-bool
+int
 VifManager::ifmgr_shutdown()
 {
-    bool ret_value;
+    int ret_value;
 
     incr_shutdown_requests_n();
 
@@ -285,7 +285,7 @@ VifManager::ifmgr_shutdown()
     // will be called.
     //
 
-    return ret_value;
+    return (ret_value);
 }
 
 void
@@ -305,8 +305,8 @@ VifManager::updates_made()
     string error_msg;
     IfMgrIfTree::IfMap::const_iterator ifmgr_iface_iter;
     IfMgrIfAtom::VifMap::const_iterator ifmgr_vif_iter;
-    IfMgrVifAtom::V4Map::const_iterator a4_iter;
-    IfMgrVifAtom::V6Map::const_iterator a6_iter;
+    IfMgrVifAtom::IPv4Map::const_iterator a4_iter;
+    IfMgrVifAtom::IPv6Map::const_iterator a6_iter;
 
     //
     // Update the local copy of the interface tree
@@ -317,8 +317,8 @@ VifManager::updates_made()
     //
     // Remove vifs that don't exist anymore
     //
-    for (ifmgr_iface_iter = _old_iftree.ifs().begin();
-	 ifmgr_iface_iter != _old_iftree.ifs().end();
+    for (ifmgr_iface_iter = _old_iftree.interfaces().begin();
+	 ifmgr_iface_iter != _old_iftree.interfaces().end();
 	 ++ifmgr_iface_iter) {
 	const IfMgrIfAtom& ifmgr_iface = ifmgr_iface_iter->second;
 	const string& ifmgr_iface_name = ifmgr_iface.name();
@@ -335,8 +335,8 @@ VifManager::updates_made()
     //
     // Add new vifs, update existing ones and remove old addresses
     //
-    for (ifmgr_iface_iter = _iftree.ifs().begin();
-	 ifmgr_iface_iter != _iftree.ifs().end();
+    for (ifmgr_iface_iter = _iftree.interfaces().begin();
+	 ifmgr_iface_iter != _iftree.interfaces().end();
 	 ++ifmgr_iface_iter) {
 	const IfMgrIfAtom& ifmgr_iface = ifmgr_iface_iter->second;
 	const string& ifmgr_iface_name = ifmgr_iface.name();
@@ -362,8 +362,9 @@ VifManager::updates_made()
 
 	    //
 	    // Update or set the vif flags
-	    // XXX: discard property needs to be inherited from
-	    // the parent interface, once it has been marshaled.
+	    // XXX: Other properties (discard, unreachable, management)
+	    // need to be inherited from the parent interface, once it has
+	    // been marshaled.
 	    //
 	    bool is_up = ifmgr_iface.enabled() && ifmgr_vif.enabled();
 	    is_up &= (! ifmgr_iface.no_carrier());	// XXX: the link state
@@ -375,7 +376,7 @@ VifManager::updates_made()
 						ifmgr_vif.multicast_capable(),
 						ifmgr_vif.broadcast_capable(),
 						is_up,
-						ifmgr_iface.mtu_bytes(),
+						ifmgr_iface.mtu(),
 						error_msg)
 		    != XORP_OK) {
 		    XLOG_ERROR("Cannot update the flags for vif %s: %s",
@@ -388,7 +389,7 @@ VifManager::updates_made()
 		vif.set_multicast_capable(ifmgr_vif.multicast_capable());
 		vif.set_broadcast_capable(ifmgr_vif.broadcast_capable());
 		vif.set_underlying_vif_up(is_up);
-		vif.set_mtu(ifmgr_iface.mtu_bytes());
+		vif.set_mtu(ifmgr_iface.mtu());
 	    }
 
 	    //
@@ -457,8 +458,8 @@ VifManager::updates_made()
     //
     // Add new vif addresses, and update existing ones
     //
-    for (ifmgr_iface_iter = _iftree.ifs().begin();
-	 ifmgr_iface_iter != _iftree.ifs().end();
+    for (ifmgr_iface_iter = _iftree.interfaces().begin();
+	 ifmgr_iface_iter != _iftree.interfaces().end();
 	 ++ifmgr_iface_iter) {
 	const IfMgrIfAtom& ifmgr_iface = ifmgr_iface_iter->second;
 	const string& ifmgr_iface_name = ifmgr_iface.name();
