@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/xrl_target.cc,v 1.59 2007/02/16 22:47:12 pavlin Exp $"
+#ident "$XORP: xorp/rib/xrl_target.cc,v 1.64 2008/07/23 05:11:33 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -370,8 +370,8 @@ XrlRibTarget::rib_0_1_add_route4(const string&	protocol,
     debug_msg("add_route4 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      XORP_UINT_CAST(metric));
@@ -420,8 +420,8 @@ XrlRibTarget::rib_0_1_add_route6(const string&	protocol,
     debug_msg("add_route6 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      XORP_UINT_CAST(metric));
@@ -472,8 +472,8 @@ XrlRibTarget::rib_0_1_replace_route4(const string&	protocol,
     debug_msg("replace_route4 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      XORP_UINT_CAST(metric));
@@ -520,8 +520,8 @@ XrlRibTarget::rib_0_1_replace_route6(const string&	protocol,
     debug_msg("replace_route6 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      XORP_UINT_CAST(metric));
@@ -565,8 +565,8 @@ XrlRibTarget::rib_0_1_delete_route4(const string&	protocol,
     debug_msg("delete_route4 protocol: %s unicast: %s multicast: %s "
 	      "network %s\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str());
 
     if (_rib_manager->profile().enabled(profile_route_ribin)) {
@@ -600,8 +600,8 @@ XrlRibTarget::rib_0_1_delete_route6(const string&	protocol,
     debug_msg("delete_route6 protocol: %s unicast: %s multicast: %s "
 	      "network %s\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str());
 
     if (_rib_manager->profile().enabled(profile_route_ribin)) {
@@ -640,8 +640,8 @@ XrlRibTarget::rib_0_1_add_interface_route4(const string&	protocol,
     debug_msg("add_interface_route4 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
@@ -694,8 +694,8 @@ XrlRibTarget::rib_0_1_add_interface_route6(const string&	protocol,
     debug_msg("add_interface_route6 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
@@ -748,8 +748,8 @@ XrlRibTarget::rib_0_1_replace_interface_route4(const string&	    protocol,
     debug_msg("replace_interface_route4 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
@@ -802,8 +802,8 @@ XrlRibTarget::rib_0_1_replace_interface_route6(const string&	    protocol,
     debug_msg("replace_interface_route6 protocol: %s unicast: %s multicast: %s "
 	      "network %s nexthop %s ifname %s vifname %s metric %u\n",
 	      protocol.c_str(),
-	      unicast ? "true" : "false",
-	      multicast ? "true" : "false",
+	      bool_c_str(unicast),
+	      bool_c_str(multicast),
 	      network.str().c_str(),
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
@@ -1164,6 +1164,7 @@ XrlRibTarget::rib_0_1_register_interest4(// Input values,
 	    break;
 	case EXTERNAL_NEXTHOP:
 	case DISCARD_NEXTHOP:
+	case UNREACHABLE_NEXTHOP:
 	    resolves = false;
 	    break;
 	}
@@ -1228,6 +1229,7 @@ XrlRibTarget::rib_0_1_register_interest6(// Input values,
 	    break;
 	case EXTERNAL_NEXTHOP:
 	case DISCARD_NEXTHOP:
+	case UNREACHABLE_NEXTHOP:
 	    resolves = false;
 	    break;
 	}
@@ -1253,6 +1255,139 @@ XrlRibTarget::rib_0_1_deregister_interest6(// Input values,
     return XrlCmdError::OKAY();
 }
 
+XrlCmdError
+XrlRibTarget::rib_0_1_get_protocol_admin_distances(
+    // Input values,
+    const bool&     ipv4,
+    const bool&     unicast,
+    // Output values,
+    XrlAtomList&    protocols,
+    XrlAtomList&    admin_distances)
+{
+
+    if (ipv4 && unicast) {
+	// ipv4 unicast
+	map<string, uint32_t>& rad = _urib4.get_protocol_admin_distances();
+	map<string, uint32_t>::iterator iter;
+	for (iter = rad.begin(); iter != rad.end(); ++iter) {
+	    protocols.append(XrlAtom(iter->first));
+	    admin_distances.append(XrlAtom(iter->second));
+	}
+    } else if (ipv4 && !unicast) {
+	// ipv4 multicast
+	map<string, uint32_t>& rad = _mrib4.get_protocol_admin_distances();
+	map<string, uint32_t>::iterator iter;
+	for (iter = rad.begin(); iter != rad.end(); ++iter) {
+	    protocols.append(XrlAtom(iter->first));
+	    admin_distances.append(XrlAtom(iter->second));
+	}
+    } else if (!ipv4 && unicast) {
+	// ipv6 unicast
+	map<string, uint32_t>& rad = _urib6.get_protocol_admin_distances();
+	map<string, uint32_t>::iterator iter;
+	for (iter = rad.begin(); iter != rad.end(); ++iter) {
+	    protocols.append(XrlAtom(iter->first));
+	    admin_distances.append(XrlAtom(iter->second));
+	}
+    } else if (!ipv4 && !unicast) {
+	// ipv6 multicast
+	map<string, uint32_t>& rad = _mrib6.get_protocol_admin_distances();
+	map<string, uint32_t>::iterator iter;
+	for (iter = rad.begin(); iter != rad.end(); ++iter) {
+	    protocols.append(XrlAtom(iter->first));
+	    admin_distances.append(XrlAtom(iter->second));
+	}
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::rib_0_1_get_protocol_admin_distance(
+    // Input values,
+    const string& protocol,
+    const bool&	ipv4,
+    const bool&	unicast,
+    // Output values,
+    uint32_t& admin_distance)
+{
+
+    if (ipv4 && unicast) {
+	admin_distance = _urib4.get_protocol_admin_distance(protocol);
+    } else if (ipv4 && !unicast) {
+	admin_distance = _mrib4.get_protocol_admin_distance(protocol);
+    } else if (!ipv4 && unicast) {
+	admin_distance = _urib6.get_protocol_admin_distance(protocol);
+    } else if (!ipv4 && !unicast) {
+	admin_distance = _mrib6.get_protocol_admin_distance(protocol);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlRibTarget::rib_0_1_set_protocol_admin_distance(
+    // Input values,
+    const string& protocol,
+    const bool& ipv4,
+    const bool& ipv6,
+    const bool&	unicast,
+    const bool&	multicast,
+    const uint32_t& admin_distance)
+{
+
+    // Only the RIB may set an admin distance outside of the
+    // ranges 1 to 255, as 0 is reserved for directly-connected
+    // routes, and anything >= 255 will never make it into the FIB.
+    if (admin_distance <= CONNECTED_ADMIN_DISTANCE ||
+	admin_distance > UNKNOWN_ADMIN_DISTANCE) {
+	string err = c_format("Admin distance %d out of range for %s"
+			      "%s protocol \"%s\"; must be between "
+			      "1 and 255 inclusive.",
+			      admin_distance, "unicast", "IPv4",
+			      protocol.c_str());
+	return XrlCmdError::BAD_ARGS(err);
+    }
+
+    if (ipv4 && unicast &&
+	_urib4.set_protocol_admin_distance(protocol, admin_distance)
+	!= XORP_OK) {
+	string err = c_format("Could not set admin distance for %s "
+			      "%s protocol \"%s\"",
+			      "IPv4", "unicast", protocol.c_str());
+	return XrlCmdError::COMMAND_FAILED(err);
+    }
+
+    if (ipv4 && multicast &&
+	_mrib4.set_protocol_admin_distance(protocol, admin_distance)
+	!= XORP_OK) {
+	string err = c_format("Could not set admin distance for %s "
+			      "%s protocol \"%s\"",
+			      "IPv4", "multicast", protocol.c_str());
+	return XrlCmdError::COMMAND_FAILED(err);
+    }
+
+    if (ipv6 && unicast &&
+	_urib6.set_protocol_admin_distance(protocol, admin_distance)
+	!= XORP_OK) {
+	string err = c_format("Could not set admin distance for %s "
+			      "%s protocol \"%s\"",
+			      "IPv6", "unicast", protocol.c_str());
+	return XrlCmdError::COMMAND_FAILED(err);
+    }
+
+    if (ipv6 && multicast &&
+	_mrib6.set_protocol_admin_distance(protocol, admin_distance)
+	!= XORP_OK) {
+	string err = c_format("Could not set admin distance for %s "
+			      "%s protocol \"%s\"",
+			      "IPv6", "multicast", protocol.c_str());
+	return XrlCmdError::COMMAND_FAILED(err);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
 XrlCmdError
 XrlRibTarget::finder_event_observer_0_1_xrl_target_birth(
         const string&	target_class,
