@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8: 
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/output_updates.cc,v 1.17 2007/02/16 22:47:14 pavlin Exp $"
+#ident "$XORP: xorp/rip/output_updates.cc,v 1.20 2008/07/23 05:11:35 pavlin Exp $"
 
 #include "output_updates.hh"
 #include "packet_assembly.hh"
@@ -66,7 +66,10 @@ OutputUpdates<A>::output_packet()
 	    continue;
 
 	RouteEntryOrigin<A>* origin = NULL;
-	RouteEntry<A>* copy = new RouteEntry<A>(r->net(), p.first, p.second,
+	string ifname, vifname;		// XXX: not set, because not needed
+	RouteEntry<A>* copy = new RouteEntry<A>(r->net(), p.first,
+						ifname, vifname,
+						p.second,
 						origin, r->tag(),
 						r->policytags());
 
