@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/packet_queue.cc,v 1.11 2007/02/16 22:47:14 pavlin Exp $"
+#ident "$XORP: xorp/rip/packet_queue.cc,v 1.14 2008/07/23 05:11:35 pavlin Exp $"
 
 #include "rip_module.h"
 
@@ -41,7 +41,9 @@ void
 PacketQueue<A>::enqueue_packet(const RipPacket<A>* pkt)
 {
     while (_buffered_bytes + pkt->data_bytes() >= _max_buffered_bytes
-	   && drop_old() == true);
+	   && drop_old() == true) {
+	// XXX: Empty body
+    }
     _buffered_bytes += pkt->data_bytes();
     _ready_packets.push_back(pkt);
 }
