@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/redist.cc,v 1.12 2007/02/16 22:47:15 pavlin Exp $"
+#ident "$XORP: xorp/rip/redist.cc,v 1.16 2008/07/23 05:11:36 pavlin Exp $"
 
 #include "rip_module.h"
 #include "libxorp/xlog.h"
@@ -51,11 +51,8 @@ RedistRouteOrigin<A>::deletion_secs() const
 // RouteRedistributor
 
 template <typename A>
-RouteRedistributor<A>::RouteRedistributor(RouteDB<A>&	rdb,
-					  const string&	protocol,
-					  uint16_t	cost,
-					  uint16_t	tag)
-    : _route_db(rdb), _protocol(protocol), _cost(cost), _tag(tag), _wdrawer(0)
+RouteRedistributor<A>::RouteRedistributor(RouteDB<A>&	rdb)
+    : _route_db(rdb), _wdrawer(0)
 {
     _rt_origin = new RedistRouteOrigin<A>();
 }
@@ -67,57 +64,32 @@ RouteRedistributor<A>::~RouteRedistributor()
     delete _wdrawer;
 }
 
-template <typename A>
-bool
-RouteRedistributor<A>::add_route(const Net&  net, const Addr& nexthop,
-				 const PolicyTags& policytags)
-{
-    _route_db.add_rib_route(net, nexthop, _cost, _tag, _rt_origin, policytags);
-    return _route_db.update_route(net, nexthop, _cost, _tag, _rt_origin,
-				  policytags, false);
-}
-
 template <typename A>
 bool
 RouteRedistributor<A>::add_route(const Net&  	net,
 				 const Addr& 	nexthop,
+				 const string&	ifname,
+				 const string&	vifname,
 				 uint16_t 	cost,
 				 uint16_t 	tag,
 				 const PolicyTags& policytags)
 {
-    _route_db.add_rib_route(net, nexthop, cost, tag, _rt_origin, policytags);
-    return _route_db.update_route(net, nexthop, cost, tag, _rt_origin,
-				  policytags, false);
+    _route_db.add_rib_route(net, nexthop, ifname, vifname, cost, tag,
+			    _rt_origin, policytags);
+    return _route_db.update_route(net, nexthop, ifname, vifname, cost, tag,
+				  _rt_origin, policytags, false);
 }
 
 template <typename A>
 bool
 RouteRedistributor<A>::expire_route(const Net& net)
 {
+    string ifname, vifname;		// XXX: not set, because not needed
+
     _route_db.delete_rib_route(net);
-    return _route_db.update_route(net, A::ZERO(), RIP_INFINITY,
-				  _tag, _rt_origin, PolicyTags(), false);
-}
-
-template <typename A>
-const string&
-RouteRedistributor<A>::protocol() const
-{
-    return _protocol;
-}
-
-template <typename A>
-uint16_t
-RouteRedistributor<A>::tag() const
-{
-    return _tag;
-}
-
-template <typename A>
-uint16_t
-RouteRedistributor<A>::cost() const
-{
-    return _cost;
+    return _route_db.update_route(net, A::ZERO(), ifname, vifname,
+				  RIP_INFINITY, 0, _rt_origin, PolicyTags(),
+				  false);
 }
 
 template <typename A>
@@ -160,8 +132,8 @@ RouteRedistributor<A>::withdraw_batch()
     const RouteEntry<A>* r = _wdrawer->current_route();
     while (r != 0) {
 	if (r->origin() == _rt_origin) {
-	    _route_db.update_route(r->net(), r->nexthop(),
-				   RIP_INFINITY, r->tag(),
+	    _route_db.update_route(r->net(), r->nexthop(), r->ifname(),
+				   r->vifname(), RIP_INFINITY, r->tag(),
 				   _rt_origin, r->policytags(), false);
 	}
 	r = _wdrawer->next_route();
