@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/test_outputs.cc,v 1.29 2007/02/16 22:47:16 pavlin Exp $"
+#ident "$XORP: xorp/rip/test_outputs.cc,v 1.34 2008/07/23 05:11:37 pavlin Exp $"
 
 #include <set>
 
@@ -69,7 +69,7 @@ static const char *program_name         = "test_output_update";
 static const char *program_description  = "Test RIP Port processing";
 static const char *program_version_id   = "0.1";
 static const char *program_date         = "August, 2003";
-static const char *program_copyright    = "See file LICENSE.XORP";
+static const char *program_copyright    = "See file LICENSE";
 static const char *program_return_value = "0 on success, 1 if test error, "
 					  "2 if internal error";
 
@@ -241,7 +241,7 @@ struct ResponseReader {
 
     bool get(IPNet<A>& n, A& nh, uint32_t& cost, uint32_t& tag);
 
-    inline uint32_t packet_entry() const		{ return _pe; }
+    uint32_t packet_entry() const		{ return _pe; }
 protected:
     const RipPacket<A>* _rp;
     uint32_t		_pe; 	// Current packet entry
@@ -315,15 +315,9 @@ public:
     /* Final check responses valid */
     virtual bool valid_in_sum() const = 0;
 
-    inline uint32_t total_routes_seen() const
-    {
-	return _total_routes_seen;
-    }
+    uint32_t total_routes_seen() const { return _total_routes_seen; }
 
-    inline uint32_t test_peer_routes_seen() const
-    {
-	return _test_peer_routes_seen;
-    }
+    uint32_t test_peer_routes_seen() const { return _test_peer_routes_seen; }
 
 protected:
     uint32_t _total_routes_seen;
@@ -525,6 +519,8 @@ public:
     int
     run_test(RipHorizon horizon, HorizonValidatorBase<A>& validator)
     {
+	string ifname, vifname;		// XXX: not set, because not needed
+
 	_pm.test_port()->set_horizon(horizon);
 
 	RouteDB<A>&    rdb = _rip_system.route_db();
@@ -537,8 +533,8 @@ public:
 	for (typename set<IPNet<A> >::const_iterator n = _tpn.begin();
 	     n != _tpn.end(); n++) {
 	    RouteEntryOrigin<A>* reo = _pm.test_peer();
-	    if (rdb.update_route(*n, A::ZERO(), 5u, 0u, reo, PolicyTags(),
-				 false) == false) {
+	    if (rdb.update_route(*n, A::ZERO(), ifname, vifname, 5u, 0u, reo,
+				 PolicyTags(), false) == false) {
 		verbose_log("Failed to add route for %s\n",
 			    n->str().c_str());
 		return 1;
@@ -549,8 +545,8 @@ public:
 	for (typename set<IPNet<A> >::const_iterator n = this->_opn.begin();
 	     n != this->_opn.end(); n++) {
 	    RouteEntryOrigin<A>* reo = _pm.other_peer();
-	    if (rdb.update_route(*n, A::ZERO(), 5u, 0u, reo, PolicyTags(),
-				 false) == false) {
+	    if (rdb.update_route(*n, A::ZERO(), ifname, vifname, 5u, 0u, reo,
+				 PolicyTags(), false) == false) {
 		verbose_log("Failed to add route for %s\n",
 			    n->str().c_str());
 		return 1;
