@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/test_route_walk.cc,v 1.18 2007/02/16 22:47:16 pavlin Exp $"
+#ident "$XORP: xorp/rip/test_route_walk.cc,v 1.22 2008/07/23 05:11:37 pavlin Exp $"
 
 #include <set>
 
@@ -46,7 +46,7 @@ static const char *program_name         = "test_route_walk";
 static const char *program_description  = "Test RIP route walking";
 static const char *program_version_id   = "0.1";
 static const char *program_date         = "July, 2003";
-static const char *program_copyright    = "See file LICENSE.XORP";
+static const char *program_copyright    = "See file LICENSE";
 static const char *program_return_value = "0 on success, 1 if test error, 2 if internal error";
 
 
@@ -188,6 +188,7 @@ public:
     int
     run_test()
     {
+	string ifname, vifname;		// XXX: not set, because not needed
 	const uint32_t n_routes = 20000;
 
 	verbose_log("Creating routes for nets\n");
@@ -200,7 +201,8 @@ public:
 	_e.current_time(tv_add_start);
 
 	for_each(nets.begin(), nets.end(),
-		 RouteInjector<A>(rdb, A::ZERO(), 5, _pm.the_peer()));
+		 RouteInjector<A>(rdb, A::ZERO(), ifname, vifname, 5,
+				  _pm.the_peer()));
 
 	TimeVal tv_add_end;
 	_e.current_time(tv_add_end);
