@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/tools/ripng_announcer.cc,v 1.12 2007/02/16 22:47:20 pavlin Exp $"
+#ident "$XORP: xorp/rip/tools/ripng_announcer.cc,v 1.15 2008/07/23 05:11:40 pavlin Exp $"
 
 #include "rip/rip_module.h"
 
@@ -155,8 +155,8 @@ init_rip_socket(int if_num)
 	fd.clear();
 	return fd;
     }
-     if (comm_set_ttl(fd, 255) != XORP_OK) {
-	 cerr << "comm_set_ttl failed" << endl;
+     if (comm_set_multicast_ttl(fd, 255) != XORP_OK) {
+	 cerr << "comm_set_multicast_ttl failed" << endl;
 	 comm_close(fd);
 	 fd.clear();
 	 return fd;
