@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/tools/show_stats.cc,v 1.13 2007/02/16 22:47:20 pavlin Exp $"
+#ident "$XORP: xorp/rip/tools/show_stats.cc,v 1.17 2008/07/23 05:11:40 pavlin Exp $"
 
 #include <iomanip>
 
@@ -132,9 +132,9 @@ pretty_print_counters(const XrlAtomList* descriptions,
 
 template <typename A>
 struct GetListAtom {
-    inline GetListAtom(const XrlAtomList& xal) : _xal(xal) {}
-    inline GetListAtom(const XrlAtomList* pxal) : _xal(*pxal) {}
-    inline const A& at(uint32_t index);
+    GetListAtom(const XrlAtomList& xal) : _xal(xal) {}
+    GetListAtom(const XrlAtomList* pxal) : _xal(*pxal) {}
+    const A& at(uint32_t index);
 
 private:
     const XrlAtomList& _xal;
@@ -174,9 +174,9 @@ public:
 		   const A& 		addr)
 	: XrlJobBase(jq), _ifname(ifname), _vifname(vifname), _addr(addr)
     {}
-    inline const string& ifname() const 		{ return _ifname; }
-    inline const string& vifname()const			{ return _vifname; }
-    inline const A& addr() const			{ return _addr;	}
+    const string& ifname() const 		{ return _ifname; }
+    const string& vifname()const		{ return _vifname; }
+    const A& addr() const			{ return _addr;	}
 
 protected:
     string	_ifname;
@@ -354,7 +354,7 @@ template <>
 bool
 GetAllAddressStats<IPv6>::dispatch()
 {
-    XrlRipV0p1Client cl(queue().sender());
+    XrlRipngV0p1Client cl(queue().sender());
     return cl.send_get_all_addresses(queue().target().c_str(),
 		callback(this, &GetAllAddressStats<IPv6>::cmd_callback)
 				     );
