@@ -119,50 +119,6 @@ rip_policy_redist4_delete_route4()
     call_xrl_wrapper -p all "${XRL}"
 }
 
-rip_redist4_add_route()
-{
-    if [ $# -ne 8 ] ; then
-        echo "Usage: rip_redist4_add_route <dst:ipv4net> <nexthop:ipv4> <ifname:txt> <vifname:txt> <metric:u32> <admin_distance:u32> <cookie:txt> <protocol_origin:txt>"
-        exit 1
-    fi
-
-    XRL="finder://rip/redist4/0.1/add_route?dst:ipv4net=$1&nexthop:ipv4=$2&ifname:txt=$3&vifname:txt=$4&metric:u32=$5&admin_distance:u32=$6&cookie:txt=$7&protocol_origin:txt=$8"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-rip_redist4_delete_route()
-{
-    if [ $# -ne 8 ] ; then
-        echo "Usage: rip_redist4_delete_route <dst:ipv4net> <nexthop:ipv4> <ifname:txt> <vifname:txt> <metric:u32> <admin_distance:u32> <cookie:txt> <protocol_origin:txt>"
-        exit 1
-    fi
-
-    XRL="finder://rip/redist4/0.1/delete_route?dst:ipv4net=$1&nexthop:ipv4=$2&ifname:txt=$3&vifname:txt=$4&metric:u32=$5&admin_distance:u32=$6&cookie:txt=$7&protocol_origin:txt=$8"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-rip_redist4_starting_route_dump()
-{
-    if [ $# -ne 1 ] ; then
-        echo "Usage: rip_redist4_starting_route_dump <cookie:txt>"
-        exit 1
-    fi
-
-    XRL="finder://rip/redist4/0.1/starting_route_dump?cookie:txt=$1"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-rip_redist4_finishing_route_dump()
-{
-    if [ $# -ne 1 ] ; then
-        echo "Usage: rip_redist4_finishing_route_dump <cookie:txt>"
-        exit 1
-    fi
-
-    XRL="finder://rip/redist4/0.1/finishing_route_dump?cookie:txt=$1"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
 rip_rip_add_rip_address()
 {
     if [ $# -ne 3 ] ; then
@@ -625,47 +581,36 @@ rip_rip_get_peer_counters()
     call_xrl_wrapper -p all "${XRL}"
 }
 
-rip_rip_redist_protocol_routes()
-{
-    if [ $# -ne 3 ] ; then
-        echo "Usage: rip_rip_redist_protocol_routes <protocol_name:txt> <cost:u32> <tag:u32>"
-        exit 1
-    fi
-
-    XRL="finder://rip/rip/0.1/redist_protocol_routes?protocol_name:txt=$1&cost:u32=$2&tag:u32=$3"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-rip_rip_no_redist_protocol_routes()
-{
-    if [ $# -ne 1 ] ; then
-        echo "Usage: rip_rip_no_redist_protocol_routes <protocol_name:txt>"
-        exit 1
-    fi
-
-    XRL="finder://rip/rip/0.1/no_redist_protocol_routes?protocol_name:txt=$1"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
 rip_socket4_user_recv_event()
 {
-    if [ $# -ne 4 ] ; then
-        echo "Usage: rip_socket4_user_recv_event <sockid:txt> <src_host:ipv4> <src_port:u32> <data:binary>"
+    if [ $# -ne 6 ] ; then
+        echo "Usage: rip_socket4_user_recv_event <sockid:txt> <if_name:txt> <vif_name:txt> <src_host:ipv4> <src_port:u32> <data:binary>"
         exit 1
     fi
 
-    XRL="finder://rip/socket4_user/0.1/recv_event?sockid:txt=$1&src_host:ipv4=$2&src_port:u32=$3&data:binary=$4"
+    XRL="finder://rip/socket4_user/0.1/recv_event?sockid:txt=$1&if_name:txt=$2&vif_name:txt=$3&src_host:ipv4=$4&src_port:u32=$5&data:binary=$6"
     call_xrl_wrapper -p all "${XRL}"
 }
 
-rip_socket4_user_connect_event()
+rip_socket4_user_inbound_connect_event()
 {
     if [ $# -ne 4 ] ; then
-        echo "Usage: rip_socket4_user_connect_event <sockid:txt> <src_host:ipv4> <src_port:u32> <new_sockid:txt>"
+        echo "Usage: rip_socket4_user_inbound_connect_event <sockid:txt> <src_host:ipv4> <src_port:u32> <new_sockid:txt>"
         exit 1
     fi
 
-    XRL="finder://rip/socket4_user/0.1/connect_event?sockid:txt=$1&src_host:ipv4=$2&src_port:u32=$3&new_sockid:txt=$4"
+    XRL="finder://rip/socket4_user/0.1/inbound_connect_event?sockid:txt=$1&src_host:ipv4=$2&src_port:u32=$3&new_sockid:txt=$4"
+    call_xrl_wrapper -p all "${XRL}"
+}
+
+rip_socket4_user_outgoing_connect_event()
+{
+    if [ $# -ne 1 ] ; then
+        echo "Usage: rip_socket4_user_outgoing_connect_event <sockid:txt>"
+        exit 1
+    fi
+
+    XRL="finder://rip/socket4_user/0.1/outgoing_connect_event?sockid:txt=$1"
     call_xrl_wrapper -p all "${XRL}"
 }
 
@@ -680,14 +625,14 @@ rip_socket4_user_error_event()
     call_xrl_wrapper -p all "${XRL}"
 }
 
-rip_socket4_user_close_event()
+rip_socket4_user_disconnect_event()
 {
-    if [ $# -ne 2 ] ; then
-        echo "Usage: rip_socket4_user_close_event <sockid:txt> <reason:txt>"
+    if [ $# -ne 1 ] ; then
+        echo "Usage: rip_socket4_user_disconnect_event <sockid:txt>"
         exit 1
     fi
 
-    XRL="finder://rip/socket4_user/0.1/close_event?sockid:txt=$1&reason:txt=$2"
+    XRL="finder://rip/socket4_user/0.1/disconnect_event?sockid:txt=$1"
     call_xrl_wrapper -p all "${XRL}"
 }
 
