@@ -119,50 +119,6 @@ ripng_policy_redist6_delete_route6()
     call_xrl_wrapper -p all "${XRL}"
 }
 
-ripng_redist6_add_route()
-{
-    if [ $# -ne 8 ] ; then
-        echo "Usage: ripng_redist6_add_route <dst:ipv6net> <nexthop:ipv6> <ifname:txt> <vifname:txt> <metric:u32> <admin_distance:u32> <cookie:txt> <protocol_origin:txt>"
-        exit 1
-    fi
-
-    XRL="finder://ripng/redist6/0.1/add_route?dst:ipv6net=$1&nexthop:ipv6=$2&ifname:txt=$3&vifname:txt=$4&metric:u32=$5&admin_distance:u32=$6&cookie:txt=$7&protocol_origin:txt=$8"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-ripng_redist6_delete_route()
-{
-    if [ $# -ne 8 ] ; then
-        echo "Usage: ripng_redist6_delete_route <dst:ipv6net> <nexthop:ipv6> <ifname:txt> <vifname:txt> <metric:u32> <admin_distance:u32> <cookie:txt> <protocol_origin:txt>"
-        exit 1
-    fi
-
-    XRL="finder://ripng/redist6/0.1/delete_route?dst:ipv6net=$1&nexthop:ipv6=$2&ifname:txt=$3&vifname:txt=$4&metric:u32=$5&admin_distance:u32=$6&cookie:txt=$7&protocol_origin:txt=$8"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-ripng_redist6_starting_route_dump()
-{
-    if [ $# -ne 1 ] ; then
-        echo "Usage: ripng_redist6_starting_route_dump <cookie:txt>"
-        exit 1
-    fi
-
-    XRL="finder://ripng/redist6/0.1/starting_route_dump?cookie:txt=$1"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-ripng_redist6_finishing_route_dump()
-{
-    if [ $# -ne 1 ] ; then
-        echo "Usage: ripng_redist6_finishing_route_dump <cookie:txt>"
-        exit 1
-    fi
-
-    XRL="finder://ripng/redist6/0.1/finishing_route_dump?cookie:txt=$1"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
 ripng_ripng_add_rip_address()
 {
     if [ $# -ne 3 ] ; then
@@ -581,47 +537,36 @@ ripng_ripng_get_peer_counters()
     call_xrl_wrapper -p all "${XRL}"
 }
 
-ripng_ripng_redist_protocol_routes()
-{
-    if [ $# -ne 3 ] ; then
-        echo "Usage: ripng_ripng_redist_protocol_routes <protocol_name:txt> <cost:u32> <tag:u32>"
-        exit 1
-    fi
-
-    XRL="finder://ripng/ripng/0.1/redist_protocol_routes?protocol_name:txt=$1&cost:u32=$2&tag:u32=$3"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
-ripng_ripng_no_redist_protocol_routes()
-{
-    if [ $# -ne 1 ] ; then
-        echo "Usage: ripng_ripng_no_redist_protocol_routes <protocol_name:txt>"
-        exit 1
-    fi
-
-    XRL="finder://ripng/ripng/0.1/no_redist_protocol_routes?protocol_name:txt=$1"
-    call_xrl_wrapper -p all "${XRL}"
-}
-
 ripng_socket6_user_recv_event()
 {
-    if [ $# -ne 4 ] ; then
-        echo "Usage: ripng_socket6_user_recv_event <sockid:txt> <src_host:ipv6> <src_port:u32> <data:binary>"
+    if [ $# -ne 6 ] ; then
+        echo "Usage: ripng_socket6_user_recv_event <sockid:txt> <if_name:txt> <vif_name:txt> <src_host:ipv6> <src_port:u32> <data:binary>"
         exit 1
     fi
 
-    XRL="finder://ripng/socket6_user/0.1/recv_event?sockid:txt=$1&src_host:ipv6=$2&src_port:u32=$3&data:binary=$4"
+    XRL="finder://ripng/socket6_user/0.1/recv_event?sockid:txt=$1&if_name:txt=$2&vif_name:txt=$3&src_host:ipv6=$4&src_port:u32=$5&data:binary=$6"
     call_xrl_wrapper -p all "${XRL}"
 }
 
-ripng_socket6_user_connect_event()
+ripng_socket6_user_inbound_connect_event()
 {
     if [ $# -ne 4 ] ; then
-        echo "Usage: ripng_socket6_user_connect_event <sockid:txt> <src_host:ipv6> <src_port:u32> <new_sockid:txt>"
+        echo "Usage: ripng_socket6_user_inbound_connect_event <sockid:txt> <src_host:ipv6> <src_port:u32> <new_sockid:txt>"
         exit 1
     fi
 
-    XRL="finder://ripng/socket6_user/0.1/connect_event?sockid:txt=$1&src_host:ipv6=$2&src_port:u32=$3&new_sockid:txt=$4"
+    XRL="finder://ripng/socket6_user/0.1/inbound_connect_event?sockid:txt=$1&src_host:ipv6=$2&src_port:u32=$3&new_sockid:txt=$4"
+    call_xrl_wrapper -p all "${XRL}"
+}
+
+ripng_socket6_user_outgoing_connect_event()
+{
+    if [ $# -ne 1 ] ; then
+        echo "Usage: ripng_socket6_user_outgoing_connect_event <sockid:txt>"
+        exit 1
+    fi
+
+    XRL="finder://ripng/socket6_user/0.1/outgoing_connect_event?sockid:txt=$1"
     call_xrl_wrapper -p all "${XRL}"
 }
 
@@ -636,14 +581,14 @@ ripng_socket6_user_error_event()
     call_xrl_wrapper -p all "${XRL}"
 }
 
-ripng_socket6_user_close_event()
+ripng_socket6_user_disconnect_event()
 {
-    if [ $# -ne 2 ] ; then
-        echo "Usage: ripng_socket6_user_close_event <sockid:txt> <reason:txt>"
+    if [ $# -ne 1 ] ; then
+        echo "Usage: ripng_socket6_user_disconnect_event <sockid:txt>"
         exit 1
     fi
 
-    XRL="finder://ripng/socket6_user/0.1/close_event?sockid:txt=$1&reason:txt=$2"
+    XRL="finder://ripng/socket6_user/0.1/disconnect_event?sockid:txt=$1"
     call_xrl_wrapper -p all "${XRL}"
 }
 
