@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/xorp_rip_main.cc,v 1.15 2007/02/16 22:47:17 pavlin Exp $"
+#ident "$XORP: xorp/rip/xorp_rip_main.cc,v 1.20 2008/07/23 05:11:37 pavlin Exp $"
 
 #include "rip_module.h"
 #include "libxorp/xlog.h"
@@ -73,8 +73,7 @@ public:
 	: _st(in_state), _o(out)
     {}
 
-    inline void operator() (const ServiceBase* sb)
-    {
+    void operator() (const ServiceBase* sb) {
 	if (sb->status() != _st) return;
 	if (_o.empty() == false)
 	    _o += ", ";
@@ -115,7 +114,7 @@ public:
 
     bool add_service(ServiceBase* sb)
     {
-	if (sb->set_observer(this)) {
+	if (sb->set_observer(this) == XORP_OK) {
 	    _services.push_back(sb);
 	    update_status();
 	    return true;
@@ -175,9 +174,7 @@ public:
 	XLOG_ASSERT(_st == SERVICE_RUNNING);
 	_t.set_status(PROC_READY, "");
     }
-    inline bool have_status(ServiceStatus st) {
-	return _st & st;
-    }
+    bool have_status(ServiceStatus st) { return _st & st; }
 
 protected:
     typename XrlTarget<A>::Type&	_t;
@@ -269,7 +266,7 @@ XorpRip<A>::run(const string& finder_host, uint16_t finder_port)
 				    FinderConstants::FINDER_DEFAULT_HOST(),
 				    FinderConstants::FINDER_DEFAULT_PORT());
 	XrlPortManager<A>	xpm(rip_system, xsr, ixm);
-	XrlRedistManager<A>	xrm(rip_system, xsr);
+	XrlRedistManager<A>	xrm(rip_system);
 
 	XrlTargetType xrlt(e, xsr, xps, xpm, xrm, _stop_flag, rip_system);
 
