@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/xrl_port_io.cc,v 1.22 2007/02/16 22:47:17 pavlin Exp $"
+#ident "$XORP: xorp/rip/xrl_port_io.cc,v 1.29 2008/07/23 05:11:37 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 
@@ -72,18 +72,8 @@ static SocketManager<IPv6> socket_manager;
 
 #ifdef INSTANTIATE_IPV4
 
-#include "xrl/interfaces/socket4_locator_xif.hh"
 #include "xrl/interfaces/socket4_xif.hh"
 
-template <>
-bool
-XrlPortIO<IPv4>::request_socket_server()
-{
-    XrlSocket4LocatorV0p1Client cl(&_xr);
-    return cl.send_find_socket_server_for_addr(xrl_fea_name(), _addr,
-		callback(this, &XrlPortIO<IPv4>::socket_server_cb));
-}
-
 template <>
 bool
 XrlPortIO<IPv4>::request_open_bind_socket()
@@ -91,19 +81,19 @@ XrlPortIO<IPv4>::request_open_bind_socket()
     XrlSocket4V0p1Client cl(&_xr);
     return cl.send_udp_open_and_bind(
 		_ss.c_str(), _xr.instance_name(), IPv4::ANY(),
-		RIP_AF_CONSTANTS<IPv4>::IP_PORT, false,
+		RIP_AF_CONSTANTS<IPv4>::IP_PORT,
 		callback(this, &XrlPortIO<IPv4>::open_bind_socket_cb)
 		);
 }
 
 template <>
 bool
-XrlPortIO<IPv4>::request_ttl_one()
+XrlPortIO<IPv4>::request_ttl()
 {
     XrlSocket4V0p1Client cl(&_xr);
     return cl.send_set_socket_option(
-		_ss.c_str(), socket_id(), "multicast_ttl", 1,
-		callback(this, &XrlPortIO<IPv4>::ttl_one_cb));
+		_ss.c_str(), socket_id(), "multicast_ttl", RIP_TTL,
+		callback(this, &XrlPortIO<IPv4>::ttl_cb));
 }
 
 template <>
@@ -185,18 +175,8 @@ XrlPortIO<IPv4>::send(const IPv4& 		dst_addr,
 
 #ifdef INSTANTIATE_IPV6
 
-#include "xrl/interfaces/socket6_locator_xif.hh"
 #include "xrl/interfaces/socket6_xif.hh"
 
-template <>
-bool
-XrlPortIO<IPv6>::request_socket_server()
-{
-    XrlSocket6LocatorV0p1Client cl(&_xr);
-    return cl.send_find_socket_server_for_addr(xrl_fea_name(), _addr,
-		callback(this, &XrlPortIO<IPv6>::socket_server_cb));
-}
-
 template <>
 bool
 XrlPortIO<IPv6>::request_open_bind_socket()
@@ -204,19 +184,19 @@ XrlPortIO<IPv6>::request_open_bind_socket()
     XrlSocket6V0p1Client cl(&_xr);
     return cl.send_udp_open_and_bind(
 		_ss.c_str(), _xr.instance_name(), IPv6::ANY(),
-		RIP_AF_CONSTANTS<IPv6>::IP_PORT, false,
+		RIP_AF_CONSTANTS<IPv6>::IP_PORT,
 		callback(this, &XrlPortIO<IPv6>::open_bind_socket_cb)
 		);
 }
 
 template <>
 bool
-XrlPortIO<IPv6>::request_ttl_one()
+XrlPortIO<IPv6>::request_ttl()
 {
     XrlSocket6V0p1Client cl(&_xr);
     return cl.send_set_socket_option(
-		_ss.c_str(), socket_id(), "multicast_hops", 1,
-		callback(this, &XrlPortIO<IPv6>::ttl_one_cb));
+		_ss.c_str(), socket_id(), "multicast_ttl", RIP_NG_HOP_COUNT,
+		callback(this, &XrlPortIO<IPv6>::ttl_cb));
 }
 
 template <>
@@ -323,22 +303,22 @@ XrlPortIO<A>::pending() const
 }
 
 template <typename A>
-bool
+int
 XrlPortIO<A>::startup()
 {
     _pending = true;
     set_status(SERVICE_STARTING);
-    if (request_socket_server() == false) {
+    if (startup_socket() == false) {
 	set_status(SERVICE_FAILED,
 		   "Failed to find appropriate socket server.");
-	return false;
+	return (XORP_ERROR);
     }
 
-    return true;
+    return (XORP_OK);
 }
 
 template <typename A>
-bool
+int
 XrlPortIO<A>::shutdown()
 {
     _pending = true;
@@ -348,19 +328,14 @@ XrlPortIO<A>::shutdown()
 	set_status(SERVICE_SHUTDOWN);
     }
 
-    return true;
+    return (XORP_OK);
 }
 
 template <typename A>
-void
-XrlPortIO<A>::socket_server_cb(const XrlError& e, const string* pss)
+bool
+XrlPortIO<A>::startup_socket()
 {
-    if (e != XrlError::OKAY()) {
-	set_status(SERVICE_FAILED);
-	return;
-    }
-
-    _ss = *pss;
+    _ss = xrl_fea_name();
 
     _sid = socket_manager.sockid(_ss);
     if (_sid == SocketManager<A>::no_entry) {
@@ -368,8 +343,8 @@ XrlPortIO<A>::socket_server_cb(const XrlError& e, const string* pss)
 	// If we succeed here the path is:
 	// request_open_bind_socket()
 	// -> open_bind_socket_cb()
-	//    -> request_ttl_one()
-	//	 -> ttl_one_cb()
+	//    -> request_ttl()
+	//	 -> ttl_cb()
 	//	     -> request_no_loop()
 	//		-> no_loop_cb()
 	//		   ->request_socket_join()
@@ -377,6 +352,7 @@ XrlPortIO<A>::socket_server_cb(const XrlError& e, const string* pss)
 	if (request_open_bind_socket() == false) {
 	    set_status(SERVICE_FAILED,
 		       "Failed sending RIP socket open request.");
+	    return false;
 	}
     } else {
 	// RIP socket exists, join appropriate interface to multicast
@@ -384,8 +360,11 @@ XrlPortIO<A>::socket_server_cb(const XrlError& e, const string* pss)
 	if (request_socket_join() == false) {
 	    set_status(SERVICE_FAILED,
 		       "Failed sending multicast join request.");
+	    return false;
 	}
     }
+
+    return true;
 }
 
 template <typename A>
@@ -400,17 +379,17 @@ XrlPortIO<A>::open_bind_socket_cb(const XrlError& e, const string* psid)
     _sid = *psid;
     socket_manager.add_sockid(_ss, _sid);
 
-    if (request_ttl_one() == false) {
-	set_status(SERVICE_FAILED, "Failed requesting ttl/hops of 1.");
+    if (request_ttl() == false) {
+	set_status(SERVICE_FAILED, "Failed requesting ttl/hops.");
     }
 }
 
 template <typename A>
 void
-XrlPortIO<A>::ttl_one_cb(const XrlError& e)
+XrlPortIO<A>::ttl_cb(const XrlError& e)
 {
     if (e != XrlError::OKAY()) {
-	XLOG_WARNING("Failed to set ttl/hops to 1");
+	XLOG_WARNING("Failed to set ttl/hops.");
     }
     if (request_no_loop() == false) {
 	set_status(SERVICE_FAILED,
