@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/xrl_port_manager.cc,v 1.24 2007/02/16 22:47:17 pavlin Exp $"
+#ident "$XORP: xorp/rip/xrl_port_manager.cc,v 1.31 2008/07/23 05:11:37 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 
@@ -46,7 +46,7 @@ address_exists(const IfMgrIfTree&	iftree,
     debug_msg("Looking for %s/%s/%s\n",
 	      ifname.c_str(), vifname.c_str(), addr.str().c_str());
 
-    const IfMgrIfAtom* ia = iftree.find_if(ifname);
+    const IfMgrIfAtom* ia = iftree.find_interface(ifname);
     if (ia == NULL)
 	return false;
 
@@ -75,7 +75,7 @@ address_enabled(const IfMgrIfTree&	iftree,
     debug_msg("Looking for %s/%s/%s\n",
 	      ifname.c_str(), vifname.c_str(), addr.str().c_str());
 
-    const IfMgrIfAtom* ia = iftree.find_if(ifname);
+    const IfMgrIfAtom* ia = iftree.find_interface(ifname);
     if (ia == 0 || ia->enabled() == false || ia->no_carrier()) {
 	debug_msg("if %s exists ? %d ?\n", ifname.c_str(), (ia ? 1 : 0));
 	return false;
@@ -102,8 +102,8 @@ address_enabled(const IfMgrIfTree&	iftree,
  */
 template <typename A>
 struct port_has_address {
-    inline port_has_address(const A& addr) : _addr(addr) {}
-    inline bool operator() (const Port<A>* p) {
+    port_has_address(const A& addr) : _addr(addr) {}
+    bool operator() (const Port<A>* p) {
 	const PortIOBase<A>* io = p->io_handler();
 	return io && io->address() == _addr;
     }
@@ -117,7 +117,7 @@ private:
  */
 template <typename A>
 struct port_has_io_in_state {
-    inline port_has_io_in_state(ServiceStatus st) : _st(st) {}
+    port_has_io_in_state(ServiceStatus st) : _st(st) {}
 
     bool operator() (const Port<A>* p) const
     {
@@ -140,17 +140,24 @@ protected:
  */
 template <typename A>
 struct is_port_for {
-    inline is_port_for(const string* sockid, const A* addr, IfMgrXrlMirror* im)
-	: _psid(sockid), _pa(addr), _pim(im)
+    is_port_for(const string* sockid, const string* ifname,
+		const string* vifname, const A* addr, IfMgrXrlMirror* im)
+	: _psid(sockid),
+	  _ifname(ifname),
+	  _vifname(vifname),
+	  _pa(addr),
+	  _pim(im)
     {}
 
-    inline bool operator() (Port<A>*& p);
+    bool operator() (Port<A>*& p);
 
 protected:
-    inline bool link_addr_valid() const;
+    bool link_addr_valid() const;
 
 private:
     const string* 	_psid;
+    const string*	_ifname;
+    const string*	_vifname;
     const A* 		_pa;
     IfMgrXrlMirror* 	_pim;
 };
@@ -195,6 +202,14 @@ is_port_for<A>::operator() (Port<A>*& p)
     if (xio->address() == *_pa)
 	return false;
 
+    // Check the incoming interface and vif name (if known)
+    if ((! _ifname->empty()) && (! _vifname->empty())) {
+	if (xio->ifname() != *_ifname)
+	    return false;
+	if (xio->vifname() != *_vifname)
+	    return false;
+    }
+
     //
     // Packet has arrived on multicast socket and is not one of ours.
     //
@@ -222,18 +237,18 @@ is_port_for<A>::operator() (Port<A>*& p)
 // XrlPortManager
 
 template <typename A>
-bool
+int
 XrlPortManager<A>::startup()
 {
     set_status(SERVICE_STARTING);
     // Transition to SERVICE_RUNNING occurs when tree_complete() is called, ie
     // we have interface/vif/address state available.
 
-    return true;
+    return (XORP_OK);
 }
 
 template <typename A>
-bool
+int
 XrlPortManager<A>::shutdown()
 {
     set_status(SERVICE_SHUTTING_DOWN);
@@ -263,7 +278,7 @@ XrlPortManager<A>::shutdown()
 	}
     }
 
-    return true;
+    return (XORP_OK);
 }
 
 template <typename A>
@@ -296,7 +311,7 @@ XrlPortManager<A>::updates_made()
 	    XLOG_INFO("Detected iftree change on %s %s %s setting transport enabled %s",
 		      xio->ifname().c_str(), xio->vifname().c_str(),
 		      xio->address().str().c_str(),
-		      (fea_en ? "true" : "false"));
+		      bool_c_str(fea_en));
 	    xio->set_enabled(fea_en);
 	}
     }
@@ -370,33 +385,36 @@ XrlPortManager<A>::remove_rip_address(const string& 	/* ifname */,
 template <typename A>
 bool
 XrlPortManager<A>::deliver_packet(const string& 		sockid,
+				  const string&			ifname,
+				  const string&			vifname,
 				  const A& 			src_addr,
 				  uint16_t 			src_port,
 				  const vector<uint8_t>& 	pdata)
 {
     typename PortManagerBase<A>::PortList& pl = this->ports();
     typename PortManagerBase<A>::PortList::iterator i;
+    Port<A>* p = NULL;
 
-    debug_msg("Packet on %s from %s/%u %u bytes\n",
-	      sockid.c_str(), src_addr.str().c_str(), src_port,
-	      XORP_UINT_CAST(pdata.size()));
+    debug_msg("Packet on %s from interface %s vif %s %s/%u %u bytes\n",
+	      sockid.c_str(), ifname.c_str(), vifname.c_str(),
+	      src_addr.str().c_str(), src_port, XORP_UINT_CAST(pdata.size()));
 
     i = find_if(pl.begin(), pl.end(),
-		is_port_for<A>(&sockid, &src_addr, &_ifm));
-
+		is_port_for<A>(&sockid, &ifname, &vifname, &src_addr, &_ifm));
     if (i == this->ports().end()) {
 	debug_msg("Discarding packet %s/%u %u bytes\n",
 		  src_addr.str().c_str(), src_port,
 		  XORP_UINT_CAST(pdata.size()));
 	return false;
     }
-    Port<A>* p = *i;
-    p->port_io_receive(src_addr, src_port, &pdata[0], pdata.size());
-
+    p = *i;
     XLOG_ASSERT(find_if(++i, pl.end(),
-			is_port_for<A>(&sockid, &src_addr, &_ifm))
+			is_port_for<A>(&sockid, &ifname, &vifname, &src_addr,
+				       &_ifm))
 		== pl.end());
 
+    p->port_io_receive(src_addr, src_port, &pdata[0], pdata.size());
+
     return true;
 }
 
