@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -713,85 +713,41 @@ XrlRipTarget::rip_0_1_get_peer_counters(const string&	ifn,
 					   descs, vals, peer_last_active);
 }
 
-XrlCmdError
-XrlRipTarget::rip_0_1_redist_protocol_routes(const string&	protocol_name,
-					     const uint32_t&	cost,
-					     const uint32_t&	tag)
-{
-    return _ct->ripx_0_1_redist_protocol_routes(protocol_name, cost, tag);
-}
-
-XrlCmdError
-XrlRipTarget::rip_0_1_no_redist_protocol_routes(const string&	protocol_name)
-{
-    return _ct->ripx_0_1_no_redist_protocol_routes(protocol_name);
-}
-
-XrlCmdError
-XrlRipTarget::redist4_0_1_add_route(const IPv4Net&	net,
-				    const IPv4&		nexthop,
-				    const string&	ifname,
-				    const string&	vifname,
-				    const uint32_t&	metric,
-				    const uint32_t&	admin_distance,
-				    const string&	cookie,
-				    const string&	protocol_origin)
-{
-    return _ct->redistx_0_1_add_route(net, nexthop, ifname, vifname, metric,
-				      admin_distance, cookie, protocol_origin);
-}
-
-XrlCmdError
-XrlRipTarget::redist4_0_1_delete_route(const IPv4Net&	net,
-				       const IPv4&	nexthop,
-				       const string&	ifname,
-				       const string&	vifname,
-				       const uint32_t&	metric,
-				       const uint32_t&	admin_distance,
-				       const string&	cookie,
-				       const string&	protocol_origin)
-{
-    return _ct->redistx_0_1_delete_route(net, nexthop, ifname, vifname, metric,
-					 admin_distance, cookie,
-					 protocol_origin);
-}
-
-XrlCmdError
-XrlRipTarget::redist4_0_1_starting_route_dump(const string& /* cookie */)
-{
-    return XrlCmdError::OKAY();
-}
-
-XrlCmdError
-XrlRipTarget::redist4_0_1_finishing_route_dump(const string& /* cookie */)
-{
-    return XrlCmdError::OKAY();
-}
-
 
 XrlCmdError
 XrlRipTarget::socket4_user_0_1_recv_event(
 					const string&		sockid,
+					const string&		if_name,
+					const string&		vif_name,
 					const IPv4&		src_host,
 					const uint32_t&		src_port,
 					const vector<uint8_t>&	pdata
 					)
 {
-    return _ct->socketx_user_0_1_recv_event(sockid, src_host, src_port, pdata);
+    return _ct->socketx_user_0_1_recv_event(sockid, if_name, vif_name,
+					    src_host, src_port, pdata);
 }
 
 XrlCmdError
-XrlRipTarget::socket4_user_0_1_connect_event(const string&	sockid,
-					     const IPv4&	src_host,
-					     const uint32_t&	src_port,
-					     const string&	new_sockid,
-					     bool&		accept)
+XrlRipTarget::socket4_user_0_1_inbound_connect_event(
+    const string&	sockid,
+    const IPv4&		src_host,
+    const uint32_t&	src_port,
+    const string&	new_sockid,
+    bool&		accept)
 {
-    return _ct->socketx_user_0_1_connect_event(sockid,
-					       src_host,
-					       src_port,
-					       new_sockid,
-					       accept);
+    return _ct->socketx_user_0_1_inbound_connect_event(sockid,
+						       src_host,
+						       src_port,
+						       new_sockid,
+						       accept);
+}
+
+XrlCmdError
+XrlRipTarget::socket4_user_0_1_outgoing_connect_event(
+    const string&	sockid)
+{
+    return _ct->socketx_user_0_1_outgoing_connect_event(sockid);
 }
 
 XrlCmdError
@@ -803,10 +759,9 @@ XrlRipTarget::socket4_user_0_1_error_event(const string&	sockid,
 }
 
 XrlCmdError
-XrlRipTarget::socket4_user_0_1_close_event(const string&	sockid,
-					   const string&	reason)
+XrlRipTarget::socket4_user_0_1_disconnect_event(const string&	sockid)
 {
-    return _ct->socketx_user_0_1_close_event(sockid, reason);
+    return _ct->socketx_user_0_1_disconnect_event(sockid);
 }
 
 XrlCmdError
