@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +13,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/main_rtrmgr.cc,v 1.71 2007/02/16 22:47:22 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/main_rtrmgr.cc,v 1.77 2008/07/23 05:11:41 pavlin Exp $"
 
 #include "rtrmgr_module.h"
 
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/daemon.h"
 #include "libxorp/eventloop.hh"
 #include "libxorp/utils.hh"
 
@@ -65,6 +67,7 @@ static bool default_do_exec = true;
 static bool default_do_restart = false;
 static bool default_verbose = false;
 
+
 //
 // Local state
 //
@@ -72,12 +75,19 @@ static volatile bool	running = false;
 static string	template_dir;
 static string	xrl_targets_dir;
 static string	boot_file;
+static bool     do_logfile = false;
+static bool     do_pidfile = false;
+static bool     do_syslog = false;
 static bool	do_exec = default_do_exec;
 static bool	do_restart = default_do_restart;
 static bool	verbose = default_verbose;
 list<IPv4>	bind_addrs;
 uint16_t	bind_port = FinderConstants::FINDER_DEFAULT_PORT();
+static string	syslogspec;
+static string	logfilename;
+static string	pidfilename;
 int32_t		quit_time = -1;
+static bool	daemon_mode = false;
 
 static void cleanup_and_exit(int errcode);
 
@@ -93,7 +103,11 @@ usage(const char* argv0)
     fprintf(stderr, "Usage: %s [options]\n", xorp_basename(argv0));
     fprintf(stderr, "Options:\n");
     fprintf(stderr, "  -a <allowed host> Host allowed by the finder\n");
+    fprintf(stderr, "  -d        Run as a UNIX daemon (detach from tty)\n");
+    fprintf(stderr, "  -l <file> Log to file <file>\n");
+    fprintf(stderr, "  -L <facility.priority> Log to syslog facility\n");
     fprintf(stderr, "  -n <allowed net>  Subnet allowed by the finder\n");
+    fprintf(stderr, "  -P <pid>  Write process ID to file <pid>\n");
     fprintf(stderr, "  -h        Display this information\n");
     fprintf(stderr, "  -v        Print verbose information\n");
     fprintf(stderr, "  -b <file> Specify boot file\n");
@@ -104,6 +118,7 @@ usage(const char* argv0)
     fprintf(stderr, "  -q <secs> Set forced quit period\n");
     fprintf(stderr, "  -t <dir>  Specify templates directory\n");
     fprintf(stderr, "  -x <dir>  Specify Xrl targets directory\n");
+    fprintf(stderr, "  -d        Daemon mode, run in the background\n");
 }
 
 static void
@@ -117,11 +132,11 @@ display_defaults()
     fprintf(stderr, "  Xrl targets directory      := %s\n",
 	    xorp_xrl_targets_dir().c_str());
     fprintf(stderr, "  Execute Xrls               := %s\n",
-	    default_do_exec ? "true" : "false");
+	    bool_c_str(default_do_exec));
     fprintf(stderr, "  Restart failed processes   := %s\n",
-	    default_do_restart ? "true" : "false");
+	    bool_c_str(default_do_restart));
     fprintf(stderr, "  Print verbose information  := %s\n",
-	    default_verbose ? "true" : "false");
+	    bool_c_str(default_verbose));
 }
 
 // the following two functions are an ugly hack to cause the C code in
@@ -150,7 +165,8 @@ Rtrmgr::Rtrmgr(const string& template_dir,
 	       bool	do_exec,
 	       bool	do_restart,
 	       bool	verbose,
-	       int32_t quit_time)
+	       int32_t quit_time,
+	       bool daemon_mode)
     : _template_dir(template_dir),
       _xrl_targets_dir(xrl_targets_dir),
       _boot_file(boot_file),
@@ -160,6 +176,7 @@ Rtrmgr::Rtrmgr(const string& template_dir,
       _do_restart(do_restart),
       _verbose(verbose),
       _quit_time(quit_time),
+      _daemon_mode(daemon_mode),
       _ready(false),
       _mct(NULL)
 {
@@ -196,11 +213,11 @@ Rtrmgr::run()
     XLOG_TRACE(_verbose, "Xrl targets directory      := %s\n",
 	       xrl_targets_dir.c_str());
     XLOG_TRACE(_verbose, "Execute Xrls               := %s\n",
-	       do_exec ? "true" : "false");
+	       bool_c_str(do_exec));
     XLOG_TRACE(_verbose, "Restart failed processes   := %s\n",
-	       do_restart ? "true" : "false");
+	       bool_c_str(do_restart));
     XLOG_TRACE(_verbose, "Print verbose information  := %s\n",
-	       _verbose ? "true" : "false");
+	       bool_c_str(_verbose));
 
 
     XRLdb* xrldb = NULL;
@@ -282,6 +299,9 @@ Rtrmgr::run()
 
 	_mct = new MasterConfigTree(boot_file, tt, mmgr, xclient, _do_exec,
 				    _verbose);
+	if (_daemon_mode) {
+	    _mct->set_task_completed(callback(this, &Rtrmgr::daemonize));
+	}
 	//
 	// XXX: note that theoretically we may receive an XRL before
 	// we call XrlRtrmgrInterface::set_master_config_tree()
@@ -368,6 +388,83 @@ Rtrmgr::module_status_changed(const string& module_name,
     _xrt->module_status_changed(module_name, status);
 }
 
+void
+open_logfile()
+{
+    if (logfilename.empty()) {
+	fprintf(stderr, "Empty log filename specified\n");
+	return;
+    }
+    FILE* logfile;
+    if ((logfile = fopen(logfilename.c_str(), "a")) != NULL) {
+       xlog_add_output(logfile);
+    } else {
+	fprintf(stderr, "Failed to open log file %s\n", logfilename.c_str());
+    }
+}
+
+void
+open_syslog()
+{
+    if (syslogspec.empty()) {
+	fprintf(stderr, "Empty syslog spec\n");
+	return;
+    }
+    int retval = xlog_add_syslog_output(syslogspec.c_str());
+    if (retval == -1)
+	fprintf(stderr, "Failed to open syslog spec %s\n", logfilename.c_str());
+}
+
+void
+write_pidfile(pid_t thepid)
+{
+    if (pidfilename.empty()) {
+	fprintf(stderr, "Empty PID filename specified\n");
+	return;
+    }
+    FILE *pidfile;
+    if ((pidfile = fopen(pidfilename.c_str(), "w")) != NULL) {
+	fprintf(pidfile, "%u\n", thepid);
+	fclose(pidfile);
+    } else {
+	fprintf(stderr, "Failed to write pid file\n");
+    }
+}
+
+void
+Rtrmgr::daemonize()
+{
+    // If not daemonizing, do nothing.
+    if (! daemon_mode)
+	return;
+
+#ifndef HOST_OS_WINDOWS
+    // Daemonize the XORP process. Close open stdio descriptors,
+    // but don't chdir -- we need to stay where we're told to go.
+    int newpid = xorp_daemonize(DAEMON_NOCHDIR, DAEMON_CLOSE);
+    if (newpid == -1) {
+	fprintf(stderr, "Failed to start as a daemon process\n");
+	cleanup_and_exit(1);
+    }
+
+    // Make sure we open the pid file in the parent, and the
+    // log file in the child. Close fds but don't chdir.
+    if (newpid == 0) {
+	// We are now in the child.
+#if 0
+	if (do_logfile)
+	    open_logfile();
+#endif
+	return;
+    }
+
+    // We are in the parent. Write the PID file and exit.
+    if (do_pidfile)
+	write_pidfile(newpid);
+    _exit(0);
+#endif
+}
+
 int
 main(int argc, char* const argv[])
 {
@@ -399,8 +496,11 @@ main(int argc, char* const argv[])
     boot_file		= xorp_boot_file();
 
     int c;
-    while ((c = getopt(argc, argv, "a:n:t:b:x:i:p:q:Nrvh")) != EOF) {
+    while ((c = getopt(argc, argv, "da:l:L:n:t:b:x:i:P:p:q:Nrvh")) != EOF) {
 	switch(c) {
+	case 'd':
+	    daemon_mode = true;
+	    break;
 	case 'a':
 	    //
 	    // User is specifying an IPv4 address to accept finder
@@ -414,6 +514,14 @@ main(int argc, char* const argv[])
 		cleanup_and_exit(1);
 	    }
 	    break;
+	case 'l':
+	    do_logfile = true;
+	    logfilename = optarg;
+	    break;
+	case 'L':
+	    do_syslog = true;
+	    syslogspec = optarg;
+	    break;
 	case 'n':
 	    //
 	    // User is specifying a network address to accept finder
@@ -455,6 +563,10 @@ main(int argc, char* const argv[])
 	case 'v':
 	    verbose = true;
 	    break;
+	case 'P':
+	    do_pidfile = true;
+	    pidfilename = optarg;
+	    break;
 	case 'p':
 	    bind_port = static_cast<uint16_t>(atoi(optarg));
 	    if (bind_port == 0) {
@@ -492,11 +604,27 @@ main(int argc, char* const argv[])
 	}
     }
 
+    // If not daemonizing, open the pid file now.
+    if (! daemon_mode) {
+	if (do_pidfile)
+	    write_pidfile(getpid());
+    }
+
+    // Open the new log facility now so that all output, up to when we
+    // daemonize, will go to the specified log facility.
+    if (do_logfile || do_syslog)
+        xlog_remove_default_output();
+    if (do_logfile)
+	open_logfile();
+    if (do_syslog)
+	open_syslog();
+
     //
     // The main procedure
     //
     Rtrmgr rtrmgr(template_dir, xrl_targets_dir, boot_file, bind_addrs,
-		  bind_port, do_exec, do_restart, verbose, quit_time);
+		  bind_port, do_exec, do_restart, verbose, quit_time,
+		  daemon_mode);
     errcode = rtrmgr.run();
 
     cleanup_and_exit(errcode);
