@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/master_conf_tree.cc,v 1.77 2007/03/15 07:39:53 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/master_conf_tree.cc,v 1.81 2008/07/23 05:11:41 pavlin Exp $"
 
 #include "rtrmgr_module.h"
 
@@ -265,6 +265,12 @@ MasterConfigTree::config_done(bool success, string error_msg)
 	_config_failed_msg = error_msg2;
 	return;
     }
+
+    if ( !_task_completed_cb.is_empty()) {
+	_task_completed_cb->dispatch();
+	_task_completed_cb.release();
+    }
+
     debug_msg("MasterConfigTree::config_done returning\n");
 }
 
@@ -463,10 +469,10 @@ MasterConfigTree::order_module_list(const set<string>& module_set,
 	    if (satisfied.find(curr_iter->second) != satisfied.end()) {
 		// Rule is now satisfied.
 		string module = curr_iter->first;
-		depends.erase(curr_iter);
-		progress_made = true;
 		debug_msg("dependency of %s on %s satisfied\n",
 		       module.c_str(), curr_iter->second.c_str());
+		depends.erase(curr_iter);
+		progress_made = true;
 		if (depends.find(module) == depends.end()) {
 		    // This was the last dependency
 		    satisfied.insert(module);
