@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/xorp_client.cc,v 1.26 2007/02/16 22:47:27 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/xorp_client.cc,v 1.29 2008/07/23 05:11:46 pavlin Exp $"
 
 
 #include "rtrmgr_module.h"
@@ -136,6 +136,8 @@ XorpClient::fake_return_args(const string& xrl_return_spec)
 	case xrlatom_list:
 	case xrlatom_boolean:
 	case xrlatom_binary:
+	case xrlatom_uint64:
+	case xrlatom_int64:
 	    XLOG_UNFINISHED();
 	    break;
 	}
