@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/xorpsh_main.cc,v 1.66 2007/02/16 22:47:27 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/xorpsh_main.cc,v 1.71 2008/07/23 05:11:46 pavlin Exp $"
 
 #include "rtrmgr_module.h"
 
@@ -79,7 +79,8 @@ announce_waiting()
 }
 
 static bool
-wait_for_xrl_router_ready(EventLoop& eventloop, XrlRouter& xrl_router)
+wait_for_xrl_router_ready(EventLoop& eventloop, XrlRouter& xrl_router,
+			  bool exit_on_error)
 {
     XorpTimer announcer = eventloop.new_oneoff_after_ms(
 				3 * 1000, callback(&announce_waiting)
@@ -91,6 +92,8 @@ wait_for_xrl_router_ready(EventLoop& eventloop, XrlRouter& xrl_router)
 	    return false;
 	    break;
 	}
+	if (xrl_router.connect_failed() && exit_on_error)
+	    return false;
     }
     return true;
 }
@@ -153,7 +156,7 @@ XorpShell::XorpShell(EventLoop& eventloop,
     XLOG_TRACE(_verbose, "Xrl targets directory      := %s\n",
 	       xrl_targets_dir.c_str());
     XLOG_TRACE(_verbose, "Print verbose information  := %s\n",
-	       _verbose ? "true" : "false");
+	       bool_c_str(_verbose));
 
     // Read the router config template files
     _tt = new TemplateTree(xorp_root_dir, _verbose);
@@ -195,7 +198,7 @@ XorpShell::~XorpShell()
 }
 
 void
-XorpShell::run(const string& commands)
+XorpShell::run(const string& commands, bool exit_on_error)
 {
     bool success;
     string error_msg;
@@ -238,7 +241,8 @@ XorpShell::run(const string& commands)
     _cli_node.set_cli_client_delete_callback(callback(exit_handler));
 
     _is_connected_to_finder = false;
-    if (wait_for_xrl_router_ready(_eventloop, _xrl_router) == false) {
+    if (wait_for_xrl_router_ready(_eventloop, _xrl_router, exit_on_error)
+	== false) {
 	// RtrMgr contains finder
 	error_msg = c_format("Failed to connect to the router manager");
 	xorp_throw(InitError, error_msg);
@@ -797,6 +801,7 @@ usage(const char *argv0)
     fprintf(stderr, "Usage: %s [options]\n", xorp_basename(argv0));
     fprintf(stderr, "Options:\n");
     fprintf(stderr, "  -c        Specify command(s) to execute\n");
+    fprintf(stderr, "  -e        Exit immediately if cannot connect to the rtrmgr\n");
     fprintf(stderr, "  -h        Display this information\n");
     fprintf(stderr, "  -v        Print verbose information\n");
     fprintf(stderr, "  -t <dir>  Specify templates directory\n");
@@ -812,7 +817,7 @@ display_defaults()
     fprintf(stderr, "  Xrl targets directory      := %s\n",
 	    xorp_xrl_targets_dir().c_str());
     fprintf(stderr, "  Print verbose information  := %s\n",
-	    default_verbose ? "true" : "false");
+	    bool_c_str(default_verbose));
 }
 
 int
@@ -820,6 +825,7 @@ main(int argc, char *argv[])
 {
     int errcode = 0;
     string commands;
+    bool exit_on_error = false;
 
     //
     // Initialize and start xlog
@@ -848,10 +854,15 @@ main(int argc, char *argv[])
     string xrl_targets_dir	= xorp_xrl_targets_dir();
 
     int c;
-    while ((c = getopt(argc, argv, "c:t:x:vh")) != EOF) {
+    while ((c = getopt(argc, argv, "c:et:x:vh")) != EOF) {
 	switch(c) {
 	case 'c':
-	    commands = optarg;
+	    // XXX: Append the arguments to allow multiple "-c cmd" commands
+	    commands += optarg;
+	    commands += "\n";
+	    break;
+	case 'e':
+	    exit_on_error = true;
 	    break;
 	case 't':
 	    template_dir = optarg;
@@ -892,7 +903,7 @@ main(int argc, char *argv[])
 					   hostname);
 	XorpShell xorpsh(eventloop, xname, xorp_binary_root_dir(),
 			 template_dir, xrl_targets_dir, verbose);
-	xorpsh.run(commands);
+	xorpsh.run(commands, exit_on_error);
     } catch (const InitError& e) {
 	XLOG_ERROR("xorpsh exiting due to an init error: %s", e.why().c_str());
 	errcode = 1;
