@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/static_routes/static_routes_node.cc,v 1.36 2007/02/16 22:47:29 pavlin Exp $"
+#ident "$XORP: xorp/static_routes/static_routes_node.cc,v 1.40 2008/07/23 05:11:48 pavlin Exp $"
 
 //
 // StaticRoutes node implementation.
@@ -46,7 +46,7 @@ StaticRoutesNode::~StaticRoutesNode()
     shutdown();
 }
 
-bool
+int
 StaticRoutesNode::startup()
 {
     //
@@ -54,10 +54,10 @@ StaticRoutesNode::startup()
     //
     if ((ServiceBase::status() == SERVICE_STARTING)
 	|| (ServiceBase::status() == SERVICE_RUNNING)) {
-	return true;
+	return (XORP_OK);
     }
     if (ServiceBase::status() != SERVICE_READY) {
-	return false;
+	return (XORP_ERROR);
     }
 
     //
@@ -82,10 +82,10 @@ StaticRoutesNode::startup()
     //
     rib_register_startup();
 
-    return true;
+    return (XORP_OK);
 }
 
-bool
+int
 StaticRoutesNode::shutdown()
 {
     //
@@ -94,14 +94,14 @@ StaticRoutesNode::shutdown()
     if ((ServiceBase::status() == SERVICE_SHUTDOWN)
 	|| (ServiceBase::status() == SERVICE_SHUTTING_DOWN)
 	|| (ServiceBase::status() == SERVICE_FAILED)) {
-	return true;
+	return (XORP_OK);
     }
     if ((ServiceBase::status() != SERVICE_RUNNING)
 	&& (ServiceBase::status() != SERVICE_STARTING)
 	&& (ServiceBase::status() != SERVICE_PAUSING)
 	&& (ServiceBase::status() != SERVICE_PAUSED)
 	&& (ServiceBase::status() != SERVICE_RESUMING)) {
-	return false;
+	return (XORP_ERROR);
     }
 
     //
@@ -131,7 +131,7 @@ StaticRoutesNode::shutdown()
     //
     update_status();
 
-    return true;
+    return (XORP_OK);
 }
 
 void
@@ -317,7 +317,7 @@ StaticRoutesNode::updates_made()
 	    const IfMgrIfAtom* if_atom;
 	    const IfMgrVifAtom* vif_atom;
 
-	    if_atom = _iftree.find_if(static_route.ifname());
+	    if_atom = _iftree.find_interface(static_route.ifname());
 	    vif_atom = _iftree.find_vif(static_route.ifname(),
 					static_route.vifname());
 	    if ((if_atom != NULL) && (if_atom->enabled())
@@ -326,7 +326,7 @@ StaticRoutesNode::updates_made()
 		is_old_up = true;
 	    }
 
-	    if_atom = ifmgr_iftree().find_if(static_route.ifname());
+	    if_atom = ifmgr_iftree().find_interface(static_route.ifname());
 	    vif_atom = ifmgr_iftree().find_vif(static_route.ifname(),
 					       static_route.vifname());
 	    if ((if_atom != NULL) && (if_atom->enabled())
@@ -1152,7 +1152,7 @@ StaticRoutesNode::is_accepted_by_nexthop(const StaticRoute& route) const
 	const IfMgrVifAtom* vif_atom;
 	bool is_up = false;
 
-	if_atom = _iftree.find_if(route.ifname());
+	if_atom = _iftree.find_interface(route.ifname());
 	vif_atom = _iftree.find_vif(route.ifname(), route.vifname());
 	if ((if_atom != NULL) && (if_atom->enabled())
 	    && (! if_atom->no_carrier())
