@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2007 International Computer Science Institute
+// Copyright (c) 2001-2008 XORP, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/static_routes/xrl_static_routes_node.cc,v 1.37 2007/02/16 22:47:30 pavlin Exp $"
+#ident "$XORP: xorp/static_routes/xrl_static_routes_node.cc,v 1.42 2008/07/23 05:11:48 pavlin Exp $"
 
 #include "static_routes_module.h"
 
@@ -40,8 +40,6 @@ XrlStaticRoutesNode::XrlStaticRoutesNode(EventLoop&	eventloop,
 		   finder_port),
       XrlStaticRoutesTargetBase(&xrl_router()),
       _eventloop(eventloop),
-      _class_name(xrl_router().class_name()),
-      _instance_name(xrl_router().instance_name()),
       _xrl_rib_client(&xrl_router()),
       _finder_target(finder_target),
       _fea_target(fea_target),
@@ -73,13 +71,13 @@ XrlStaticRoutesNode::~XrlStaticRoutesNode()
     _ifmgr.unset_observer(dynamic_cast<StaticRoutesNode*>(this));
 }
 
-bool
+int
 XrlStaticRoutesNode::startup()
 {
     return StaticRoutesNode::startup();
 }
 
-bool
+int
 XrlStaticRoutesNode::shutdown()
 {
     return StaticRoutesNode::shutdown();
@@ -141,7 +139,7 @@ XrlStaticRoutesNode::fea_register_startup()
     // Register interest in the FEA with the Finder
     //
     success = _xrl_finder_client.send_register_class_event_interest(
-	_finder_target.c_str(), _instance_name, _fea_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _fea_target,
 	callback(this, &XrlStaticRoutesNode::finder_register_interest_fea_cb));
 
     if (! success) {
@@ -172,7 +170,7 @@ XrlStaticRoutesNode::finder_register_interest_fea_cb(const XrlError& xrl_error)
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot register interest in finder events: %s",
+	XLOG_FATAL("Cannot register interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -181,7 +179,7 @@ XrlStaticRoutesNode::finder_register_interest_fea_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -206,7 +204,7 @@ XrlStaticRoutesNode::finder_register_interest_fea_cb(const XrlError& xrl_error)
 	// (unless the timer is already running).
 	//
 	if (! _fea_register_startup_timer.scheduled()) {
-	    XLOG_ERROR("Failed to register interest in finder events: %s. "
+	    XLOG_ERROR("Failed to register interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _fea_register_startup_timer = _eventloop.new_oneoff_after(
@@ -247,7 +245,7 @@ XrlStaticRoutesNode::fea_register_shutdown()
     // De-register interest in the FEA with the Finder
     //
     success = _xrl_finder_client.send_deregister_class_event_interest(
-	_finder_target.c_str(), _instance_name, _fea_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _fea_target,
 	callback(this, &XrlStaticRoutesNode::finder_deregister_interest_fea_cb));
 
     if (! success) {
@@ -285,7 +283,7 @@ XrlStaticRoutesNode::finder_deregister_interest_fea_cb(
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot deregister interest in finder events: %s",
+	XLOG_FATAL("Cannot deregister interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -294,7 +292,7 @@ XrlStaticRoutesNode::finder_deregister_interest_fea_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -320,7 +318,7 @@ XrlStaticRoutesNode::finder_deregister_interest_fea_cb(
 	// (unless the timer is already running).
 	//
 	if (! _fea_register_shutdown_timer.scheduled()) {
-	    XLOG_ERROR("Failed to deregister interest in finder events: %s. "
+	    XLOG_ERROR("Failed to deregister interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _fea_register_shutdown_timer = _eventloop.new_oneoff_after(
@@ -361,7 +359,7 @@ XrlStaticRoutesNode::rib_register_startup()
     // Register interest in the RIB with the Finder
     //
     success = _xrl_finder_client.send_register_class_event_interest(
-	_finder_target.c_str(), _instance_name, _rib_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _rib_target,
 	callback(this, &XrlStaticRoutesNode::finder_register_interest_rib_cb));
 
     if (! success) {
@@ -393,7 +391,7 @@ XrlStaticRoutesNode::finder_register_interest_rib_cb(const XrlError& xrl_error)
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot register interest in finder events: %s",
+	XLOG_FATAL("Cannot register interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -402,7 +400,7 @@ XrlStaticRoutesNode::finder_register_interest_rib_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -427,7 +425,7 @@ XrlStaticRoutesNode::finder_register_interest_rib_cb(const XrlError& xrl_error)
 	// (unless the timer is already running).
 	//
 	if (! _rib_register_startup_timer.scheduled()) {
-	    XLOG_ERROR("Failed to register interest in finder events: %s. "
+	    XLOG_ERROR("Failed to register interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _rib_register_startup_timer = _eventloop.new_oneoff_after(
@@ -471,7 +469,7 @@ XrlStaticRoutesNode::rib_register_shutdown()
     // De-register interest in the RIB with the Finder
     //
     success = _xrl_finder_client.send_deregister_class_event_interest(
-	_finder_target.c_str(), _instance_name, _rib_target,
+	_finder_target.c_str(), xrl_router().instance_name(), _rib_target,
 	callback(this, &XrlStaticRoutesNode::finder_deregister_interest_rib_cb));
 
     if (! success) {
@@ -505,7 +503,7 @@ XrlStaticRoutesNode::finder_deregister_interest_rib_cb(
 	// If a command failed because the other side rejected it, this is
 	// fatal.
 	//
-	XLOG_FATAL("Cannot deregister interest in finder events: %s",
+	XLOG_FATAL("Cannot deregister interest in Finder events: %s",
 		   xrl_error.str().c_str());
 	break;
 
@@ -514,7 +512,7 @@ XrlStaticRoutesNode::finder_deregister_interest_rib_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -540,7 +538,7 @@ XrlStaticRoutesNode::finder_deregister_interest_rib_cb(
 	// (unless the timer is already running).
 	//
 	if (! _rib_register_shutdown_timer.scheduled()) {
-	    XLOG_ERROR("Failed to deregister interest in finder events: %s. "
+	    XLOG_ERROR("Failed to deregister interest in Finder events: %s. "
 		       "Will try again.",
 		       xrl_error.str().c_str());
 	    _rib_register_shutdown_timer = _eventloop.new_oneoff_after(
@@ -566,8 +564,8 @@ XrlStaticRoutesNode::send_rib_add_tables()
 	success = _xrl_rib_client.send_add_igp_table4(
 	    _rib_target.c_str(),
 	    StaticRoutesNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    true,	/* unicast */
 	    true,	/* multicast */
 	    callback(this,
@@ -583,8 +581,8 @@ XrlStaticRoutesNode::send_rib_add_tables()
 	success = _xrl_rib_client.send_add_igp_table6(
 	    _rib_target.c_str(),
 	    StaticRoutesNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    true,	/* unicast */
 	    true,	/* multicast */
 	    callback(this,
@@ -635,7 +633,7 @@ XrlStaticRoutesNode::rib_client_send_add_igp_table4_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -699,7 +697,7 @@ XrlStaticRoutesNode::rib_client_send_add_igp_table6_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -751,8 +749,8 @@ XrlStaticRoutesNode::send_rib_delete_tables()
 	success4 = _xrl_rib_client.send_delete_igp_table4(
 	    _rib_target.c_str(),
 	    StaticRoutesNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    true,	/* unicast */
 	    true,	/* multicast */
 	    callback(this, &XrlStaticRoutesNode::rib_client_send_delete_igp_table4_cb));
@@ -768,8 +766,8 @@ XrlStaticRoutesNode::send_rib_delete_tables()
 	success6 = _xrl_rib_client.send_delete_igp_table6(
 	    _rib_target.c_str(),
 	    StaticRoutesNode::protocol_name(),
-	    _class_name,
-	    _instance_name,
+	    xrl_router().class_name(),
+	    xrl_router().instance_name(),
 	    true,	/* unicast */
 	    true,	/* multicast */
 	    callback(this, &XrlStaticRoutesNode::rib_client_send_delete_igp_table6_cb));
@@ -813,7 +811,7 @@ XrlStaticRoutesNode::rib_client_send_delete_igp_table4_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -877,7 +875,7 @@ XrlStaticRoutesNode::rib_client_send_delete_igp_table6_cb(
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
@@ -927,7 +925,7 @@ XrlStaticRoutesNode::common_0_1_get_target_name(
     // Output values, 
     string&	name)
 {
-    name = my_xrl_target_name();
+    name = xrl_router().class_name();
 
     return XrlCmdError::OKAY();
 }
@@ -967,7 +965,7 @@ XrlStaticRoutesNode::common_0_1_shutdown()
 {
     string error_msg;
 
-    if (shutdown() != true) {
+    if (shutdown() != XORP_OK) {
 	error_msg = c_format("Failed to shutdown StaticRoutes");
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
@@ -994,7 +992,7 @@ XrlStaticRoutesNode::finder_event_observer_0_1_xrl_target_birth(
 	// IfMgrHintObserver::tree_complete() will be called.
 	//
 	_is_fea_alive = true;
-	if (_ifmgr.startup() != true) {
+	if (_ifmgr.startup() != XORP_OK) {
 	    StaticRoutesNode::ServiceBase::set_status(SERVICE_FAILED);
 	    StaticRoutesNode::update_status();
 	}
@@ -1986,7 +1984,7 @@ XrlStaticRoutesNode::send_rib_route_change_cb(const XrlError& xrl_error)
     case SEND_FAILED:
 	//
 	// A communication error that should have been caught elsewhere
-	// (e.g., by tracking the status of the finder and the other targets).
+	// (e.g., by tracking the status of the Finder and the other targets).
 	// Probably we caught it here because of event reordering.
 	// In some cases we print an error. In other cases our job is done.
 	//
