@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001-2007 International Computer Science Institute
+# Copyright (c) 2001-2008 XORP, Inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 # notice is a summary of the XORP LICENSE file; the license in that file is
 # legally binding.
 
-# $XORP: xorp/tests/bgp/test_bgp_config.py,v 1.19 2007/02/16 22:47:32 pavlin Exp $
+# $XORP: xorp/tests/bgp/test_bgp_config.py,v 1.23 2008/07/23 05:11:50 pavlin Exp $
 
 import sys
 sys.path.append("..")
@@ -207,6 +207,42 @@ set peer-port 20001
 set next-hop 127.0.0.1
 set local-ip 127.0.0.1
 set as 75
+commit
+"""
+
+    if not xorpsh(builddir, xorpsh_commands):
+        return False
+
+    return True
+
+def conf_bug_740(builddir):
+    """
+    Configure an import and an export policy
+    """
+
+    # Configure the xorpsh
+    xorpsh_commands = \
+"""configure
+load empty.boot
+
+create policy policy-statement out term a from protocol connected
+create policy policy-statement in term a from neighbor 10.1.2.18..10.1.2.18
+
+create protocol bgp
+edit protocol bgp
+set bgp-id 1.2.3.4
+set local-as 65001
+set export "out"
+set import "in"
+
+create peer peer1
+edit peer peer1
+set local-port 10001
+set peer-port 20001
+set next-hop 127.0.0.1
+set local-ip 127.0.0.1
+set as 75
+
 commit
 """
 
@@ -246,6 +282,56 @@ set next-hop 127.0.0.1
 set local-ip 127.0.0.1
 set as 65002
 
+commit
+"""
+
+    if not xorpsh(builddir, xorpsh_commands):
+        return False
+
+    return True
+
+def conf_test_dump_1(builddir):
+    """
+    Configure two EBGP peerings
+    """
+
+    # Configure the xorpsh
+    xorpsh_commands = \
+"""configure
+load empty.boot
+create protocol bgp
+edit protocol bgp
+set bgp-id 1.2.3.4
+set local-as 65010
+
+create peer peer1
+edit peer peer1
+set local-port 10001
+set peer-port 20001
+set next-hop 127.0.0.1
+set local-ip 127.0.0.1
+set as 65000
+up
+
+create peer peer2
+edit peer peer2
+set local-port 10002
+set peer-port 20002
+set next-hop 127.0.0.1
+set local-ip 127.0.0.1
+set as 65001
+up
+
+up
+up
+
+create protocols static
+edit protocols static
+create route 192.150.187.0/24
+edit route 192.150.187.0/24
+set next-hop 127.0.0.1
+up
+
 commit
 """
 
