@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001-2007 International Computer Science Institute
+# Copyright (c) 2001-2008 XORP, Inc.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 # notice is a summary of the XORP LICENSE file; the license in that file is
 # legally binding.
 
-# $XORP: xorp/tests/test_main.py,v 1.3 2007/02/16 22:47:30 pavlin Exp $
+# $XORP: xorp/tests/test_main.py,v 1.6 2008/07/23 05:11:49 pavlin Exp $
 
 # A common main to be used by all test programs plus the entry point into
 # running the tests.
@@ -21,6 +21,7 @@ import getopt
 import sys
 import os
 import time
+import re
 from test_builddir import builddir
 from test_call_xrl import call_xrl
 
@@ -66,7 +67,7 @@ def delay(seconds):
         raise Exception, 'Delay was too short should have been %s was %s' % \
               (seconds, slept)
 
-def coord(command):
+def coord(command, noblock=False):
     """
     Send a command to the coordinator
     """
@@ -74,6 +75,9 @@ def coord(command):
     print command
     status, message = call_xrl(builddir(1), "finder://coord/coord/0.1/command?command:txt=%s" % command)
     if 0 != status:
+        if noblock:
+            print message
+            return
         raise Exception, message
 
     # Wait up to five seconds for this command to complete
@@ -95,6 +99,18 @@ def pending():
     else:
         return True
 
+def status(peer):
+    """
+    Get the status of a test peer.
+    """
+
+    status, message = call_xrl(builddir(1), "finder://coord/coord/0.1/status?peer:txt=" + peer)
+
+    message = re.sub('^status:txt=', '', message)
+    message = re.sub('\+', ' ', message)
+
+    return message
+
 def run_test(test, single, configure, TESTS, config_module, test_module):
     """
     Run the provided test
